// A minimal program that writes all pages it gets to disk.
// Adapted to XdTV. You can choose several options you will

#include <stdio.h>
#include <stdlib.h>
#include <locale.h>
#include <string.h>

#include "vt.h"
#include "fdset.h"
#include "vbi.h"
#include "export.h"

int debug = 0;
int vbifd = -1;

/* dummy command to allow compilation... */
int do_command_single(char * cmd) {return 0;}

static void
usage(FILE *fp, int exitval)
{
    fprintf(fp, "\nUsage: %s [options] ppp[.ss]...\n", prgname);
    fprintf(fp,
	    "\n"
	    "Capture all Teletext pages on the current TV station\n"
	    "\n"
	    "  Valid options:\t\t\t\t\t\tDefault:\n"
	    "    --help or -h\n"
	    "    --version or -v\n"
	    "    --copyright or -c\n"
	    "    -vbi or -dev <vbidev>\t\t\t\t\t/dev/vbi\n"
	    "    -finetune or -f <-4..4|auto>\t\t\t\t0\n"
	    "    -charset or -latin latin-1/2/koi8-r/koi/iso8859-7/el\tlatin-1\n"
	    "    -format or -fmt <fmt[,options]>\t\t\t\tascii\n"
	    "    -format help\n"
	    "    -[no]fixvbioffset or -[n]fvbio\t\t\t\tenable\n"
#ifdef HAVE_DVB
	    "    -progname name\n"
	    "    -pid pid\n"
	    "\n"
	    "  To use the DVB interface, use\n"
	    "  -vbi /dev/dvb/adapter0/dvr0\n"
	    "\n"
	    "  The -progname or -pid option specify\n"
	    "  the program or teletext PID if using\n"
	    "  the DVB interface.\n"
#endif
	);
    exit(exitval);
}

static void
exp_help(FILE *fp)
{
    struct export_module **ep;
    char **cp, c;

    fprintf(fp,
	    "\nSyntax: -format Name[,Options]\n"
	    "\n"
	    "    Name\tExt.\tOptions\n"
	    "    --------------------------------\n"
	);
    for (ep = modules; *ep; ep++)
    {
	fprintf(fp, "    %-7s\t.%-4s", (*ep)->fmt_name, (*ep)->extension);
	for (c = '\t', cp = (*ep)->options; cp && *cp; cp++, c = ',')
	    fprintf(fp, "%c%s", c, *cp);
	fprintf(fp, "\n");
    }
    fprintf(fp,
	    "\n"
	    "Common options: reveal,hide\n"
	    "\n"
	    "Example: -format ansi,reveal,bg=none\n"
	    "\n"
	);
    exit(0);
}

static void
event(void *fmt, struct vt_event *ev)
{
    if (ev->type == EV_PAGE)
    {
	struct vt_page *vtp = ev->p1;
	char *name = export_mkname(fmt, "ttext-%p.%e", vtp, "");
	
	export(fmt, vtp, name);
	free(name);
    }
}

static int
option(int argc, char **argv, int *ind, char **arg)
{
    static struct { char *nam, *altnam; int arg; } opts[] = {
	{ "--help", "-h", 0 },
	{ "--version", "-v", 0 },
	{ "--copyright", "-c", 0 },
	{ "-vbi", "-dev", 1 },
	{ "-finetune", "-f", 1 },
	{ "-charset", "-latin", 1 },
	{ "-format", "-fmt", 1 },
	{ "-fixvbioffset", "-fvbio", 0 },
	{ "-nofixvbioffset", "-nfvbio", 0 },
#ifdef HAVE_DVB
	{ "-progname", "-pn", 1 },
	{ "-pid", "--pid", 1 },
#endif
    };
    int i;

    if (*ind >= argc)
	return 0;

    *arg = argv[(*ind)++];
    for (i = 0; i < NELEM(opts); ++i)
	if (streq(*arg, opts[i].nam) || streq(*arg, opts[i].altnam))
	{
	    if (opts[i].arg) {
		if (*ind < argc)
		    *arg = argv[(*ind)++];
		else
		    fatal("option %s requires an argument", *arg);
	    }
	    return i+1;
	}

    if (**arg == '-')
    {
	fatal("%s: invalid option", *arg);
	usage(stderr, 2);
    }

    return -1;
}

int
main(int argc, char **argv)
{
    char *vbi_name = "/dev/vbi";
    int fine_tune = 1;
    int opt, ind;
    char *arg;
    struct vbi *vbi;
    char *out_fmt = "ascii";
    struct export *fmt = 0;
    char *progname = NULL;
    int txtpid = 0;

    setlocale (LC_CTYPE, "");
    setprgname(argv[0]);
    fdset_init(fds);

    ind = 1;
    while ( (opt = option(argc, argv, &ind, &arg)) )
	switch (opt)
	{
	    case 1:	// help
		usage(stdout, 0);
		break;
	    case 2:	// version
		printf("AleVT-Capall Version "VERSION" adapted to xdTV\nA simple capture software for all teletext pages\n");
		exit(0);
	    case 3:	// copyright
		printf("Copyright 2000 by E. Toernig, froese@gmx.de\n");
		exit(0);
	    case 4:	// vbi
		vbi_name = arg;
		break;
	    case 5:	// finetune
		if (streq(arg, "auto"))
		    fine_tune = 999;
		else
		    fine_tune = strtol(arg, 0, 10);
		break;
            case 6:    // charset
		if (streq(arg, "latin-1") || streq(arg, "1"))
		    latin1 = 1;
		else if (streq(arg, "latin-2") || streq(arg, "2"))
		    latin1 = 0;
		else if (streq(arg, "koi8-r") || streq(arg, "koi"))
		    latin1 = KOI8;
		else if (streq(arg, "iso8859-7") || streq(arg, "el"))
		    latin1 = GREEK;
		else
		    fatal("bad charset (not latin-1/2/koi8-r/koi/iso8859-7/el)");
		break;
	    case 7:	// format
		if (streq(arg, "help") || streq(arg, "?") || streq(arg, "list"))
		    exp_help(stdout);
		out_fmt = arg;
		fmt = 0;
		break;
	    case 8:	// Fix The wrong vbi offset value for kernel >=2.6.16
        	fixvbioffset = 1;
		break;
	    case 9:	// Keep the default vbi offset driver value
        	fixvbioffset = 0;
		break;
#ifdef HAVE_DVB
  	    case 10:    // DVB progname
		progname = arg;
		break;
  	    case 11:    // DVB pid
		txtpid = strtoul(arg, NULL, 0);
		break;
#endif
	    case -1:	// non-option arg
		break;
	}
    
    fmt = export_open(out_fmt);
          
    if (not(vbi = vbi_open(vbi_name, 0, fine_tune, -1 , progname, txtpid))) // Does it works?????? To test!!
	fatal("cannot open %s", vbi_name);
    
    vbi_add_handler(vbi, event, fmt);    
    
    if (!fmt || !vbi) exit(1);

    for (;;)
	fdset_select(fds, -1);
}
