/*****************************************************************************
 * channel.c: channel for Bt848 frame grabber driver
 *****************************************************************************
 * $Id: channel.c,v 1.138 2006/12/26 16:06:01 estre Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 * Sun 09 Nov 2003 07:20:47 PM EET Lucian Langa
 *	Added eventmap rc file config
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************
 *
 *  Original code:
 *  channel for Bt848 frame grabber driver
 *
 *  Copyright (C) 1996,97 Marcus Metzler (mocm@thp.uni-koeln.de)
 *
 *  many changes by Gerd Knorr <kraxel@cs.tu-berlin.de>
 *      [ hmm, think by now nearly nothing left from the original code ... ]
 *
 *****************************************************************************/

#include "config.h"

#include "channel.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#ifndef NO_X11
# include <X11/StringDefs.h>
# include <X11/Xaw/XawInit.h>
# include <X11/Xaw/Command.h>
#endif

#include "grab.h"
#include "x11.h"
#include "channels.h"
#include "channel.h"
#include "mixer.h"
#include "audio.h"
#include "memcpy.h"
#include "deinterlace.h"
#include "divx_ui.h"
#include "subtitles_ui.h"
#include "grab_ui.h"
#include "divx.h"
#include "alevtparams_ui.h"
#include "xosdparams_ui.h"
#include "x11params_ui.h"
#include "blackborder.h"
#include "commands.h"
#include "vop.h"

#include "event.h"

#ifdef HAVE_DVB
#include "dvbtools.h"
#endif

/* ----------------------------------------------------------------------- */
/* misc common stuff, not only channel related                             */

struct CHANNEL defaults = {
  "defaults",        // name
  NULL,              // key
  "33",              // cname
  0,                 // channel
  0,                 // fine
  0,                 // freq
  CAPTURE_GRABDISPLAY,   // capture
  0,                 // source
  2,                 // norm
  19000,             // color
  34000,             // bright
  32768,             // hue
  28000,             // contrast
  0,                 // pixmap
  0,                 // button
  0,                 // ckey
  DEINTERLACE_LINEARBLEND,  // deinterlace
  "888",             // subpage        
  ANALOG	     // channel type 
#ifdef HAVE_DVB
  ,	  
  /* DVB Part */
  0,		     // dvb frequency
  0,		     // inversion
  0,		     // bandwidth
  0,  		     // FEC high
  0,		     // FEC low
  0,		     // constellation
  0,		     // transmission
  0,		     // guard interval
  0,		     // hierarchy
  0,		     // video pid
  0,		     // audio pid
  0		     // subtitle pid
#endif
};

struct CHANNEL **channels = NULL;

int count = 0;

int cur_sender = -1, cur_channel = 5, cur_fine = 0;
int cur_norm = -1, cur_input = -1, cur_freq = -1;
char *cur_subpage;

int cur_maxwidth = 768, cur_maxheight = 576, cur_secam = 0;
int cur_maxfpsnum = 25, cur_maxfpsden = 1;
int chan_tab = 4;


extern char user_dir[255];
extern int have_config;
extern struct GRABBER *grabbers[];
extern int grabber;
extern int fs_width, fs_height;
extern int fs_xoff, fs_yoff;
extern int pix_width, pix_height;
extern int owidth, oheight;
extern int wwidth, wheight;
extern int width_capture_max, height_capture_max;
extern struct STRTAB deinterlace_list[];
extern struct STRTAB subpageno_list[];
extern int deinterlace_method;
extern void set_deinterlace(int j);
extern int xawpopup;
void set_blackborder (int j);
extern int message_timer;
extern int decoration; 
#ifdef HAVE_XV
extern int cmdline_xv_img_port;
#endif

//#ifdef USE_LIBXOSD <= xosd inside now
extern char *font, *font_vtx, *colour, *outline_colour;
extern int shadow_offset, outline_offset;
extern char main_font[50], main_size[50];
extern char vtx_font[50], vtx_size[50];
extern int libxosd;
//#endif <= xosd inside now
#ifndef NO_X11
extern Display *dpy;
void button_cb (Widget widget, XtPointer clientdata, XtPointer call_data);

extern Widget app_shell, chan_box, chan_viewport, tv, opt_paned, c_stayontop, c_decoration;
#endif

extern divx_t divx;
#ifdef HAVE_FFMPEG
extern ffmpeg_t ffmpeg;
#endif
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
extern xvid_t xvid;
#endif
#endif
extern char *pplayer;
extern int stay;
extern int subactive;
extern int switch_method;
extern char *filepath;
#ifdef HAVE_LIBJPEG
extern int jpegpcent;
#endif

extern char* current_theme;


const struct STRTAB channel_type_list[] = {
  {ANALOG,"ANALOG"},
  {DVB_T,"DVB_T"},
  {-1, NULL}
};


extern int memcpy_method;
struct STRTAB memcpy_list[] = {
  {MEMCPY_PROBE,      "probe"},
  {MEMCPY_GLIBC,      "glibc"},
  {MEMCPY_KERNEL,     "kernel"},
  {MEMCPY_FPU_512,    "FPU_512"},
  {MEMCPY_FPU_1K,     "FPU_1K"},
  {MEMCPY_FPU_2K,     "FPU_2K"},
  {MEMCPY_FPU_4K,     "FPU_4K"},
  {MEMCPY_FPU_8K,     "FPU_8K"},
  {MEMCPY_FPU_16K,    "FPU_16K"},
  {MEMCPY_FPU_32K,    "FPU_32K"},
  {MEMCPY_MMX,        "mmx"},
  {MEMCPY_MMX_512,    "mmx_512"},
  {MEMCPY_MMX_1K,     "mmx_1K"},
  {MEMCPY_MMX_2K,     "mmx_2K"},
  {MEMCPY_MMX_4K,     "mmx_4K"},
  {MEMCPY_MMX_8K,     "mmx_8K"},
  {MEMCPY_MMX_16K,    "mmx_16K"},
  {MEMCPY_MMX_32K,    "mmx_32K"},
  {MEMCPY_MMXEXT,     "mmxext"},
  {MEMCPY_MMXEXT_1K,  "mmxext_1K"},
  {MEMCPY_MMXEXT_2K,  "mmxext_2K"},
  {MEMCPY_MMXEXT_4K,  "mmxext_4K"},
  {MEMCPY_MMXEXT_8K,  "mmxext_8K"},
  {MEMCPY_MMXEXT_16K, "mmxext_16K"},
  {MEMCPY_MMXEXT_32K, "mmxext_32K"},
  {MEMCPY_MMXEXT_64K, "mmxext_64K"},
  {MEMCPY_SSE,        "sse"},
  {MEMCPY_SSE_1K,     "sse_1K"},
  {MEMCPY_SSE_2K,     "sse_2K"},
  {MEMCPY_SSE_4K,     "sse_4K"},
  {MEMCPY_SSE_8K,     "sse_8K"},
  {MEMCPY_SSE_16K,    "sse_16K"},
  {MEMCPY_SSE_32K,    "sse_32K"},
  {MEMCPY_SSE_64K,    "sse_64K"},
  {-1, NULL}
};

/* ----------------------------------------------------------------------- */
void save_config (void);
/* ----------------------------------------------------------------------- */

int
lookup_channel (char *channel)
{
  int i, nr1, nr2;
  char tag1[5], tag2[5];

  if (isdigit (channel[0]))
    {
      tag1[0] = 0;
      nr1 = atoi (channel);
    }
  else
    {
      sscanf (channel, "%4[A-Za-z]%d", tag1, &nr1);
    }

  for (i = 0; i < CHAN_ENTRIES; i++)
    {
      if (isdigit (tvtuner[i].name[0]))
        {
          tag2[0] = 0;
          nr2 = atoi (tvtuner[i].name);
        }
      else
        {
          sscanf (tvtuner[i].name, "%4[A-Za-z]%d", tag2, &nr2);
        }

      /* First simply test for name equality */
      if (strcmp(channel, tvtuner[i].name) == 0)
        break;

      /* If it doesn't match, try to make a guess */
      if (tag1[0] && tag2[0])
        if (nr1 == nr2 && 0 == strcmp (tag1, tag2))
          break;
      if (!tag1[0] && !tag2[0])
        if (nr1 == nr2)
          break;
    }
  if (i == CHAN_ENTRIES)
    return -1;

  return i;
}

int
get_freq (int i)
{
  if (!tvtuner[i].freq[chan_tab])
    return -1;
  return tvtuner[i].freq[chan_tab] * 16 / 1000;
}

int
cf2freq (char *name, int fine)
{
  int i;

  if (-1 == (i = lookup_channel (name)))
    return -1;
  return get_freq (i) + fine;
}

void
get_font_infos (char *font, char *font_type, char *font_size)
{
char *tok;
int i = 0;
char *cmd;
cmd = malloc(strlen(font)+1);
strcpy(cmd, font);
tok = strtok(cmd,"-"); 
  while(tok != NULL) 
  {
    i = i + 1;
    if (i == 2) { strcpy(font_type,tok); }
    if (i == 8) { strcpy(font_size,tok); }
    tok = strtok(NULL,"-");
  }
}


/* ----------------------------------------------------------------------- */

struct STRTAB captab[] = {
  {CAPTURE_OFF, "off"},
  {CAPTURE_OFF, "no"},
  {CAPTURE_OFF, "false"},
  {CAPTURE_OVERLAY, "on"},
  {CAPTURE_OVERLAY, "yes"},
  {CAPTURE_OVERLAY, "true"},
  {CAPTURE_OVERLAY, "overlay"},
  {CAPTURE_GRABDISPLAY, "grab"},
  {CAPTURE_GRABDISPLAY, "grabdisplay"},
  {-1, NULL}
};

/* just malloc memory for a new channel ... */
struct CHANNEL *
add_channel (char *name)
{
  static int alloc_count = 0;
  struct CHANNEL *channel;

  if (alloc_count == count)
    {
      alloc_count += 16;
      if (alloc_count == 16)
        channels = malloc (sizeof (struct CHANNEL *) * alloc_count);
      else
        channels =
          realloc (channels, sizeof (struct CHANNEL *) * alloc_count);
    }
  channel = channels[count++] = malloc (sizeof (struct CHANNEL));
  memcpy (channel, &defaults, sizeof (struct CHANNEL));
  channel->name = strdup (name);
  return channel;
}

/* add channel to the list and copy parameters which are not the defaults one ...*/
struct CHANNEL *
add_channel_copy (struct CHANNEL* src)
{
  struct CHANNEL *channel=NULL;
  if (!src) return NULL;
  
  channel=add_channel(src->name);
  
  if (src->channel && defaults.channel != src->channel) channel->channel = src->channel;
  if (src->freq && defaults.freq != src->freq) channel->freq = src->freq;
  if (src->fine && defaults.fine != src->fine) channel->fine = src->fine;
  if (src->capture && defaults.capture != src->capture) channel->capture = src->capture;
  if (src->source && defaults.source != src->source) channel->source = src->source;
  if (src->norm && defaults.norm != src->norm) channel->norm = src->norm;
  if (src->color && defaults.color != src->color) channel->color = src->color;
  if (src->bright && defaults.bright != src->bright) channel->bright = src->bright;
  if (src->hue && defaults.hue != src->hue) channel->hue = src->hue;
  if (src->contrast && defaults.contrast != src->contrast) channel->contrast = src->contrast;
  if (src->ckey && defaults.ckey != src->ckey) channel->ckey = src->ckey;
  if (src->deinterlace && defaults.deinterlace != src->deinterlace) channel->deinterlace = src->deinterlace;
  if (src->channel_type && defaults.channel_type != src->channel_type) channel->channel_type = src->channel_type;
#ifdef HAVE_DVB
  if (channel->channel_type == DVB_T){
  	channel->dvb_freq  = src->dvb_freq ;
  	channel->inversion  = src->inversion ;
  	channel->bandwidth  = src->bandwidth ;
  	channel->code_rate_high  = src->code_rate_high ;
  	channel->code_rate_low  = src->code_rate_low ;
  	channel->constellation  = src->constellation ;
  	channel->transmission_mode  = src->transmission_mode ;
  	channel->guard_interval  = src->guard_interval ;
  	channel->hierarchy  = src->hierarchy ;
  	channel->vpid  = src->vpid ;
  	channel->apid  = src->apid ;
  	channel-> spid = src->spid ;
  }
#endif // HAVE_DVB
  if (src->key) {
	  if (NULL == defaults.key) // au cas ou on change les valeurs par defaut ...
  		channel->key=strdup(src->key);
	  else if (0 != strcmp(src->key,defaults.key))
  		channel->key=strdup(src->key);
  }
  if (src->cname){
	  if (NULL == defaults.cname) // au cas ou on change les valeurs par defaut ...
  		channel->cname=strdup(src->cname);
	  else if (0 != strcmp(src->cname,defaults.cname))
  		channel->cname=strdup(src->cname);
  }
  if (src->subpage){
	  if (NULL == defaults.subpage) // au cas ou on change les valeurs par defaut ...
  		channel->subpage=strdup(src->subpage);
	  else if (0 != strcmp(src->subpage,defaults.subpage))
  		channel->subpage=strdup(src->subpage);
  }

  if (channel->channel_type == ANALOG)
  	channel->channel = lookup_channel (channel->cname); // in case of wrong src->channel ...

  configure_channel(channel); // to create button/pixmap ...

  return channel;
}
#ifndef NO_X11
void
hotkey_channel (struct CHANNEL *channel)
{
  char str[100], key[32], ctrl[16];

  if (NULL == channel->key)
    return;
  if (2 == sscanf (channel->key, "%15[A-Za-z0-9_]+%31[A-Za-z0-9_]",
                   ctrl, key))
    sprintf (str, "%s<Key>%s: SetChannel(\"%s\")", ctrl, key, channel->name);
  else
    sprintf (str, "<Key>%s: SetChannel(\"%s\")", channel->key, channel->name);
  XtOverrideTranslations (tv, XtParseTranslationTable (str));
  XtOverrideTranslations (opt_paned, XtParseTranslationTable (str));
  XtOverrideTranslations (chan_viewport, XtParseTranslationTable (str));
}
#endif

/* ... and initalize later */
void
configure_channel (struct CHANNEL *channel)
{
  channel->freq = get_freq (channel->channel) + channel->fine;
#ifndef NO_X11
  channel->button =
    XtVaCreateManagedWidget (channel->name,
                             commandWidgetClass, chan_box,
                             XtNwidth, pix_width,
                             XtNheight, pix_height, NULL);
  XtAddCallback (channel->button, XtNcallback, button_cb,
                 (XtPointer *) channel);
  hotkey_channel (channel);
#endif
}

/* delete channel */
void
del_channel (int i)
{
#ifndef NO_X11
  if(channels[i]->button) XtDestroyWidget (channels[i]->button);
#endif
  free (channels[i]->name);
  if (channels[i]->key)
    free (channels[i]->key);
  free (channels[i]);
  count--;
  if (i < count)
    memmove (channels + i, channels + i + 1,
             (count - i) * sizeof (struct CHANNEL *));
}


void
update_channel(struct CHANNEL *dest, struct CHANNEL *src)
{
  Pixmap p;
  Widget b;
  int ck;

  p = dest->pixmap;
  b = dest->button;
  ck = dest->ckey;
  memcpy(dest, src, sizeof(struct CHANNEL));
  dest->pixmap = p;
  dest->button = b;
  dest->ckey = ck;
}

void
debug_channel(const struct CHANNEL *chan)
{
  printf("\n");
  printf("channel.name        = %s\n", chan->name);
  printf("channel.key         = %s\n", chan->key);
  printf("channel.cname       = %s\n", chan->cname);
  printf("channel.channel     = %d\n", chan->channel);
  printf("channel.fine        = %d\n", chan->fine);
  printf("channel.freq        = %d\n", chan->freq);
  printf("channel.capture     = %s\n", int_to_str(chan->capture, captab));
  printf("channel.source      = %s\n", int_to_str(chan->source, grabbers[grabber]->inputs));
  printf("channel.norm        = %s\n", int_to_str(chan->norm, grabbers[grabber]->norms));
  printf("channel.color       = %d\n", chan->color);
  printf("channel.bright      = %d\n", chan->bright);
  printf("channel.hue         = %d\n", chan->hue);
  printf("channel.contrast    = %d\n", chan->contrast);
  printf("channel.pixmap      = %ld\n", chan->pixmap);
  printf("channel.button      = %p\n", chan->button);
  printf("channel.ckey        = %d\n", chan->ckey);                
  printf("channel.deinterlace = %d\n", chan->deinterlace);                
  printf("channel.subpage     = %s\n", chan->subpage);
  printf("channel.channel_type= %s\n", int_to_str(chan->channel_type,channel_type_list));
#ifdef HAVE_DVB  
  if (chan->channel_type == DVB_T){
	printf("channel.dvb_freq\t\t= %d\n", chan->dvb_freq );
	printf("channel.inversion\t\t= %s\n",int_to_str(chan->inversion,inversion_list));
	printf("channel.bandwidth\t\t= %s\n",int_to_str(chan->bandwidth,bw_list));
	printf("channel.code_rate_high\t\t= %s\n",int_to_str(chan->code_rate_high,fec_list));
	printf("channel.code_rate_low\t\t= %s\n", int_to_str(chan->code_rate_low,fec_list));
	printf("channel.constellation\t\t= %s\n", int_to_str(chan->constellation,constellation_list));
	printf("channel.transmission_mode\t= %s\n", int_to_str(chan->transmission_mode,transmissionmode_list));
	printf("channel.guard_interval\t\t= %s\n",int_to_str(chan->guard_interval, guard_list));
	printf("channel.hierarchy\t\t= %s\n", int_to_str(chan->hierarchy,hierarchy_list));
	printf("channel.vpid\t\t\t= %d\n", chan->vpid);
	printf("channel.apid\t\t\t= %d\n", chan->apid);
	printf("channel.spid\t\t\t= %d\n", chan->spid);                
  }
#endif
  printf("\n");
}

#define PAR_FROM_STRTAB(v,l) \
if (-1 != (i = str_to_int (val, l))) \
  v = i; \
else \
  fprintf (stderr, "%s:%d: invalid value for %s: %s\n", \
           filename, nr, tag, val);


/* ----------------------------------------------------------------------- */
int
read_channel(char *filename, char *line, int nr, struct CHANNEL **pt_current, char *tag, char *val, int flag)
{
  char *h;
  int i;
  struct CHANNEL *current = *pt_current;

  if (1 == sscanf (line, "[%99[^]]]", val))
    {
      if (flag == True)
        {
          current = add_channel (val);
          *pt_current = current;
        }
      else
        current->name = strdup(val);

      //continue;
      return 0;
    }

  if (2 != sscanf (line, " %31[^= ] = %99[^\n]", tag, val))
    {
      fprintf (stderr, "%s:%d: parse error\n", filename, nr);
      //continue;
      return 0;
    }

  for (h = val + strlen (val); h > val && *h == ' '; h--)
    *h = '\0';

  if (0 == strcmp (tag, "key"))
    {
      current->key = strdup (val);
    }
  else if (0 == strcmp (tag, "capture"))
    {
      if (-1 != (i = str_to_int (val, captab)))
        current->capture = i;
      else
        fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                 filename, nr, tag, val);
    }
  else if (0 == strcmp (tag, "source"))
    {PAR_FROM_STRTAB(current->source,grabbers[grabber]->inputs);}
  else if (0 == strcmp (tag, "norm"))
    {PAR_FROM_STRTAB(current->norm,grabbers[grabber]->norms);}
  else if (0 == strcmp (tag, "channel"))
    {
      current->cname = strdup (val);
      current->channel = lookup_channel (current->cname);
      if (-1 == current->channel)
        fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                 filename, nr, tag, val);
    }
  // Patch provided by Henk
  //This makes finetuning of freq. configured channels possible
  else if (0 == strcmp(tag, "freq"))
    {
      int n;
      unsigned d;

      current->freq = (int)(atof(val) * 1000);
      n = 0;
      d = (unsigned)-1;
      for (i = 0; i < CHAN_ENTRIES; i++)
        {
          if (abs(tvtuner[i].freq[chan_tab] - current->freq) < d)
            {
              d = abs(tvtuner[i].freq[chan_tab] - current->freq);
              n = i;
            }
        }
      current->cname = strdup(tvtuner[n].name);
      current->channel = lookup_channel(current->cname);
      current->fine = (current->freq - tvtuner[n].freq[chan_tab]) * 16 / 1000;
      current->freq = tvtuner[n].freq[chan_tab] * 16 / 1000;
      // debug_channel(current);
    }
  // End of Patch provided by Henk
  else if (0 == strcmp (tag, "fine"))
    {
      current->fine = atoi (val);
    }
  else if (0 == strcmp (tag, "color"))
    {
        current->color = atoi (val);
    }
  else if (0 == strcmp (tag, "bright"))
    {
      current->bright = atoi (val);
    }
  else if (0 == strcmp (tag, "hue"))
    {
      current->hue = atoi (val);
    }
  else if (0 == strcmp (tag, "contrast"))
    {
      current->contrast = atoi (val);
    }
  else if (0 == strcmp (tag, "deinterlace")) {
    if(grabbers[grabber]->name=="avi")
      current->deinterlace = DEINTERLACE_NONE;
    else
      {PAR_FROM_STRTAB(current->deinterlace, deinterlace_list);}
  }
  else if (0 == strcmp (tag, "subpage"))
    {
      current->subpage = strdup (val);
    }
  else if (0 == strcmp (tag, "type"))
    {
      PAR_FROM_STRTAB(current->channel_type,channel_type_list);
    }
#ifdef HAVE_DVB
  else if (0 == strcmp (tag, "frequency"))
    {
      current->dvb_freq = atoi (val);
    }
  else if (0 == strcmp (tag, "inversion"))
    {
      PAR_FROM_STRTAB(current->inversion ,inversion_list);
    }
  else if (0 == strcmp (tag, "bandwidth"))
    {
      PAR_FROM_STRTAB(current->bandwidth ,bw_list);
    }
  else if (0 == strcmp (tag, "code_rate_high"))
    {
      PAR_FROM_STRTAB(current->code_rate_high ,fec_list);
    }
  else if (0 == strcmp (tag, "code_rate_low"))
    {
      PAR_FROM_STRTAB(current->code_rate_low ,fec_list);
    }
  else if (0 == strcmp (tag, "constellation"))
    {
      PAR_FROM_STRTAB(current->constellation ,constellation_list);
    }
  else if (0 == strcmp (tag, "transmission_mode"))
    {
      PAR_FROM_STRTAB(current->transmission_mode ,transmissionmode_list);
    }
  else if (0 == strcmp (tag, "guard_interval"))
    {
      PAR_FROM_STRTAB(current->guard_interval ,guard_list);
    }
  else if (0 == strcmp (tag, "hierarchy"))
    {
      PAR_FROM_STRTAB(current->hierarchy ,hierarchy_list);
    }
 else if (0 == strcmp (tag, "vpid"))
    {
      current->vpid = atoi (val);
    }
 else if (0 == strcmp (tag, "apid"))
    {
      current->apid = atoi (val);
    }
 else if (0 == strcmp (tag, "spid"))
    {
      current->spid = atoi (val);
    }
#endif // HAVE_DVB
 else
    {
      // nothing has been find
      return -1;
    }

  // something relevant has been find and hopefully interpreted
  return 0;
}

// this function read special parameters like decoration
// that must be extract before several inits
void
read_first(void)
{
  FILE *fp;
  char filename[100], line[100], tag[32], val[100];
  int nr = 0, i;
  
  sprintf (filename, "%s/%s", user_dir, "xdtvrc");
  fprintf (stderr, "filename = %s\n",filename);  
  fp = fopen (filename, "r");
  if (NULL == fp)
    return;
  
    while (NULL != fgets (line, 99, fp))
    {
      nr++;
      sscanf (line, " %31[^= ] = %99[^\n]", tag, val);
      if (line[0] == '\n' || line[0] == '#' || line[0] == '%')
        continue;
      if (0 == count && 0 == strcmp (tag, "decoration"))
        {
	  if ( 0 == strcmp (val, "off") )
	   decoration = 0;
	 else if ( 0 == strcmp (val, "on") )
	   decoration = 1;
	 else
	   fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		    filename, nr, tag, val);
        }
      else if (0 == count && 0 == strcmp (tag, "stayontop"))
        {
	  if ( 0 == strcmp (val, "off") )
	   stay = 0;
	 else if ( 0 == strcmp (val, "on") )
	   stay = 1;
	 else
	   fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		    filename, nr, tag, val);
        }
//#ifdef USE_LIBXOSD <= xosd inside now
      else if (0 == count && 0 == strcmp (tag, "xosd"))
        {
	  if ( 0 == strcmp (val, "off") )
	   libxosd = 0;
	 else if ( 0 == strcmp (val, "on") )
	   libxosd = 1;
	 else
	   fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		    filename, nr, tag, val);
        }
//#endif <= xosd inside now
      else if (0 == count && 0 == strcmp (tag, "subtitles"))
        {
	  if ( 0 == strcmp (val, "off") )
	   subactive = 0;
	 else if ( 0 == strcmp (val, "on") )
	   subactive = 1;
	 else
	   fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		    filename, nr, tag, val);
        }
      else if (0 == count && 0 == strcmp (tag, "theme"))
        {
	  if ( strlen( val ) < 22 )
		  current_theme=strdup(val);
	 else
	   fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		    filename, nr, tag, val);
        }

      else if (0 == count && 0 == strcmp (tag, "fullscreen_mode"))
        {
	 switch_method = atoi(val);
	 if (switch_method >=1 && switch_method <=3)
		 switch_method--; // to have 1,2,3 -> 0,1,2
	 else {
	   fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		    filename, nr, tag, val);
	   	switch_method=2; //default;
	 }
        }		
#ifdef HAVE_XV
      else if (0 == count && 0 == strcmp (tag, "xvport"))
	cmdline_xv_img_port = atoi(val);
#endif
      else if (0 == count && 0 == strcmp (tag, "mixer_dev"))
	{
	  mixer_dev = strdup (val);
	}
      else if (0 == count && 0 == strcmp (tag, "mixer_tvchan"))
	{
	  mixer_tvchan = strdup (val);
	}
      else if (0 == count && 0 == strcmp (tag, "mixer_pcmchan"))
	{
	  mixer_pcmchan = strdup (val);
	}
      else if (0 == count && 0 == strcmp (tag, "audio_dev"))
	{
	  audio_dev = strdup (val);
	}
      else if (0 == count && 0 == strcmp (tag, "restoresnd"))
	{PAR_FROM_STRTAB(restoresnd, booltab);}
      else if (0 == count && 0 == strcmp (tag, "respectnullsnd"))
	{PAR_FROM_STRTAB(respectnullsnd, booltab);}
      else if (0 == count && 0 == strcmp (tag, "vop_autograb"))
	{
	  if ( 0 == strcmp (val, "off") )
	    vop_autograb = 0;
	  else if ( 0 == strcmp (val, "on") )
	    vop_autograb = 1;
	  else
	    fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		     filename, nr, tag, val);
	}
    }
  fclose (fp);
}

void
read_config (void)
{
  FILE *fp;
  char filename[100], line[100], tag[32], val[100], mytag[32];
  int i, j, nr = 0;
  char map[100];
  int rate = 0;
  struct CHANNEL *current = &defaults;

  if(int_to_str(current->norm,grabbers[grabber]->norms)==NULL)
    current->norm = 0;
  sprintf (filename, "%s/%s", user_dir, "xdtvrc");
  fp = fopen (filename, "r");
  if (NULL == fp)
    {
      fprintf (stderr, "Can't open config file %s:", filename);
      fprintf (stderr, "writing a default one.\n");
      fprintf (stderr, "Please edit %s to suit your needs\n", filename);
      /* Write a default config file */
      save_config ();

      /* And reread it ! */
      if ((fp = fopen (filename, "r")) == (FILE *) NULL)
        return;
    }
  have_config = 1;

  while (NULL != fgets (line, 99, fp))
    {
      nr++;
      if (line[0] == '\n' || line[0] == '#' || line[0] == '%')
        continue;

      if (read_channel(filename, line, nr, &current, tag, val, True) == 0)
        {
          continue;
        }
      else if (0 == count && 0 == strcmp (tag, "freqtab"))
        {
          if (-1 != (i = str_to_int (val, chan_names)))
            chan_tab = i;
          else {
	    /* temporary compatibilities with old names */
	    struct STRTAB freqtab_compat[] = {
	      {0, "ntsc-bcast"},
	      {1, "ntsc-cable"},
	      {2, "ntsc-bcast-jp"},
	      {3, "ntsc-cable-jp"},
	      {6, "pal-europe"},
	      {-1, NULL}};
	    if(-1 != (i = str_to_int (val, freqtab_compat))) {
	      fprintf(stderr, 
		      "freqtab=%s should be replaced by freqtab=%s\n",
		      val,int_to_str(i,chan_names));
	      chan_tab = i;
	    }
	  }
	  if(i==-1)
            fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
        }
      else if (0 == count && 0 == strcmp (tag, "fullscreen"))
        {
          if (2 != sscanf (val, "%d x %d", &fs_width, &fs_height))
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                       filename, nr, tag, val);
              fs_width = fs_height = 0;
            }
        }
      else if (0 == count && 0 == strcmp (tag, "windowsize"))
        {
          if (2 == sscanf (val, "%d x %d", &i, &j))
            {
              if(grabbers[grabber]->name!="avi")
                video_set_size (i, j);
            }
          else
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                       filename, nr, tag, val);
            }
        }
      else if (0 == count && 0 == strcmp (tag, "pixsize"))
        {
	  if (2 == sscanf (val, "%d x %d", &pix_width, &pix_height))
	  {
            pix_height = pix_width * 3 / 4;
	  }
          else
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                       filename, nr, tag, val);
              pix_width = 128;
              pix_height = 96;
            }
        }
      else if (0 == count && 0 == strcmp (tag, "wm-off-by"))
        {
          if (2 != sscanf (val, "%d %d", &fs_xoff, &fs_yoff))
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                       filename, nr, tag, val);
              fs_xoff = fs_yoff = 0;
            }
        }
      else if (0 == count && 0 == strcmp (tag, "colorkey"))
        {
          if (1 == sscanf (val, "%d", &i))
            {
              set_colorkey(dpy, i);
            }
          else
            fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
        }

      else if (0 == count && 0 == strcmp (tag, "capture_size"))
        {
          if (2 != sscanf (val, "%d x %d", &width_capture_max, &height_capture_max))
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                       filename, nr, tag, val);
              width_capture_max = 768;
              height_capture_max = 576;
            }
        }

      else if (0 == count && 0 == strcmp (tag, "blackborder"))
        {
          if (1 == sscanf (val, "%d", &i))
            {
              set_ybar_ratio(i);
            }
	  else
	    fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		     filename, nr, tag, val);
        }
      else if (0 == count && 0 == strcmp (tag, "container"))
	{PAR_FROM_STRTAB(divx.container, container_list);}
      else if (0 == count && 0 == strcmp (tag, "codec"))
	{PAR_FROM_STRTAB(divx.codec, codec_list);}
      else if (0 == count && 0 == strcmp (tag, "width"))
        {
          if (1 != sscanf(val,"%d",&divx.width))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "height"))
        {
          if (1 != sscanf(val,"%d",&divx.height))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "bitrate"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    divx.bitrate = i*1000;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "maxsize"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    divx.maxsize = i;
	  }
        }
     else if (0 == count && 0 == strcmp (tag, "quality"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    divx.quality = i;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "stereo_mode"))
	{PAR_FROM_STRTAB(divx.stereo, booltab);}
      else if (0 == count && 0 == strcmp (tag, "audio_codec"))
	{PAR_FROM_STRTAB(divx.audio_codec, audio_codec_list);}
      else if (0 == count && 0 == strcmp (tag, "mp3_bitrate"))
        {
          if (1 != sscanf(val,"%d",&divx.mp3_bitrate))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "mp3_quality"))
        {
          if (1 != sscanf(val,"%d",&divx.mp3_quality))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "mp3_vbr_mode"))
	{PAR_FROM_STRTAB(divx.mp3_vbr, booltab);}
      else if (0 == count && 0 == strcmp (tag, "mp3_vbr_quality"))
        {
          if (1 != sscanf(val,"%d",&divx.mp3_vbr_quality))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "fps"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    divx.fps = i;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "min_quantizer"))
        {
          if (1 != sscanf(val,"%d",&divx.min_quantizer))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "max_quantizer"))
        {
          if (1 != sscanf(val,"%d",&divx.max_quantizer))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "audio_freq"))
        {
          if (1 != sscanf(val,"%d",&divx.audio_freq))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		       filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "audio_fmt"))
        {
	  if (-1 != (i = str_to_int_noatoi (val, audio_fmt_names)))
	    divx.audio_fmt = i;
	  else
	    fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
		     filename, nr, tag, val);
	}
      else if (0 == count && 0 == strcmp (tag, "audio_buffer_size"))
        {
          if (1 != sscanf(val,"%d",&divx.audio_buffer_size))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "max_gap"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    divx.maxgap = i/1000.0;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "record_delay"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    divx.delay = i/1000.0;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "audio_fragments"))
        {
          if (1 != sscanf(val,"%d",&divx.audio_fragments))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "audio_sizefragment"))
        {
          if (1 != sscanf(val,"%d",&divx.audio_sizefragment))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "display_frame"))
	{PAR_FROM_STRTAB(divx.display_frames, booltab);}
      else if (0 == count && 0 == strcmp (tag, "record_sub"))
	{PAR_FROM_STRTAB(divx.sub, booltab);}
      else if (0 == count && 0 == strcmp (tag, "record_chg"))
	{PAR_FROM_STRTAB(divx.chg, booltab);}
      else if (0 == count && 0 == strcmp (tag, "streaming_mode"))
	{PAR_FROM_STRTAB(divx.stream, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xawpopup"))
	{PAR_FROM_STRTAB(xawpopup, booltab);}
      else if (0 == count && 0 == strcmp (tag, "streaming_http_port"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    divx.http_port = i;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "preview_player"))
        {
		pplayer = strdup(val);
        }	
      /* read divx file path into xdtvrc and give it to DivX GUI */
      else if (0 == count && 0 == strcmp (tag, "divx_path"))
	 divx.filename = strdup(val);
      else if (0 == count && 0 == strcmp (tag, "message_timer"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    message_timer = i;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "grab_filepath"))
        {
		filepath = strdup(val);
        }
#ifdef HAVE_LIBJPEG
      else if (0 == count && 0 == strcmp (tag, "jpeg_compression"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    if (i <= 100 && i >= 0)
	      jpegpcent = i;
	    else
	      jpegpcent = 100;
	  }
        }
#endif	
//#ifdef USE_LIBXOSD <= xosd inside now
      else if (0 == count && 0 == strcmp (tag, "xosd_main_color"))
        {
		colour = strdup(val);
        }
      else if (0 == count && 0 == strcmp (tag, "xosd_outline_color"))
        {
		outline_colour = strdup(val);
        }
      else if (0 == count && 0 == strcmp (tag, "xosd_outline_offset"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    outline_offset = i;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "xosd_shadow_offset"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    shadow_offset = i;
	  }
        }	
      else if (0 == count && 0 == strcmp (tag, "xosd_main_font"))
        {
		font = strdup(val);
		if (debug)
		  fprintf(stderr,"Main font = %s\n",font);		
		get_font_infos (font, main_font, main_size);
		if (debug) {
		  fprintf(stderr,"Main font type = %s\n",main_font);
                  fprintf(stderr,"Main font size = %s\n",main_size); 
		}
        }
       else if (0 == count && 0 == strcmp (tag, "xosd_vtx_font"))
        {
		font_vtx = strdup(val);
		if (debug)		
		  fprintf(stderr,"Teletext font = %s\n",font_vtx);		
		get_font_infos (font_vtx, vtx_font, vtx_size);
		if (debug) {		
		  fprintf(stderr,"Teletext font type = %s\n",vtx_font);
		  fprintf(stderr,"Teletext font size = %s\n",vtx_size); 
		}
        }
//#endif <= xosd inside now

      /* search for eventmap stuff in the config file */
      else if (0 == count && 0 == strncmp (tag, "lirc-key-", 9))
        {
	    fprintf(stderr, "ev config event:%s\n", tag);
	    sscanf(tag, "%s", mytag);
	    strcpy(map, val);
	    if (debug)
		fprintf(stderr, "map:%s\n", map);
	    event_register(mytag, map, 0);
	}
      else if (0 == count && 0 == strcmp (tag, "repeat"))
    	{
	    sscanf(val, "%d", &rate);
	    if (debug)
		fprintf(stderr, "repeat_rate:%d\n", rate);
	    change_repeat_last_registered(rate);
        }

	/* search for alevt parameters into the config file */

      else if (0 == count && 0 == strcmp (tag, "alevt_defaultpage"))
        {
	 alevt_defaultpage_str = strdup(val);
        }
      else if (0 == count && 0 == strcmp (tag, "alevt_finetune_mode"))
        {
         if ( 0 == strcmp (val, "none") )
	   alevt_finetune_mode = 0;
	 else if ( 0 == strcmp (val, "auto") )
	   alevt_finetune_mode = 1;
	 else if ( 0 == strcmp (val, "manual") )
	   alevt_finetune_mode = 2;
	 else
            fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
        }

      else if (0 == count && 0 == strcmp (tag, "alevt_finetune"))
        {
	 alevt_finetune_str = strdup(val);
        }

      else if (0 == count && 0 == strcmp (tag, "alevt_error_reduction"))
	{PAR_FROM_STRTAB(alevt_error_reduction, booltab);}

      else if (0 == count && 0 == strcmp (tag, "alevt_error_bell"))
	{PAR_FROM_STRTAB(alevt_error_bell, booltab);}

      else if (0 == count && 0 == strcmp (tag, "alevt_vbioffset"))
	{PAR_FROM_STRTAB(alevt_vbioffset, booltab);}

      else if (0 == count && 0 == strcmp (tag, "alevt_charset"))
        {
         if ( 0 == strcmp (val, "latin-1") )
	   alevt_charset = 0;
	 else if ( 0 == strcmp (val, "latin-2") )
	   alevt_charset = 1;
	 else if ( 0 == strcmp (val, "koi8-r") )
	   alevt_charset = 2;
	 else if ( 0 == strcmp (val, "iso8859-7") )
	   alevt_charset = 3;
	 else
            fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
        }
#ifdef HAVE_FFMPEG
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_v4mv"))
	{PAR_FROM_STRTAB(ffmpeg.v4mv, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_naq"))
	{PAR_FROM_STRTAB(ffmpeg.naq, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_gray"))
	{PAR_FROM_STRTAB(ffmpeg.gray, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_gmc"))
	{PAR_FROM_STRTAB(ffmpeg.gmc, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_qpel"))
	{PAR_FROM_STRTAB(ffmpeg.qpel, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_ildct"))
	{PAR_FROM_STRTAB(ffmpeg.ildct, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_keyint"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.keyint))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_vmax_b_frames"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.vmax_b_frames))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_vlelim"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.vlelim))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_vcelim"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.vcelim))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_lumi_mask"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    ffmpeg.lumi_mask = i/100.0;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_dark_mask"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    ffmpeg.dark_mask = i/100.0;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_tcplx_mask"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    ffmpeg.tcplx_mask = i/100.0;
	  }
        }
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_scplx_mask"))
        {
          if (1 != sscanf(val,"%d",&i))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
	  else
	  {
	    ffmpeg.scplx_mask = i/100.0;
	  }
        }
#if LIBAVCODEC_BUILD >= 4647
	      else if (0 == count && 0 == strcmp (tag, "ffmpeg_dia"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.dia))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
	      else if (0 == count && 0 == strcmp (tag, "ffmpeg_cmp"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.cmp))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
	      else if (0 == count && 0 == strcmp (tag, "ffmpeg_subcmp"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.subcmp))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
#endif
#if LIBAVCODEC_BUILD >= 4648
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_trell"))
	{PAR_FROM_STRTAB(ffmpeg.trell, booltab);}
#endif
#if LIBAVCODEC_BUILD >= 4649
	      else if (0 == count && 0 == strcmp (tag, "ffmpeg_last_pred"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.last_pred))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
#endif
#if LIBAVCODEC_BUILD >= 4651
	      else if (0 == count && 0 == strcmp (tag, "ffmpeg_predia"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.predia))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
	      else if (0 == count && 0 == strcmp (tag, "ffmpeg_precmp"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.precmp))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
#endif
#if LIBAVCODEC_BUILD >= 4661
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_umv"))
	{PAR_FROM_STRTAB(ffmpeg.umv, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_aic"))
	{PAR_FROM_STRTAB(ffmpeg.aic, booltab);}
#endif
#if LIBAVCODEC_BUILD >= 4673
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_mbd"))
        {
          if (1 != sscanf(val,"%d",&ffmpeg.mbd))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
#endif
#if LIBAVCODEC_BUILD >= 4681
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_cbp"))
	{PAR_FROM_STRTAB(ffmpeg.cbp, booltab);}
#endif
#if LIBAVCODEC_BUILD >= 4694
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_obmc"))
	{PAR_FROM_STRTAB(ffmpeg.obmc, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_ss"))
	{PAR_FROM_STRTAB(ffmpeg.ss, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_aiv"))
	{PAR_FROM_STRTAB(ffmpeg.aiv, booltab);}
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_loop"))
	{PAR_FROM_STRTAB(ffmpeg.loop, booltab);}
#endif
#if LIBAVCODEC_BUILD >= 4698
      else if (0 == count && 0 == strcmp (tag, "ffmpeg_ilme"))
	{PAR_FROM_STRTAB(ffmpeg.ilme, booltab);}
#endif
#endif
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
      else if (0 == count && 0 == strcmp (tag, "xvid_vhq"))
        {
          if (1 != sscanf(val,"%d",&xvid.vhq))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
#if XVID_API >= XVID_MAKE_API(4,1)
      else if (0 == count && 0 == strcmp (tag, "xvid_bvhq"))
	{PAR_FROM_STRTAB(xvid.bvhq, booltab);}
#endif
      else if (0 == count && 0 == strcmp (tag, "xvid_chromame"))
	{PAR_FROM_STRTAB(xvid.chromame, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_chroma_opt"))
	{PAR_FROM_STRTAB(xvid.chroma_opt, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_hqacpred"))
	{PAR_FROM_STRTAB(xvid.hqacpred, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_trell"))
	{PAR_FROM_STRTAB(xvid.trell, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_cartoon"))
	{PAR_FROM_STRTAB(xvid.cartoon, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_gray"))
	{PAR_FROM_STRTAB(xvid.gray, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_interlacing"))
	{PAR_FROM_STRTAB(xvid.interlacing, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_turbo"))
	{PAR_FROM_STRTAB(xvid.turbo, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_qpel"))
	{PAR_FROM_STRTAB(xvid.qpel, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_gmc"))
	{PAR_FROM_STRTAB(xvid.gmc, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_packed"))
	{PAR_FROM_STRTAB(xvid.packed, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_closed_gop"))
	{PAR_FROM_STRTAB(xvid.closed_gop, booltab);}
      else if (0 == count && 0 == strcmp (tag, "xvid_keyint"))
        {
          if (1 != sscanf(val,"%d",&xvid.keyint))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_frame_drop_ratio"))
        {
          if (1 != sscanf(val,"%d",&xvid.frame_drop_ratio))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_vmax_b_frames"))
        {
          if (1 != sscanf(val,"%d",&xvid.vmax_b_frames))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_bquant_ratio"))
        {
          if (1 != sscanf(val,"%d",&xvid.bquant_ratio))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_bquant_offset"))
        {
          if (1 != sscanf(val,"%d",&xvid.bquant_offset))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_bframe_threshold"))
        {
          if (1 != sscanf(val,"%d",&xvid.bframe_threshold))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_reaction_delay_factor"))
        {
          if (1 != sscanf(val,"%d",&xvid.reaction_delay_factor))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_averaging_period"))
        {
          if (1 != sscanf(val,"%d",&xvid.averaging_period))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_buffer"))
        {
          if (1 != sscanf(val,"%d",&xvid.buffer))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_max_I_quantizer"))
        {
          if (1 != sscanf(val,"%d",&xvid.max_quant[0]))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_min_I_quantizer"))
        {
          if (1 != sscanf(val,"%d",&xvid.min_quant[0]))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_max_P_quantizer"))
        {
          if (1 != sscanf(val,"%d",&xvid.max_quant[1]))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_min_P_quantizer"))
        {
          if (1 != sscanf(val,"%d",&xvid.min_quant[1]))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_max_B_quantizer"))
        {
          if (1 != sscanf(val,"%d",&xvid.max_quant[2]))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
      else if (0 == count && 0 == strcmp (tag, "xvid_min_B_quantizer"))
        {
          if (1 != sscanf(val,"%d",&xvid.min_quant[2]))
            {
	      fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                     filename, nr, tag, val);
	    }
        }
#endif
#endif
    }
  fclose (fp);

  /* calculate channel frequencies */
  defaults.channel = lookup_channel (defaults.cname);
  defaults.freq = get_freq (defaults.channel) + defaults.fine;
  for (i = 0; i < count; i++)
    configure_channel (channels[i]);
}

/* ----------------------------------------------------------------------- */

void
save_channel(FILE *fp, int i)
{
  fprintf (fp, "[%s]\n", channels[i]->name);

  fprintf (fp, "channel = %s\n", tvtuner[channels[i]->channel].name);

  if (defaults.fine != channels[i]->fine)
    fprintf (fp, "fine = %d\n", channels[i]->fine);

  if (defaults.norm != channels[i]->norm)
    fprintf (fp, "norm = %s\n",
             int_to_str (channels[i]->norm, grabbers[grabber]->norms));

  if (channels[i]->key != NULL)
    fprintf (fp, "key = %s\n", channels[i]->key);

  if (defaults.capture != channels[i]->capture)
    fprintf (fp, "capture = %s\n",
             int_to_str (channels[i]->capture, captab));

  if (defaults.source != channels[i]->source)
    fprintf (fp, "source = %s\n",
             int_to_str (channels[i]->source, grabbers[grabber]->inputs));

  if (defaults.color != channels[i]->color)
    fprintf (fp, "color = %d\n", channels[i]->color);

  if (defaults.bright != channels[i]->bright)
    fprintf (fp, "bright = %d\n", channels[i]->bright);

  if (defaults.hue != channels[i]->hue)
    fprintf (fp, "hue = %d\n", channels[i]->hue);

  if (defaults.contrast != channels[i]->contrast)
    fprintf (fp, "contrast = %d\n", channels[i]->contrast);
  
  if (defaults.deinterlace != channels[i]->deinterlace)
    fprintf (fp, "deinterlace = %s\n", int_to_str(channels[i]->deinterlace, deinterlace_list));

  if (strcmp(defaults.subpage, channels[i]->subpage))
    fprintf (fp, "subpage = %s\n", channels[i]->subpage); // page 888 par defaut
  
  if (defaults.channel_type != channels[i]->channel_type)
    fprintf (fp, "type = %s\n", int_to_str(channels[i]->channel_type, channel_type_list));
#ifdef HAVE_DVB  
  if (channels[i]->channel_type == DVB_T) {
	fprintf(fp,"frequency = %d\n",channels[i]->dvb_freq);
	fprintf(fp,"inversion = %s\n",int_to_str(channels[i]->inversion,inversion_list));
	fprintf(fp,"bandwidth = %s\n",int_to_str(channels[i]->bandwidth,bw_list));
	fprintf(fp,"code_rate_high = %s\n",int_to_str(channels[i]->code_rate_high,fec_list));
	fprintf(fp,"code_rate_low = %s\n", int_to_str(channels[i]->code_rate_low,fec_list));
	fprintf(fp,"constellation = %s\n", int_to_str(channels[i]->constellation,constellation_list));
	fprintf(fp,"transmission_mode = %s\n", int_to_str(channels[i]->transmission_mode,transmissionmode_list));
	fprintf(fp,"guard_interval = %s\n",int_to_str(channels[i]->guard_interval, guard_list));
	fprintf(fp,"hierarchy = %s\n", int_to_str(channels[i]->hierarchy,hierarchy_list));
	fprintf(fp,"vpid = %d\n", channels[i]->vpid);
	fprintf(fp,"apid = %d\n", channels[i]->apid);
	fprintf(fp,"spid = %d\n", channels[i]->spid);                
  }
#endif
  fprintf (fp, "\n");
}

void
save_alevt(FILE *fp)
{
  fprintf(fp,"\n");
  fprintf (fp, "#\n# alevt options\n#\n");
  update_defaultpage();
  fprintf (fp,"alevt_defaultpage = %s\n",alevt_defaultpage_str);

  switch(alevt_finetune_mode)
  {
    case 0:
      fprintf (fp,"alevt_finetune_mode = none\n");
      break;
    case 1:
      fprintf (fp,"alevt_finetune_mode = auto\n");
      break;
    case 2:
      fprintf (fp,"alevt_finetune_mode = manual\n");
      break;
  }
  update_finetune();
  fprintf (fp,"alevt_finetune = %s\n",alevt_finetune_str);
    switch(alevt_error_reduction)
  {
    case 0:
      fprintf (fp,"alevt_error_reduction = no\n");
      break;
    case 1:
      fprintf (fp,"alevt_error_reduction = yes\n");
      break;
  }
    switch(alevt_error_bell)
  {
    case 0:
      fprintf (fp,"alevt_error_bell = no\n");
      break;
    case 1:
      fprintf (fp,"alevt_error_bell = yes\n");
      break;
  }
    switch(alevt_vbioffset)
  {
    case 0:
      fprintf (fp,"alevt_vbioffset = no\n");
      break;
    case 1:
      fprintf (fp,"alevt_vbioffset = yes\n");
      break;
  }
  switch(alevt_charset)
  {
    case 0:
      fprintf (fp,"alevt_charset = latin-1\n");
      break;
    case 1:
      fprintf (fp,"alevt_charset = latin-2\n");
      break;
    case 2:
      fprintf (fp,"alevt_charset = koi8-r\n");
      break;
    case 3:
      fprintf (fp,"alevt_charset = iso8859-7\n");
      break;
  }
}

void
save_record(FILE *fp)
{
   fprintf (fp, "#\n# record options\n#\n");
   fprintf (fp, "container = %s\n", int_to_str(divx.container, container_list));
   fprintf (fp, "codec = %s\n", int_to_str(divx.codec, codec_list));
   fprintf (fp,"width = %d\n",divx.width);
   fprintf (fp,"height = %d\n",divx.height);
   fprintf (fp,"bitrate = %ld\n",divx.bitrate/1000);
   fprintf (fp,"quality = %d\n", divx.quality);
   fprintf (fp,"stereo_mode = %s\n", int_to_str(divx.stereo,booltab));
   fprintf (fp,"audio_codec = %s\n",
	    int_to_str(divx.audio_codec, audio_codec_list));
   fprintf (fp,"mp3_bitrate = %d\n",divx.mp3_bitrate);
   fprintf (fp,"mp3_quality = %d\n",divx.mp3_quality);
   fprintf (fp,"mp3_vbr_mode = %s\n",int_to_str(divx.mp3_vbr, booltab));
   fprintf (fp,"mp3_vbr_quality = %d\n",divx.mp3_vbr_quality);
   fprintf (fp,"fps = %d\n",divx.fps);
   fprintf (fp,"max_gap = %d\n",(int)(divx.maxgap*1000.0+0.5));
   fprintf (fp,"audio_fragments = %d\n",divx.audio_fragments);
   fprintf (fp,"audio_sizefragment = %d\n",divx.audio_sizefragment);
   fprintf (fp,"min_quantizer = %d\n",divx.min_quantizer);
   fprintf (fp,"max_quantizer = %d\n",divx.max_quantizer);
   fprintf (fp,"audio_buffer_size = %d\n",divx.audio_buffer_size);
   fprintf (fp,"audio_freq = %d\n",divx.audio_freq);
   fprintf (fp,"audio_fmt = %s\n",int_to_str(divx.audio_fmt, audio_fmt_names));

/* write divx file path into xdtvrc */
   update_divxfilename();
   if (divx.filename)
   {
     if(strcmp(divx.filename, "/dev/null")!=0)
       fprintf (fp,"divx_path = %s\n",divx.filename);
     else
       fprintf (fp,"divx_path = %s\n",getcwd(NULL,0));
   }
   if(divx.maxsize)
     fprintf (fp,"maxsize = %d\n",divx.maxsize);
   fprintf (fp,"display_frame = %s\n",
	    int_to_str(divx.display_frames, booltab));
   fprintf (fp,"record_sub = %s\n",int_to_str(divx.sub, booltab));
   fprintf (fp,"record_chg = %s\n",int_to_str(divx.chg, booltab));
   fprintf (fp,"record_delay = %d\n",(int)(divx.delay*1000.0+0.5));
   fprintf (fp,"streaming_mode = %s\n",int_to_str(divx.stream, booltab));
   update_http_port();
   fprintf (fp,"streaming_http_port = %d\n",divx.http_port);
   if (pplayer)
     fprintf (fp, "preview_player = %s\n", pplayer);   
   fprintf(fp,"\n");
}


void
save_record_advanced(FILE *fp)
{
   fprintf (fp, "#\n# record advanced options\n#\n");
#ifdef HAVE_FFMPEG
   fprintf (fp,"ffmpeg_v4mv = %s\n",int_to_str(ffmpeg.v4mv, booltab));
   fprintf (fp,"ffmpeg_naq = %s\n",int_to_str(ffmpeg.naq, booltab));
   fprintf (fp,"ffmpeg_gray = %s\n",int_to_str(ffmpeg.gray, booltab));
   fprintf (fp,"ffmpeg_gmc = %s\n",int_to_str(ffmpeg.gmc, booltab));
   fprintf (fp,"ffmpeg_qpel = %s\n",int_to_str(ffmpeg.qpel, booltab));
   fprintf (fp,"ffmpeg_ildct = %s\n",int_to_str(ffmpeg.ildct, booltab));
   fprintf (fp,"ffmpeg_keyint = %d\n",ffmpeg.keyint);
   fprintf (fp,"ffmpeg_vmax_b_frames = %d\n",ffmpeg.vmax_b_frames);
   fprintf (fp,"ffmpeg_vlelim = %d\n",ffmpeg.vlelim);
   fprintf (fp,"ffmpeg_vcelim = %d\n",ffmpeg.vcelim);
   fprintf (fp,"ffmpeg_lumi_mask = %d\n",(int)(ffmpeg.lumi_mask*100+0.5));
   fprintf (fp,"ffmpeg_dark_mask = %d\n",(int)(ffmpeg.dark_mask*100+0.5));
   fprintf (fp,"ffmpeg_tcplx_mask = %d\n",(int)(ffmpeg.tcplx_mask*100+0.5));
   fprintf (fp,"ffmpeg_scplx_mask = %d\n",(int)(ffmpeg.scplx_mask*100+0.5));
#if LIBAVCODEC_BUILD >= 4647   
   fprintf (fp,"ffmpeg_dia = %d\n",ffmpeg.dia);
   fprintf (fp,"ffmpeg_cmp = %d\n",ffmpeg.cmp);
   fprintf (fp,"ffmpeg_subcmp = %d\n",ffmpeg.subcmp);
#endif
#if LIBAVCODEC_BUILD >= 4648
   fprintf (fp,"ffmpeg_trell = %s\n",int_to_str(ffmpeg.trell, booltab));
#endif
#if LIBAVCODEC_BUILD >= 4649
   fprintf (fp,"ffmpeg_last_pred = %d\n",ffmpeg.last_pred);
#endif
#if LIBAVCODEC_BUILD >= 4651
   fprintf (fp,"ffmpeg_predia = %d\n",ffmpeg.predia);
   fprintf (fp,"ffmpeg_precmp = %d\n",ffmpeg.precmp);
#endif
#if LIBAVCODEC_BUILD >= 4661
   fprintf (fp,"ffmpeg_umv = %s\n",int_to_str(ffmpeg.umv, booltab));
   fprintf (fp,"ffmpeg_aic = %s\n",int_to_str(ffmpeg.aic, booltab));
#endif
#if LIBAVCODEC_BUILD >= 4673
   fprintf (fp,"ffmpeg_mbd = %d\n",ffmpeg.mbd);
#endif
#if LIBAVCODEC_BUILD >= 4681
   fprintf (fp,"ffmpeg_cbp = %s\n",int_to_str(ffmpeg.cbp, booltab));
#endif
#if LIBAVCODEC_BUILD >= 4694
   fprintf (fp,"ffmpeg_obmc = %s\n",int_to_str(ffmpeg.obmc, booltab));
   fprintf (fp,"ffmpeg_ss = %s\n",int_to_str(ffmpeg.ss, booltab));
   fprintf (fp,"ffmpeg_aiv = %s\n",int_to_str(ffmpeg.aiv, booltab));
   fprintf (fp,"ffmpeg_loop = %s\n",int_to_str(ffmpeg.loop, booltab));
#endif
#if LIBAVCODEC_BUILD >= 4698
   fprintf (fp,"ffmpeg_ilme = %s\n",int_to_str(ffmpeg.ilme, booltab));
#endif
#endif
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
fprintf (fp,"xvid_vhq = %d\n",xvid.vhq);
#if XVID_API >= XVID_MAKE_API(4,1)
fprintf (fp,"xvid_bvhq = %s\n",int_to_str(xvid.bvhq, booltab));
#endif
fprintf (fp,"xvid_chromame = %s\n",int_to_str(xvid.chromame, booltab));
fprintf (fp,"xvid_chroma_opt = %s\n",int_to_str(xvid.chroma_opt, booltab));
fprintf (fp,"xvid_hqacpred = %s\n",int_to_str(xvid.hqacpred, booltab));
fprintf (fp,"xvid_trell = %s\n",int_to_str(xvid.trell, booltab));
fprintf (fp,"xvid_cartoon = %s\n",int_to_str(xvid.cartoon, booltab));
fprintf (fp,"xvid_gray = %s\n",int_to_str(xvid.gray, booltab));
fprintf (fp,"xvid_interlacing = %s\n",int_to_str(xvid.interlacing, booltab));
fprintf (fp,"xvid_turbo = %s\n",int_to_str(xvid.turbo, booltab));
fprintf (fp,"xvid_qpel = %s\n",int_to_str(xvid.qpel, booltab));
fprintf (fp,"xvid_gmc = %s\n",int_to_str(xvid.gmc, booltab));
fprintf (fp,"xvid_keyint = %d\n",xvid.keyint);
fprintf (fp,"xvid_frame_drop_ratio = %d\n",xvid.frame_drop_ratio);
fprintf (fp,"xvid_vmax_b_frames = %d\n",xvid.vmax_b_frames);
fprintf (fp,"xvid_bquant_ratio = %d\n",xvid.bquant_ratio);
fprintf (fp,"xvid_bquant_offset = %d\n",xvid.bquant_offset);
fprintf (fp,"xvid_packed = %s\n",int_to_str(xvid.packed, booltab));
fprintf (fp,"xvid_closed_gop = %s\n",int_to_str(xvid.closed_gop, booltab));
fprintf (fp,"xvid_bframe_threshold = %d\n",xvid.bframe_threshold);
fprintf (fp,"xvid_reaction_delay_factor = %d\n",xvid.reaction_delay_factor);
fprintf (fp,"xvid_averaging_period = %d\n",xvid.averaging_period);
fprintf (fp,"xvid_buffer = %d\n",xvid.buffer);
fprintf (fp,"xvid_max_I_quantizer = %d\n",xvid.max_quant[0]);
fprintf (fp,"xvid_min_I_quantizer = %d\n",xvid.min_quant[0]);
fprintf (fp,"xvid_max_P_quantizer = %d\n",xvid.max_quant[1]);
fprintf (fp,"xvid_min_P_quantizer = %d\n",xvid.min_quant[1]);
fprintf (fp,"xvid_max_B_quantizer = %d\n",xvid.max_quant[2]);
fprintf (fp,"xvid_min_B_quantizer = %d\n",xvid.min_quant[2]);
#endif
#endif
   fprintf(fp,"\n");
}

void
save_config (void)
{
  char filename1[100], filename2[100];
  FILE *fp;
  int i;
  Dimension w = 0;
  Dimension h = 0;

  sprintf (filename1, "%s/%s", user_dir, "xdtvrc");
  sprintf (filename2, "%s/%s", user_dir, "xdtvrc.bak");

  /* delete old backup */
  unlink (filename2);

  /* current becomes backup */
  if (0 == link (filename1, filename2))
    unlink (filename1);

  /* write new one... */
  fp = fopen (filename1, "w");
  if (NULL == fp)
    {
      fprintf (stderr, "can't open config file %s\n", filename1);
      return;
    }

  /* write defaults */
  fprintf(fp, "#\n# Global options\n#\n");
  if(int_to_str (defaults.norm, grabbers[grabber]->norms)==NULL)
    defaults.norm = 0;
  fprintf (fp, "norm = %s\n",
           int_to_str (defaults.norm, grabbers[grabber]->norms));

  fprintf (fp, "capture = %s\n", int_to_str (defaults.capture, captab));

  fprintf (fp, "source = %s\n",
           int_to_str (defaults.source, grabbers[grabber]->inputs));

  if ((defaults.color != 19000) && (defaults.color != 0))
    fprintf (fp, "color = %d\n", defaults.color);

  if ((defaults.bright != 34000) && (defaults.bright != 0))
    fprintf (fp, "bright = %d\n", defaults.bright);

  if ((defaults.hue != 32768) && (defaults.hue != 0))
    fprintf (fp, "hue = %d\n", defaults.hue);

  if ((defaults.contrast != 28000) && (defaults.contrast != 0))
    fprintf (fp, "contrast = %d\n", defaults.contrast);

  if (defaults.deinterlace != DEINTERLACE_LINEARBLEND)
    fprintf (fp, "deinterlace = %s\n", int_to_str(defaults.deinterlace, deinterlace_list));

  fprintf (fp, "subpage = %s\n", defaults.subpage);

  fprintf (fp, "freqtab = %s\n", int_to_str (chan_tab, chan_names));  
  
  fprintf (fp, "\n");   
  
  if (mixer_tvchan)
    fprintf (fp, "mixer_tvchan = %s\n", mixer_tvchan);
  else {
    fprintf (fp, "# You should uncomment this line\n");
    fprintf (fp, "# and modify it by your own specific channel:\n");
    fprintf (fp, "# mixer_tvchan = xxxxxx\n\n"); }
  if (audio_dev)
    fprintf (fp, "audio_dev = %s\n", audio_dev);
  if (mixer_dev)
    fprintf (fp, "mixer_dev = %s\n", mixer_dev);
  if (mixer_pcmchan)
    fprintf (fp, "mixer_pcmchan = %s\n", mixer_pcmchan);
  fprintf (fp, "restoresnd = %s\n", restoresnd ? "on" : "off");

  fprintf (fp, "respectnullsnd = %s\n", respectnullsnd ? "on" : "off");

  fprintf (fp, "vop_autograb = %s\n", vop_autograb ? "on" : "off");

  fprintf (fp, "\n");     
      
  if (xawpopup)
    fprintf (fp, "xawpopup = on\n");
  else
    fprintf (fp, "xawpopup = off\n");

  if (message_timer)
    fprintf (fp, "message_timer = %d\n", message_timer);     
    
  if (decoration)
    fprintf (fp, "decoration = on\n");    
  else
    fprintf (fp, "decoration = off\n");
    
  if (stay)
    fprintf (fp, "stayontop = on\n");    
  else
    fprintf (fp, "stayontop = off\n");    
         
  if (subactive)
    fprintf (fp, "subtitles = on\n");    
  else
    fprintf (fp, "subtitles = off\n");        

#ifdef HAVE_XV
  if(cmdline_xv_img_port)
    fprintf(fp, "xvport = %d\n", cmdline_xv_img_port);
#endif

  fprintf (fp,"theme = %s\n",current_theme);
  
  fprintf (fp, "\n");  
  fprintf(fp, "#\n# Fullscreen options\n#\n");    
  
  if (fs_width && fs_height)
    fprintf (fp, "fullscreen = %d x %d\n", fs_width, fs_height);
  else
    fprintf (fp, "fullscreen = 640 x 480\n");

  if (switch_method == 1)
    fprintf (fp, "fullscreen_mode = 2\n");    
  else if (switch_method == 0)
    fprintf (fp, "fullscreen_mode = 1\n");
  else if (switch_method == 2)
    fprintf (fp, "fullscreen_mode = 3\n");    
  
  update_wmoffby();
  if (fs_xoff || fs_yoff)
    fprintf (fp, "wm-off-by = %+d%+d\n", fs_xoff, fs_yoff);

  update_pixmaps_size();
  fprintf (fp, "pixsize = %d x %d\n", pix_width, pix_height);

  if (fs_width && fs_height)
  {
    XtVaGetValues (tv, XtNwidth, &w, XtNheight, &h, NULL);
    fprintf (fp, "windowsize = %d x %d\n", w, h);
  }
  else fprintf (fp, "windowsize = 384 x 288\n");

  update_colorkey();
  if ((i = get_colorkey(dpy)) > 0)
    fprintf (fp, "colorkey = %d\n", i);
  else
    fprintf (fp, "colorkey = 123456\n");    

  if (width_capture_max && height_capture_max)
    fprintf (fp, "capture_size = %d x %d\n", width_capture_max, height_capture_max);
  else
    /*maybe use CAPT_WIDTH_MAX, CAPT_HEIGHT_MAX instead*/
    fprintf (fp, "capture_size = 768 x 576\n");

  if (get_ybar_ratio())
    fprintf (fp, "blackborder = %d\n", get_ybar_ratio());

  fprintf (fp, "\n");
  fprintf(fp, "#\n# Grab options\n#\n");
  
  update_grabfilename();
  if (filepath)
    fprintf (fp, "grab_filepath = %s\n", filepath);
  else
    fprintf (fp, "grab_filepath = %s\n", getcwd(NULL,0));
#ifdef HAVE_LIBJPEG    
  if (jpegpcent)
    fprintf (fp, "jpeg_compression = %d\n", jpegpcent);  
#endif  
//#ifdef USE_LIBXOSD <= xosd inside now

  fprintf (fp, "\n");
  fprintf(fp, "#\n# XOSD options\n#\n");  

  if (libxosd)
    fprintf (fp, "xosd = on\n");    
  else
    fprintf (fp, "xosd = off\n");  
  
  if (colour)
    fprintf (fp, "xosd_main_color = %s\n", colour);

  if (outline_colour)
    fprintf (fp, "xosd_outline_color = %s\n", outline_colour);    

  if (outline_offset)
    fprintf (fp, "xosd_outline_offset = %d\n", outline_offset);     

  if (shadow_offset)
    fprintf (fp, "xosd_shadow_offset = %d\n", shadow_offset);       
            
  if (font)
    fprintf (fp, "xosd_main_font = %s\n", font);

  if (font_vtx)
    fprintf (fp, "xosd_vtx_font = %s\n", font_vtx);
//#endif <= xosd inside now

  fprintf (fp, "\n");

  save_record(fp);
  
  save_record_advanced(fp);

  save_user_events(fp);

  save_alevt(fp);

  fprintf (fp, "\n");

  /* write channels */
  if (count > 0)
    {
      fprintf(fp, "#\n# Channel options\n#\n");
      for (i = 0; i < count; i++)
        {
          save_channel(fp, i);
        }
    }
  else
    {
      fprintf (fp,
               "# For each channel define at least following information\n");
      fprintf (fp, "# [Channel Name]\n");
      fprintf (fp, "# channel = Channel Number\n");
      fprintf (fp, "# norm = PAL|NTSC|SECAM|AUTO\n");
      fprintf (fp,
               "# key = key to press to switch the channel on (KP_End, KP_Next,...)\n");
      fprintf (fp, "# capture = off|grabdisplay|overlay\n");
      fprintf (fp, "\n");
    }



  fclose (fp);
}

int
load_last_channel(struct CHANNEL *last_channel)
{
  FILE *fp;
  char filename[100], line[100], tag[32], val[100];
  int nr = 0;
  int r, status = 0;
  int w, h;

  sprintf (filename, "%s/%s", user_dir, "last_channel");
  fp = fopen (filename, "r");
  if (NULL == fp)
    {
      fprintf(stderr, "load_last_channel: can't open %s\n", filename);
      return -1;
    }

  memcpy(last_channel, &defaults, sizeof(struct CHANNEL));

  while (NULL != fgets (line, 99, fp))
    {
      nr++;
      if (line[0] == '\n' || line[0] == '#' || line[0] == '%')
        continue;


      if ((r=read_channel(filename, line, nr, &last_channel, tag, val, False)) == 0)
        {
          continue;
        }
      else if (0 == strcmp (tag, "windowsize"))
        {
          if (2 == sscanf (val, "%d x %d", &w, &h))
            {
              video_set_size (w, h);
            }
          else
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                       filename, nr, tag, val);
            }
        }
      else
        {
          // Nothing was found;
          status += r;
        }
    }
  fclose(fp);

  if (nr == 0)
    return -1; // file was empty !!!
  else if (status == 0)
    {
      /* calculate channel frequencies */
      last_channel->freq = get_freq (last_channel->channel) + last_channel->fine;
   //   hotkey_channel (last_channel);
      return 0; // everything went fine
    }
  else
    return -1;
}

void
save_last_channel(int last_channel)
{
  FILE *fp;
  char filename[100];
  //Dimension w, h;
  
  sprintf (filename, "%s/%s", user_dir, "last_channel");
  fp = fopen (filename, "w");
  if (NULL == fp)
    {
      fprintf(stderr, "save_last_channel: can't open %s\n", filename);
      return;
    }

  // Causes more troubles than real added value
  //XtVaGetValues (tv, XtNwidth, &w, XtNheight, &h, NULL);
  //fprintf (fp, "windowsize = %d x %d\n", w, h);

  save_channel(fp, last_channel);

  fclose(fp);
}


void
save_memcpy_method(int method)
{
  FILE *fp;
  char filename[100];

  sprintf (filename, "%s/%s", user_dir, "memcpy_method");
  fp = fopen (filename, "w");

  fprintf(fp, "# memcpy_method: probe | glibc | kernel | FPU_xx | mmx | mmx_xx | mmxext | mmxext_xx | sse | sse_xx\n"
              "version = %s\n"
              "memcpy_method = %s\n", VERSION, int_to_str(method, memcpy_list));

  fclose(fp);
}


void
load_memcpy_method(void)
{
  FILE *fp;
  char filename[100];
  char line[128];
  int nr;
  char *h;
  char tag[32], val[128];
  int i;
  // Version found in file vs current version
  int version=0; // 0: not found, 1: same, -1: different

  sprintf (filename, "%s/%s", user_dir, "memcpy_method");
  fp = fopen (filename, "r");
  if (NULL == fp)
    {
      fprintf(stderr, "load_memcpy_method: can't open %s\n", filename);
      return;
    }

  nr = 0;
  while (NULL != fgets (line, 127, fp))
    {
      nr++;
      if (line[0] == '\n' || line[0] == '#' || line[0] == '%')
        continue;

      if (2 != sscanf (line, " %31[^= ] = %127[^\n]", tag, val))
        {
          fprintf (stderr, "%s:%d: parse error\n", filename, nr);
          continue;
        }

      for (h = val + strlen (val); h > val && *h == ' '; h--)
        *h = '\0';
      

      if (0 == strcmp (tag, "memcpy_method"))
        { // Found a method
          if (version != -1)
          { // version "not found" or "same"
              if (-1 != (i = str_to_int (val, memcpy_list)))
                { // Found a licit method
                  memcpy_method = i;
                  fprintf (stderr, "Method %s\n", int_to_str (i, memcpy_list)) ;
                }
              else
                { // Found an illicit method
                  memcpy_method = MEMCPY_PROBE;
                  fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                           filename, nr, tag, val);
                }
          }
        }

       if (0 == strcmp (tag, "version"))
        { // Found a version
          if (strcmp(val, VERSION))
            { // Not the same version, have to probe
              memcpy_method = MEMCPY_PROBE;
              version = -1; // version set to "different"
              fprintf (stderr, "Changing version from %s to %s: probing memcpy methods\n",
                       val, VERSION);
            }
          else
            {
                version = 1; // version set to "same"
            }
        }
    }

    if (!version)
    { // No version found after all, have to probe
     memcpy_method = MEMCPY_PROBE;
     fprintf (stderr, "New memcpy methods available: probing methods\n");
    }
  fclose(fp);
}
