/*****************************************************************************
 * conf.c: channel editor
 *****************************************************************************
 * $Id: conf.c,v 1.61 2007/02/19 16:59:32 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 1998 Gerd Knorr <kraxel@cs.tu-berlin.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <sys/time.h>
#include <dlfcn.h>
#include <pthread.h>

#include "config.h"

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Toggle.h>

#ifdef HAVE_XPM
# include <X11/xpm.h>
#endif

#include "channel.h"
#include "channels.h"
#include "grab.h"
#include "x11.h"
#include "xdtv.h"
#include "scan_common.h"
#include "popup_ui.h"
#include "xosd.h"
#include "dvb_init_ui.h"

#ifdef HAVE_XPM
# include "xdtv.xpm"
#endif

extern struct GRABBER grab_dummy;
extern struct GRABBER grab_avi;
#ifdef HAVE_DVB
extern struct GRABBER grab_dvb;
extern Widget dvbinit_shell;
extern int dvb_grabber_running;
extern void DVBinitAction (Widget, XEvent *, String *, Cardinal *);
#endif
Widget c_chan_opendvbinit;

// internationalization
#include "translation.h"
#include "pixmaps_loader.h"

#ifdef BUGTEXTXAW3D
#define STRDUP(x) strdup(x)
#else
#define STRDUP(x) (x)
#endif

void pixit (void);
void set_channel (struct CHANNEL *channel);
void channel_menu (void);

extern void set_capture (int capture, int save);
extern void OptionsAction (Widget, XEvent *, String *, Cardinal *);
extern void action_cb (Widget widget, XtPointer clientdata, XtPointer call_data);
extern void set_timer_title (void);
extern Widget app_shell, conf_shell;
extern Display *dpy;
extern Atom wm_protocols[2];
extern int debug;
extern void ExitCB();
extern char title[256];
extern int count;

// For the Xosd messages
extern int message_timer;
extern int outline_offset;
extern int shadow_offset;
extern char *font, *colour, *outline_colour;
extern int libxosd_ok;
extern void update_main_xosd();
xosd *channel_xosd;

Widget conf_channel, conf_name, conf_key, conf_list;
Widget c_chan_openmain;
Widget c_chan_move;
Widget c_chan_move_up;
Widget c_chan_move_down;
Widget c_chan_shortcuts;
Widget c_chan_rc;
Widget c_chan_scantv;
Widget c_chan_fullscan;
Widget c_chan_add;
Widget c_chan_delete;
Widget c_chan_update;
Widget c_chan_save;
Widget label_channelnum;
Widget label_choosehotkey;
Widget label_stationname;

Widget last_command, viewport;
static String *channel_list;
static int scantv = 0;
int complete_scan = 0;
#ifdef HAVE_XPM
  Pixmap icon_pixmap;
  Pixmap icon_shapemask;
#endif
pthread_t scan_thread;
pthread_t update_thread;



static struct STRTAB shortcuts_list[]={
	{0,"KP_End"},
	{1,"KP_Down"},
	{2,"KP_Next"},
	{3,"KP_Left"},
	{4,"KP_Begin"},
	{5,"KP_Right"},
	{6,"KP_Home"},
	{7,"KP_Up"},
	{8,"KP_Prior"},
	{9,"KP_Insert"},
	{10,"Shift+KP_End"},
	{11,"Shift+KP_Down"},
	{12,"Shift+KP_Next"},
	{13,"Shift+KP_Left"},
	{14,"Shift+KP_Begin"},
	{15,"Shift+KP_Right"},
	{16,"Shift+KP_Home"},
	{17,"Shift+KP_Up"},
	{18,"Shift+KP_Prior"},
	{19,"Shift+KP_Insert"},
	{20,"Ctrl+KP_End"},
	{21,"Ctrl+KP_Down"},
	{22,"Ctrl+KP_Next"},
	{23,"Ctrl+KP_Left"},
	{24,"Ctrl+KP_Begin"},
	{25,"Ctrl+KP_Right"},
	{26,"Ctrl+KP_Home"},
	{27,"Ctrl+KP_Up"},
	{28,"Ctrl+KP_Prior"},
	{29,"Ctrl+KP_Insert"},
	{-1,NULL}
};


/*-------------------------------------------------------------------------*/

struct CALL_ACTION call_options_1 = { 0, "Options", {NULL} };

// DVB!!!!
#ifdef HAVE_DVB
struct CALL_ACTION call_dvbinit = { 0, "DVBinit", {NULL} };
#endif

/*--------------------------------------------------------------------------*/

static void waitforchild(int i)
{
        waitpid(-1, NULL, WNOHANG);
        signal(SIGCHLD, waitforchild);
}
/*
static void
launch_it (Widget w, XtPointer client_data, XtPointer call_data)
{
fputs(client_data,popen("/bin/sh","w"));
fflush(popen("/bin/sh","w"));
}
*/
static void execute_it (Widget w, XtPointer client_data, XtPointer call_data)
{
  char *cmd = (char *)client_data;
  pthread_t thread;
  pthread_create (&thread, NULL, (void *(*)(void *)) system, cmd);
}

void editor_station_switched ();

static void
list_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  if (channel_list[0] != "empty") {
	XawListReturnStruct *lr = call_data;

	pixit ();
	cur_sender = lr->list_index;
	set_channel (channels[cur_sender]);
  }
}

void
scantv_xosd (char * message1, char * message2)
{
  Dimension x, y;
  Dimension w, h;

  XtVaGetValues (app_shell, XtNx, &x, XtNy, &y,
                XtNwidth, &w, XtNheight, &h, NULL);

  update_main_xosd ();

  if (xosd_is_onscreen(channel_xosd)) xosd_hide (channel_xosd);
  channel_xosd = xosd_create (2);

  if (!channel_xosd) {
      fprintf (stderr, "Error initializing osd: %s\n", xosd_error);
      libxosd_ok = 0;
      return;}

  if (xosd_set_font(channel_xosd, font))
    if (xosd_set_font(channel_xosd, osd_default_font)) {
        fprintf (stderr, "ABORT: %s\n", xosd_error);
	libxosd_ok = 0;
	return;}

  if (!font)
    font = strdup("-*-ledfixed-*-*-*-*-*-*-*-*-*-*-*-*");
  if (!colour)
    colour = strdup("white");
  if (!outline_colour)
    outline_colour = strdup("black");  

  xosd_set_font (channel_xosd, font);
  xosd_set_shadow_offset (channel_xosd, shadow_offset);
  xosd_set_colour (channel_xosd, colour);
  xosd_set_outline_offset(channel_xosd, outline_offset);
  xosd_set_outline_colour(channel_xosd, outline_colour);
  xosd_set_align(channel_xosd, XOSD_center);
  xosd_set_pos (channel_xosd, XOSD_bottom);
  xosd_set_vertical_offset (channel_xosd, -y - h + sheight);
  xosd_set_align (channel_xosd, XOSD_center);
  xosd_set_horizontal_offset (channel_xosd,x + w/2 - swidth/2);
  xosd_set_bar_length(channel_xosd, w/20);
  xosd_display (channel_xosd, 0,XOSD_string,message1);
  xosd_display (channel_xosd, 1,XOSD_string,message2);
  xosd_set_timeout(channel_xosd, message_timer/1000);
}

static void
add_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  struct CHANNEL *channel;
  char *name, *key;

  if(scantv) {
    popup_mess(translate("You can't delete, add or update \nany channel during a TV scanning\n"));
    return;}

  XtVaGetValues (conf_name, XtNstring, STRDUP(&name), NULL);
  XtVaGetValues (conf_key, XtNlabel, &key, NULL);

  if (0 == strlen (name)) return;

  channel = add_channel (name);
  if (strlen (key) > 0) channel->key = strdup (key);
  channel->cname = (cur_channel != -1) ? tvtuner[cur_channel].name : "???";
  channel->channel = cur_channel;
  channel->fine = cur_fine;

  if (channel_list[0] == "empty") XawListUnhighlight (conf_list);
  configure_channel(channel);
  channel_menu ();
  scantv_xosd(translate("Add a channel:"), translate(name));
}



static void
del_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  char *name;
  int last_num=count-1;

  if(scantv) {
    popup_mess(translate("You can't delete, add or update \nany channel during a TV scanning\n"));
    return;
  }
  if (cur_sender == -1 || last_num == -1) return;

  //Xosd: before deleting, an xosd message is send.
  XtVaGetValues (conf_name, XtNstring, STRDUP(&name), NULL);
  scantv_xosd(translate("Delete a channel:"), translate(name));

/* 3 cas se presentent : 
 * 	1) on est au debut       => les elements sont supprimes par le bas
 * 	2) on est au milieu      => les elements sont supprimes par en bas
 * 	2) on est �la fin       => les elements sont supprim� par le haut
 */

  if (cur_sender == 0){	// this channel is the first one
	del_channel (cur_sender);
	channel_menu ();
	if (0 == last_num)
  		cur_sender = -1;
  }else if ( cur_sender < last_num) { // on se trouve au sein de la liste
	del_channel (cur_sender);
 	if (cur_sender != 0) XawListUnhighlight (conf_list);
	channel_menu ();
  }else if ( cur_sender == last_num ){  // on se trouve �la fin de la liste
	del_channel (cur_sender);
 	if (cur_sender != 0) XawListUnhighlight (conf_list);
	channel_menu ();
  	cur_sender -- ; 
  }else {
  	fprintf(stderr, "conf.c : should never appear ... return\n");
	return;
  }
  
  editor_station_switched ();
}

static void
modify_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  char *name, *key;

  if(scantv) {
    popup_mess(translate("You can't delete, add or update \nany channel during a TV scanning\n"));
    return;}

  XtVaGetValues (conf_name, XtNstring, STRDUP(&name), NULL);
  XtVaGetValues (conf_key, XtNlabel, &key, NULL);

  if (0 == strlen (name))
    return;

  /* no channel yet ... */
  if (-1 == cur_sender) {
      add_cb (widget, clientdata, call_data);
      return;
    }

  free (channels[cur_sender]->name);
  channels[cur_sender]->name = strdup (name);

  if (channels[cur_sender]->key)
    free (channels[cur_sender]->key);
  if (0 != strlen (key))
    channels[cur_sender]->key = strdup (key);
  else
    channels[cur_sender]->key = 0;
  hotkey_channel (channels[cur_sender]);

  channel_menu ();
  editor_station_switched ();
  scantv_xosd(translate("Update a channel:"), translate(name));
}

static void
moveup_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  struct CHANNEL *tmp=NULL; 
  
  if (cur_sender == -1 || cur_sender == 0) return;

  if(scantv) {
    popup_mess(translate("You can't delete, add or update \nany channel during a TV scanning\n"));
    return;}

  if (channels[cur_sender] == NULL || channels[cur_sender-1] == NULL ){
//  	fprintf(stderr,"NULL pointer ! problem !\n");
	return;
  }
  tmp=channels[cur_sender];
  channels[cur_sender]=channels[cur_sender-1];
  channels[cur_sender-1]=tmp;
  cur_sender--;

  channel_menu ();
  editor_station_switched ();
}

static void* auto_update(void* arg){

	chan_handler_t* chan_hdl=get_chan_handler();

	chan_hdl_is_in_use(chan_hdl); // to be sure that chan_hdl is not deleted if this thread isnt finished !

	while (chan_hdl->scan_running == 0){
		if (debug>2) fprintf(stderr, "scan not running yet ...\n");
		usleep(100000);
	}
	if (debug > 2) fprintf(stderr, "scan running now !\n");

#ifdef HAVE_DVB
	if ((grabbers[grabber]) != (&grab_dvb))
#endif
	  { 
		while (chan_hdl->scan_running != 2){
			char current_freq[20];
			pthread_mutex_lock(&(chan_hdl->scan_mutex));
			snprintf(current_freq,20,"%f", (chan_hdl->current_freq));
		//	scantv_xosd(translate("Current Freq:"),current_freq);
			
			if (new_channel_available(chan_hdl)) {
				struct CHANNEL* cur_chan=NULL;
				struct CHANNEL* added_chan=NULL;
				cur_chan=&(chan_hdl->new_channels[chan_hdl->chan_number]);
				scantv_xosd(translate("A new channel detected:"), cur_chan->name);
				added_chan=add_channel_copy(cur_chan);
				channel_menu();
				editor_station_switched ();
				new_channel_accepted(chan_hdl);
			}
			pthread_mutex_unlock(&(chan_hdl->scan_mutex));
			usleep (10000);
		}	
		if (debug>2)fprintf(stderr,"** number of detected channels : %d\n",chan_hdl->chan_number);
	}
#ifdef HAVE_DVB
else { // for the dvb scan, the update isn't real-time .. for the moment.
		int i=0;
		while (chan_hdl->scan_running != 2){
			usleep (10000);
		}
		if (debug>2) fprintf(stderr,"** number of detected channels : %d\n",chan_hdl->chan_number);
		for(i=0;i<chan_hdl->chan_number;i++){
			struct CHANNEL* cur_chan=NULL;
			struct CHANNEL* added_chan=NULL;
			cur_chan=&(chan_hdl->new_channels[i]);
			added_chan=add_channel_copy(cur_chan);
			channel_menu();
			editor_station_switched ();
			new_channel_accepted(chan_hdl);
		}
	}
#endif


	chan_hdl_is_not_in_use(chan_hdl);
	chan_hdl=NULL;
	if (strcmp (channel_list[0],"empty")==0 ) 
		XawListUnhighlight (conf_list);
	else {
 		cur_sender = 0 ;
 		set_channel (channels[cur_sender]);
	}
	
	channel_menu ();
  	editor_station_switched ();
	if (debug) fprintf(stderr,"\n ** scan finished. **\n");
	scantv_xosd(translate("Scanning..."), translate("Finished"));
	scantv = 0;
	return NULL;
}
static void
fullscan (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  complete_scan = !complete_scan;

  if (!complete_scan ) {
    sprintf (title, translate("Fullscan Off"));
    create_editor_off_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
  }
  else {
    sprintf (title, translate("Fullscan On"));
    create_editor_on_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
  }

  if (c_chan_fullscan)
    XtVaSetValues (c_chan_fullscan, XtNlabel, title, NULL);

}


static void
set_shortcuts (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int i=0;

  if (cur_sender == -1) return;

  if(scantv) {
    popup_mess(translate("You can't delete, add or update \nany channel during a TV scanning\n"));
    return;}

  for (i=0; i< count;i++ ){
	if (channels[i]->key)
    		free (channels[i]->key);

  	channels[i]->key=strdup(int_to_str(i, shortcuts_list));
	hotkey_channel (channels[i]);
  }

  channel_menu ();
  editor_station_switched ();
}


static void
builtinscan (Widget widget, XtPointer clientdata, XtPointer call_data)
{
 char* current_source=NULL;
// set_capture(CAPTURE_OFF,0);

 
  if(scantv) {
    popup_mess(translate("TV scanning already started\n"));
    return;
  }

  if (!grabbers )  { 
	  fprintf(stderr,"don't have any grabber ?!?\n");
	  return;
  }
  if (!grabbers[grabber]){
	  fprintf(stderr,"don't have a valid grabber ?!?\n");
	  return;
  }
  if (!grabbers[grabber]->inputs){
	  fprintf(stderr,"don't have a valid grabber->inputs ?!?\n");
	  return;
  }
  if (grabbers[grabber] == &grab_avi){
	  fprintf(stderr,"no builtinscan with avi grabber\n");
	  return;
  }
  if (grabbers[grabber] == &grab_dummy){
	  fprintf(stderr,"no builtinscan with dummy grabber\n");
	  return;
  }

  current_source = int_to_str(cur_input,grabbers[grabber]->inputs) ;
  
  if (!current_source) return;
  
  if (strcmp(current_source , "Television") != 0){
  	  fprintf(stderr,translate("Error : you cannot perform a scan on source %s !\n"),current_source);
	  return;
  }
  
  scantv =1;
  scantv_xosd(translate("Scanning..."), translate("Started"));

#ifdef HAVE_DVB
  if ((grabbers[grabber]) != (&grab_dvb))
#endif
  {
	pthread_create (&scan_thread,NULL,scan_v4l, &complete_scan);
  	//pthread_create (&update_thread,NULL,auto_update , NULL);
  	auto_update ( NULL);
  }
#ifdef HAVE_DVB
else { /// DVB CHOICE
	  struct stat buf;
	  char* init_filename=get_dvbinit_file_name();
	  int exist=0;
	  
  	  if (0 == stat (init_filename , &buf)){
		if (S_ISREG(buf.st_mode)){
			if (debug)
				fprintf(stderr,"start dvb SCAN !\n");
			grab_dvb.grab_close();
			usleep(100000); 
	  		exist=1;
			pthread_create (&scan_thread,NULL,scan_dvb , NULL);
  		//	pthread_create (&update_thread,NULL,auto_update , NULL);
  			auto_update ( NULL);
			return;
		}else {
			fprintf(stderr,"%s exist but isn't a regular file !\n",init_filename );
			return;
		}
	  }else {
		 
	  	fprintf(stderr,"NO DVB init file  => we start the GUI!\n");
	 	XtCallActionProc (dvbinit_shell, "DVBinit", NULL, NULL, 0);
		scantv =0;
		
	  }
  }
#endif
	
}

static void
movedown_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  struct CHANNEL *tmp=NULL; 
  int last_num=count-1;

  if (cur_sender == -1 || last_num == -1) return;

  if (!channels){
// 	fprintf(stderr,"no channel ! return \n");
  	return;
  }

  if(scantv) {
    popup_mess(translate("You can't delete, add or update \nany channel during a TV scanning\n"));
    return;}

  if (channels[cur_sender] == NULL || channels[cur_sender+1] == NULL ){
	return;
  }

  if (last_num == cur_sender ) {
//  	fprintf(stderr,"twe are at the end ! return \n");
  	// we are at the end ! 
	return;
  }

  tmp=channels[cur_sender];
  channels[cur_sender]=channels[cur_sender+1];
  channels[cur_sender+1]=tmp;
  cur_sender++;

  channel_menu ();
  editor_station_switched ();
}



/*static*/ void
save_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  save_config ();
  sprintf (title, translate("Configuration Saved"));  
  set_timer_title ();
}

static void
close_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  XtCallActionProc (conf_shell, "Editor", NULL, NULL, 0);
}

static void
key_eh (Widget widget, XtPointer client_data, XEvent * event, Boolean * cont)
{
  XKeyEvent *ke = (XKeyEvent *) event;
  KeySym sym;
  char *key;
  char line[64];

  sym = XKeycodeToKeysym (dpy, ke->keycode, 0);
  if (NoSymbol == sym)
    {
      fprintf (stderr, "can't translate keycode %d\n", ke->keycode);
      return;
    }
  key = XKeysymToString (sym);

  line[0] = '\0';
  if (ke->state & ShiftMask)
    strcpy (line, "Shift+");
  if (ke->state & ControlMask)
    strcpy (line, "Ctrl+");
  strcat (line, key);
  XtVaSetValues (conf_key, XtNlabel, line, NULL);
}

/*-------------------------------------------------------------------------*/

#define FIX_RIGHT_TOP       \
    XtNleft,XawChainRight,  \
    XtNright,XawChainRight, \
    XtNtop,XawChainTop,     \
    XtNbottom,XawChainTop

void
create_editorwin ()
{
  Widget form, label, command;

  conf_shell = XtVaAppCreateShell ("Editor", "XdTV",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell,
				   XtNvisual,vinfo->visual,
				   XtNcolormap,colormap, XtNdepth,vinfo->depth,
				   NULL);

  form = XtVaCreateManagedWidget ("form", formWidgetClass, conf_shell, NULL);

  /* list */
  viewport =
    XtVaCreateManagedWidget ("viewport", viewportWidgetClass, form,
                             XtNleft, XawChainLeft,
                             XtNright, XawChainRight,
                             XtNtop, XawChainTop,
                             XtNbottom, XawChainBottom, NULL);
  conf_list =
    XtVaCreateManagedWidget ("list", listWidgetClass, viewport, NULL);
  XtAddCallback (conf_list, XtNcallback, list_cb, (XtPointer) NULL);

/* ------------------------------------------------------------------------- */
  label_channelnum = label =
    XtVaCreateManagedWidget ("lchannel", labelWidgetClass, form,
                             FIX_RIGHT_TOP, XtNfromHoriz, viewport, NULL);
  conf_channel =
    XtVaCreateManagedWidget ("channel", labelWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, label, NULL);
  label_choosehotkey = label =
    XtVaCreateManagedWidget ("lkey", labelWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, conf_channel, NULL);
  conf_key =
    XtVaCreateManagedWidget ("key", labelWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, label, NULL);
label_stationname = label =
    XtVaCreateManagedWidget ("lname", labelWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, conf_key, NULL);
  conf_name =
    XtVaCreateManagedWidget ("name", asciiTextWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, label,
			     XtNtype, XawAsciiString,
			     NULL);
  XtAddEventHandler (conf_key, KeyPressMask, False, key_eh, NULL);

  /* buttons */

  c_chan_move_up = command =
    XtVaCreateManagedWidget ("moveup", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz,viewport,
                             XtNfromVert, conf_name, NULL);
  XtAddCallback (command, XtNcallback, moveup_cb, (XtPointer) NULL);

  c_chan_move_down = command =
    XtVaCreateManagedWidget ("movedown", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                            XtNfromHoriz,command,
                             XtNfromVert,conf_name, NULL);

  XtAddCallback (command, XtNcallback, movedown_cb, (XtPointer) NULL);

  c_chan_shortcuts = command =
    XtVaCreateManagedWidget ("shortcuts", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
  XtAddCallback (command, XtNcallback, set_shortcuts, (XtPointer) NULL);



  c_chan_rc = command =
    XtVaCreateManagedWidget ("editrc", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert,command , NULL);
  signal(SIGCHLD, waitforchild);
  XtAddCallback (command, XtNcallback, execute_it, translate("xterm -geometry 80x35+0+0 -title xdtvrc -e vi $HOME/.xdtv/xdtvrc"));
  c_chan_scantv = command =
    XtVaCreateManagedWidget ("scantv", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
    XtAddCallback (command, XtNcallback, builtinscan, (XtPointer) NULL);

   c_chan_fullscan = command =
	  XtVaCreateManagedWidget("fullscan",toggleWidgetClass ,form,
	       FIX_RIGHT_TOP,
               XtNfromHoriz, viewport,
               XtNfromVert,command, NULL);


  XtAddCallback(c_chan_fullscan, XtNcallback,fullscan ,(XtPointer)  NULL) ;

  c_chan_openmain = command =
    XtVaCreateManagedWidget ("openmain", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
    XtAddCallback (command, XtNcallback, action_cb, (XtPointer) & call_options_1);   
#ifdef HAVE_DVB
  c_chan_opendvbinit = command =
    XtVaCreateManagedWidget ("opendvbinit", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
    XtAddCallback (command, XtNcallback, action_cb, (XtPointer) & call_dvbinit);  
#endif

  c_chan_add = command =
    XtVaCreateManagedWidget ("add", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
  XtAddCallback (command, XtNcallback, add_cb, (XtPointer) NULL);
  c_chan_delete = command =
    XtVaCreateManagedWidget ("delete", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
  XtAddCallback (command, XtNcallback, del_cb, (XtPointer) NULL);
  c_chan_update = command =
    XtVaCreateManagedWidget ("modify", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
  XtAddCallback (command, XtNcallback, modify_cb, (XtPointer) NULL);
  c_chan_save = command =
    XtVaCreateManagedWidget ("save", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
  XtAddCallback (command, XtNcallback, save_cb, (XtPointer) NULL);
  last_command = command =
    XtVaCreateManagedWidget ("close", commandWidgetClass, form,
                             FIX_RIGHT_TOP,
                             XtNfromHoriz, viewport,
                             XtNfromVert, command, NULL);
  XtAddCallback (command, XtNcallback, close_cb, (XtPointer) NULL);

  XtInstallAllAccelerators (conf_name, conf_shell);
#ifdef HAVE_XPM
  XpmCreatePixmapFromData(XtDisplay(conf_shell),
                          RootWindowOfScreen(XtScreen(conf_shell)),
                          icon_xpm,
                          &icon_pixmap,
                          &icon_shapemask, NULL);
  XtVaSetValues(conf_shell, XtNiconPixmap, icon_pixmap, NULL);

  create_editor_common_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
  if (!complete_scan)
    create_editor_off_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
  else
    create_editor_on_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));

#ifdef HAVE_DVB
   create_editor_dvb_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
#endif

#endif
}

void
EditorAction (Widget widget, XEvent * event,
              String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (conf_shell), XtWindow (conf_shell), NULL,
                       0);
          return;
        }
    }

  if (mapped)
    {
      XtPopdown (conf_shell);
      mapped = 0;
    }
  else
    {
      XtPopup (conf_shell, XtGrabNone);
      mapped = 1;
      if (first)
        {
          XSetWMProtocols (XtDisplay (conf_shell), XtWindow (conf_shell),
                           wm_protocols, 2);
          first = 0;
        }
    }
}
/*-------------------------------------------------------------------------*/

void
editor_station_switched ()
{
  char line[128] = "???";

  /* channel */
  if (cur_channel != -1) {
     strcpy (line, tvtuner[cur_channel].name);
     if (cur_fine != 0)
        sprintf (line + strlen (line), " (%+d)", cur_fine);
  }
  XtVaSetValues (conf_channel, XtNlabel, line, NULL);

  if (cur_sender == -1)
    {
      XtVaSetValues (conf_key, XtNlabel, "", NULL);
      XtVaSetValues (conf_name, XtNstring, STRDUP(""), NULL);
      XawListUnhighlight (conf_list);
    }
  else
    {
      if (channels[cur_sender]->key)
        XtVaSetValues (conf_key, XtNlabel, channels[cur_sender]->key, NULL);
      else
        XtVaSetValues (conf_key, XtNlabel, "", NULL);
      XtVaSetValues (conf_name, XtNstring, STRDUP(channels[cur_sender]->name), NULL);
      XawListHighlight (conf_list, cur_sender);
    }
}

void
editor_list_update ()
{
  int i;

  if (channel_list)
    free (channel_list);

//  if (cur_sender != 0)
//	  XawListUnhighlight (conf_list); // nico : sometimes, there is a segfault ...

  if (count)
    {
      /* rebuild list */
      channel_list = malloc ((count + 1) * sizeof (String));
      for (i = 0; i < count; i++)
        channel_list[i] = channels[i]->name;
      channel_list[i] = NULL;
    }
  else
    {
      /* empty list */
      channel_list = malloc (2 * sizeof (String));
      channel_list[0] = "empty";
      channel_list[1] = NULL;
    }
  XtVaSetValues (conf_list, XtNlist, channel_list, XtNnumberStrings, 0, NULL);
}

/*-------------------------------------------------------------------------*/
