/*****************************************************************************
 * container-avi.c : support for AVI format output
 *****************************************************************************
 * $Id: container-avi.c,v 1.9 2005/12/10 17:52:10 pingus77 Exp $
 *****************************************************************************
 * Program under GNU General Public License (see ../COPYING)
 *****************************************************************************/

#include <config.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <pthread.h>
#include <limits.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "container.h"
#include "divx.h"

#define HEADERBYTES 2048

static  int open_ofile (struct CONTAINER* This, char* filename,int maxsize)
{
	
	This->avfile = AVI_open_output_file(filename,maxsize);
	if (This->avfile) 
		return 0; 
	else 	return -1;
}
static  int    write_frame(struct CONTAINER* This, char* data, long bytes, long flags)
{
	return AVI_write_frame(This->avfile,data,bytes,flags);
}
static  int    write_audio(struct CONTAINER* This, char *data, long bytes)
{
	return AVI_write_audio(This->avfile,data,bytes);
}
static  int 	 close_ofile      (struct CONTAINER* This)
{
		return AVI_close(This->avfile);
}

static  unsigned 
  char*  get_header (struct CONTAINER* This,int* ret_size)
{
	unsigned char* header;
	if(NULL==(header=(unsigned char*)malloc(HEADERBYTES*sizeof(*header)))){
		fprintf(stderr,"get_header : malloc problem \n");
		return NULL;
	}
	*ret_size=HEADERBYTES;
	AVI_get_header(This->avfile,header);
	return header;
}
static void set_extradata(struct CONTAINER* This, unsigned char* extradata, int extradata_size){
	AVI_set_extradata(This->avfile,extradata, extradata_size);
}

static struct STRTAB avi_codec_list[] = {
#ifdef HAVE_XVID
  {XVID, "XviD"},
#endif
#ifdef HAVE_FFMPEG
#ifdef HAVE_FFMPEG_XVID
#if LIBAVCODEC_BUILD >= 4719
  {FFMPEG_XVID,     "FFMpeg XviD"},
#endif
#endif
  {FFMPEG_MPEG4,     "FFMpeg Mpeg4"},
#if LIBAVCODEC_BUILD >= 4676
  {FFMPEG_MPEG2,     "FFMpeg Mpeg2"},
#endif
  {FFMPEG_MPEG1,     "FFMpeg Mpeg1"},
  {FFMPEG_MJPEG,     "FFMpeg Mjpeg"},
#if LIBAVCODEC_BUILD >= 4670
  {FFMPEG_LJPEG,     "FFMpeg Lowless Mjpeg"},
  {FFMPEG_FLV1,      "FFMpeg Flash Video"},
#endif
#if LIBAVCODEC_BUILD >= 4735
  {FFMPEG_RV20,      "FFMpeg RealVideo 2.0"},
#endif
  {FFMPEG_RV10,      "FFMpeg RealVideo 1.0"},
#ifdef HAVE_FFMPEG_X264
#if LIBAVCODEC_BUILD >= 4748
  {FFMPEG_X264,     "FFMpeg H.264"},
#endif
#endif
  {FFMPEG_H263P,     "FFMpeg H.263+"},
  {FFMPEG_H263,      "FFMpeg H.263"},
#if LIBAVCODEC_BUILD >= 4729
  {FFMPEG_H261,      "FFMpeg H.261"},
#endif
  {FFMPEG_MSMPEG4V3, "FFMpeg MS Mpeg4 V3"},
  {FFMPEG_MSMPEG4V2, "FFMpeg MS Mpeg4 V2"},
/* Already implemented into the 0.4.6 version 
but only work correctly with the 0.4.7 version */
#if LIBAVCODEC_BUILD >= 4675
  {FFMPEG_WMV2,      "FFMpeg WMV2-WMV8"},
#endif
  {FFMPEG_WMV1,      "FFMpeg WMV1-WMV7"},
#if LIBAVCODEC_BUILD >= 4677
  {FFMPEG_ASV2,      "FFMpeg Asus V2"},
#endif
#if LIBAVCODEC_BUILD >= 4668
  {FFMPEG_ASV1,      "FFMpeg Asus V1"},
#endif
/* appeared with the 4719 version
but don't work correctly actually */
#if LIBAVCODEC_BUILD >= 16777215
  {FFMPEG_SVQ1,      "FFMpeg SVQ"},
#endif
/* appeared with the 4669 version
but only work correctly with the 4755 version */
#if LIBAVCODEC_BUILD >= 4755
  {FFMPEG_FFV1,      "FFMpeg FFV1"},
#endif
#if LIBAVCODEC_BUILD >= 4719
  {FFMPEG_SNOW,      "FFMpeg SNOW"},
#endif
#if LIBAVCODEC_BUILD >= 4734
  {FFMPEG_FFVHUFF, "FFMpeg Ffvhuff"},
#endif
  {FFMPEG_HUFFYUV, "FFMpeg HuffYUV"},
#endif // HAVE_FFMPEG
#ifdef HAVE_DIVX4LINUX
  {DIVX4LINUX, "DivX"},
#endif
  {UNCOMPRESSED, "UnCompressed"},
  {-1, NULL}
};


static struct STRTAB avi_audio_codec_list[] = {
#ifdef HAVE_LAME
  {LAME, "LAME MP3"},
#endif
#ifdef HAVE_FFMPEG
  {FFMPEG_MP2, "FFMpeg MP2"},
#ifndef HAVE_LAME
#ifdef HAVE_FFMPEG_MP3
  {FFMPEG_MP3, "FFMpeg MP3"},
#endif
#endif
#ifdef HAVE_FFMPEG_AC3
  {FFMPEG_AC3, "FFMpeg AC3"},
#endif
#endif // HAVE_FFMPEG
  {AUDIO_UNCOMPRESSED, "UnCompressed"},
  {-1, NULL}
};

struct CONTAINER cont_avi = {
	"AVI", 
	"avi",
	AVI_BUILTIN,
	"avi",
	avi_codec_list,
	avi_audio_codec_list,
	NULL,
	open_ofile, 
	set_audio,
	set_video,
	write_frame,
	write_audio,
	close_ofile,
	get_header,
	set_extradata
};
  
