/*****************************************************************************
 * container-ogg.c : support for OGG format output
 *****************************************************************************
 * $Id: container-ogg.c,v 1.8 2006/01/13 21:19:24 estre Exp $
 *****************************************************************************
 * Program under GNU General Public License (see ../COPYING)
 *****************************************************************************
 * for a large part, this code comes from encoder_example.c in libtheora 
 * 1.0-alpha5 
 * */


#include <config.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <pthread.h>
#include <limits.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "audio.h"
#include "container.h"
#include "divx.h"


#ifdef HAVE_OGG
#include <theora/theora.h>
#include <vorbis/codec.h>
#include <vorbis/vorbisenc.h>

extern int debug;
extern pthread_mutex_t avi_mutex;

extern divx_t divx;

/* video Muxing Params */
static ogg_stream_state to; /* take physical pages, weld into a logical stream of packets */

static ogg_page         videopage;   /* one Ogg bitstream page.  Theora packets are inside */
static ogg_packet       videopacket; /* one raw packet of data for decode */

static theora_state     td;
static theora_info      ti;
static theora_comment   tc;


static int video_x=0;
static int video_y=0;
static int frame_x_offset=0;
static int frame_y_offset=0;

static int want_video=1; /* should be only for debug */

/* Audio Muxing Params */
static ogg_stream_state vo; /* take physical pages, weld into a logical stream of packets */
static ogg_page         audiopage;  /* one Ogg bitstream page.  Vorbis packets are inside */
static ogg_packet       audiopacket; /* one raw packet of data for decode */


static vorbis_info      vi; /* struct that stores all the static vorbis bitstream settings */
static vorbis_comment   vc; /* struct that stores all the user comments */
static vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
static vorbis_block     vb; /* local working space for packet->PCM decode */

static int audio_channels;
static int audio_fmt_factor;

static int want_audio=1; /* should be only for debug */

/* Common params */
  
static FILE* outfile;

static void print_info(theora_info ti){
	fprintf(stderr,"width %d\n",	  ti.width);
	fprintf(stderr,"height 	%d\n",  ti.height);
	fprintf(stderr,"frame_width %d\n",	ti.frame_width);
	fprintf(stderr,"frame_height %d\n",	  ti.frame_height);
	fprintf(stderr,"offset_x %d\n",	  ti.offset_x);
	fprintf(stderr,"offset_y %d\n",	  ti.offset_y);
	fprintf(stderr,"fps_numerator %d\n",	  ti.fps_numerator);
	fprintf(stderr,"fps_denominator	 %d\n",  ti.fps_denominator);
	fprintf(stderr,"aspect_numerator %d\n",	  ti.aspect_numerator);
	fprintf(stderr,"aspect_denominator %d\n",	  ti.aspect_denominator);
	fprintf(stderr,"colorspace %d\n",	  ti.colorspace);
	fprintf(stderr,"pixelformat %d\n",	  ti.pixelformat);
	fprintf(stderr,"target_bitrate %d\n",	  ti.target_bitrate);
	fprintf(stderr,"quality	%d\n",  ti.quality);
	fprintf(stderr,"dropframes_p %d\n",	  ti.dropframes_p);
	fprintf(stderr,"quick_p %d\n",	  ti.quick_p);
	fprintf(stderr,"keyframe_auto_p	%d\n",  ti.keyframe_auto_p);
	fprintf(stderr,"keyframe_frequency %d\n",	  ti.keyframe_frequency);
	fprintf(stderr,"keyframe_frequency_force %d\n",	  ti.keyframe_frequency_force);
	fprintf(stderr,"keyframe_data_target_bitrate %d\n",	  ti.keyframe_data_target_bitrate);
	fprintf(stderr,"keyframe_auto_threshold %d\n",	  ti.keyframe_auto_threshold);
	fprintf(stderr,"keyframe_mindistance %d\n",	  ti.keyframe_mindistance);
	fprintf(stderr,"noise_sensitivity %d\n",	  ti.noise_sensitivity);
}
static  int open_ofile (struct CONTAINER* This, char* filename,int maxsize)
{
	
  int ret=0;
  ogg_packet header;
  ogg_packet header_comm;
  ogg_packet header_code;


  if(NULL == (outfile=fopen(filename,"wb"))){
     fprintf(stderr,"Unable to open output file '%s'\n",filename);
     return -1;
  }

  /* yayness.  Set up Ogg output stream */
  srand(time(NULL));
  {
    /* need two inequal serial numbers */
    int serial1, serial2;
    serial1 = rand();
    serial2 = rand();
    if (serial1 == serial2) serial2++;
    if (want_video)
    	ogg_stream_init(&to,serial1);
    if (want_audio)
    	ogg_stream_init(&vo,serial2);
  }

   /* Theora has a divisible-by-sixteen restriction for the encoded video size */
  /* scale the frame size up to the nearest /16 and calculate offsets */
  video_x=((divx.width  + 15) >>4)<<4;
  video_y=((divx.height + 15) >>4)<<4;
  /* We force the offset to be even.
     This ensures that the chroma samples align properly with the luma
      samples. */
  frame_x_offset=((video_x-divx.width )/2)&~1;
  frame_y_offset=((video_y-divx.height)/2)&~1;

  
  if (want_video){
	  theora_info_init(&ti);
	  ti.width=divx.width;
	  ti.height=divx.height;
	  ti.frame_width=video_x;
	  ti.frame_height=video_y;
	  ti.offset_x=frame_x_offset;
	  ti.offset_y=frame_y_offset;
	  ti.fps_numerator= divx.fps;
	  ti.fps_denominator=1000;
	  ti.aspect_numerator=1;
	  ti.aspect_denominator=1;
	  ti.colorspace=OC_CS_UNSPECIFIED;
	  ti.pixelformat=OC_PF_420; // ok in theora 1.0 alpha5 ?
	  ti.target_bitrate=divx.bitrate ; //*1000;
	  ti.quality= divx.quality*6.3; // xdtv : 0 -> 9 , theora : 0 < quality < 63

	  ti.dropframes_p=0;
	  ti.quick_p=1;
	  ti.keyframe_auto_p=1;
	  ti.keyframe_frequency=64;
	  ti.keyframe_frequency_force=64;
	  ti.keyframe_data_target_bitrate=ti.target_bitrate*1.5;
	  ti.keyframe_auto_threshold=80;
	  ti.keyframe_mindistance=8;
	  ti.noise_sensitivity=1;

	  print_info(ti);
	  theora_encode_init(&td,&ti);
	  theora_info_clear(&ti);
  }

  if (want_audio){
	  vorbis_info_init(&vi);

	  audio_channels=divx.stereo?2:1;
	  audio_fmt_factor = divx.audio_fmt==AUDIO_U8 ? 1 : 2 ;

	  // in XdTV, quality is between 1 and 9, in vorbis codec, between -1 and 1 
	  if (divx.mp3_vbr){
	    float quality_vorbis=divx.mp3_vbr_quality/5.-1;  // ==> ~ -0.8 .. 0.8 
	    ret=vorbis_encode_init_vbr(&vi, audio_channels ,divx.audio_freq, quality_vorbis);
	  }else{
	    ret=vorbis_encode_init(&vi, audio_channels,divx.audio_freq,-1,divx.mp3_bitrate*1000,-1);
	  }
	  if(ret){
	      fprintf(stderr,"The Vorbis encoder could not set up a mode according to\n"
		      "the requested quality or bitrate.\n\n");
	      fclose(outfile);
	      return -1;
	  }
	    
	  /* add a comment */
	  vorbis_comment_init(&vc);
	  vorbis_comment_add_tag(&vc,"ENCODER","XdTV encoder");

	  /* set up the analysis state and auxiliary encoding storage */
	  vorbis_analysis_init(&vd,&vi);
	  vorbis_block_init(&vd,&vb);
  }
 
  /* write the bitstream header packets with proper page interleave */

  if (want_video){
	  /* first packet will get its own page automatically */
	  theora_encode_header(&td,&videopacket);
	  ogg_stream_packetin(&to,&videopacket);
	  if(ogg_stream_pageout(&to,&videopage)!=1){
	    fprintf(stderr,"Internal Ogg library error.\n");
	    exit(1);
	  }

			
	  fwrite(videopage.header,1,videopage.header_len,outfile);
	  fwrite(videopage.body,1,videopage.body_len,outfile);

	  /* create the remaining theora headers */
	  theora_comment_init(&tc);
	  theora_encode_comment(&tc,&videopacket);
	  ogg_stream_packetin(&to,&videopacket);
	  theora_encode_tables(&td,&videopacket);
	  ogg_stream_packetin(&to,&videopacket);
  }

  if (want_audio){
	  vorbis_analysis_headerout(&vd,&vc,&header,&header_comm,&header_code);
	  ogg_stream_packetin(&vo,&header); /* automatically placed in its own
						 page */
	  if(ogg_stream_pageout(&vo,&audiopage)!=1){
	     fprintf(stderr,"Internal Ogg library error.\n");
	     fclose(outfile);
	     return -1;
	  }
	  fwrite(audiopage.header,1,audiopage.header_len,outfile);
	  fwrite(audiopage.body,1,audiopage.body_len,outfile);

	    /* remaining vorbis header packets */
	  ogg_stream_packetin(&vo,&header_comm);
	  ogg_stream_packetin(&vo,&header_code);
  }
  /* Flush the rest of our headers. This ensures
     the actual data in each stream will start
     on a new page, as per spec. */
  if(want_video){
	  while(1){
	    int result = ogg_stream_flush(&to,&videopage);
	      if(result<0){
		/* can't get here */
		fprintf(stderr,"Internal Ogg library error.\n");
		fclose(outfile);
		return -1;
	      }
	    if(result==0)break;
	    fwrite(videopage.header,1,videopage.header_len,outfile);
	    fwrite(videopage.body,1,videopage.body_len,outfile);
	  }
  }
  if(want_audio){
	  while(1){
	    int result=ogg_stream_flush(&vo,&audiopage);
	    if(result<0){
	      /* can't get here */
	      fprintf(stderr,"Internal Ogg library error.\n");
	      fclose(outfile);
	      return -1;
	    }
	    if(result==0)break;
	    fwrite(audiopage.header,1,audiopage.header_len,outfile);
	    fwrite(audiopage.body,1,audiopage.body_len,outfile);
	  }
  }

  if (debug)
  	fprintf(stderr,"ogg initialisation finished!\n"); 

   /* now we just have to contruct an avi_t struct */
    if (NULL == (This->avfile = (avi_t *) malloc(sizeof(avi_t)))){
	    fclose(outfile);
	    return -1;
    }
    memset((void *)This->avfile,0,sizeof(avi_t));
    This->avfile->init = 0;
    This->avfile->mode = AVI_MODE_WRITE; /* open for writing */
    This->avfile->seekable = 1;
    This->avfile->send_data = NULL;
    pthread_mutex_init (&avi_mutex, NULL);

    return 0; 
}

static  int    write_frame(struct CONTAINER* This, char* data, long bytes, long flags)
{

  yuv_buffer          yuv;
  long pos;

  pos = This->avfile->pos;

  if (want_video){
		  yuv.y_width=video_x;
		  yuv.y_height=video_y;
		  yuv.y_stride=video_x;

		  yuv.uv_width=video_x/2;
		  yuv.uv_height=video_y/2;
		  yuv.uv_stride=video_x/2;

		  yuv.y = (unsigned char*) data;
		  yuv.u = (unsigned char*) data+ video_x*video_y;
		  yuv.v = (unsigned char*) data+ video_x*video_y*5/4;

		  theora_encode_YUVin(&td,&yuv);
		  theora_encode_packetout(&td,0,&videopacket);
		  ogg_stream_packetin(&to,&videopacket);

		  if(ogg_stream_pageout(&to,&videopage)>0){
			long bytes;
		        pthread_mutex_lock(&avi_mutex);
			bytes=fwrite(videopage.header,1,videopage.header_len,outfile);
			bytes+=fwrite(videopage.body,1,videopage.body_len,outfile);
			
			This->avfile->pos += bytes;
			This->avfile->last_pos = pos;
			This->avfile->last_len = bytes;
			This->avfile->video_frames++;
		        pthread_mutex_unlock(&avi_mutex);
		  }
  }

    return 0;

}

static int write_audio(struct CONTAINER* This, char *data, long size)
{
 /* expose the buffer to submit data */
  
   int samples;
   int i,j;
   float **buffer=NULL;
   int count=0;
   long bytes;

   if (want_audio){
	   samples=size/ audio_fmt_factor / audio_channels;
           buffer=vorbis_analysis_buffer(&vd,samples);
	      /* uninterleave samples */
	   for(i=0;i<samples;i++){
	     for(j=0;j<audio_channels;j++){
		buffer[j][i]=((data[count+1]<<8)|(0x00ff&(int)data[count]))/32768.f;
		count+=2;
	     }
	   }

	   vorbis_analysis_wrote(&vd,i);
	      
	   while(vorbis_analysis_blockout(&vd,&vb)==1){

	      /* analysis, assume we want to use bitrate management */
	      vorbis_analysis(&vb,NULL);
	      vorbis_bitrate_addblock(&vb);

	      while(vorbis_bitrate_flushpacket(&vd,&audiopacket)){
		int result;
		
		/* weld the packet into the bitstream */
		ogg_stream_packetin(&vo,&audiopacket);

		result=ogg_stream_pageout(&vo,&audiopage);
		if(result==0)break;
		
	        pthread_mutex_lock(&avi_mutex);
		fwrite(audiopage.header,1,audiopage.header_len, outfile);
		fwrite(audiopage.body,1,audiopage.body_len,outfile);
		bytes= audiopage.header_len+audiopage.body_len;
		This->avfile->pos += bytes;
   		This->avfile->audio_bytes += bytes;
	        pthread_mutex_unlock(&avi_mutex);
		
	      }
	    }
   }
   return 0;
}
static  int 	 close_ofile      (struct CONTAINER* This)
{
    if (want_audio){
	ogg_stream_clear(&vo);
	vorbis_block_clear(&vb);
	vorbis_dsp_clear(&vd);
	vorbis_comment_clear(&vc);
	vorbis_info_clear(&vi);
    }

    if (want_video){
    	ogg_stream_clear(&to);
    	theora_clear(&td);
    }
    fclose(outfile);
    pthread_mutex_destroy(&avi_mutex);
    return 0;
}

static  unsigned 
  char*  get_header (struct CONTAINER* This,int* ret_size)
{
	return NULL;
}

static struct STRTAB ogg_codec_list[] = {
  {THEORA, "Theora"},
  {-1, NULL}
};


static struct STRTAB ogg_audio_codec_list[] = {
  {VORBIS, "Vorbis"},
  {-1, NULL}
};

struct CONTAINER cont_ogg = {
	"OGG", 
	"ogg",
	OGG_BUILTIN,
	"ogg",
	ogg_codec_list,
	ogg_audio_codec_list,
	NULL,
	open_ofile, 
	set_audio,
	set_video,
	write_frame,
	write_audio,
	close_ofile,
	get_header
};
  
#endif
