/*****************************************************************************
 * cpu_accel.c: MMX accelerated functions
 *****************************************************************************
 * $Id: cpu_accel.c,v 1.6 2005/11/06 12:37:11 alainjj Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************
 *
 * This software is highly based on mpeg2dec. Those portions are
 * Copyright (C) 1999-2001 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
 *
 * This file is part of mpeg2dec, a free MPEG-2 video stream decoder.
 *
 * Portions of this file are Copyright (C) 1998-2004 VideoLAN
 *****************************************************************************/
#include <stdio.h>
 
#include "config.h"

#include "cpu_accel.h"

#ifdef HAVE_MMX

#ifdef HAVE_SIGNAL_H
#include <signal.h>                            /* SIGHUP, SIGINT, SIGKILL */
#include <setjmp.h>                                    /* longjmp, setjmp */

#ifdef HAVE_SIGRELSE
int sigrelse(int i_signal);
#endif

static void SigHandler   ( int );

static jmp_buf env;
static int     i_illegal;
static char   *psz_capability;
#endif

static uint32_t x86_accel (void)
{
    volatile uint32_t i_capabilities = 0;

    volatile unsigned long i_eax, i_ebx, i_ecx, i_edx;
    volatile unsigned char b_amd;

#  ifdef __x86_64__
#    define REG_a "rax"
#    define REG_b "rbx"
#  else
#    define REG_a "eax"
#    define REG_b "ebx"
#  endif

    /* Needed for x86 CPU capabilities detection */
#   define cpuid( reg )                    \
        asm volatile ( "push %%"REG_b"\n\t"   \
                       "cpuid\n\t"         \
                       "mov %%"REG_b",%1\n\t" \
                       "pop %%"REG_b"\n\t"    \
                     : "=a" ( i_eax ),     \
                       "=r" ( i_ebx ),     \
                       "=c" ( i_ecx ),     \
                       "=d" ( i_edx )      \
                     : "a"  ( reg )        \
                     : "cc" );

#   ifdef HAVE_SIGNAL_H
    void (*pf_sigill) (int) = signal( SIGILL, SigHandler );
#   endif

    asm volatile ( "push %%"REG_b"\n\t"
                   "pushf\n\t"
                   "pop %%"REG_a"\n\t"
                   "movl %%eax, %%ebx\n\t"
                   "xorl $0x200000, %%eax\n\t"
                   "push %%"REG_a"\n\t"
                   "popf\n\t"
                   "pushf\n\t"
                   "pop %%"REG_a"\n\t"
                   "mov %%"REG_b",%1\n\t"
                   "pop %%"REG_b"\n\t"
                 : "=a" ( i_eax ),
                   "=r" ( i_ebx )
                 :
                 : "cc" );

    if( i_eax == i_ebx )
    {
#   ifdef HAVE_SIGNAL_H
        signal( SIGILL, pf_sigill );
#   endif
        return i_capabilities;
    }

    /* the CPU supports the CPUID instruction - get its level */
    cpuid( 0x00000000 );

    if( !i_eax )
    {
#   ifdef HAVE_SIGNAL_H
        signal( SIGILL, pf_sigill );
#   endif
        return i_capabilities;
    }

    /* borrowed from mpeg2dec */
    b_amd = ( i_ebx == 0x68747541 ) && ( i_ecx == 0x444d4163 )
                    && ( i_edx == 0x69746e65 );

    /* test for the MMX flag */
    cpuid( 0x00000001 );

    if( ! (i_edx & 0x00800000) )
    {
        return i_capabilities;
    }

    i_capabilities |= MM_ACCEL_X86_MMX;

    if( i_edx & 0x02000000 )
    {
        i_capabilities |= MM_ACCEL_X86_MMXEXT;

        /* We test if OS supports the SSE instructions */
        psz_capability = "SSE";
        i_illegal = 0;

        if( setjmp( env ) == 0 )
        {
            /* Test a SSE instruction */
            __asm__ __volatile__ ( "xorps %%xmm0,%%xmm0\n" : : );
        }

        if( i_illegal == 0 )
        {
            i_capabilities |= MM_ACCEL_X86_SSE;
        }
    }

    if( i_edx & 0x04000000 )
    {
        /* We test if OS supports the SSE instructions */
        psz_capability = "SSE2";
        i_illegal = 0;

        if( setjmp( env ) == 0 )
        {
            /* Test a SSE2 instruction */
            __asm__ __volatile__ ( "movupd %%xmm0, %%xmm0\n" : : );
        }

        if( i_illegal == 0 )
        {
            i_capabilities |= MM_ACCEL_X86_SSE2;
        }
    }

    /* test for additional capabilities */
    cpuid( 0x80000000 );

    if( i_eax < 0x80000001 )
    {
#   ifdef HAVE_SIGNAL_H
        signal( SIGILL, pf_sigill );
#   endif
        return i_capabilities;
    }

    /* list these additional capabilities */
    cpuid( 0x80000001 );

    if( i_edx & 0x80000000 )
    {
        psz_capability = "3D Now!";
        i_illegal = 0;

        if( setjmp( env ) == 0 )
        {
            /* Test a 3D Now! instruction */
            __asm__ __volatile__ ( "pfadd %%mm0,%%mm0\n" "femms\n" : : );
        }

        if( i_illegal == 0 )
        {
            i_capabilities |= MM_ACCEL_X86_3DNOW;
        }
    }

    if( b_amd && ( i_edx & 0x00400000 ) )
    {
        i_capabilities |= MM_ACCEL_X86_MMXEXT;
    }

#   ifdef HAVE_SIGNAL_H
    signal( SIGILL, pf_sigill );
#   endif
    return i_capabilities;
}

#if defined( HAVE_SIGNAL_H )
static void SigHandler( int i_signal )
{
    /* Acknowledge the signal received */
    i_illegal = 1;

#if defined( __i386__ )
    fprintf( stderr, "warning: your CPU has %s instructions, but not your "
                     "operating system.\n", psz_capability );
    fprintf( stderr, "         some optimizations will be disabled unless "
                     "you upgrade your OS\n" );
#   if defined( SYS_LINUX )
    fprintf( stderr, "         (for instance Linux kernel 2.4.x or later)\n" );
#   endif
#endif

    longjmp( env, 1 );
}
#endif
#endif

uint32_t mm_accel (void)
{
#ifdef HAVE_MMX
    static int got_accel = 0;
    static uint32_t accel;

    if (!got_accel) {
	got_accel = 1;
	accel = x86_accel ();
    }

    return accel;
#else
#ifdef HAVE_MLIB
    return MM_ACCEL_MLIB;
#else
    return 0;
#endif
#endif
}

/*
uint32_t mm_support (void)
{
  return mm_accel();
}
*/
