/*****************************************************************************
 * divx.c: DivX recording
 *****************************************************************************
 * $Id: divx.c,v 1.204 2007/02/19 15:04:27 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <pthread.h>
#include <math.h>
#include <sys/time.h>
#include "config.h"
#include "audio.h"
#include "writefile.h"
#include "aop.h"
#include "mixer.h"
#include "fourcc.h"

#include "divx.h"
#include "popup_ui.h"

#include <X11/Intrinsic.h>
#include "strtab.h"
#include "channel.h"
#include "blackborder.h"
#include "memcpy.h"
#include "grab.h"
#include "divx_ui.h"
#include "divx_advanced_ui.h"
#include "http.h"
#include "xdtv.h"

#ifdef HAVE_FFMPEG
#if LIBAVCODEC_BUILD < 4641
#  error we dont support libavcodec prior to build 4641, get the latest libavcodec CVS
#endif
#if LIBAVCODEC_BUILD < 4684
#define FF_QP2LAMBDA 1
#endif
#endif

// internationalization
#include "translation.h"

#include "container.h"

/*-------------------------------------------------------------------------*/
extern struct CONTAINER* cont_handler;
extern struct GRABBER *grabbers[],grab_avi;

#ifdef HAVE_DVB
extern struct GRABBER grab_dvb;
#endif

extern int grabber;
extern void file_wait_write_audio(void);
extern int fs, force_recfile;
extern char channel_title[256];
extern int cur_input;

#undef  NOAUDIO
#ifndef NOAUDIO
#define SYNCHRO
#endif

void video_get_size(int *w, int *h);
void video_set_size(int w, int h, int force);
void video_get_capture_size(int *w, int *h);
void video_set_capture_size(int w, int h);
void video_lock_size(void);
void video_unlock_size(void);
void set_capture_temp_grab(int grab, int mask);
char dvvideo_mode[8] ={'\0'};

extern pthread_mutex_t avi_mutex;

divx_t divx = {
AVI_BUILTIN,
#if defined (HAVE_XVID)
 XVID,         // codec to use
#elif defined (HAVE_FFMPEG)
 FFMPEG_MPEG4, // codec to use
#elif defined (HAVE_DIVX4LINUX)
 DIVX4LINUX,   // codec to use
#else
 UNCOMPRESSED,
#endif
    384, // width
    288, // height
 900000, // bitrate (bits/second)
      1, // quality
      2, // min_quantizer
      8, // max_quantizer
  44100, // audio_freq
 AUDIO_S16LE,  // audio_fmt
  4*441, // audio_buffer_size
     48, // audio_fragments
   2048, // audio_sizefragment
      0, // stereo
#ifdef HAVE_LAME
    LAME, // audio_codec
#else
   AUDIO_UNCOMPRESSED, // audio_codec
#endif
    128, // mp3_bitrate
      5, // mp3_quality
      0, // mp3_vbr
      8, // mp3_vbr_quality
  25000, // fps * 1000
   0.08, // maxgap between audio and video in seconds.
   0.00, // delay between audio and video
      1, // display_frames
      0, // sub
      0, // chg
   NULL, // filename
      0, // stream
  63427, // http_port
      0, // maxsize, in Mb
};

#ifdef HAVE_FFMPEG
ffmpeg_t ffmpeg = {
      0, // v4mv, 4MV allows 8x8 motion vectors for small movement.
      0, // naq, Normalize Adaptive Quantization.
      0, // gray, Only encode grayscale.
      0, // gmc, Global Motion Compensation allows better motion vectors for full image zooms, pans & rotation.
      0, // qpel, Use the Quarter Pel Motion Compensation.
      0, // ildct, Use the Interlaced DCT.
    250, // keyint, Keyframe Interval
      0, // vmax_b_frames, Maximum number of B-frames between non-B-frames.
      0, // vlelim, Sets single coefficient elimination threshold for luminance.
      0, // vcelim, Sets single coefficient elimination threshold for chrominance.
    0.0, // lumi_mask, Luminance masking: value = 0.0.5 is adviced  by Avidemux.
    0.0, // dark_mask, Darkness masking: value = 0.01 is adviced  by Avidemux.
    0.0, // tcplx_mask, Temporal complexity masking: value = 0.5 is adviced  by Avidemux.
    0.0, // scplx_mask, Spatial complexity masking: value = 0.5 is adviced  by Avidemux.
#if LIBAVCODEC_BUILD >= 4647
      0, // dia, Diamond type & size for motion estimation.
      0, // cmp, Sets the comparison function for full pel motion estimation.
      0, // subcmp, Sets the comparison function for sub pel motion estimation.
#endif
#if LIBAVCODEC_BUILD >= 4648
      0, // trell, Use the Trellis searched quantization.
#endif
#if LIBAVCODEC_BUILD >= 4649
      0, // Last_pred, amount of motion predictors from the previous frame.
#endif
#if LIBAVCODEC_BUILD >= 4651
      0, // predia, Diamond type & size for motion estimation pre-pass.
      0, // precmp, Sets the comparison function for motion estimation pre pass.
#endif
#if LIBAVCODEC_BUILD >= 4661
      0, // umv, Use the Unlimited Motion Vector for H.263+.
      0, // aic, Use the Advanced Intra Prediction for H.263+.
#endif
#if LIBAVCODEC_BUILD >= 4673
      0, // mbd, Macroblock decision algorithm.
#endif
#if LIBAVCODEC_BUILD >= 4681
      0, // cbp, Use the rate distorted optimal for Coded Block Pattern.
#endif
#if LIBAVCODEC_BUILD >= 4694
      0, // obmc, Use the Overlapped Block Motion Compensation.
      0, // ss, Use the Slice Structured mode for H.263+.
      0, // aiv, Use the Alternative Inter Vlc for H.263+.
      0, // loop, Use the loop filter for H.263+.
#endif
#if LIBAVCODEC_BUILD >= 4698
      0, // ilme,  Use the Interlaced Motion Estimation.
#endif
};
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
xvid_t xvid = {
         0, // vhq
#if XVID_API >= XVID_MAKE_API(4,1)
         0, // bvhq
#endif
         0, // chromame
         0, // chroma_opt
         0, // hqacpred
         0, // trell
         0, // cartoon
         0, // gray
         0, // interlacing
         1, // turbo
         0, // qpel
         0, // gmc
       250, // keyint
         0, // frame_drop_ratio
         0, // vmax_b_frames
       150, // bquant_ratio 
       100, // bquant_offset
         1, // packed
         1, // close_gop
         0, // bframe_threshold
        16, // reaction_delay_factor
       100, // averaging_period
       100, // buffer
{12,12,12}, // max_quant
   {2,2,2}, // min_quant
};
#endif
#endif

struct STRTAB container_list[] = {
	{AVI_BUILTIN, "AVI"},
#ifdef HAVE_OGG
	{OGG_BUILTIN, "OGG"},
#endif
#ifdef HAVE_FFMPEG
#if LIBAVCODEC_BUILD >= 4754
 	{MPEG_FFMPEG, "MPEG"},
 	{MPEGTS_FFMPEG, "MPEGTS"},
	{MP4_FFMPEG, "MP4"},
 	{ASF_FFMPEG, "ASF" },
 	{DV_FFMPEG, "DV"},
 	{VOB_FFMPEG,"VOB" },
 	{MOV_FFMPEG,"MOV" },
	{_3GP_FFMPEG,"3GP" },
	{_3G2_FFMPEG,"3G2" },
	{SWF_FFMPEG,"SWF" },
	{RM_FFMPEG,"RM" },
#endif
#endif // HAVE_FFMPEG
  {-1, NULL}
};

struct STRTAB codec_list[] = {
#ifdef HAVE_XVID
  {XVID, "XviD"},
#endif
#ifdef HAVE_OGG
  {THEORA, "Theora"},
#endif
#ifdef HAVE_FFMPEG_XVID
#if LIBAVCODEC_BUILD >= 4719
  {FFMPEG_XVID,     "FFMpeg XviD"},
#endif
#endif
#ifdef HAVE_FFMPEG
  {FFMPEG_MPEG4,     "FFMpeg Mpeg4"},
#if LIBAVCODEC_BUILD >= 4676
  {FFMPEG_MPEG2,     "FFMpeg Mpeg2"},
#endif
  {FFMPEG_MPEG1,     "FFMpeg Mpeg1"},
  {FFMPEG_MJPEG,     "FFMpeg Mjpeg"},
#if LIBAVCODEC_BUILD >= 4670
  {FFMPEG_LJPEG,     "FFMpeg Lowless Mjpeg"},
  {FFMPEG_FLV1,      "FFMpeg Flash Video"},
#endif
#if LIBAVCODEC_BUILD >= 4735
  {FFMPEG_RV20,      "FFMpeg RealVideo 2.0"},
#endif
  {FFMPEG_RV10,      "FFMpeg RealVideo 1.0"},
#ifdef HAVE_FFMPEG_X264
#if LIBAVCODEC_BUILD >= 4748
  {FFMPEG_X264,     "FFMpeg H.264"},
#endif
#endif
  {FFMPEG_H263P,     "FFMpeg H.263+"},
  {FFMPEG_H263,      "FFMpeg H.263"},
#if LIBAVCODEC_BUILD >= 4729
  {FFMPEG_H261,      "FFMpeg H.261"},
#endif
  {FFMPEG_MSMPEG4V3, "FFMpeg MS Mpeg4 V3"},
  {FFMPEG_MSMPEG4V2, "FFMpeg MS Mpeg4 V2"},
/* Already implemented into the 0.4.6 version 
but only work correctly with the 0.4.7 version */
#if LIBAVCODEC_BUILD >= 4675
  {FFMPEG_WMV2,      "FFMpeg WMV2-WMV8"},
#endif
  {FFMPEG_WMV1,      "FFMpeg WMV1-WMV7"},
#if LIBAVCODEC_BUILD >= 4677
  {FFMPEG_ASV2,      "FFMpeg Asus V2"},
#endif
#if LIBAVCODEC_BUILD >= 4668
  {FFMPEG_ASV1,      "FFMpeg Asus V1"},
#endif
/* appeared with the 4719 version
but don't work correctly actually */
#if LIBAVCODEC_BUILD >= 16777215
  {FFMPEG_SVQ1,      "FFMpeg SVQ"},
#endif
/* appeared with the 4669 version
but only work correctly with the 4755 version */
#if LIBAVCODEC_BUILD >= 4755
  {FFMPEG_FFV1,      "FFMpeg FFV1"},
#endif
#if LIBAVCODEC_BUILD >= 4719
  {FFMPEG_SNOW,      "FFMpeg SNOW"},
#endif
#if LIBAVCODEC_BUILD >= 4734
  {FFMPEG_FFVHUFF, "FFMpeg Ffvhuff"},
#endif
  {FFMPEG_HUFFYUV, "FFMpeg HuffYUV"},
#if LIBAVCODEC_BUILD >= 4707
  {FFMPEG_DVVIDEO,"FFMpeg DV"},
#endif
#endif //HAVE_FFMPEG
#ifdef HAVE_DIVX4LINUX
  {DIVX4LINUX, "DivX"},
#endif
  {UNCOMPRESSED, "UnCompressed"},
  {-1, NULL}
};


struct STRTAB audio_codec_list[] = {
#ifdef HAVE_LAME
  {LAME, "LAME MP3"},
#endif
#ifdef HAVE_OGG
  {VORBIS, "Vorbis"},
#endif
#ifdef HAVE_FFMPEG
  {FFMPEG_MP2, "FFMpeg MP2"},
#ifdef HAVE_FFMPEG_MP3
  {FFMPEG_MP3, "FFMpeg MP3"},
#endif
#ifdef HAVE_FFMPEG_AC3
  {FFMPEG_AC3, "FFMpeg AC3"},
#endif
#ifdef HAVE_FFMPEG_AAC
  {FFMPEG_AAC, "FFMpeg AAC"},
#endif
  {FFMPEG_PCMS16LE, "FFMpeg PCMS16LE"},
#ifdef HAVE_FFMPEG_AMR_NB
  {FFMPEG_AMR_NB, "FFMpeg AMR NB"},
#endif
#ifdef HAVE_FFMPEG_AMR_WB
  {FFMPEG_AMR_WB, "FFMpeg AMR WB"},
#endif
#endif // HAVE_FFMPEG
  {AUDIO_UNCOMPRESSED, "UnCompressed"},
  {-1,NULL}
};

static char *av_buffer = NULL, *av_buffer0 = NULL;
static int  av_buffer_size = 0, av_buffer0_l;

#ifdef HAVE_FFMPEG
static AVCodec *av_codec = NULL;
AVCodecContext *audio_codec_context = NULL;
int xdtv_to_audioffmpeg(int x) {
  switch(x) {
  case FFMPEG_MP2: return CODEC_ID_MP2;
#ifdef HAVE_FFMPEG_MP3
  case FFMPEG_MP3: return CODEC_ID_MP3;
#endif
#ifdef HAVE_FFMPEG_AC3
  case FFMPEG_AC3: return CODEC_ID_AC3;
#endif
#ifdef HAVE_FFMPEG_AAC
  case FFMPEG_AAC: return CODEC_ID_AAC;
#endif
  case FFMPEG_PCMS16LE: return CODEC_ID_PCM_S16LE;
#ifdef HAVE_FFMPEG_AMR_NB
  case FFMPEG_AMR_NB : return CODEC_ID_AMR_NB;
#endif
#ifdef HAVE_FFMPEG_AMR_WB
  case FFMPEG_AMR_WB : return CODEC_ID_AMR_WB;
#endif
  }
  return -1;
}
#endif


static int       slw; //status of the last AVI_write
static int displaynextframe = 1;

extern int debug;

char xawmessage [1024];
char xawmesstmp [256];
char lastfilename[256] = {'\0'};
char *pplayer = "mplayer -nofs -quiet -nosound ";
extern int xawpopup;

#ifndef NOAUDIO
/*****************************************************************************
 *                       RAW AUDIO DEFINITIONS
 ****************************************************************************/
#define DEC_FREQ    (divx.audio_freq)

static pthread_t audio_thread;
static int       achans;  // nb of audio channels
static int       aformat = 0; // Audio format
static unsigned long audio_frames;
unsigned long audio_buffers;
static int       fps_num, fps_den;

static void *    audio_record(void *);

/*****************************************************************************
 *                       MP3 AUDIO DEFINITIONS
 ****************************************************************************/
#ifdef HAVE_LAME

#include "lame/lame.h"

static lame_global_flags *gfp = NULL;
static char *    lame_buffer = NULL;
static int       lame_buffer_size = 0;

inline static int       lame_encode(short int *raw_buffer, int samples_per_channel);
static void      lame_stop(void);

#endif // HAVE_LAME

#ifdef SYNCHRO
static long long nbsamplesaudios;
unsigned long dropped_audio_buffers, dropped_video_frames;
static long long dropped_audio_samples;
extern double videostampmin,videostampmax;
double audiostamp=0.0;
#endif // SYNCHRO
#endif // NOAUDIO


/*****************************************************************************
 *                       DIVX4 VIDEO DEFINITIONS
 ****************************************************************************/
static void *    handle = NULL;

#ifdef HAVE_DIVX4LINUX
# if ENCORE_VERSION >= 20021024
   static DivXBitmapInfoHeader *format =NULL;
   static SETTINGS *settings = NULL;
# else
   static ENC_PARAM param;
# endif
static ENC_FRAME frame;
#endif

#ifdef HAVE_FFMPEG
static AVCodec *codec = NULL;
AVCodecContext *codec_context = NULL;
static AVFrame *picture_420p = NULL;
int xdtv_to_ffmpeg(int x) {
  switch(x) {
#ifdef HAVE_FFMPEG_XVID
#if LIBAVCODEC_BUILD >= 4719
  case FFMPEG_XVID: return CODEC_ID_XVID;
#endif
#endif
  case FFMPEG_MPEG4: return CODEC_ID_MPEG4;
  case FFMPEG_MPEG1: return CODEC_ID_MPEG1VIDEO;
#if LIBAVCODEC_BUILD >= 4676
  case FFMPEG_MPEG2: return CODEC_ID_MPEG2VIDEO;
#endif
  case FFMPEG_MJPEG: return CODEC_ID_MJPEG;
#if LIBAVCODEC_BUILD >= 4670
  case FFMPEG_LJPEG: return CODEC_ID_LJPEG;
  case FFMPEG_FLV1: return CODEC_ID_FLV1;
#endif
#if LIBAVCODEC_BUILD >= 4735
  case FFMPEG_RV20: return CODEC_ID_RV20;
#endif
  case FFMPEG_RV10: return CODEC_ID_RV10;
#ifdef HAVE_FFMPEG_X264
#if LIBAVCODEC_BUILD >= 4748
  case FFMPEG_X264: return CODEC_ID_H264;
#endif
#endif
  case FFMPEG_H263P: return CODEC_ID_H263P;
  case FFMPEG_H263: return CODEC_ID_H263;
#if LIBAVCODEC_BUILD >= 4729
  case FFMPEG_H261: return CODEC_ID_H261;
#endif
  case FFMPEG_MSMPEG4V3: return CODEC_ID_MSMPEG4V3;
  case FFMPEG_MSMPEG4V2: return CODEC_ID_MSMPEG4V2;
/* Already implemented into the 0.4.6 version 
but only work correctly with the 0.4.7 version */
#if LIBAVCODEC_BUILD >= 4675
  case FFMPEG_WMV2: return CODEC_ID_WMV2;
#endif
  case FFMPEG_WMV1: return CODEC_ID_WMV1;
#if LIBAVCODEC_BUILD >= 4677
  case FFMPEG_ASV2: return CODEC_ID_ASV2;
#endif
#if LIBAVCODEC_BUILD >= 4668
  case FFMPEG_ASV1: return CODEC_ID_ASV1;
#endif
/* appeared with the 4719 version
but don't work correctly actually */
#if LIBAVCODEC_BUILD >= 16777215
  case FFMPEG_SVQ1: return CODEC_ID_SVQ1;
#endif
/* appeared with the 4669 version
but only work correctly with the 4755 version */
#if LIBAVCODEC_BUILD >= 4755
  case FFMPEG_FFV1: return CODEC_ID_FFV1;
#endif
#if LIBAVCODEC_BUILD >= 4719
  case FFMPEG_SNOW: return CODEC_ID_SNOW;
#endif
#if LIBAVCODEC_BUILD >= 4734
  case FFMPEG_FFVHUFF: return CODEC_ID_FFVHUFF;
#endif
  case FFMPEG_HUFFYUV: return CODEC_ID_HUFFYUV;
#if LIBAVCODEC_BUILD >= 4707
  case FFMPEG_DVVIDEO: return CODEC_ID_DVVIDEO;
#endif
  default: return -1;
  }
}
#endif
#ifdef HAVE_XVID
#ifdef HAVE_XVID09
static XVID_ENC_FRAME  xframe;
static XVID_ENC_PARAM  xparam;
static int const xmotion_presets[7] = {
        0,                                                        // Q 0
        PMV_EARLYSTOP16,                                          // Q 1
        PMV_EARLYSTOP16,                                          // Q 2
        PMV_EARLYSTOP16 | PMV_HALFPELREFINE16,                    // Q 3
        PMV_EARLYSTOP16 | PMV_HALFPELREFINE16,                    // Q 4
        PMV_EARLYSTOP16 | PMV_HALFPELREFINE16 | PMV_EARLYSTOP8 |  // Q 5
        PMV_HALFPELREFINE8,
        PMV_EARLYSTOP16 | PMV_HALFPELREFINE16 | PMV_EXTSEARCH16 | // Q 6
        PMV_USESQUARES16 | PMV_EARLYSTOP8 | PMV_HALFPELREFINE8
};
static int const xgeneral_presets[7] = {
        XVID_H263QUANT,                               // Q 0
        XVID_MPEGQUANT,                               // Q 1
        XVID_H263QUANT,                               // Q 2
        XVID_H263QUANT | XVID_HALFPEL,                // Q 3
        XVID_H263QUANT | XVID_HALFPEL | XVID_INTER4V, // Q 4
        XVID_H263QUANT | XVID_HALFPEL | XVID_INTER4V, // Q 5
        XVID_H263QUANT | XVID_HALFPEL | XVID_INTER4V  // Q 6
};
#endif

#ifdef HAVE_XVID10
static xvid_enc_frame_t  xframe;
static xvid_enc_create_t xparam;
static xvid_enc_plugin_t xparam_plugin;
static xvid_enc_stats_t xstats;
static int const xmotion_presets[7] = {
  /* quality 0 */
  0,
  /* quality 1 */
  XVID_ME_ADVANCEDDIAMOND16,
  /* quality 2 */
  XVID_ME_ADVANCEDDIAMOND16 | XVID_ME_HALFPELREFINE16,
  /* quality 3 */
  XVID_ME_ADVANCEDDIAMOND16 | XVID_ME_HALFPELREFINE16 |
  XVID_ME_ADVANCEDDIAMOND8 | XVID_ME_HALFPELREFINE8,
  /* quality 4 */
  XVID_ME_ADVANCEDDIAMOND16 | XVID_ME_HALFPELREFINE16 |
  XVID_ME_ADVANCEDDIAMOND8 | XVID_ME_HALFPELREFINE8 |
  XVID_ME_CHROMA_PVOP | XVID_ME_CHROMA_BVOP,
  /* quality 5 */
  XVID_ME_ADVANCEDDIAMOND16 | XVID_ME_HALFPELREFINE16 |
  XVID_ME_ADVANCEDDIAMOND8 | XVID_ME_HALFPELREFINE8 |
  XVID_ME_CHROMA_PVOP | XVID_ME_CHROMA_BVOP,
  /* quality 6 */
  XVID_ME_ADVANCEDDIAMOND16 | XVID_ME_HALFPELREFINE16 | XVID_ME_EXTSEARCH16 |
  XVID_ME_ADVANCEDDIAMOND8 | XVID_ME_HALFPELREFINE8 | XVID_ME_EXTSEARCH8 |
  XVID_ME_CHROMA_PVOP | XVID_ME_CHROMA_BVOP,
};
static int const xvop_presets[7] = {
  /* quality 0 */
  0,
  /* quality 1 */
  0,
  /* quality 2 */
  XVID_VOP_HALFPEL,
  /* quality 3 */
  XVID_VOP_HALFPEL | XVID_VOP_INTER4V,
  /* quality 4 */
  XVID_VOP_HALFPEL | XVID_VOP_INTER4V,
  /* quality 5 */
  XVID_VOP_HALFPEL | XVID_VOP_INTER4V |
  XVID_VOP_TRELLISQUANT,
  /* quality 6 */
  XVID_VOP_HALFPEL | XVID_VOP_INTER4V |
  XVID_VOP_TRELLISQUANT | XVID_VOP_HQACPRED,
};
#endif
#endif /* HAVE_XVID*/

/************** conversion format tables *****************/
#ifdef HAVE_DIVX4LINUX
static video_fmt* fmts_conv_divx=NULL;
static video_fmt fmts_divx[]={
  VIDEO_YVU420,
  VIDEO_YUYV,
  VIDEO_RGB24B,
  VIDEO_RGB32B,
  0};
#if ENCORE_VERSION >= 20021024
#define FOURCC(A, B, C, D) ( ((uint8_t) (A)) | (((uint8_t) (B))<<8) | (((uint8_t) (C))<<16) | (((uint8_t) (D))<<24) )
static int fmts_xaw_to_biCompression[MAX_VIDEO_FMT] = {
  [VIDEO_YVU420] = FOURCC('Y','V','1','2'),
  [VIDEO_YUYV]   = FOURCC('Y','U','Y','2'),
  [VIDEO_RGB24B] = 0,
  [VIDEO_RGB32B] = 0,
};
static int fmts_xaw_to_biBitCount[MAX_VIDEO_FMT] = {
  [VIDEO_YVU420] = 0,
  [VIDEO_YUYV]   = 0,
  [VIDEO_RGB24B] = 24,
  [VIDEO_RGB32B] = 32,
};
#else
static int fmts_xaw_to_divxcolorspace[MAX_VIDEO_FMT]= {
  [VIDEO_YVU420] = ENC_CSP_YV12,
  [VIDEO_YUYV]   = ENC_CSP_YUY2,
  [VIDEO_RGB24B] = ENC_CSP_RGB24,
  [VIDEO_RGB32B] = ENC_CSP_RGB32,
};
#endif
#endif

#ifdef HAVE_XVID
static video_fmt fmts_xvid[]={
  VIDEO_RGB15,
  VIDEO_RGB16,
#ifdef HAVE_XVID09
  VIDEO_RGB32B,
  VIDEO_RGB24B,
#endif
#ifdef HAVE_XVID10
  VIDEO_RGB32,
  VIDEO_RGB24,
#endif
  VIDEO_YUYV,
  VIDEO_YVU420,
  0};
static video_fmt* fmts_conv_xvid=NULL;
int fmts_xaw_to_xvidcolorspace[MAX_VIDEO_FMT]= {
  [VIDEO_RGB15]  = XVID_CSP_RGB555,
  [VIDEO_RGB16]  = XVID_CSP_RGB565,
#ifdef HAVE_XVID09
  [VIDEO_RGB24B] = XVID_CSP_RGB24,
  [VIDEO_RGB32B] = XVID_CSP_RGB32,
  [VIDEO_RGB24]  = XVID_CSP_RGB24, /* for grab-avi */
  [VIDEO_RGB32]  = XVID_CSP_RGB32,
#endif
#ifdef HAVE_XVID10
  [VIDEO_RGB24]  = XVID_CSP_BGR,
  [VIDEO_RGB32]  = XVID_CSP_BGRA,
#endif
  [VIDEO_YUYV]   = XVID_CSP_YUY2,
  [VIDEO_UYVY]   = XVID_CSP_YUY2,
  [VIDEO_YVU420] = XVID_CSP_YV12,
};
#endif

static video_fmt *fmts_conv_uncompressed=NULL;
static video_fmt fmts_uncompressed[]= {
  VIDEO_RGB15,
  VIDEO_RGB16,
  VIDEO_RGB24,
  VIDEO_RGB32,
  VIDEO_YUYV,
  VIDEO_UYVY,
  VIDEO_YVU420,
  VIDEO_YUV420,
  0};

struct STRTAB fourccs[] = {
#ifdef HAVE_XVID
  {XVID,             "XVID"},
#endif
#ifdef HAVE_OGG
  {THEORA,             "theo"},
#endif
#ifdef HAVE_FFMPEG_XVID
#if LIBAVCODEC_BUILD >= 4719
  {FFMPEG_XVID,      "XVID"},
#endif
#endif
#ifdef HAVE_FFMPEG
  {FFMPEG_MPEG4,     "DIVX"},
#if LIBAVCODEC_BUILD >= 4676
  {FFMPEG_MPEG2,     "mpg2"},
#endif
  {FFMPEG_MPEG1,     "mpg1"},
  {FFMPEG_MJPEG,     "MJPG"},
#if LIBAVCODEC_BUILD >= 4670
  {FFMPEG_LJPEG,     "LJPG"},
  {FFMPEG_FLV1,      "FLV1"},
#endif
#if LIBAVCODEC_BUILD >= 4735
  {FFMPEG_RV20,      "RV20"},
#endif
  {FFMPEG_RV10,      "RV10"},
#ifdef HAVE_FFMPEG_X264
#if LIBAVCODEC_BUILD >= 4748
  {FFMPEG_X264,     "H264"},
#endif
#endif
  {FFMPEG_H263P,     "H263P"},
  {FFMPEG_H263,      "H263"},
#if LIBAVCODEC_BUILD >= 4729
  {FFMPEG_H261,      "H261"},
#endif
  {FFMPEG_MSMPEG4V3, "MP43"},
  {FFMPEG_MSMPEG4V2, "MP42"},
/* Already implemented into the 0.4.6 version 
but only work correctly with the 0.4.7 version */
#if LIBAVCODEC_BUILD >= 4675
  {FFMPEG_WMV2,      "WMV2"},
#endif
  {FFMPEG_WMV1,      "WMV1"},
#if LIBAVCODEC_BUILD >= 4677
  {FFMPEG_ASV2,      "ASV2"},
#endif
#if LIBAVCODEC_BUILD >= 4668
  {FFMPEG_ASV1,      "ASV1"},
#endif
/* appeared with the 4719 version
but don't work correctly actually */
#if LIBAVCODEC_BUILD >= 16777215
  {FFMPEG_SVQ1,      "SVQ1"},
#endif
/* appeared with the 4669 version
but only work correctly with the 4755 version */
#if LIBAVCODEC_BUILD >= 4755
  {FFMPEG_FFV1,      "FFV1"},
#endif
#if LIBAVCODEC_BUILD >= 4719
  {FFMPEG_SNOW,      "SNOW"},
#endif
#if LIBAVCODEC_BUILD >= 4734
  {FFMPEG_FFVHUFF,   "FFVH"},
#endif
  {FFMPEG_HUFFYUV,   "HFYU"},
#if LIBAVCODEC_BUILD >= 4707
  {FFMPEG_DVVIDEO, dvvideo_mode}, // DVSD : PAL, DVCS : NTSC
#endif
#endif //HAVE_FFMPEG
#ifdef HAVE_DIVX4LINUX
#if (ENCORE_MAJOR_VERSION >= 5010)
  {DIVX4LINUX,       "DX50"},
#else
  {DIVX4LINUX,       "DIVX"},
#endif
#endif
  {-1, NULL}
};


struct STRTAB fourccs_raw[] = {
  {VIDEO_RGB15_LE,"\017RGB"},
  {VIDEO_RGB16_LE,"\020RGB"},
  {VIDEO_RGB24_LE,"\030RGB"},
  {VIDEO_RGB32_LE,"\040RGB"},
  {VIDEO_RGB15_BE,"\017BGR"},
  {VIDEO_RGB16_BE,"\020BGR"},
  {VIDEO_RGB24_BE,"\030BGR"},
  {VIDEO_RGB32_BE,"\040BGR"},
  {VIDEO_RGB08,  "\010RGB"},
  {VIDEO_GRAY1,  "\001RGB"},
  {VIDEO_YUYV,   "YUY2"},
  {VIDEO_UYVY,   "UYVY"},
  {VIDEO_YVU420, "YV12"},
  {VIDEO_YUV420, "I420"},
  {VIDEO_YUV420, "IYUV"},
  {-1,NULL}
};

int audio_fourcc(int x) {
  switch(x) {
  case AUDIO_UNCOMPRESSED: return WAVE_FORMAT_PCM;
#ifdef HAVE_LAME
  case LAME: return WAVE_FORMAT_MPEGLAYER3;
#endif
#ifdef HAVE_OGG
  case VORBIS: return get_fourcc("vrbs") ;
#endif
#ifdef HAVE_FFMPEG
  case FFMPEG_MP2: return 0x0050;
#ifdef HAVE_FFMPEG_MP3
  case FFMPEG_MP3: return 0x0055;
#endif
#ifdef HAVE_FFMPEG_AC3
  case FFMPEG_AC3:  return 0x2000;
#endif
#ifdef HAVE_FFMPEG_AAC
  case FFMPEG_AAC:  return 0x00ff;
#endif
  case FFMPEG_PCMS16LE: return 0x0000;
#ifdef HAVE_FFMPEG_AMR_NB
  case FFMPEG_AMR_NB :  return get_fourcc("samr");
#endif
#ifdef HAVE_FFMPEG_AMR_WB
  case FFMPEG_AMR_WB : return get_fourcc("sawb");
#endif
#endif //HAVE_FFMPEG
  default:
	fprintf(stderr," no fourcc found ! should never happend !\n");
	return -1;
  }
}

/************** end conversion format tables *****************/

  // Doc says max encoded frame size is around 6 bytes per pixel...
  // I like the "around" ....
#define ENC_FRAME_SIZE (6*768*576)
static unsigned char * enc_frame = NULL;

static FILE *    subfile = NULL;

static int       prev_w = 0;
static int       prev_h = 0;
static int       prev_w_capture = 0;
static int       prev_h_capture = 0;
static unsigned long lastsubend;
static struct timeval tv_start;
static int width_rec, height_rec;
   /* recorded for the case where divx.width/height change...*/
static video_fmt fmt_conv;

extern int       AVI_errno;
static int       divx_in_progress = 0, divx_audio=0, divx_audio_must_start=0;
static int restart=0;

/*****************************************************************************
 *                       DIVX VIDEO FUNCTIONS
 ****************************************************************************/

char * get_filename(void) {
  char *filename=NULL;
  char begin_name[256];
  struct stat buf;
  char *divx_file;
  update_divxfilename();

  if (stat (divx.filename, &buf) == 0 &&  S_ISDIR(buf.st_mode)) {
    
    // Get TV Program Title if Nxtvepg send it to xdTV    
    if (cur_input==0)
    {
    	if (channel_title[0] != 0)
      	sprintf(begin_name, "movie-%s", channel_title);
    	else
      	strcpy(begin_name,"movie");
      	divx_file = snap_filename (begin_name, cur_sender!=-1 ? channels[cur_sender]->name : "unknown", cont_handler->ext);
    
    }
    else 
    {
    	sprintf (begin_name, "capture");
	divx_file = snap_filename (begin_name, "AV", cont_handler->ext);
    
    };
    /* case divx.filename is a directory */  
    filename=malloc(strlen(divx.filename)+strlen(divx_file)+2);
    strcpy(filename,divx.filename);
    strcat(filename, "/");
    strcat(filename, divx_file);
  } else {
    int l;
    char* c=NULL;
    int ext_size_ori=0;
    filename=strdup(divx.filename);
    l=strlen(filename);
    c=filename+l-1;
    while (*c != '.' && c!=filename){
	    c--;
	    ext_size_ori++;
    } 
    
    if (c==filename) /* it was no '.' in the name -> extension will be added in the end */
	    ext_size_ori=0;
    else
	    ext_size_ori++; /* +1 for the '.' */

    if(l>=4 ){
      static int nb;
      if(!restart) {
	char *f2=filename;
	filename=malloc(l+10);
	*(f2+l-ext_size_ori)=0;
	sprintf(filename,"%s.%s",f2,cont_handler->ext);
	free(f2);
	nb=1;
      }
      else {
	/* when recording is restarted automatically.
	   (when the file reaches the 2Gb/4Gb limit
	   or when there's a new http client)
	   filename.avi becomes filename_2.avi...
	*/
	char *f2=filename;
	filename=malloc(l+10);
	*(f2+l-ext_size_ori)=0; // +1 for the '.'
	sprintf(filename,"%s_%d.%s",f2,nb,cont_handler->ext);
	free(f2);
	nb++;
      }
    }
  }
  if(debug){
    fprintf(stderr, "divx.filename = %s\n",divx.filename);
    fprintf(stderr, "filename      = %s\n",filename);
  }

  if (!force_recfile && divx.stream) strcpy(filename,("/dev/null"));

  return filename;
}

static char* get_subfilename(char *avi_filename) {
  int l=strlen(avi_filename);
  char *f=NULL;
  if(l<4 || strcmp(avi_filename+l-4,".avi")!=0){
    f=malloc(l+5);
    strcpy(f,avi_filename);
    strcat(f, ".sub");
  } else {
    f=strdup(avi_filename);
    strcpy(f+l-3,"sub");
  }
  return f;
}

static void activated_mess(char *option) {
  sprintf(xawmesstmp, translate("           option %s activated\n"),option);
  strcat(xawmessage,xawmesstmp);
}

static void equal_int_mess(char *option, int ffmpeg_value) {
  sprintf(xawmesstmp, translate("           option %s = %d\n"),option, ffmpeg_value);
  strcat(xawmessage,xawmesstmp);
}

#ifdef HAVE_FFMPEG
static void equal_float_mess(char *option, float ffmpeg_value) {
  sprintf(xawmesstmp, translate("           option %s = %f\n"),option, ffmpeg_value);
  strcat(xawmessage,xawmesstmp);
}
#endif

void divx_init(int width, int height)
{

#ifdef HAVE_DIVX4LINUX
# if defined(ENC_OPT_VERSION) && defined (ENCORE_VERSION)
   int version_binary;
#endif
#endif // HAVE_DIVX4LINUX

#ifdef HAVE_XVID
#ifdef HAVE_XVID09
  XVID_INIT_PARAM xinit;
#endif
#ifdef HAVE_XVID10
  xvid_gbl_init_t xinit;
  xvid_plugin_single_t single;
#endif
#endif // HAVE_XVID

#if defined (HAVE_DIVX4LINUX) || defined (HAVE_XVID)
  int ret;
#endif

  struct timezone tz;
  time_t t;

#ifndef SYNCHRO
  int fps = (double)cur_maxfpsnum * 1000 / cur_maxfpsden + 0.5;
#else
  int fps = divx.fps;
#endif

  char *fourcc = NULL;
  char *filename = NULL;
  char *codec_name = NULL;
  struct timeval tv;

  strcpy(xawmessage,translate("Record Informations:\n\n"));

  if (handle != NULL)
  {
    popup_mess(translate("divx_init: already started\n"));
    return;
  }

  if ( divx.min_quantizer > divx.max_quantizer) 
  {
    divx.max_quantizer = 8;
    divx.min_quantizer = 2;
    fprintf(stderr, "\nMin. Quantizer must be <= Min. Quantizer\n");
    fprintf(stderr, "So, XdTV will use default values\n\n");
    update_divx_scrollbarwidget(l_divxmaxquantizer, s_divxmaxquantizer, divx.max_quantizer, 64, translate("Max Quantizer"));
    update_divx_scrollbarwidget(l_divxminquantizer, s_divxminquantizer, divx.min_quantizer, 64, translate("Min Quantizer"));
  }

  if (debug) fprintf(stderr, "DivX init...\n");
  codec_name=int_to_str(divx.codec,codec_list);

  if (debug)
  {
    fprintf(stderr, "divx.container         = %s\n",int_to_str(divx.container,container_list));
    fprintf(stderr, "divx.codec             = %s\n", codec_name);
    fprintf(stderr, "divx.width             = %d\n", divx.width);
    fprintf(stderr, "divx.height            = %d\n", divx.height);
    fprintf(stderr, "divx.bitrate           = %ld\n", divx.bitrate);
    fprintf(stderr, "divx.quality           = %d\n", divx.quality);
    fprintf(stderr, "divx.min_quantizer     = %d\n", divx.min_quantizer);
    fprintf(stderr, "divx.max_quantizer     = %d\n", divx.max_quantizer);
    fprintf(stderr, "divx.audio_buffer_size = %d\n", divx.audio_buffer_size);
    fprintf(stderr, "divx.stereo            = %d\n", divx.stereo);
    fprintf(stderr, "divx.audio_codec       = %s\n", int_to_str(divx.audio_codec,audio_codec_list));
    fprintf(stderr, "divx.mp3_bitrate       = %d\n", divx.mp3_bitrate);
    fprintf(stderr, "divx.mp3_quality       = %d\n", divx.mp3_quality);
    fprintf(stderr, "divx.mp3_vbr           = %d\n", divx.mp3_vbr);
    fprintf(stderr, "divx.mp3_vbr_quality   = %d\n", divx.mp3_vbr_quality);
#ifdef SYNCHRO
    fprintf(stderr, "divx.fps               = %d\n", divx.fps);
    fprintf(stderr, "divx.maxgap            = %f\n", divx.maxgap);
    fprintf(stderr, "divx.delay             = %f\n", divx.delay);
#endif
    fprintf(stderr, "divx.display_frames    = %d\n", divx.display_frames);
    fprintf(stderr, "divx.sub               = %d\n", divx.sub);
    fprintf(stderr, "divx.chg               = %d\n", divx.chg);
    fprintf(stderr, "divx.filename          = %s\n", divx.filename);
    fprintf(stderr, "divx.stream            = %d\n", divx.stream);
    fprintf(stderr, "divx.http_port         = %d\n", divx.http_port);
    fprintf(stderr, "divx.maxsize           = %d\n", divx.maxsize);
  }

  prev_w = width;
  prev_h = height;
  width_rec = divx.width;
  height_rec = divx.height;

  if(width_rec>cur_maxwidth) width_rec = cur_maxwidth;
  if(height_rec>cur_maxheight) height_rec = cur_maxheight;
  width = width_rec; height = height_rec;
  if(fps / 1000.0 > (double)cur_maxfpsnum / cur_maxfpsden - 0.01) {
    fps_num = cur_maxfpsnum;
    fps_den = cur_maxfpsden;
  } else {
    fps_num = fps;
    fps_den = 1000;
  }

  if (debug) fprintf(stderr, "divx_init: requested video size: w = %d, h = %d\n", width, height);

  video_get_capture_size(&prev_w_capture, &prev_h_capture);
  video_set_capture_size(width, height);

  if (debug) fprintf(stderr, "divx_init: allocated video size: w = %d, h = %d\n", width, height);

  height_rec = height -= 2 * get_ybar(height);

  /******************** UNCOMPRESSED init ***********/
  if (divx.codec==UNCOMPRESSED) {
    fprintf(stderr, "divx_init: encoded video size:   w = %d, h = %d\n", width, height);
    if(fmts_conv_uncompressed==NULL) {
      fmts_conv_uncompressed=malloc(MAX_VIDEO_FMT*sizeof(video_fmt));
      setpreferred_list(fmts_uncompressed,fmts_conv_uncompressed);}
    strcat(xawmessage,translate("divx_init: Uncompressed Codec used\n"));
    fmt_conv = fmts_conv_uncompressed[x11_pixmap_format];}

  /*************** DIVX4LINUX init **********************/
#ifdef HAVE_DIVX4LINUX
  else if(divx.codec==DIVX4LINUX) {
    if(fmts_conv_divx==NULL) {
      fmts_conv_divx=malloc(MAX_VIDEO_FMT*sizeof(video_fmt));
      setpreferred_list(fmts_divx,fmts_conv_divx);
    }
  // Check for DivX codec version
#if defined(ENC_OPT_VERSION) && defined (ENCORE_VERSION)
    version_binary = encore(0, ENC_OPT_VERSION, 0, 0);
    sprintf(xawmesstmp, translate("divx_init: divx4linux DivX encoder version = %d\n"), version_binary);
    strcat(xawmessage,xawmesstmp);
    if (version_binary != ENCORE_VERSION)
    {
      //...interfaces are incompatible, encoding can't be performed
      sprintf(xawmesstmp, translate("divx4linux DivX encoder claim its version is %d and I was expecting encoder version %d: I give up...\n"),
                  version_binary, ENCORE_VERSION);
      strcat(xawmessage,xawmesstmp);
      divx_stop();
      return;
    }
#else
    strcat(xawmessage,translate("divx_init: Warning: at compile time, divx4linux was too old to perform version checking\n"));
#endif

#if ENCORE_VERSION >= 20021024
    if ((settings = malloc(sizeof(SETTINGS)))==NULL) {
      strcat(xawmessage,translate("out of memory\n"));
      divx_stop();
      return;
    }
    if ((format = malloc(sizeof(DivXBitmapInfoHeader)))==NULL) {
      strcat(xawmessage,translate("out of memory\n"));
      divx_stop();
      return;
    }
    memset (settings, 0, sizeof(SETTINGS));
    memset (format, 0, sizeof(DivXBitmapInfoHeader));

    settings->vbr_mode = RCMODE_502_1PASS;
    settings->bitrate = divx.bitrate;
    settings->quantizer = (float)(divx.bitrate/1000);
    settings->use_bidirect = 0;
    settings->input_clock = fps_num;
    settings->input_frame_period = fps_den;
    settings->quality = divx.quality;
    settings->complexity_modulation = 0.5;
    settings->enable_crop = 0;
    settings->enable_resize = 0;

    format->biSize = sizeof(DivXBitmapInfoHeader);
    format->biWidth = width;
    format->biHeight = height;
#else
    memset(&param, 0, sizeof(param));
    param.x_dim = width;
    param.y_dim = height;
    param.framerate = (float)fps_num/fps_den;
    param.min_quantizer = divx.min_quantizer;
    param.max_quantizer = divx.max_quantizer;
    param.quality = divx.quality; // (1 - fastest, 5 - best)
    param.bitrate = divx.bitrate;

    ret = encore(0, ENC_OPT_INIT, &param, 0);
    if (ret != ENC_OK)
    {
      strcat(xawmessage,translate("divx_init: divx4linux encore initialization failed\n"));
      divx_stop();
      return;
    }

    handle=param.handle;
#endif
    enc_frame = (unsigned char *)malloc(ENC_FRAME_SIZE);
    if (enc_frame == NULL)
    {
      strcat(xawmessage,translate("divx_init: couldn't allocate memory for encoded frame: releasing divx4linux DivX encoder...\n"));
      divx_stop();
      return;
    }
    memset(&frame, 0, sizeof(frame));
    frame.bitstream = enc_frame;

    fmt_conv = fmts_conv_divx[x11_pixmap_format];
#if ENCORE_VERSION >= 20021024
    format->biCompression = fmts_xaw_to_biCompression[fmt_conv];
    format->biBitCount = fmts_xaw_to_biBitCount[fmt_conv];
    ret = encore((void*)&handle, ENC_OPT_INIT, format, settings);
    if (ret != ENC_OK)
    {
      strcat(xawmessage,translate("divx_init: divx4linux encore initialization failed\n"));
      sprintf(xawmesstmp, translate("error value is :%d\n"),ret);
      strcat(xawmessage,xawmesstmp);
      divx_stop();
      return;
    }
#else
     memset(&frame, 0, sizeof(frame));
     frame.bitstream = enc_frame;
     frame.colorspace = fmts_xaw_to_divxcolorspace[fmt_conv];
#endif
  }
#endif // HAVE_DIVX4LINUX

  /*************** FFMPEG init **************************/

#ifdef HAVE_FFMPEG
  else if(xdtv_to_ffmpeg(divx.codec)!=-1) {
    avcodec_init();
#if LIBAVCODEC_BUILD <= 4759
    sprintf(xawmesstmp, translate("divx_init: initializing %s version %d (build %d)\n"), codec_name, avcodec_version(), avcodec_build());
#else
    {div_t A_div, B_div;
    A_div=div(LIBAVCODEC_BUILD,65536);
    B_div=div(A_div.rem,256); 
    sprintf(xawmesstmp, translate("divx_init: initializing %s - build %i.%i.%i\n"), codec_name, A_div.quot, B_div.quot, B_div.rem);}
#endif
    strcat(xawmessage,xawmesstmp);
    switch(divx.codec) {
#ifdef HAVE_FFMPEG_XVID
#if LIBAVCODEC_BUILD >= 4719
      case FFMPEG_XVID: register_avcodec(&xvid_encoder); break;
#endif
#endif
      case FFMPEG_MPEG4: register_avcodec(&mpeg4_encoder); break;
      case FFMPEG_MPEG1: register_avcodec(&mpeg1video_encoder); break;
#if LIBAVCODEC_BUILD >= 4676
      case FFMPEG_MPEG2: register_avcodec(&mpeg2video_encoder); break;
#endif
      case FFMPEG_MJPEG: register_avcodec(&mjpeg_encoder); break;
#if LIBAVCODEC_BUILD >= 4670
      case FFMPEG_LJPEG: register_avcodec(&ljpeg_encoder); break;
      case FFMPEG_FLV1: register_avcodec(&flv_encoder); break;
#endif
#if LIBAVCODEC_BUILD >= 4735
      case FFMPEG_RV20: register_avcodec(&rv20_encoder); break;
#endif
      case FFMPEG_RV10: register_avcodec(&rv10_encoder); break;
#ifdef HAVE_FFMPEG_X264
#if LIBAVCODEC_BUILD >= 4748
      case FFMPEG_X264: register_avcodec(&x264_encoder); break;
#endif
#endif
      case FFMPEG_H263P: register_avcodec(&h263p_encoder); break;
      case FFMPEG_H263: register_avcodec(&h263_encoder); break;
#if LIBAVCODEC_BUILD >= 4729
      case FFMPEG_H261: register_avcodec(&h261_encoder); break;
#endif
      case FFMPEG_MSMPEG4V3: register_avcodec(&msmpeg4v3_encoder); break;
      case FFMPEG_MSMPEG4V2: register_avcodec(&msmpeg4v2_encoder); break;
/* Already implemented into the 0.4.6 version but only work correctly with the 0.4.7 version */
#if LIBAVCODEC_BUILD >= 4675
      case FFMPEG_WMV2: register_avcodec(&wmv2_encoder); break;
#endif
      case FFMPEG_WMV1: register_avcodec(&wmv1_encoder); break;
#if LIBAVCODEC_BUILD >= 4677
      case FFMPEG_ASV2: register_avcodec(&asv2_encoder); break;
#endif
#if LIBAVCODEC_BUILD >= 4668
      case FFMPEG_ASV1: register_avcodec(&asv1_encoder); break;
#endif
/* appeared with the 4719 version but don't work correctly actually */
#if LIBAVCODEC_BUILD >= 16777215
      case FFMPEG_SVQ1: register_avcodec(&svq1_encoder); break;
#endif
/* appeared with the 4669 version
but only work correctly with the 4755 version */
#if LIBAVCODEC_BUILD >= 4755
      case FFMPEG_FFV1: register_avcodec(&ffv1_encoder); break;
#endif
#if LIBAVCODEC_BUILD >= 4719
      case FFMPEG_SNOW: register_avcodec(&snow_encoder); break;
#endif

#if LIBAVCODEC_BUILD >= 4734
      case FFMPEG_FFVHUFF: register_avcodec(&ffvhuff_encoder); break;
#endif
      case FFMPEG_HUFFYUV: register_avcodec(&huffyuv_encoder); break;
#if LIBAVCODEC_BUILD >= 4707
      case FFMPEG_DVVIDEO: register_avcodec(&dvvideo_encoder); break;
#endif
    }
    codec = avcodec_find_encoder(xdtv_to_ffmpeg(divx.codec));
    if (codec == NULL)
    {
      sprintf(xawmesstmp, translate("divx_init: couldn't find %s codec\n"), codec_name);
      strcat(xawmessage,xawmesstmp);
      divx_stop();
      return;
    }

    if (codec_context == NULL) codec_context = handle = avcodec_alloc_context();
    picture_420p = avcodec_alloc_frame();

// FFMpeg Parameters from MPlayer libmpcodecs/ve_lavc.c -------------------------------------

    codec_context->codec_id              = xdtv_to_ffmpeg(divx.codec);
    codec_context->codec_type            = CODEC_TYPE_VIDEO;
    codec_context->bit_rate              = divx.bitrate;
    codec_context->bit_rate_tolerance    = divx.bitrate*10;
    codec_context->flags                 = 0;
    codec_context->me_method             = ME_ZERO+ divx.quality;
    codec_context->width                 = width;
    codec_context->height                = height;
    codec_context->gop_size              = ffmpeg.keyint;
    codec_context->pix_fmt               = PIX_FMT_YUV420P;
    fmt_conv                             = VIDEO_YUV420;
#if LIBAVCODEC_BUILD >= 4754
    codec_context->time_base             = (AVRational){fps_den,fps_num};
#elif LIBAVCODEC_BUILD >= 4662
    codec_context->frame_rate            = fps_num;
    codec_context->frame_rate_base       = fps_den;
#else
    codec_context->frame_rate            = fps_num*FRAME_RATE_BASE/fps_den;
#endif 
    codec_context->qcompress             = 0.5;
    codec_context->qblur                 = 0.5;
    codec_context->qmin                  = divx.min_quantizer;
    codec_context->qmax                  = divx.max_quantizer;
    codec_context->max_qdiff             = 3;
    codec_context->max_b_frames          = ffmpeg.vmax_b_frames;
    codec_context->b_quant_factor        = 1.25;
    codec_context->rc_strategy           = 2;
    codec_context->b_frame_strategy      = 0;
    codec_context->rtp_payload_size      = 0;
    codec_context->luma_elim_threshold   = ffmpeg.vlelim;
    codec_context->chroma_elim_threshold = ffmpeg.vcelim;
    codec_context->strict_std_compliance = -1; // To follow strictly the standard
    codec_context->b_quant_offset        = (int)(FF_QP2LAMBDA * 1.25 + 0.5);
    codec_context->mpeg_quant            = 0;
    codec_context->rc_qsquish            = 1.0;
    codec_context->rc_qmod_amp           = 0;
    codec_context->rc_qmod_freq          = 0;
    codec_context->rc_eq                 = "tex^qComp";
    codec_context->rc_max_rate           = 0*1000;
    codec_context->rc_min_rate           = 0*1000;
    codec_context->rc_buffer_size        = 0*1000;
    codec_context->rc_buffer_aggressivity= 1.0;
    codec_context->i_quant_factor        = 0.8;
    codec_context->i_quant_offset        = (int)(FF_QP2LAMBDA * 0.0 + 0.5);
    codec_context->rc_initial_cplx       = 0;
    codec_context->dct_algo              = 0; // Auto Mode ON
    codec_context->lumi_masking          = ffmpeg.lumi_mask;
    codec_context->temporal_cplx_masking = ffmpeg.tcplx_mask;
    codec_context->spatial_cplx_masking  = ffmpeg.scplx_mask;
    codec_context->p_masking             = 0.0;
    codec_context->dark_masking          = ffmpeg.dark_mask;
    codec_context->idct_algo             = 0; // Auto Mode ON
    codec_context->prediction_method     = FF_PRED_LEFT;

#if LIBAVCODEC_BUILD >= 4642
    codec_context->debug                 = 0;
#endif    
#if LIBAVCODEC_BUILD >= 4646
    codec_context->mb_qmin               = divx.min_quantizer;
    codec_context->mb_qmax               = divx.max_quantizer;
#endif 
#if LIBAVCODEC_BUILD >= 4647
    codec_context->me_cmp                = ffmpeg.cmp;
    codec_context->me_sub_cmp            = ffmpeg.subcmp;
    codec_context->mb_cmp                = 0;
#ifdef FF_CMP_VSAD
    codec_context->ildct_cmp             = FF_CMP_VSAD;
#endif
    codec_context->dia_size              = ffmpeg.dia;
#endif
#if LIBAVCODEC_BUILD >= 4649
    codec_context->last_predictor_count  = ffmpeg.last_pred;
#endif
#if LIBAVCODEC_BUILD >= 4650
    codec_context->pre_me                = 1;
#endif
#if LIBAVCODEC_BUILD >= 4651
    codec_context->me_pre_cmp            = ffmpeg.precmp;
    codec_context->pre_dia_size          = ffmpeg.predia;
#endif
#if LIBAVCODEC_BUILD >= 4652
    codec_context->me_subpel_quality     = 8;
#endif
#if LIBAVCODEC_BUILD >= 4659
    codec_context->me_range              = 0;
#endif
#if LIBAVCODEC_BUILD >= 4663
    codec_context->intra_quant_bias      = FF_DEFAULT_QUANT_BIAS;
    codec_context->inter_quant_bias      = FF_DEFAULT_QUANT_BIAS;
#endif
#if LIBAVCODEC_BUILD >= 4669
    codec_context->coder_type            = 0;
    codec_context->context_model         = 0;
#endif
#if LIBAVCODEC_BUILD >= 4673
    codec_context->mb_decision           = ffmpeg.mbd; // 0 = Realtime encoding
#endif
#if LIBAVCODEC_BUILD >= 4680
    codec_context->scenechange_threshold = 0;
#endif
#if LIBAVCODEC_BUILD >= 4684
    codec_context->lmin                     = (int)(FF_QP2LAMBDA * divx.min_quantizer + 0.5);
    codec_context->lmax                     = (int)(FF_QP2LAMBDA * divx.max_quantizer + 0.5);
#endif
#if LIBAVCODEC_BUILD >= 4690
    codec_context->noise_reduction       = 0; // Realtime encoding
#endif
#if LIBAVCODEC_BUILD >= 4694
    codec_context->rc_initial_buffer_occupancy = codec_context->rc_buffer_size * 0.9;
    codec_context->inter_threshold       = 0;
#endif
#if LIBAVCODEC_BUILD >= 4700
    codec_context->quantizer_noise_shaping= 0; // Realtime encoding
#endif
#if LIBAVCODEC_BUILD >= 4711
    codec_context->intra_dc_precision    = 0;
#endif
#if LIBAVCODEC_BUILD >= 4716
    codec_context->nsse_weight           = 8;
#endif
#if LIBAVCODEC_BUILD >= 4741
    codec_context->border_masking        = 0.0;
#endif

// <=> 51.1.0
#if LIBAVCODEC_BUILD >= 3342592
codec_context->brd_scale                 = 0;
#endif
// <=> 51.2.0
#if LIBAVCODEC_BUILD >= 3342848
      codec_context->scenechange_factor = 1;
#endif
// <=> 51.10.0
#if LIBAVCODEC_BUILD >= 3344896
codec_context->b_sensitivity             = 40;
codec_context->mv0_threshold             = 256;
#endif

// FFMPEG Flags & Advanced Options Managements -----------------------------------------------------------------

    if (ffmpeg.v4mv == 1 && (divx.codec == FFMPEG_MPEG4 
	|| divx.codec == FFMPEG_H263 
	|| divx.codec == FFMPEG_H263P 
#if LIBAVCODEC_BUILD >= 4670
	|| divx.codec == FFMPEG_FLV1))
#else
    ))
#endif
    {
      codec_context->flags |= CODEC_FLAG_4MV;
      activated_mess("V4mv : 4 Motion Vector");}

    if (ffmpeg.naq == 1) {
      codec_context->flags |= CODEC_FLAG_NORMALIZE_AQP;
      activated_mess("Normalize Quantization");}

// Bad color with ASV2 ASV1 & RV20 but no segfault.
// I Hope it will be fixed in future.
    if (ffmpeg.gray == 1) {
      codec_context->flags |= CODEC_FLAG_GRAY;
      activated_mess("Grayscale Encoding");}

    if (ffmpeg.gmc == 1) {
      codec_context->flags |= CODEC_FLAG_GMC;
      activated_mess("Global Motion Compensation");}

    if (ffmpeg.qpel == 1 && divx.codec == FFMPEG_MPEG4) {
      codec_context->flags |= CODEC_FLAG_QPEL;
      activated_mess("Quarter PixEL");}

    if (ffmpeg.ildct == 1 && (divx.codec == FFMPEG_MPEG4 
#if LIBAVCODEC_BUILD >= 4676
	|| divx.codec == FFMPEG_MPEG2))
#else
    ))
#endif
    {
      codec_context->flags |= CODEC_FLAG_INTERLACED_DCT;
      activated_mess("InterLaced DCT Encoding");}

#if LIBAVCODEC_BUILD >= 4698
    if (ffmpeg.ilme == 1 && (divx.codec == FFMPEG_MPEG4 
#if LIBAVCODEC_BUILD >= 4676
	|| divx.codec == FFMPEG_MPEG2))
#else
    ))
#endif
    {
      codec_context->flags |= CODEC_FLAG_INTERLACED_ME;
      activated_mess("InterLaced Motion Estimation");}
#endif

// Trell segfault with these two codecs!!
#if LIBAVCODEC_BUILD >= 4648
    if (ffmpeg.trell == 1 && divx.codec != FFMPEG_MJPEG 
#if LIBAVCODEC_BUILD >= 4729
	&& divx.codec != FFMPEG_H261)
#else
    )
#endif
    {
      codec_context->flags |= CODEC_FLAG_TRELLIS_QUANT;
      activated_mess("Trellis Quantization");}
#endif

// Trell segfault with these two codecs!!
#if LIBAVCODEC_BUILD >= 4681
    if (ffmpeg.cbp == 1  && divx.codec != FFMPEG_MJPEG 
#if LIBAVCODEC_BUILD >= 4729
	&& divx.codec != FFMPEG_H261)
#else
    )
#endif
    {
      codec_context->flags |= CODEC_FLAG_TRELLIS_QUANT;
      codec_context->flags |= CODEC_FLAG_CBP_RD;
      activated_mess("Coded Block Pattern");}
#endif

#if LIBAVCODEC_BUILD >= 4647
    if (ffmpeg.dia != 0) equal_int_mess("DIAmond", ffmpeg.dia);
    if (ffmpeg.cmp != 0) equal_int_mess("Full pel ME CoMParison", ffmpeg.cmp);
    if (ffmpeg.subcmp != 0) equal_int_mess("SUB pel ME CoMParison", ffmpeg.subcmp);
#endif
#if LIBAVCODEC_BUILD >= 4649
    if (ffmpeg.last_pred != 0) equal_int_mess("LAST frame motion PREdictors", ffmpeg.last_pred);
#endif
#if LIBAVCODEC_BUILD >= 4651
    if (ffmpeg.predia > 1) codec_context->pre_me = 2;
    if (ffmpeg.predia != 0) equal_int_mess("PRE-pass DIAmond", ffmpeg.predia);
    if (ffmpeg.precmp != 0) equal_int_mess("ME PRE-pass CoMParison", ffmpeg.precmp);
#endif
#if LIBAVCODEC_BUILD >= 4673
    if (ffmpeg.mbd >= 2) codec_context->b_frame_strategy = 1;
    if (ffmpeg.mbd != 0) equal_int_mess("MacroBlock Decision", ffmpeg.mbd);
#endif
    if (ffmpeg.keyint != 250) equal_int_mess("KEYframe INTernal", ffmpeg.keyint);
    if (ffmpeg.vlelim != 0) equal_int_mess("ELIMination Luminance", ffmpeg.vlelim);
    if (ffmpeg.vlelim != 0) equal_int_mess("ELIMination Chrominance", ffmpeg.vlelim);
    if (ffmpeg.lumi_mask != 0.0) equal_float_mess("LUMInance MASKing", ffmpeg.lumi_mask);
    if (ffmpeg.dark_mask != 0.0) equal_float_mess("DARKness MASKing", ffmpeg.dark_mask);
    if (ffmpeg.tcplx_mask != 0.0) equal_float_mess("Temporal ComPLeXity MASKing", ffmpeg.tcplx_mask);
    if (ffmpeg.scplx_mask != 0.0) equal_float_mess("Spatial ComPLeXity MASKing", ffmpeg.scplx_mask);
    if (ffmpeg.vmax_b_frames != 0) {
#if LIBAVCODEC_BUILD >= 4700
      codec_context->scenechange_threshold = 1000000000;
      codec_context->flags |= CODEC_FLAG_CLOSED_GOP;
#endif
      equal_int_mess("MAX B-Frames", ffmpeg.vmax_b_frames);}

    if (ffmpeg.vmax_b_frames >= 2)
      codec_context->b_frame_strategy = 1; // That help!!

// Switchs only used by the H263+ codec!!!!!
    if (divx.codec == FFMPEG_H263P) {
#if LIBAVCODEC_BUILD >= 4661
      if (ffmpeg.umv == 1) {
        codec_context->flags |= CODEC_FLAG_H263P_UMV;
        activated_mess("UMV");}

      if (ffmpeg.aic == 1) {
        if (divx.min_quantizer < 8) codec_context->qmin = 8;
        codec_context->flags |= CODEC_FLAG_H263P_AIC;
        activated_mess("AIC");}
#endif
#if LIBAVCODEC_BUILD >= 4694
      if (ffmpeg.obmc == 1) {
        codec_context->flags |= CODEC_FLAG_OBMC;
        activated_mess("OBMC");  }

      if (ffmpeg.ss == 1) {
        codec_context->flags |= CODEC_FLAG_H263P_SLICE_STRUCT;
        activated_mess("SS");  }

      if (ffmpeg.aiv == 1) {
        codec_context->flags |= CODEC_FLAG_H263P_AIV;
        activated_mess("AIV");}

      if (ffmpeg.loop == 1) {
        codec_context->flags |= CODEC_FLAG_LOOP_FILTER;
        activated_mess("LOOP");}
#endif
    }

// FFMPEG Exceptions managements ------------------------------------------------------------

// 1) HUFFYUV/FFVHUF Codecs
#if LIBAVCODEC_BUILD >= 4734
    if (divx.codec == FFMPEG_FFVHUFF) {
#if LIBAVCODEC_BUILD >= 4754 
      codec_context->strict_std_compliance=-2; // allow non standarized experimental things
#else
      codec_context->strict_std_compliance=-1; // allow inofficial extensions
#endif
      if(divx.bitrate < 5000000) codec_context->context_model = 1;}
    if (divx.codec == FFMPEG_HUFFYUV) {
      fmt_conv = VIDEO_YUV422P;
      codec_context->pix_fmt=PIX_FMT_YUV422P;
      codec_context->prediction_method = FF_PRED_MEDIAN;}
#else
    if (divx.codec == FFMPEG_HUFFYUV) {
      codec_context->strict_std_compliance=-1; // allow inofficial extensions
      codec_context->prediction_method = FF_PRED_MEDIAN;}
#endif

// 2) MJPEG/LJPEG exceptions
    if (divx.codec == FFMPEG_MJPEG 
#if LIBAVCODEC_BUILD >= 4670
      || divx.codec == FFMPEG_LJPEG)
#else
    )
#endif
      codec_context->strict_std_compliance=-1; // allow inofficial extensions

// 3) SVQ Codec
/* appeared with the 4719 version but don't work correctly actually */
#if LIBAVCODEC_BUILD >= 16777215
    if (divx.codec == FFMPEG_SVQ1) codec_context->pix_fmt=PIX_FMT_YUV410P;
#endif

// 4) Experimental video Codecs
/* appeared with the 4669 version but only work correctly with the 4755 version */
#if LIBAVCODEC_BUILD >= 4755
    if (divx.codec == FFMPEG_FFV1 
#if LIBAVCODEC_BUILD >= 4719
      || divx.codec == FFMPEG_SNOW)
#else
    )
#endif
      codec_context->strict_std_compliance=-2; // allow non standarized experimental things
#endif

// 5) qmin exception
    if (divx.codec == FFMPEG_MSMPEG4V3) codec_context->qmin = 3;


// 6) ffmpeg.vmax_b_frames exceptions
    if (divx.codec != FFMPEG_MPEG4 
#if LIBAVCODEC_BUILD >= 4676
     && divx.codec != FFMPEG_MPEG2 
#endif
#ifdef HAVE_FFMPEG_XVID
#if LIBAVCODEC_BUILD >= 4719
     && divx.codec != FFMPEG_XVID
#endif
#endif
#ifdef HAVE_FFMPEG_X264
#if LIBAVCODEC_BUILD >= 4748
     && divx.codec != FFMPEG_X264 
#endif
#endif
#if LIBAVCODEC_BUILD >= 4668
     && divx.codec != FFMPEG_ASV2
#endif
#if LIBAVCODEC_BUILD >= 4668
     && divx.codec != FFMPEG_ASV1
#endif
#if LIBAVCODEC_BUILD >= 4755
     && divx.codec != FFMPEG_FFV1
#endif
#if LIBAVCODEC_BUILD >= 4719
     && divx.codec != FFMPEG_SNOW
#endif
#if LIBAVCODEC_BUILD >= 4734
     && divx.codec != FFMPEG_FFVHUFF
#endif
     && divx.codec != FFMPEG_HUFFYUV
     && divx.codec != FFMPEG_MPEG1) {
       codec_context->max_b_frames = 0;
       update_divxa_commandwidget(c_ffmpeg_vmax_b_frames,0, translate("MAX B-Frames"));}

// 7) ffmpeg.cmp & ffmpeg.subcmp exceptions
#if LIBAVCODEC_BUILD >= 4647
    if (divx.codec == FFMPEG_MPEG1
#if LIBAVCODEC_BUILD >= 4676
     || divx.codec == FFMPEG_MPEG2 
#endif
#if LIBAVCODEC_BUILD >= 4670
     || divx.codec == FFMPEG_FLV1  
#endif
     || divx.codec == FFMPEG_RV10 
     || divx.codec == FFMPEG_H263  
     || divx.codec == FFMPEG_H263P) {
       codec_context->me_cmp = 0;
       codec_context->me_sub_cmp = 0;
       update_divxa_commandwidget(c_ffmpeg_cmp,0, translate("Full pel ME CoMParison"));
       update_divxa_commandwidget(c_ffmpeg_subcmp,0, translate("SUB pel ME CoMParison"));}
#endif

// 8 for SNOW
#if LIBAVCODEC_BUILD >= 4719
    if(divx.codec == FFMPEG_SNOW)
// <=> 51.1.0
#if LIBAVCODEC_BUILD >= 3342592
      codec_context->refs = 1;
#endif
#endif

// 9 for XviD
// <=> 51.2.0
#ifdef HAVE_FFMPEG_XVID
#if LIBAVCODEC_BUILD >= 3342848
    if(divx.codec == FFMPEG_XVID)
      codec_context->rc_strategy = 0;
#endif
#endif

// END FFMpeg Parameters --------------------------------------------------------------------
// We are using only a one pass compression because of realtime encoding

#if LIBAVCODEC_BUILD >= 4734
    if (divx.codec != FFMPEG_FFVHUFF && divx.codec != FFMPEG_HUFFYUV)
#else
    if (divx.codec != FFMPEG_HUFFYUV)
#endif
      codec_context->flags |= CODEC_FLAG_PASS1;

    picture_420p->linesize[0] = codec_context->width;
    picture_420p->linesize[1] = codec_context->width / 2;
    picture_420p->linesize[2] = codec_context->width / 2;

    if (avcodec_open(codec_context, codec) < 0)
    {
      sprintf(xawmesstmp, translate("divx_init: couldn't open %s codec\n"), codec_name);
      strcat(xawmessage,xawmesstmp);
      divx_stop();
      return;
    }

    if (codec_context->codec->encode == NULL)
    {
      sprintf(xawmesstmp, translate("divx_init: %s codec initialization failed (ctx->codec->encode == NULL)!\n"), codec_name);
      strcat(xawmessage,xawmesstmp);
      divx_stop();
      return;
    }

    enc_frame = (unsigned char *)malloc(ENC_FRAME_SIZE);
    if (enc_frame == NULL)
    {
      strcat(xawmessage,translate("divx_init: couldn't allocate memory for encoded frame: releasing ffmpeg DivX encoder...\n"));
      divx_stop();
      return;
    }
  }
#endif // HAVE_FFMPEG
      
  /*************** XVID init ****************************/
#ifdef HAVE_XVID
  else if(divx.codec==XVID) {
    if(fmts_conv_xvid==NULL) {
      fmts_conv_xvid=malloc(MAX_VIDEO_FMT*sizeof(video_fmt));
      setpreferred_list(fmts_xvid,fmts_conv_xvid);
    }
    memset(&xinit, 0, sizeof(xinit));
#ifdef HAVE_XVID09
    xvid_init(NULL, 0, &xinit, NULL);
    sprintf(xawmesstmp, translate("divx_init: initializing XviD core version %d\n"), xinit.core_build);
    strcat(xawmessage,xawmesstmp);
    if (debug)
      fprintf(stderr, "divx_init: XviD API version = %d.%d\n", xinit.api_version>>16, xinit.api_version&0xFF);
    if (xinit.api_version != API_VERSION)
    {
      //...interfaces are incompatible, encoding can't be performed
      sprintf(xawmesstmp, translate("XviD core claim its API version is %d.%d and I was expecting API version %d%.d: I give up...\n"),
                  xinit.api_version>>16, xinit.api_version&0xFF,
                  API_VERSION>>16, API_VERSION&0xFF);
      strcat(xawmessage,xawmesstmp);
      divx_stop();
      return;
    }
#endif

#ifdef HAVE_XVID10
    sprintf(xawmesstmp, translate("divx_init: XviD version=%d.%d.%d API=%d.%d\n"),
		XVID_VERSION_MAJOR(XVID_VERSION),
		XVID_VERSION_MINOR(XVID_VERSION),
		XVID_VERSION_PATCH(XVID_VERSION),
		XVID_API_MAJOR(XVID_API),
		XVID_API_MINOR(XVID_API));
    strcat(xawmessage,xawmesstmp);
    xinit.version = XVID_VERSION;
    if(xvid_global(NULL, XVID_GBL_INIT, &xinit, NULL)<0) {
      strcat(xawmessage,translate("xvid initialization failed\n"));
      divx_stop();
    }
    if(xinit.version != XVID_VERSION) {
      sprintf(xawmesstmp, translate("divx_init: XviD executed version=%d.%d.%d\n"),
		XVID_VERSION_MAJOR(xinit.version),
		XVID_VERSION_MINOR(xinit.version),
		XVID_VERSION_PATCH(xinit.version));
      strcat(xawmessage,xawmesstmp);
    }
#endif

    memset(&xparam, 0, sizeof(xparam));
    xparam.width = width;
    xparam.height = height;
    xparam.fincr = fps_den;
    xparam.fbase = fps_num;

#ifdef HAVE_XVID09
    xparam.rc_bitrate = divx.bitrate;
    xparam.rc_reaction_delay_factor = -1; // forces default value
    xparam.rc_averaging_period = -1; // forces default value
    xparam.rc_buffer = -1; // forces default value
    xparam.max_quantizer = divx.max_quantizer;
    xparam.min_quantizer = divx.min_quantizer;
    xparam.max_key_interval = 10*fps_num/fps_den;
#endif

#ifdef HAVE_XVID10
    memset(&single, 0, sizeof(xvid_plugin_single_t));  

    single.version = XVID_VERSION;
    single.bitrate = divx.bitrate;

    // Single Pass parameters (CBR)
    if (xvid.reaction_delay_factor != 16) {
      single.reaction_delay_factor = xvid.reaction_delay_factor;
      equal_int_mess("Reaction Delay Factor", xvid.reaction_delay_factor);}
    if (xvid.averaging_period != 100) {
      single.averaging_period = xvid.averaging_period;
      equal_int_mess("Averaging period", xvid.averaging_period);}
    if (xvid.buffer != 100) {
      single.buffer = xvid.buffer;
      equal_int_mess("Smoother", xvid.buffer);}

    xparam_plugin.func = xvid_plugin_single;
    xparam_plugin.param = &single;
    xparam.num_plugins = 1;
    xparam.plugins = &xparam_plugin;
    xparam.version = XVID_VERSION;

    // Quantizer Range / Restrictions
    {char letter[3][2]={"I", "P", "B"}; // Different quantizer types
      char messtmp[32]="";
      int i;

      for (i=0;i<3;i++) {
      if (xvid.max_quant[i]!=12) {
        xparam.max_quant[i] = xvid.max_quant[i];
        sprintf(messtmp,"Max %s-frame quantizer",letter[i]);
        equal_int_mess(messtmp, xvid.max_quant[i]);}
      if (xvid.min_quant[i]!=2) {
        xparam.min_quant[i] = xvid.min_quant[i];
        sprintf(messtmp,"Min %s-frame quantizer",letter[i]);
      equal_int_mess(messtmp, xvid.min_quant[i]);}}}

    // B-Frames (Advanced Simple Profile Specific Settings)
    if (xvid.vmax_b_frames) {

      xparam.max_bframes = xvid.vmax_b_frames;
      equal_int_mess("MAX B-Frames", xvid.vmax_b_frames);

      if (xvid.bquant_ratio != 150) {
        xparam.bquant_ratio = xvid.bquant_ratio;
        equal_int_mess("Quantizer Ratio", xvid.bquant_ratio);}

      if (xvid.bquant_offset != 100) {
        xparam.bquant_offset = xvid.bquant_offset;
        equal_int_mess("Quantizer Offset", xvid.bquant_offset);}

      if( xvid.frame_drop_ratio) {
        xparam.frame_drop_ratio = xvid.frame_drop_ratio;
        equal_int_mess("Frame Drop Tolerance", xvid.frame_drop_ratio);}

      if(xvid.packed) {
        xparam.global |= XVID_GLOBAL_PACKED;
        activated_mess("Packed bitstream");}

      if(xvid.closed_gop) {
        xparam.global |= XVID_GLOBAL_CLOSED_GOP;
        activated_mess("Closed GOP");}
    }

    if( xvid.keyint != 250 ) {
      xparam.max_key_interval = xvid.keyint;
      equal_int_mess("KEYframe INTernal", xvid.keyint);}
#endif

    ret = xvid_encore(NULL, XVID_ENC_CREATE, &xparam, NULL);
    if (ret != 0)
    {
      sprintf(xawmesstmp, translate("divx_init: XviD encore initialization failed ret=%d\n"),ret);
      strcat(xawmessage,xawmesstmp);
      divx_stop();
      return;
    }
    handle = xparam.handle;

    memset(&xframe, 0, sizeof(xframe));
    fmt_conv = fmts_conv_xvid[x11_pixmap_format];

#ifdef HAVE_XVID09
    xframe.general = xgeneral_presets[divx.quality];
    xframe.motion = xmotion_presets[divx.quality];
    xframe.quant = 0;
    xframe.intra = -1;
    xframe.colorspace = fmts_xaw_to_xvidcolorspace[fmt_conv];
#endif

#ifdef HAVE_XVID10
    xframe.version = XVID_VERSION;

    // From XviD 1.0.x examples/xvid_encraw.c & MPlayer libmpcodecs/ve_xvid(4).c & xvid4conf
    // Initialize input image fields
    xframe.input.csp = fmts_xaw_to_xvidcolorspace[fmt_conv];

    // Set up core's general features
    xframe.vol_flags = 0;
    xframe.vop_flags = 0;
    xframe.motion    = 0;

    xframe.vop_flags = xvop_presets[divx.quality];

    if (xvid.qpel) {
      xframe.vol_flags |= XVID_VOL_QUARTERPEL;
      activated_mess("Quarter PixEL");}
    if (xvid.gmc) {
      xframe.vol_flags |= XVID_VOL_GMC;
      activated_mess("Global Motion Compensation");}
    if (xvid.interlacing) {
      xframe.vol_flags |= XVID_VOL_INTERLACING;
      activated_mess("Interlaced Encoding");}

    if (xvid.gray) {
      xframe.vop_flags |= XVID_VOP_GREYSCALE;
      activated_mess("Grayscale Encoding");}
    if (xvid.cartoon) {
      xframe.vop_flags |= XVID_VOP_CARTOON;
      activated_mess("Cartoon Mode");}
    if (xvid.chroma_opt) {
      xframe.vop_flags |= XVID_VOP_CHROMAOPT;
      activated_mess("Chroma Optimizer");}
    if(xvid.vhq >= 1) {
      xframe.vop_flags |= XVID_VOP_MODEDECISION_RD;
      equal_int_mess("VHQ Mode", xvid.vhq);}
#if XVID_API >= XVID_MAKE_API(4,1)
    if(xvid.bvhq) {
      xframe.vop_flags |= XVID_VOP_RD_BVOP;
      activated_mess("BVHQ Mode");}
#endif

    // Quantizer Range / Restrictions

    if(xvid.chromame && divx.quality < 4) {
      xframe.motion |= XVID_ME_CHROMA_PVOP;
      xframe.motion |= XVID_ME_CHROMA_BVOP;
      activated_mess("Chroma Motion Estimation");}

    if(xvid.trell && divx.quality < 5) {
      xframe.vop_flags |= XVID_VOP_TRELLISQUANT;
      activated_mess("Trellis Quantization");}

    if(xvid.hqacpred && divx.quality < 6) {
      xframe.vop_flags |= XVID_VOP_HQACPRED;
      activated_mess("HQ-AC Prediction");}

    // Force the right quantizer (internally managed by RC plugins)
    xframe.quant = 0;

    // Set up motion estimation flags
    xframe.motion = xmotion_presets[divx.quality];

    if (xvid.gmc) xframe.motion |= XVID_ME_GME_REFINE;

    if (xvid.qpel) xframe.motion |= XVID_ME_QUARTERPELREFINE16;
    if (xvid.qpel && (xframe.vop_flags & XVID_VOP_INTER4V))
      xframe.motion |= XVID_ME_QUARTERPELREFINE8;

    if (xvid.turbo) {
      xframe.motion |= XVID_ME_FASTREFINE16 | XVID_ME_FASTREFINE8 | 
      XVID_ME_SKIP_DELTASEARCH | XVID_ME_FAST_MODEINTERPOLATE | 
      XVID_ME_BFRAME_EARLYSTOP;
      activated_mess("Turbo Mode");}

    if (xvid.cartoon) xframe.motion |= XVID_ME_DETECT_STATIC_MOTION;

    if(xvid.vhq >= 2) {
      xframe.motion |= XVID_ME_HALFPELREFINE16_RD;
      xframe.motion |= XVID_ME_QUARTERPELREFINE16_RD;}
    if(xvid.vhq >= 3) {
      xframe.motion |= XVID_ME_HALFPELREFINE8_RD;
      xframe.motion |= XVID_ME_QUARTERPELREFINE8_RD;
      xframe.motion |= XVID_ME_CHECKPREDICTION_RD;}
    if(xvid.vhq >= 4) xframe.motion |= XVID_ME_EXTSEARCH_RD;

    // B-Frames
    if (xvid.bframe_threshold) {
      xframe.bframe_threshold = xvid.bframe_threshold;
      equal_int_mess("B-Frame Threshold", xvid.bframe_threshold);}        

    // Quantizer Range / Restrictions
    // We don't propose to load any users matrices
    xframe.quant_intra_matrix = NULL;
    xframe.quant_inter_matrix = NULL;
#endif

    enc_frame = (unsigned char *)malloc(ENC_FRAME_SIZE);
    if (enc_frame == NULL)
    {
      strcat(xawmessage,translate("divx_init: couldn't allocate memory for encoded frame: releasing XviD encoder...\n"));
      divx_stop();
      return;
    }
    // Encode the frame
    xframe.bitstream = enc_frame;
  }
#endif // HAVE_XVID

  /*************** THEORA  init *************************/
  /* main part of theora init is done in container-ogg..*/
#ifdef HAVE_OGG
  else if(divx.codec==THEORA) {
    sprintf(xawmesstmp, translate("divx_init: Theora Codec used\n"));
    fmt_conv= VIDEO_YUV420;
  }
#endif // HAVE_OGG

  /* file must be opened AFTER codec initialisation..   */
  /******************************************************/

  if(divx.codec==UNCOMPRESSED)
    fourcc=int_to_str(fmt_conv, fourccs_raw);
  else
    fourcc=int_to_str(divx.codec,fourccs);
  if(fourcc==NULL) TRAP("should never happen\n");

  /************************ audio init *************************/

  /**** fix a segfault if the bitrate is not a modulo of 32 ****/
  div_t mp3bitrate;
  mp3bitrate=div(divx.mp3_bitrate,32);
  if (mp3bitrate.rem !=0)
  {
    fprintf(stderr, "\nIf you select CBR, the bitrate must be a modulo of 32\n\n");
    divx.mp3_bitrate = mp3bitrate.quot*32;
    update_divx_scrollbarwidget(l_divxmp3bitrate, s_divxmp3bitrate, divx.mp3_bitrate, 512, translate("Bitrate"));
  }

#ifndef NOAUDIO
  sprintf (xawmesstmp, "divx_init: audio: %s\n",
	  int_to_str(divx.audio_codec,audio_codec_list));
  strcat(xawmessage,xawmesstmp);
 
  /* the stereo mode could be automatically detected by the card !*/
  if(divx.stereo) achans = 2; else achans = 1;

  aformat=audio_fourcc(divx.audio_codec);

 /******************** lame audio init **************************/
#ifdef HAVE_LAME
  if (divx.audio_codec==LAME) {
    if(divx.audio_fmt!=AUDIO_S16LE) {
      fprintf(stderr, "@@@ only 16 bit format is mandatory for mp3 recording !!\n");
      divx_stop();
      return;
    }
    gfp = lame_init();

    lame_set_num_channels(gfp, achans);

    lame_set_in_samplerate(gfp, DEC_FREQ);
    lame_set_compression_ratio(gfp, 0.0);
    if(!divx.mp3_vbr)
      lame_set_brate(gfp, divx.mp3_bitrate);
    else {
      lame_set_VBR(gfp, vbr_mtrh);
      lame_set_VBR_q(gfp,divx.mp3_vbr_quality);
    }

    // mode = 0,1,2,3 = stereo, jstereo, dual channel (not supported), mono
    // default: lame picks based on compression ration and input channels
    if (achans == 1)
      lame_set_mode(gfp, 3);
    else
      lame_set_mode(gfp, 1);

    // quality=0..9.  0=best (very slow).  9=worst.
    // recommended:  2     near-best quality, not too slow
    //               5     good quality, fast
    //               7     ok quality, really fast
    lame_set_quality(gfp, divx.mp3_quality);

    if (lame_init_params(gfp) < 0)
    {
      strcat(xawmessage,translate("divx_init: lame_init_params failed\n"));
      divx_stop();
      return;
    }

    lame_buffer_size = (1.25*divx.audio_buffer_size)+7200;
    lame_buffer = malloc(lame_buffer_size);
    if (lame_buffer == NULL)
    {
      strcat(xawmessage,translate("divx_init: could not allocate lame buffer\n"));
      divx_stop();
      return;
    }

    aformat = WAVE_FORMAT_MPEGLAYER3;
  }
#endif // HAVE_LAME

 /*************** ffmpeg audio init ***********************/
#ifdef HAVE_FFMPEG
  if (xdtv_to_audioffmpeg(divx.audio_codec)!=-1) {
    if(divx.audio_fmt!=AUDIO_S16LE) {
      fprintf(stderr, "@@@ only 16 bit format is implemented for ffmpeg audio recording !!\n");
      divx_stop();
      return;
    }
    avcodec_init();
    switch(divx.audio_codec) {
      case FFMPEG_MP2: register_avcodec(&mp2_encoder); break;
#ifdef HAVE_FFMPEG_MP3
      case FFMPEG_MP3: register_avcodec(&mp3lame_encoder); break;
#endif
#ifdef HAVE_FFMPEG_AC3
      case FFMPEG_AC3: register_avcodec(&ac3_encoder); break;
#endif
#ifdef HAVE_FFMPEG_AAC
      case FFMPEG_AAC: register_avcodec(&faac_encoder); break;
#endif
      case FFMPEG_PCMS16LE: register_avcodec(&pcm_s16le_encoder); break;
#ifdef HAVE_FFMPEG_AMR_NB
      case FFMPEG_AMR_NB: register_avcodec(&amr_nb_encoder); break;
#endif
#ifdef HAVE_FFMPEG_AMR_WB
      case FFMPEG_AMR_WB: register_avcodec(&amr_wb_encoder); break;
#endif
    }
    if (audio_codec_context == NULL)
      audio_codec_context = avcodec_alloc_context();
    audio_codec_context->codec_id = xdtv_to_audioffmpeg(divx.audio_codec);
    audio_codec_context->codec_type= CODEC_TYPE_AUDIO;
    audio_codec_context->bit_rate = (divx.mp3_bitrate * 1000);
    audio_codec_context->sample_rate = divx.audio_freq;
    audio_codec_context->channels = achans;
    av_codec = avcodec_find_encoder(audio_codec_context->codec_id);
    if (av_codec==NULL
       || avcodec_open(audio_codec_context, av_codec) < 0
       || audio_codec_context->codec->encode == NULL) {
      strcat(xawmessage,translate("divx_init: audio: problem with ffmpeg\n"));
      sprintf(xawmesstmp, translate("divx_init: audio: av_codec = %p\n"),av_codec);
      strcat(xawmessage,xawmesstmp);
      sprintf(xawmesstmp, translate("divx_init: audio: avcodec_open = %d\n"),avcodec_open(audio_codec_context, av_codec));
      strcat(xawmessage,xawmesstmp);
      sprintf(xawmesstmp, translate("divx_init: audio: audio_codec_context->codec->encode = %p\n"),audio_codec_context->codec->encode);
      strcat(xawmessage,xawmesstmp);
      divx_stop();
      return;
    }

    av_buffer0=malloc(audio_codec_context->frame_size*2*achans);
    av_buffer0_l=0;
    av_buffer_size = 200000; /* around 50ko for vorbis ! */
    av_buffer = malloc(av_buffer_size);
  }
#endif // HAVE_FFMPEG

 /******************** Vorbis audio init **************************/
 /* main part of vorbis init si done in container-ogg.c */
#ifdef HAVE_OGG
  if (divx.audio_codec==VORBIS) {
    if(divx.audio_fmt!=AUDIO_S16LE) {
      fprintf(stderr, "@@@ only 16 bit format is mandatory for vorbis recording !!\n");
      divx_stop();
      return;
    }
  }
#endif // HAVE_OGG
#endif // NOAUDIO

  /*************** Open and setup output file *********************/

  if (divx.container != AVI_BUILTIN) {
    divx.maxsize = 0;
    update_divx_scrollbarwidget(l_divxmaxsize, s_divxmaxsize, divx.maxsize, 4096, translate("Max. Size (Mb)"));
  }

  filename = get_filename();
  sprintf(lastfilename,"%s '%s'&",pplayer, filename);

  if (debug)
    fprintf(stderr,"nxtvepg title: %s\n",channel_title);

  if (0 != (cont_handler->open_ofile(cont_handler,filename, divx.maxsize))){
      fprintf (stderr,"divx_init: open_ofile error \n");
      divx_stop();
      return;
  }

  if(divx.sub) {
    char *f=get_subfilename(filename);
    subfile=fopen(f,"w");
    free(f);
    if(subfile==NULL) perror("fopen sub");
    lastsubend=0;
  }
  if(subfile)
    fprintf(subfile,"FORMAT=%f\n\n",(double)fps_num/fps_den);
  free(filename);

/* file initialisation etc. */
#ifdef HAVE_FFMPEG  
  if(xdtv_to_ffmpeg(divx.codec)!=-1 && cont_handler->set_extradata)
    cont_handler->set_extradata(cont_handler, codec_context->extradata,codec_context->extradata_size);
#endif

  cont_handler->set_video(cont_handler, width, height, fps_num, fps_den, fourcc);
#ifdef NOAUDIO
  /* channels, rate, bits ,formats, brate,framesize,vbr */
  cont_handler->set_audio(cont_handler, 0 ,0 , 0, 0 , 0 , 0, 0 ); 
#else
/******** uncompressed case **********/
  if(divx.audio_codec==AUDIO_UNCOMPRESSED)
    cont_handler->set_audio(cont_handler, achans, DEC_FREQ, 
		 divx.audio_fmt==AUDIO_U8 ? 8 : 16, aformat, 0, 0, 0);
/******** MP3 Lame case **********/
#ifdef HAVE_LAME
  else if (divx.audio_codec==LAME)
    cont_handler->set_audio(cont_handler, achans, lame_get_out_samplerate(gfp), 16, aformat, (divx.mp3_bitrate * 1000)/8,lame_get_framesize(gfp),divx.mp3_vbr);
#endif // HAVE_LAME
/******** FFMPEG case **********/
#ifdef HAVE_FFMPEG
 else if (xdtv_to_audioffmpeg(divx.audio_codec)!=-1)
   cont_handler->set_audio(cont_handler, achans, divx.audio_freq, 16, aformat, (divx.mp3_bitrate * 1000)/8,audio_codec_context->frame_size,divx.mp3_vbr);
#endif // HAVE_FFMPEG
/******** Vorbis case **********/
#ifdef HAVE_OGG
 else if (divx.audio_codec==VORBIS)
   cont_handler->set_audio(cont_handler, achans, divx.audio_freq, 16, aformat, (divx.mp3_bitrate * 1000)/8, 50000 /*FIXME*/ ,divx.mp3_vbr);
#endif // HAVE_OGG
#endif // NOAUDIO

  /************************************************************/
  /* http must be started when the header of the avfile is fixed
     but before the write of any frame to catch the header */

  if(divx.stream)
  {
    fprintf(stderr,"GUI Streaming activation: %d\n",divx.stream);
    cont_handler->avfile->send_data = http_send_data;
    if(!restart) http_init_server();
  }

  set_capture_temp_grab(1, 1);

  // Let's go
  video_frames = 0;
  slw = 0;
#ifndef NOAUDIO
  audio_frames = 0;
  audio_buffers = 0;
#ifdef SYNCHRO
  nbsamplesaudios = 0;
  dropped_video_frames = 0;
  dropped_audio_samples = 0;
  dropped_audio_buffers = 0;
#endif
  gettimeofday(&tv, NULL);
  if(audiostamp==0.0) audiostamp=tv.tv_usec/1000000.0+tv.tv_sec;
  if(videostampmin==0.0)
    videostampmin=videostampmax=tv.tv_usec/1000000.0+tv.tv_sec;
  divx_audio_must_start=1;
#endif

  time(&t);
  sprintf(xawmesstmp, translate("Recording started %s"), ctime(&t));
  strcat(xawmessage,xawmesstmp);
  gettimeofday(&tv_start, &tz);

  divx_in_progress = 1;
} // END divx_init

void divx_restart(void) {
  restart=1;
  divx_stop();
  divx_init(prev_w, prev_h);
  restart=0;
}

void
divx_encode(unsigned char *src_frame)
{
#ifdef HAVE_DIVX4LINUX
   ENC_RESULT res;
#endif
#if defined(HAVE_DIVX4LINUX) || defined(HAVE_XVID)
  int ret;
#endif
#ifdef HAVE_FFMPEG
  int enc_size;
#endif
  int flag;

  /* started after the first frame because alsa dmix
     makes fork which cause probleme for v4l2 format change... */
  if (divx_audio_must_start &&
      !is_aop_running() && grabbers[grabber] != &grab_avi 
#ifdef HAVE_DVB
      && grabbers[grabber] != &grab_dvb
#endif      
      ){
      fprintf(stderr,"call to divx_audio_start()\n");
    divx_audio_start();
    }
  divx_audio_must_start=0;

  /* only the video thread can run divx_stop */
  if(slw)
  {
    sprintf(xawmesstmp, translate("last AVI_write_status = %d\n"),slw);
    strcat(xawmessage,xawmesstmp);
    AVI_print_error("reason");
    displaynextframe = 0;
    if(AVI_errno==AVI_ERR_SIZELIM) divx_restart();
    else divx_stop();
    return;
  }

#ifdef SYNCHRO
  if(((double)video_frames * fps_den / fps_num - videostampmax
      + divx.delay >
      (double)nbsamplesaudios / DEC_FREQ - audiostamp + divx.maxgap)
     || (videostampmin > audiostamp + 4.0) ) { //if audio is blocked
    dropped_video_frames++;
    return;
  }
#endif

  if(divx.codec==UNCOMPRESSED) {
    unsigned char *image;
    int size = size_img(fmt_conv,width_rec,height_rec);
    image=convert1
      ((char*)src_frame, x11_pixmap_format, fmt_conv, width_rec, height_rec);
    slw = cont_handler->write_frame(cont_handler, (char*)image, size, AVI_FLAG_KEYFRAME);
  }
#ifdef HAVE_DIVX4LINUX
  else if(divx.codec==DIVX4LINUX) {
      frame.image=convert1
	((char*)src_frame, x11_pixmap_format, fmt_conv, width_rec, height_rec);
      ret = encore(handle, ENC_OPT_ENCODE, &frame, &res);
      if (ret != ENC_OK)
      {
        strcat(xawmessage,translate("divx_encode: error encoding frame (divx4linux)!\n"));
        slw = cont_handler->write_frame(cont_handler, frame.bitstream, 0, AVI_FLAG_NONE);
      }
      else
      {
#if ENCORE_VERSION >= 20021024
        flag = (res.cType == 'I') ? AVI_FLAG_KEYFRAME : AVI_FLAG_NONE;
#else
	flag = res.is_key_frame ? AVI_FLAG_KEYFRAME : AVI_FLAG_NONE;
#endif
        slw = cont_handler->write_frame(cont_handler, frame.bitstream, frame.length, flag);
      }
  }
#endif // HAVE_DIVX4LINUX

#ifdef HAVE_FFMPEG
  else if(xdtv_to_ffmpeg(divx.codec)!=-1) {
      picture_420p->data[0] = convert1
	((char*)src_frame, x11_pixmap_format, fmt_conv, width_rec, height_rec);
      picture_420p->data[1] =
	picture_420p->data[0] + width_rec*height_rec;
      picture_420p->data[2] =
	picture_420p->data[1] + width_rec*height_rec / 4;
      if(fmt_conv==VIDEO_YUV422P)
	picture_420p->data[2] += width_rec*height_rec / 4;
      picture_420p->key_frame = -1;
      enc_size = avcodec_encode_video(codec_context, enc_frame, ENC_FRAME_SIZE, picture_420p);
      if (enc_size > 0) {
	flag = codec_context->coded_frame->key_frame ? AVI_FLAG_KEYFRAME : AVI_FLAG_NONE;
	slw = cont_handler->write_frame(cont_handler, (char*)enc_frame, enc_size, flag);
      } else {
	strcat(xawmessage,translate("divx_encode: error encoding frame (ffmpeg)!\n"));
	slw = cont_handler->write_frame(cont_handler, (char*)enc_frame, 0, AVI_FLAG_NONE);
      }
  }
#endif // HAVE_FFMPEG

#ifdef HAVE_XVID
  else if(divx.codec==XVID) {

#ifdef HAVE_XVID09
      xframe.image = convert1
	((char*)src_frame, x11_pixmap_format, fmt_conv, width_rec, height_rec);
      xframe.intra = -1;
      ret = xvid_encore(handle, XVID_ENC_ENCODE, &xframe, NULL);
#endif

#ifdef HAVE_XVID10
      xframe.input.plane[0] = convert1
	    ((char*)src_frame, x11_pixmap_format, fmt_conv, width_rec, height_rec);
      if(fmt_conv==VIDEO_YVU420) {
	int s=width_rec*height_rec;
	xframe.input.plane[1] = xframe.input.plane[0]+s;
	xframe.input.plane[2] = xframe.input.plane[1]+s/4;
	xframe.input.stride[0] = width_rec;
	xframe.input.stride[1] = xframe.input.stride[2] = width_rec/4;
      } else
	xframe.input.stride[0] = size_img(fmt_conv,width_rec,1);
      
      // Let core decide for us
      // motion level == 0 means no motion search which is equivalent to intra coding only
      if(divx.quality == 0) xframe.type = XVID_TYPE_IVOP;
      else xframe.type = XVID_TYPE_AUTO;
      
      xframe.length = ENC_FRAME_SIZE;
      memset(&xstats, 0, sizeof(xstats));
      xstats.version = XVID_VERSION;
      ret = xvid_encore(handle, XVID_ENC_ENCODE, &xframe, &xstats);
#endif

#ifdef HAVE_XVID09
      if(ret>0) ret=-1000-ret;
#endif
      if (ret < 0)
      {
        sprintf(xawmesstmp, translate("divx_encode: error encoding frame (XviD)! ret=%d\n"),ret);
        strcat(xawmessage,xawmesstmp);
        slw = cont_handler->write_frame(cont_handler, xframe.bitstream, 0, AVI_FLAG_NONE);
      }
      else
        {
#ifdef HAVE_XVID09
          flag = (xframe.intra > 0) ? AVI_FLAG_KEYFRAME : AVI_FLAG_NONE;
          slw = cont_handler->write_frame(cont_handler, xframe.bitstream, xframe.length, flag);
#endif

#ifdef HAVE_XVID10
          flag = (xframe.out_flags & XVID_KEYFRAME)
	    ? AVI_FLAG_KEYFRAME : AVI_FLAG_NONE;
          slw = cont_handler->write_frame(cont_handler, xframe.bitstream, ret, flag);
#endif
        }
  }
#endif // HAVE_XVID
#ifdef HAVE_OGG
  else if(divx.codec==THEORA) {

    unsigned char *image;
    int size = size_img(fmt_conv,width_rec,height_rec);
    image=convert1((char*)src_frame, x11_pixmap_format, fmt_conv, width_rec, height_rec);
    slw = cont_handler->write_frame(cont_handler,(char*) image , size  , 0);
  }
#endif

  

  video_frames++;
  if(divx.stream && !divx.display_frames)
    http_read_clients();
  //segfaults if executed when norm changes and frames are displayed
}

void
divx_stop(void)
{
#if defined(HAVE_DIVX4LINUX) || defined(HAVE_XVID)
  int ret;
#endif
  double elapsed_time;
  struct timeval tv_stop;
  struct timezone tz;
  time_t t;

  divx_in_progress = 0;

  gettimeofday(&tv_stop, &tz);
  time(&t);
  sprintf(xawmesstmp, translate("Recording stopped %s"), ctime(&t));
  strcat(xawmessage,xawmesstmp);

  if (debug)
    fprintf(stderr, "divx_stop...\n");

#ifndef NOAUDIO
  divx_audio_stop();
  if (debug)
    fprintf(stderr, "divx_stop: audio stopped\n");

  if(is_aop_running())
    aop_wait_write_audio();
  else if(grabbers[grabber] == &grab_avi) {
    file_wait_write_audio();
  }

  if(divx.audio_codec==AUDIO_UNCOMPRESSED) {
    /* nothing !*/
  }
#ifdef HAVE_LAME
  else if (divx.audio_codec==LAME)
    lame_stop();
#endif // HAVE_LAME
#ifdef HAVE_FFMPEG
  else if(xdtv_to_audioffmpeg(divx.audio_codec)!=-1) {
    if(audio_codec_context) {
      avcodec_close(audio_codec_context);
      av_free(audio_codec_context);
      audio_codec_context=NULL;
    }
    if(av_buffer) {
      free(av_buffer);
      av_buffer_size=0;
    }
    if(av_buffer0) free(av_buffer0);
  }
#endif
#endif

  if(cont_handler->avfile) cont_handler->avfile->send_data = NULL;
  if(!restart) http_close_server();

  // Set the real FPS
  elapsed_time = ((double)tv_stop.tv_sec + (double)tv_stop.tv_usec*1.0e-6)
    - ((double)tv_start.tv_sec + (double)tv_start.tv_usec*1.0e-6);

  set_capture_temp_grab(0, 1);

#ifndef SYNCHRO
  fps_den = 1000000;
  fps_num = (double)video_frames * fps_den / elapsed_time + 0.5;

  if (cont_handler->avfile) {
    cont_handler->avfile->fps_num = fps_num;
    cont_handler->avfile->fps_den = fps_den;
  }
#else
  if(cont_handler->avfile && (cont_handler->avfile->fps_num != fps_num || cont_handler->avfile->fps_den != fps_den))
  {
    strcat(xawmessage,translate("Be careful fps has changed during recording\n"));
  }
#endif

#ifndef NOAUDIO
#ifndef SYNCHRO
  sprintf(xawmesstmp, translate("%ld frames recorded in %f seconds => FPS = %f\n%ld audio frames recorded (%ld audio buffers)\n"), video_frames, elapsed_time, (double)fps_num/fps_den, audio_frames,audio_buffers);
  strcat(xawmessage,xawmesstmp);
#else
  sprintf(xawmesstmp, translate("recording time = %f seconds,  "), elapsed_time);
  strcat(xawmessage,xawmesstmp);
  sprintf(xawmesstmp, translate("fps = %f\n"),(double)fps_num/fps_den);
  strcat(xawmessage,xawmesstmp);
  sprintf(xawmesstmp, translate("%ld video frames recorded (%f seconds)\n"), video_frames, (double)video_frames*fps_den/fps_num);
  strcat(xawmessage,xawmesstmp);
  sprintf(xawmesstmp, translate("%ld audio frames recorded (%ld audio buffers, %f seconds)\n"), audio_frames,audio_buffers,(double)nbsamplesaudios / DEC_FREQ);
  strcat(xawmessage,xawmesstmp);
  sprintf(xawmesstmp, translate("%ld dropped video frames (fpscod=%f fpscapt=%f)\n"), dropped_video_frames, (double)video_frames / elapsed_time,
	 ((double)video_frames + dropped_video_frames) / elapsed_time);
  strcat(xawmessage,xawmesstmp);
  sprintf(xawmesstmp, translate("%ld dropped audio buffers (%f seconds)\n"), dropped_audio_buffers, (double)dropped_audio_samples / DEC_FREQ);
  strcat(xawmessage,xawmesstmp);

  if(video_frames==0)
    strcat(xawmessage,translate("@@@@ No video frames recorded\n"));
  else if(dropped_audio_buffers*20>audio_frames) {
    strcat(xawmessage,  translate("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n"));
    strcat(xawmessage,  translate("@@ You computer is too slow to do such recording @@\n"));
    strcat(xawmessage,  translate("@@ try to decrease image dimension               @@\n"));
    sprintf(xawmesstmp, translate("@@ or the framerate (to %.1ffps ?)               @@\n"), (double)video_frames / elapsed_time * 1.05);
    strcat(xawmessage,xawmesstmp);
    strcat(xawmessage,  translate("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n"));
  }
  else if(dropped_video_frames*20>video_frames) {
    sprintf(xawmesstmp, translate("You could try record to the better fps=%.1f\n"),
	   ((double)video_frames + dropped_video_frames)
	   / elapsed_time * 0.95);
    strcat(xawmessage,xawmesstmp);
  }
#endif
#else
  sprintf(xawmesstmp, translate("%ld frames recorded in %f seconds => FPS = %f\n"), video_frames, elapsed_time,(double)fps_num/fps_den);
  strcat(xawmessage,xawmesstmp);
#endif

  // Release allocated buffers
  if(divx.codec==UNCOMPRESSED) {
    /* nothing ! */
  }
#ifdef HAVE_DIVX4LINUX
  else if(divx.codec==DIVX4LINUX) {
      if (handle != NULL)
        {
          /* Release the encoder */
          ret = encore(handle, ENC_OPT_RELEASE, 0, 0);
          if (ret != ENC_OK)
            strcat(xawmessage,translate("divx_stop: divx4linux codec release failed\n"));
          else if (debug)
            fprintf(stderr, "divx_stop: divx4linux codec released\n");

          handle = NULL;
        }

      if (enc_frame != NULL)
        {
          free(enc_frame);
          enc_frame = NULL;
          if (debug)
            fprintf(stderr, "divx_stop: divx4linux encoded frame freed\n");
        }
  }
#endif // HAVE_DIVX4LINUX

#ifdef HAVE_FFMPEG
  else if(xdtv_to_ffmpeg(divx.codec)!=-1) {
      if (codec_context != NULL)
        {
          avcodec_close(codec_context);
          free(codec_context);
          codec_context = NULL;
          handle = NULL;
          free(picture_420p);
          picture_420p = NULL;
          if (debug)
            fprintf(stderr, "divx_stop: ffmpeg codec released\n");
        }
      if (enc_frame != NULL)
        {
          free(enc_frame);
          enc_frame = NULL;
          if (debug)
            fprintf(stderr, "divx_stop: ffmpeg encoded frame freed\n");
        }
  }
#endif // HAVE_FFMPEG

#ifdef HAVE_XVID
  else if(divx.codec==XVID) {
      if (handle != NULL)
        {
          /* Release the encoder */
          ret = xvid_encore(handle, XVID_DEC_DESTROY, NULL, NULL);
          if (ret != 0)
            strcat(xawmessage,translate("divx_stop: XviD codec release failed\n"));
          else if (debug)
            fprintf(stderr, "divx_stop: XviD codec released\n");

          handle = NULL;
        }

      if (enc_frame != NULL)
        {
          free(enc_frame);
          enc_frame = NULL;
          if (debug)
            fprintf(stderr, "divx_stop: XviD encoded frame freed\n");
        }
  }
#endif // HAVE_XVID

  if (cont_handler->avfile)
    {
      sprintf(xawmesstmp, translate("file size = %.1f Mb,  "),cont_handler->avfile->pos/1024.0/1024.0);
      strcat(xawmessage,xawmesstmp);
      sprintf(xawmesstmp, translate("mean bitrate = %.3f kbps\n"),cont_handler->avfile->pos*8.0/elapsed_time/1000);
      strcat(xawmessage,xawmesstmp);
      if ( cont_handler->close(cont_handler) != 0){
       	fprintf(stderr,"divx_stop: close error \n");
      }

      if (debug)
        fprintf(stderr, "divx_stop: divx file closed\n");
    }

  if(subfile)
    {
      write_sub(NULL);
      fclose(subfile);
      subfile=NULL;
    }

  video_set_capture_size(prev_w_capture, prev_h_capture);

  if (debug) fprintf(stderr, "...Done\n");

  if(divx.stream) fprintf(stderr, xawmessage);
  else popup_mess(xawmessage);
}

int
is_divx_in_progress(void)
{
  return divx_in_progress;
}

int
is_http_in_progress(void)
{
  return divx_in_progress && divx.stream;
}

int
divx_display_frames(void)
{
  //when divx_stop is started from divx_encode
  //the frame size changes, assigning xvimage to NULL
  if(!displaynextframe) { displaynextframe=1; return 0; }
  return divx.display_frames;
}

#ifndef NOAUDIO
/*****************************************************************************
 *                            RAW AUDIO FUNCTIONS
 ****************************************************************************/

void
divx_audio_start(void)
{
  int arg = 0;
  int ret;

  divx_audio = 1;
  if(audio_open(AUDIO_RO, divx.audio_fmt, achans, DEC_FREQ, divx.audio_fragments,
		divx.audio_sizefragment)<0) {
    fprintf(stderr,"audio_open failed\n");
    divx_audio = 0;
    return;
  }
  mixer_set_mode(MIXER_MODE_CAPTURE);
#ifdef _POSIX_THREAD_IS_GNU_PTH
  {
    pthread_attr_t attr;
    pthread_attr_init(&attr);
    /* increase if you have a "**Pth** STACK OVERFLOW" */
    pthread_attr_setstacksize(&attr, 256*1024);
    ret = pthread_create (&audio_thread, &attr, audio_record, (void *) arg);
    pthread_attr_destroy(&attr);
    /* in this case audioframetime must be equal to videoframetime */
    divx.audio_buffer_size=DEC_FREQ*achans*fps_den/fps_num;
  }
#else
  ret = pthread_create (&audio_thread, NULL, audio_record, (void *) arg);
#endif

  if (ret != 0)
    strcat(xawmessage,translate("Audio record thread failed to start.\n"));
  else if (debug)
    fprintf (stderr, "Audio record thread started.\n");

}

inline void
write_audio_to_avi(short int *buffer, int size)
{
#if defined(HAVE_LAME) || defined(HAVE_FFMPEG)
  int nsamples;
#endif

  if(slw) return;
#ifdef SYNCHRO
  if(((double)nbsamplesaudios / DEC_FREQ - audiostamp > 
      (double)video_frames * fps_den / fps_num
      - videostampmin + divx.delay + divx.maxgap)
     || (audiostamp > videostampmax + 4.0)) { //if video is blocked
    dropped_audio_samples += size / achans;
    dropped_audio_buffers++;
    return;
  }
  nbsamplesaudios += size / achans;
#endif

  if(divx.audio_codec==AUDIO_UNCOMPRESSED) {
    slw = cont_handler->write_audio
      (cont_handler, (char *)buffer, size * (divx.audio_fmt==AUDIO_U8 ? 1 : 2));
    audio_frames++;
  }
#ifdef HAVE_LAME
  else if (divx.audio_codec==LAME)
    {
      if(!divx.mp3_vbr) {
	nsamples = lame_encode(buffer, size / achans);
	slw = cont_handler->write_audio(cont_handler, lame_buffer, nsamples);
	audio_frames++;
      } else {
	/* in this case an AVI frame must contain only one MP3 frame */
	int i;
	for(i=0;i<size/achans;i+=cont_handler->avfile->a_framesize) {
	  if(i+cont_handler->avfile->a_framesize<size/achans) {
	    nsamples = lame_encode(buffer+i*achans,cont_handler->avfile->a_framesize);
	    slw = cont_handler->write_audio(cont_handler, lame_buffer, nsamples);
	    if(slw) return;
	    audio_frames++;
	  } else {
	    nsamples = lame_encode(buffer+i*achans, size / achans - i);
	    if(nsamples) {
	      slw = cont_handler->write_audio(cont_handler, lame_buffer, nsamples);
	      if(slw) return;
	      audio_frames++;
	    }
	  }
	}
      }
    }
#endif // HAVE_LAME
#ifdef HAVE_FFMPEG
  else if(xdtv_to_audioffmpeg(divx.audio_codec)!=-1) {
    int i=0; /* i = nb of treated samples on each channel */
    int lremain=size / achans;
    while(1) {
      int ll;
      if(av_buffer0_l+lremain<cont_handler->avfile->a_framesize) {
	fast_memcpy(av_buffer0+2*achans*av_buffer0_l,
		    buffer+i*achans,2*achans*lremain);
	av_buffer0_l+=lremain;
	break;
      }
      ll=cont_handler->avfile->a_framesize-av_buffer0_l;
      fast_memcpy(av_buffer0+2*achans*av_buffer0_l, buffer+i*achans,
		  2*achans*ll);
      i+=ll;
      lremain-=ll;
      nsamples = 
	avcodec_encode_audio(audio_codec_context, (unsigned char*)av_buffer,
			     av_buffer_size, (short*) av_buffer0);
      if(nsamples) {
	slw = cont_handler->write_audio(cont_handler, av_buffer, nsamples);
	if(slw) return;
      }
      audio_frames++;
      av_buffer0_l=0;
    }
  }
#endif //HAVE_FFMPEG
#ifdef HAVE_OGG
  else if (divx.audio_codec==VORBIS)
    {
	slw = cont_handler->write_audio(cont_handler, (char*) buffer, size*(divx.audio_fmt==AUDIO_U8 ? 1 : 2));
      audio_frames++;
    }
#endif 
    audio_buffers++;
}

static void*
audio_record(void *arg)
{
  int error;
  short int *buffer;
  struct timeval tv;

  pthread_setcancelstate(PTHREAD_CANCEL_DISABLE,NULL);
  buffer = (short int *)malloc (/*BUFFER_SIZE*/divx.audio_buffer_size * sizeof (short int));
  if (buffer == NULL)
    {
      error = 1;
      goto error;
    }

  while (1)
    {
      int r;
#ifdef _POSIX_THREAD_IS_GNU_PTH
      pthread_yield_np();
#endif
      r = audio_read(buffer, divx.audio_buffer_size*(divx.audio_fmt==AUDIO_U8 ? 1 : 2));
      if(r<0) {
	fprintf(stderr,"audio_read failed");
	error = 2;
	goto error;
      }
      r /= (divx.audio_fmt==AUDIO_U8 ? 1 : 2);
      gettimeofday (&tv, NULL);
      audiostamp=tv.tv_usec/1000000.0+tv.tv_sec;
      write_audio_to_avi(buffer, r);

      pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
      pthread_testcancel ();
      pthread_setcancelstate(PTHREAD_CANCEL_DISABLE,NULL);
    }
  return NULL;

 error:
  switch (error)
    {
    case 1:
      strcat(xawmessage,translate("audio_record: Error allocating audio buffer: exiting audio record thread.\n"));
      break;
    case 2:
      strcat(xawmessage,translate("audio_record: Error reading device: exiting audio record thread.\n"));
      break;
    }
  pthread_exit ((void *)&error);
}

void
divx_audio_stop(void)
{
  void *ret;

  divx_audio_must_start=0;
  if(!divx_audio) return;

  pthread_cancel (audio_thread);
  pthread_join (audio_thread, &ret);

  audio_close();
  mixer_set_mode(MIXER_MODE_TVLINE);
  divx_audio= 0;
  
}


/*****************************************************************************
 *                           MP3 AUDIO FUNCTIONS
 ****************************************************************************/
#ifdef HAVE_LAME

inline static int
lame_encode(short int *raw_buffer, int samples_per_channel)
{
  int mp3_samples;

  if(achans == 1)
    mp3_samples = lame_encode_buffer(gfp, raw_buffer, raw_buffer, samples_per_channel, (unsigned char*)lame_buffer, lame_buffer_size);
  else
    mp3_samples = lame_encode_buffer_interleaved(gfp, raw_buffer, samples_per_channel, (unsigned char*)lame_buffer, lame_buffer_size);
  return  mp3_samples;
}

static void
lame_stop(void)
{
  int i;

  if(gfp == NULL) return;

  i = lame_encode_flush(gfp, (unsigned char*)lame_buffer, lame_buffer_size);

  if (cont_handler->avfile && !slw)
    slw = cont_handler->write_audio(cont_handler, lame_buffer, i);

  lame_close(gfp);
  gfp = NULL;

  if (lame_buffer)
    {
      free(lame_buffer);
      lame_buffer = NULL;
      lame_buffer_size = 0;
    }
}
#endif // HAVE_LAME

#else //NOAUDIO

inline void write_audio_to_avi(short int *buffer, int size){}

#endif //NOAUDIO


/* function which should be executed in main.c, or by the interface ?*/
void setfps(double fps) 
{
#ifdef SYNCHRO
  divx.fps= 1000 * fps + 0.5;
#endif
}

void write_sub(char *s)
{
  static char *lastsub = NULL;
  static long lastsubbegin;
  long frame;
  int i,j,k,debutligne,lignevide,l,nlignes;
  char *s2;

  if(!subfile) return;
  frame=video_frames;
  if(lastsub!=NULL) {
    if(debug)
      printf("%ld %ld\n",lastsubbegin-lastsubend,
	     frame-lastsubbegin);
    fprintf(subfile,"%ld %ld\n",lastsubbegin-lastsubend,
	    frame-lastsubbegin);
    fputs(lastsub,subfile);
    fputs("\n\n",subfile);
    fflush(subfile);
    lastsubend=frame;
    free(lastsub);
  }
  if(s==NULL || *s=='\0') {lastsub=NULL;return;}
  l=strlen(s);
  s2=(char *)malloc(l+1);

  debutligne=0;
  lignevide=1;
  for(nlignes=0,i=0,j=0;i<l;i++) {
    if(s[i]!='\n' && i!=l-1) {
      if(s[i]!=' ' && s[i]!='#') lignevide=0;
    } else {
      if(s[i]!='\n' && s[i]!='#' && s[i]!=' ') //just case i==l-1
	lignevide=0;
      if(!lignevide && nlignes<8) {
	for(k=debutligne;k<=i;k++,j++)
	  if(s[k]!='#') s2[j]=s[k]; else s2[j]=' ';
	nlignes++;
      }
      debutligne=i+1;
      lignevide=1;
    }
  }
  if(j==0) 
    {free(s2);lastsub=NULL; return;}
  s2[j]='\0';
  lastsub=s2;
  lastsubbegin=frame;
  if(debug)
    printf("lastsub=@@%s@@\n",lastsub);
}


/* forces the capture height except if we are recording */
void force_capt_height_unless_divx(int height) {
  if(!divx_in_progress)
    video_set_capture_size(-2, height);
  else
    prev_h_capture=height;
}

int divx_exceptions (int container)
{
  int flag = 0;
  int channel_norm=0; //default
  
/* crashes on recording from freqs
 if (channels != NULL) {
	 channel_norm=channels[cur_sender]->norm;
  } 
 */
  channel_norm = cur_norm;
  switch(container)
  {
    case AVI_BUILTIN:
#ifdef HAVE_FFMPEG
      if (divx.codec == FFMPEG_MPEG1 
#if LIBAVCODEC_BUILD >= 4676
       || divx.codec == FFMPEG_MPEG2)
#else
      )
#endif
        if (divx.fps <= 8333) flag = 1;

#if LIBAVCODEC_BUILD >= 4729
      if (divx.codec == FFMPEG_H261) {
        if ((divx.width == 176 && divx.height == 144) 
         || (divx.width == 352 && divx.height == 288)) {}
        else {flag = 1;}
      }
#endif

      if (divx.codec == FFMPEG_H263) {
        if ((divx.width == 128 && divx.height == 96) 
         || (divx.width == 176 && divx.height == 144) 
         || (divx.width == 352 && divx.height == 288) 
         || (divx.width == 704 && divx.height == 576)) {}
        else {flag = 1;}
      }

#if LIBAVCODEC_BUILD >= 4707
      if (divx.codec == FFMPEG_DVVIDEO) flag = 1;
#endif
#endif

#ifdef HAVE_OGG
      if (divx.codec == THEORA) flag = 1;
      if (divx.audio_codec == VORBIS) flag = 1;
#endif

      if (flag){
      popup_mess(translate("    AVI needs these parameters    \n\n MPEG1/2 needs a video framerate > 8.33 fps\n H261 needs a video resolution = 176x144 or 352x288\n H263 needs a video resolution = 128x96 or 176x144 or 352x288 or 704x576\n\n The DVVideo Video Codec can't be used\n The Theora video codec can't be used\n The Vorbis audio codec can't be used\n\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

#ifdef HAVE_OGG
    case OGG_BUILTIN:
      if (divx.codec != THEORA) flag = 1;
      if (divx.audio_codec != VORBIS) flag = 1;
      if (flag){
      popup_mess(translate("    OGG needs these parameters    \n\n Video Codec = Theora\n Audio Codec = Vorbis\n\n"));
      return 1;}
    break;
#endif

#ifdef HAVE_FFMPEG
#if LIBAVCODEC_BUILD >= 4754
    case MPEG_FFMPEG:
      // NTSC
      if (strncmp(int_to_str (channel_norm , grabbers[grabber]->norms),"NTS",3) == 0) 
      {
        if (divx.codec == FFMPEG_MPEG1 && divx.audio_codec == FFMPEG_MP2) // VCD
        {
          if (divx.width == 352 && divx.height == 240) {}
          else {flag = 1;}
          if (divx.fps != 23980 && divx.fps != 29970) flag = 1;
          if (divx.audio_freq != 44100) flag = 1;
        }
        else if (divx.codec == FFMPEG_MPEG2 && (divx.audio_codec == FFMPEG_MP2 
#ifdef HAVE_FFMPEG_AC3
             || divx.audio_codec == FFMPEG_AC3))  // SVCD
#else
             ))
#endif
        {
          if (divx.width == 480 && divx.height == 480) {}
          else {flag = 1;}
          if (divx.fps != 23980 && divx.fps != 29970) flag = 1;
          if (divx.audio_freq != 44100 && divx.audio_freq != 48000 ) flag = 1;
        }
#ifdef HAVE_FFMPEG_MP3
        else if ((divx.codec == FFMPEG_MPEG2 || divx.codec == FFMPEG_MPEG1) && divx.audio_codec == FFMPEG_MP3)
          {if (divx.fps <= 15000)  flag = 1;}
#endif
        else {flag = 1;}
      }

      //PAL/SECAM
      else if (strncmp(int_to_str (channel_norm, grabbers[grabber]->norms),"SEC",3) == 0 ||
          strncmp(int_to_str (channel_norm, grabbers[grabber]->norms),"PAL",3) == 0) 
      {
        if (divx.codec == FFMPEG_MPEG1 && divx.audio_codec == FFMPEG_MP2) // VCD
        {
          if (divx.width == 352 && divx.height == 288) {}
          else {flag = 1;}
          if (divx.fps != 25000)  flag = 1;
          if (divx.audio_freq != 44100) flag = 1;
        }
        else if (divx.codec == FFMPEG_MPEG2 && (divx.audio_codec == FFMPEG_MP2 
#ifdef HAVE_FFMPEG_AC3
             || divx.audio_codec == FFMPEG_AC3))  // SVCD
#else
             ))
#endif
        {
          if (divx.width == 480 && divx.height == 576) {}
          else {flag = 1;}
          if (divx.fps != 25000)  flag = 1;
          if (divx.audio_freq != 44100 && divx.audio_freq != 48000) flag = 1;
        }
#ifdef HAVE_FFMPEG_MP3
        else if ((divx.codec == FFMPEG_MPEG2 || divx.codec == FFMPEG_MPEG1) && divx.audio_codec == FFMPEG_MP3)
          {if (divx.fps <= 15000)  flag = 1;}
#endif
        else {flag = 1;}
      }

      if (divx.codec != FFMPEG_MPEG1 && divx.codec != FFMPEG_MPEG2) flag = 1;

      if (divx.audio_codec != FFMPEG_MP2
#ifdef HAVE_FFMPEG_AC3
       && divx.audio_codec != FFMPEG_AC3
#endif
#ifdef HAVE_FFMPEG_MP3
       && divx.audio_codec != FFMPEG_MP3)
#else
      )
#endif
        flag = 1;

      if (flag){
      popup_mess(translate("    MPEG needs these parameters    \n    (AC3: Recordable but not playable...)\n\n VCD NTSC :\n   Video Codec = FFMpeg MPEG1\n   Audio Codec = FFMpeg MP2\n   Video resolution = 352x240\n   Video framerate = 23.98 or 29.97 fps\n   Audio Frequency = 44.1Khz\n\n SVCD NTSC:\n   Video Codec = FFMpeg MPEG2\n   Audio Codec = FFMpeg MP2 & AC3\n   Video resolution = 480x480\n   Video framerate = 23.98 or 29.97 fps\n   Audio Frequency = 44.1Khz or 48Khz\n\n VCD PAL/SECAM:\n   Video Codec = FFMpeg MPEG1\n   Audio Codec = FFMpeg MP2\n   Video resolution = 352x288\n   Video framerate = 25 fps\n   Audio Frequency = 44.1Khz\n\n SVCD PAL/SECAM:\n   Video Codec = FFMpeg MPEG2\n   Audio Codec = FFMpeg MP2 & AC3\n   Video resolution = 480x576\n   Video framerate = 25 fps\n   Audio Frequency = 44.1Khz or 48Khz\n\nOthers:\n   Video Codec = FFMpeg MPEG1 & MPEG2\n   Audio Codec = FFMpeg MP3\n   Video framerate > 15 fps\n\n You need an audio codec to use the MOV container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

    case MPEGTS_FFMPEG:

      if (divx.audio_freq <= 16000) flag = 1;

#ifdef HAVE_FFMPEG_AAC
      if (divx.audio_codec == FFMPEG_AAC) if (divx.stereo != 1) flag = 1;
#endif

      if (divx.codec != FFMPEG_MPEG4 && divx.codec != FFMPEG_MPEG1 
       && divx.codec != FFMPEG_MPEG2 
#ifdef HAVE_FFMPEG_X264
       &&  divx.codec != FFMPEG_X264)
#else
      )
#endif
        flag = 1;

      if (divx.audio_codec != FFMPEG_MP2
#ifdef HAVE_FFMPEG_AAC
      && divx.audio_codec != FFMPEG_AAC
#endif
#ifdef HAVE_FFMPEG_AC3
      && divx.audio_codec != FFMPEG_AC3
#endif
#ifdef HAVE_FFMPEG_MP3
      && divx.audio_codec != FFMPEG_MP3)
#else
      )
#endif
        flag = 1;

      if (divx.codec == FFMPEG_MPEG2 || divx.codec == FFMPEG_MPEG1)
        if (divx.fps <= 8333) flag = 1;

      if (flag){
      popup_mess(translate("    MPEG-TS needs these parameters    \n    (AAC: Recordable but not playable...)\n\n Video Codec = FFMpeg MPEG2 or MPEG1 or MPEG4 or H264\n Audio Codec = FFMpeg MP3 or MP2 or AC3 or AAC\n MPEG1/2 needs a video framerate > 8.33 fps\n AAC needs Audio Stereo\n Audio Frequency > 16Khz\n\n You need an audio codec to use the MPEG-TS container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

    case MP4_FFMPEG:

      if (divx.audio_codec == FFMPEG_MP2) if (divx.audio_freq <= 16000) flag = 1;
#ifdef HAVE_FFMPEG_AAC
      if (divx.audio_codec == FFMPEG_AAC) if (divx.audio_freq <= 16000) flag = 1;
      if (divx.audio_codec == FFMPEG_AAC) if (divx.stereo != 1) flag = 1;
#endif

#ifdef HAVE_FFMPEG_MP3
      if (divx.audio_codec == FFMPEG_MP3) if (divx.audio_freq != 44100) flag = 1;
#endif

      if (divx.codec == FFMPEG_H263) {
        if ((divx.width == 128 && divx.height == 96) 
         || (divx.width == 176 && divx.height == 144) 
         || (divx.width == 352 && divx.height == 288) 
         || (divx.width == 704 && divx.height == 576)) {}
        else {flag = 1;}
      }

      if (divx.codec == FFMPEG_MPEG2 || divx.codec == FFMPEG_MPEG1)
        if (divx.fps <= 8333) flag = 1;

#ifdef HAVE_FFMPEG_X264
      if (divx.codec == FFMPEG_X264) if (divx.fps >= 25000) flag = 1;
#endif

      if (divx.codec != FFMPEG_MPEG4 && divx.codec != FFMPEG_MPEG2
      &&  divx.codec != FFMPEG_H263 &&  divx.codec != FFMPEG_H263P 
      &&  divx.codec != FFMPEG_MSMPEG4V2 &&  divx.codec != FFMPEG_FLV1
      &&  divx.codec != FFMPEG_MJPEG &&  divx.codec != FFMPEG_MPEG1
#ifdef HAVE_FFMPEG_X264
      &&  divx.codec != FFMPEG_X264)
#else
      )
#endif
        flag = 1;

      if (divx.audio_codec != FFMPEG_MP2
#ifdef HAVE_FFMPEG_AAC
      && divx.audio_codec != FFMPEG_AAC
#endif
#ifdef HAVE_FFMPEG_MP3
      && divx.audio_codec != FFMPEG_MP3)
#else
      )
#endif
        flag = 1;

      if (flag){
      popup_mess(translate("    MP4 needs these parameters    \n    (MP2/AAC: Recordable but not playable...)\n\n Video Codec = FFMpeg MPEG4, H263, H263+, H264, FLV1, MJPEG, MPEG1,MPEG2\n               MsMPEG4v2\n Audio Codec = FFMpeg MP3 or MP2 or AAC\n\n AAC Audio Frequency > 16 Khz + Audio Stereo\n MP3 Audio Frequency = 44.1 Khz\n\n H263 needs a video resolution = 128x96 or 176x144 or 352x288 or 704x576\n MPEG1/2 needs a video framerate > 8.33 fps\n H264 needs a video framerate < 25 fps\n\n You need an audio codec to use the MP4 container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

    case ASF_FFMPEG:
      if (divx.fps <= 15000) flag = 1;
#ifdef HAVE_FFMPEG_MP3
      if (divx.audio_codec != FFMPEG_MP3) flag = 1;
#else
      flag = 1;
#endif

      if (divx.codec != FFMPEG_MSMPEG4V2) flag = 1;

      if (flag){
      popup_mess(translate("    ASF needs these parameters    \n\n Video Codec = FFMpeg MSMPEG4V2\n Audio Codec = FFMpeg MP3\n Video framerate > 15 fps\n\n You need an audio codec to use the ASF container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

    case DV_FFMPEG:
      // NTSC
      if (strncmp(int_to_str (channel_norm, grabbers[grabber]->norms),"NTS",3) == 0) 
      {
        if (divx.width == 720 && divx.height == 480) {}
        else {flag = 1;}
        sprintf(dvvideo_mode,"DVCS");
      }

      //PAL/SECAM
      if (strncmp(int_to_str (channel_norm, grabbers[grabber]->norms),"SEC",3) == 0 ||
          strncmp(int_to_str (channel_norm, grabbers[grabber]->norms),"PAL",3) == 0) 
      {
        if (divx.width == 720 && divx.height == 576) {}
        else {flag = 1;}
                sprintf(dvvideo_mode,"DVSD");
      }

      if (divx.audio_codec != FFMPEG_PCMS16LE) flag = 1;
      if (divx.codec != FFMPEG_DVVIDEO) flag = 1;
      if (divx.audio_freq != 48000) flag = 1;
      if (divx.stereo != 1) flag = 1;
      if (divx.fps <= 15000) flag = 1;

      if (flag){
      popup_mess(translate("    DV needs these parameters    \n    (Bad Sound!! To be Fixed!!)\n\n Video Codec = FFMpeg DVVIDEO\n Audio Codec = FFMpeg PCM16LE\n\n Audio Frequency = 48Khz\n Audio stereo activated\n\n Video framerate > 15 fps\n NTSC video resolution = 720x480\n PAL/SECAM video resolution = 720x576\n\n You need an audio codec to use the DV container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;


    case VOB_FFMPEG:
      // NTSC
      if (strncmp(int_to_str (channel_norm, grabbers[grabber]->norms),"NTS",3) == 0) 
      {
        if ((divx.width == 704 && divx.height == 480) 
         || (divx.width == 720 && divx.height == 480)
         || (divx.width == 352 && divx.height == 480)
         || (divx.width == 352 && divx.height == 240)) {}
        else {flag = 1;}
      }

      //PAL/SECAM
      if (strncmp(int_to_str (channel_norm, grabbers[grabber]->norms),"SEC",3) == 0 ||
          strncmp(int_to_str (channel_norm, grabbers[grabber]->norms),"PAL",3) == 0) 
      {
        if ((divx.width == 704 && divx.height == 576)
         || (divx.width == 720 && divx.height == 576)
         || (divx.width == 352 && divx.height == 576)
         || (divx.width == 352 && divx.height == 288)) {}
        else {flag = 1;}
      }

      if (divx.audio_codec != FFMPEG_MP2 
#ifdef HAVE_FFMPEG_AC3
      &&  divx.audio_codec != FFMPEG_AC3)
#else
      )
#endif
        flag = 1;

      if (divx.codec != FFMPEG_MPEG2 &&  divx.codec != FFMPEG_MPEG1) flag = 1;
      if (divx.audio_freq != 48000) flag = 1;
      if (divx.stereo != 1) flag = 1;
      if (divx.fps <= 15000) flag = 1;

      if (flag){
        popup_mess(translate("    VOB/DVD needs these parameters    \n\n Video Codec = FFMpeg MPEG2 or MPEG1\n Audio Codec = FFMpeg MP3 or AC3\n\n Audio Frequency = 48Khz\n Audio stereo activated\n\n Video framerate > 15 fps\n NTSC video resolution = 704x480, 720x480, 352x480 352x240\n PAL/SECAM video resolution = 704x576, 720x576, 352x576, 352x288\n\n You need an audio codec to use the VOB/DVD container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
        return 1;}
    break;


    case MOV_FFMPEG:
#ifdef HAVE_FFMPEG_AAC
      if (divx.audio_codec == FFMPEG_AAC) if (divx.audio_freq <= 16000) flag = 1;
      if (divx.audio_codec == FFMPEG_AAC) if (divx.stereo != 1) flag = 1;
#endif

#ifdef HAVE_FFMPEG_MP3
      if(divx.audio_codec == FFMPEG_MP3) if (divx.audio_freq != 44100) flag = 1;
#endif

      if (divx.codec == FFMPEG_H263) {
        if ((divx.width == 128 && divx.height == 96) 
         || (divx.width == 176 && divx.height == 144) 
         || (divx.width == 352 && divx.height == 288) 
         || (divx.width == 704 && divx.height == 576)) {}
        else {flag = 1;}
      }

      if (divx.codec == FFMPEG_MPEG2 || divx.codec == FFMPEG_MPEG1)
        if (divx.fps <= 8333) flag = 1;

#ifdef HAVE_FFMPEG_X264
      if (divx.codec == FFMPEG_X264) if (divx.fps >= 25000) flag = 1;
#endif

      if (divx.codec != FFMPEG_MPEG4 &&  divx.codec != FFMPEG_MSMPEG4V2 
      &&  divx.codec != FFMPEG_H263 &&  divx.codec != FFMPEG_H263P 
      &&  divx.codec != FFMPEG_MPEG2 &&  divx.codec != FFMPEG_FLV1
      &&  divx.codec != FFMPEG_MJPEG &&  divx.codec != FFMPEG_MPEG1
#ifdef HAVE_FFMPEG_X264
      &&  divx.codec != FFMPEG_X264)
#else
      )
#endif
        flag = 1;

      if (divx.audio_codec != FFMPEG_MP2
#ifdef HAVE_FFMPEG_AAC
      && divx.audio_codec != FFMPEG_AAC
#endif
#ifdef HAVE_FFMPEG_AC3
      && divx.audio_codec != FFMPEG_AC3
#endif
#ifdef HAVE_FFMPEG_MP3
      && divx.audio_codec != FFMPEG_MP3)
#else
      )
#endif
        flag = 1;

      if (flag){
      popup_mess(translate("    MOV needs these parameters    \n    (AAC/MP2/AC3: Recordable but not playable...)\n\n Video Codec = FFMpeg MPEG4, H263, H263+, H264, FLV1, MJPEGMPEG1, MPEG2\n               MsMPEG4v2\n\n Audio Codec = FFMpeg MP3 or MP2 or AAC or AC3\n\n AAC Audio Frequency > 16 Khz + Audio Stereo\n MP3 Audio Frequency = 44.1 Khz\n\n MPEG1/2 needs a video framerate > 8.33 fps\n H264 needs a video framerate < 25 fps\n H263 needs a video resolution = 128x96 or 176x144 or 352x288 or 704x576\n\n You need an audio codec to use the MOV container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

    case _3GP_FFMPEG:
#ifdef HAVE_FFMPEG_AMR_NB
      if (divx.audio_codec == FFMPEG_AMR_NB) {
        if (divx.stereo != 0) flag = 1;
        if (divx.audio_freq != 8000) flag = 1;}
#endif

#ifdef HAVE_FFMPEG_AMR_WB
      if (divx.audio_codec == FFMPEG_AMR_WB) {
        if (divx.stereo != 0) flag = 1; 
        if (divx.audio_freq != 16000) flag = 1;}
#endif

#ifdef HAVE_FFMPEG_AAC
      if (divx.audio_codec == FFMPEG_AAC) if (divx.audio_freq <= 16000) flag = 1;
      if (divx.audio_codec == FFMPEG_AAC) if (divx.stereo != 1) flag = 1;
#endif

      if (divx.codec == FFMPEG_H263) {
        if ((divx.width == 128 && divx.height == 96) 
         || (divx.width == 176 && divx.height == 144) 
         || (divx.width == 352 && divx.height == 288) 
         || (divx.width == 704 && divx.height == 576)) {}
        else {flag = 1;}
      }

#ifdef HAVE_FFMPEG_AMR_NB
      if (divx.audio_codec != FFMPEG_AMR_NB &&
#else
      if (1 &&
#endif
#ifdef HAVE_FFMPEG_AMR_WB
          divx.audio_codec != FFMPEG_AMR_WB &&
#else
          1 &&
#endif
#ifdef HAVE_FFMPEG_AAC
          divx.audio_codec != FFMPEG_AAC &&
#else
          1 &&
#endif
#ifdef HAVE_FFMPEG_MP3
          divx.audio_codec != FFMPEG_MP3)
#else
          1)
#endif
      flag = 1; // No Audio Codec => No 3GP possible

#ifdef HAVE_FFMPEG_MP3
      if (divx.audio_codec == FFMPEG_MP3) if (divx.audio_freq != 44100) flag = 1;
#endif

      if (divx.codec != FFMPEG_H263 &&  divx.codec != FFMPEG_MPEG4) flag = 1;

      if (flag){
      popup_mess(translate("    3GP needs these parameters    \n\n Audio Codec AMR-NB:\n   Audio stereo disable\n   Audio Frequency = 8 Khz\n\n Audio Codec AMR-WB:\n   Audio stereo disable\n   Audio Frequency = 16 Khz\n\n Audio Codec AAC:\n   Audio Frequency > 16 Khz + Audio Stereo\n\n Audio Codec MP3:\n   Audio Frequency = 44.1 Khz\n\n H263 needs a video resolution = 128x96 or 176x144 or 352x288 or 704x576\n\n You need an audio codec to use the 3GP container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

    case _3G2_FFMPEG:
#ifdef HAVE_FFMPEG_AAC
      if (divx.audio_codec == FFMPEG_AAC) if (divx.audio_freq <= 16000) flag = 1;
      if (divx.audio_codec == FFMPEG_AAC) if (divx.stereo != 1) flag = 1;
#endif

#ifdef HAVE_FFMPEG_MP3
      if (divx.audio_codec == FFMPEG_MP3) if (divx.audio_freq != 44100) flag = 1;
#endif

#ifdef HAVE_FFMPEG_AAC
      if (divx.audio_codec != FFMPEG_AAC &&
#else
      if (1 &&
#endif
#ifdef HAVE_FFMPEG_MP3
          divx.audio_codec != FFMPEG_MP3)
#else
          1)
#endif
      flag = 1; // No Audio Codec => No 3G2 possible

      if (divx.codec != FFMPEG_MPEG4) flag = 1;

      if (flag){
      popup_mess(translate("    3G2 needs these parameters    \n    (AAC: Recordable but not playable...)\n\n Audio Codec AAC:\n   Audio Frequency > 16 Khz + Audio Stereo\n\n Audio Codec MP3:\n   Audio Frequency = 44.1 Khz\n\n You need an audio codec to use the 3G2 container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

    case SWF_FFMPEG:
      if (divx.audio_freq != 11025 && divx.audio_freq != 22050 && divx.audio_freq != 44100) flag = 1;

#ifdef HAVE_FFMPEG_MP3
      if (divx.audio_codec != FFMPEG_MP3) flag = 1;
#else
      flag =1; // if no MP3 Codec => not possible to use SWF
#endif

      if (divx.codec != FFMPEG_MJPEG && divx.codec != FFMPEG_FLV1) flag = 1;

      if (flag){
      popup_mess(translate("    SWF needs these parameters    \n\n Video Codec = FFMpeg Flash Video or Mjpeg\n Audio Codec = FFMpeg MP3\n\n Audio Frequency = 11, 22 or 44.1Khz\n Audio stereo activated\n\n You need an audio codec to use the SWF container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;

    case RM_FFMPEG:
#ifdef HAVE_FFMPEG_AC3
      if (divx.audio_codec != FFMPEG_AC3) flag = 1;
#else
      flag =1; // if no AC3 Codec => not possible to use RM
#endif

      if (divx.codec != FFMPEG_RV20 && divx.codec != FFMPEG_RV10) flag = 1;

      if (flag){
      popup_mess(translate("    RM needs these parameters    \n    (Recordable but not playable...)\n\n Video Codec = FFMpeg RealVideo 1.0 & 2.0\n Audio Codec = FFMpeg AC3\n\n You need an audio codec to use the RM container\n For more informations about the FFMpeg compilation\n read the README.ffmpeg file that can be find into the XdTV tarball\n\n"));
      return 1;}
    break;
#endif
#endif
  }
  return 0;
}
