/*****************************************************************************
 * divx.h: DivX recording functions and other definitions
 *****************************************************************************
 * $Id: divx.h,v 1.56 2006/02/18 22:12:10 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
#ifndef _DIVX_H_
#define _DIVX_H_

#include "colorspace.h"

#ifdef HAVE_DIVX4LINUX
# include <encore2.h>
#endif

#include "container.h"

#ifdef HAVE_FFMPEG
# include "avcodec.h"
#if LIBAVCODEC_BUILD >= 4754
# include "avformat.h"
#endif
#endif

#ifdef HAVE_XVID
# include <xvid.h>
#ifdef XVID_API
#define HAVE_XVID10 /* XVID 1.0 */
#else
#define HAVE_XVID09 /* XVID 0.9 */
#endif
#endif

void divx_init(int width, int height);
int divx_exceptions(int container);
void divx_encode(unsigned char *src_frame);
void divx_stop(void);
int is_divx_in_progress(void);
int is_http_in_progress(void);
int divx_display_frames(void);
inline void write_audio_to_avi(short int *buffer, int size);
void write_sub(char *s);
void divx_audio_start(void);
void divx_audio_stop(void);
void divx_restart(void);
char* get_filename(void);

char lastfilename[256];
char *pplayer;
unsigned long dropped_audio_buffers, dropped_video_frames,video_frames, audio_buffers;

enum {
  RECORD_STOP = 0,
  RECORD_START,
  RECORD_PREVIEW,  
  RECORD_SCHEDULE,    
  RECORD_VIDEO_PARAM,
  RECORD_AUDIO_PARAM,
  STREAM_SERVER_PARAM
};

enum {
#ifdef HAVE_DIVX4LINUX
DIVX4LINUX,
#endif
#ifdef HAVE_FFMPEG
FFMPEG_MPEG4,
#if LIBAVCODEC_BUILD >= 4676
FFMPEG_MPEG2,
#endif
FFMPEG_MPEG1,
FFMPEG_MJPEG,
#if LIBAVCODEC_BUILD >= 4670
FFMPEG_LJPEG,
FFMPEG_FLV1,
#endif
#if LIBAVCODEC_BUILD >= 4735
FFMPEG_RV20,
#endif
FFMPEG_RV10,
#ifdef HAVE_FFMPEG_X264
#if LIBAVCODEC_BUILD >= 4748
FFMPEG_X264,
#endif
#endif
FFMPEG_H263P,
FFMPEG_H263,
#if LIBAVCODEC_BUILD >= 4729
FFMPEG_H261,
#endif
FFMPEG_MSMPEG4V3,
FFMPEG_MSMPEG4V2,
/* Already implemented into the 0.4.6 version 
but only work correctly with the 0.4.7 version */
#if LIBAVCODEC_BUILD >= 4675
FFMPEG_WMV2,
#endif
FFMPEG_WMV1,
#if LIBAVCODEC_BUILD >= 4677
FFMPEG_ASV2,
#endif
#if LIBAVCODEC_BUILD >= 4668
FFMPEG_ASV1,
#endif
/* appeared with the 4719 version
but don't work correctly actually */
#if LIBAVCODEC_BUILD >= 16777215
FFMPEG_SVQ1,
#endif
/* appeared with the 4669 version
but only work correctly with the 4755 version */
#if LIBAVCODEC_BUILD >= 4755
FFMPEG_FFV1,
#endif
#if LIBAVCODEC_BUILD >= 4719
FFMPEG_SNOW,
#endif
#ifdef HAVE_FFMPEG_XVID
#if LIBAVCODEC_BUILD >= 4719
FFMPEG_XVID,
#endif
#endif
#if LIBAVCODEC_BUILD >= 4734
FFMPEG_FFVHUFF,
#endif
FFMPEG_HUFFYUV,
#if LIBAVCODEC_BUILD >= 4707
FFMPEG_DVVIDEO,
#endif
#endif // HAVE_FFMPEG
#ifdef HAVE_XVID
XVID,
#endif
#ifdef HAVE_OGG
THEORA,
#endif
UNCOMPRESSED
};

enum {
#ifdef HAVE_LAME
  LAME,
#endif
#ifdef HAVE_OGG
  VORBIS,
#endif
#ifdef HAVE_FFMPEG
  FFMPEG_MP2,
#ifdef HAVE_FFMPEG_MP3
  FFMPEG_MP3,
#endif
#ifdef HAVE_FFMPEG_AC3
  FFMPEG_AC3,
#endif
#ifdef HAVE_FFMPEG_AAC
  FFMPEG_AAC,
#endif
  FFMPEG_DVAUDIO,
  FFMPEG_PCMS16LE,
#ifdef HAVE_FFMPEG_AMR_NB
  FFMPEG_AMR_NB,
#endif
#ifdef HAVE_FFMPEG_AMR_WB
  FFMPEG_AMR_WB,
#endif
#endif
  AUDIO_UNCOMPRESSED
};

typedef struct {
  int container;
  int codec;
  int width;
  int height;
  long bitrate;
  int quality;
  int min_quantizer;
  int max_quantizer;
  int audio_freq;
  int audio_fmt;
  int audio_buffer_size;
  int audio_fragments;
  int audio_sizefragment;
  int stereo;
  int audio_codec;
  int mp3_bitrate;
  int mp3_quality;
  int mp3_vbr;
  int mp3_vbr_quality;
  int fps;
  double maxgap;
  double delay;
  int display_frames;
  int sub;
  int chg;
  char *filename;
  int stream;
  int http_port;
  int maxsize;
} divx_t;

#ifdef HAVE_FFMPEG
typedef struct {
  int v4mv;
  int naq;
  int gray;
  int gmc;
  int qpel;
  int ildct;
  int keyint;
  int vmax_b_frames;
  int vlelim;
  int vcelim;
  float lumi_mask;
  float dark_mask;
  float tcplx_mask;
  float scplx_mask;
#if LIBAVCODEC_BUILD >= 4647
  int dia;
  int cmp;
  int subcmp;
#endif
#if LIBAVCODEC_BUILD >= 4648
  int trell;
#endif
#if LIBAVCODEC_BUILD >= 4649
  int last_pred;
#endif
#if LIBAVCODEC_BUILD >= 4651
  int predia;
  int precmp;
#endif
#if LIBAVCODEC_BUILD >= 4661
  int umv;
  int aic;
#endif
#if LIBAVCODEC_BUILD >= 4673
  int mbd;
#endif
#if LIBAVCODEC_BUILD >= 4681
  int cbp;
#endif
#if LIBAVCODEC_BUILD >= 4694
  int obmc;
  int ss;
  int aiv;
  int loop;
#endif
#if LIBAVCODEC_BUILD >= 4698
  int ilme;
#endif
} ffmpeg_t;

extern int xdtv_to_ffmpeg(int x);
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
typedef struct {
  int vhq;
#if XVID_API >= XVID_MAKE_API(4,1)
  int bvhq;
#endif
  int chromame;
  int chroma_opt;
  int hqacpred;
  int trell;
  int cartoon;
  int gray;
  int interlacing;
  int turbo;
  int qpel;
  int gmc;
  int keyint;
  int frame_drop_ratio;
  int vmax_b_frames;
  int bquant_ratio;
  int bquant_offset;
  int packed;
  int closed_gop;
  int bframe_threshold;
  int reaction_delay_factor;
  int averaging_period;
  int buffer;
  int max_quant[3];
  int min_quant[3];
} xvid_t;
#endif
#endif

#define WAVE_FORMAT_MPEGLAYER3 (0x0055)
extern struct STRTAB fourccs_raw[];
extern struct STRTAB container_list[];
extern struct STRTAB codec_list[];
extern struct STRTAB audio_codec_list[];
struct STRTAB *get_quality_list(int codec);

#endif // _DIVX_H_
