/*****************************************************************************
 * divx_advanced_ui.c: GUI for advanced DivX options (ffmpeg options etc...)
 *****************************************************************************
 * $Id: divx_advanced_ui.c,v 1.14 2005/12/14 21:49:17 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2004 Pingus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>

#include "config.h"

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Scrollbar.h>

#include "xdtv.h"
#include "grab.h"
#include "x11.h"
#include "divx.h"
#include "toolbox.h"
#include "divx_advanced_ui.h"
#include "divx_ui.h"

#ifdef HAVE_XPM
#include <X11/xpm.h>
#include "xdtv.xpm"
#endif

// internationalization
#include "translation.h"
#include "pixmaps_loader.h"

/*-------------------------------------------------------------------------*/
extern char* translate(char* string);
extern void translate_struct(struct STRTAB *to_translate);
extern void save_cb (Widget widget, XtPointer clientdata, XtPointer call_data);
extern void DivXAction (Widget, XEvent *, String *, Cardinal *);
extern void action_cb (Widget widget, XtPointer clientdata, XtPointer call_data);
extern Widget app_shell;
extern Display *dpy;
extern Atom wm_protocols[2];
extern int debug;
extern divx_t divx;

static float modulo = 1.0;

// for Pixmaps: must be always defined
Widget ffmpeg_paned;
Widget c_ffmpeg_me, c_ffmpeg_masking, c_ffmpeg_otherset;
Widget c_ffmpeg_quantizers, c_ffmpeg_videoopen, c_ffmpeg_quit;
Widget xvid_paned;
Widget c_xvid_me, c_xvid_quantizers;
Widget c_xvid_profset, c_xvid_otherset;
Widget c_xvid_videoopen, c_xvid_quit;

#ifdef HAVE_FFMPEG
extern ffmpeg_t ffmpeg;

Widget c_ffmpeg_vmax_b_frames;
static Widget ffmpeg_shell, c_ffmpeg_modulo;
static Widget c_ffmpeg_v4mv, c_ffmpeg_naq, c_ffmpeg_gray, c_ffmpeg_gmc;
static Widget c_ffmpeg_qpel, c_ffmpeg_ildct;
static Widget l_ffmpeg_keyint, s_ffmpeg_keyint;
static Widget l_ffmpeg_vlelim, s_ffmpeg_vlelim;
static Widget l_ffmpeg_vcelim, s_ffmpeg_vcelim;
static Widget l_ffmpeg_lumi_mask, s_ffmpeg_lumi_mask;
static Widget l_ffmpeg_dark_mask, s_ffmpeg_dark_mask;
static Widget l_ffmpeg_tcplx_mask, s_ffmpeg_tcplx_mask;
static Widget l_ffmpeg_scplx_mask, s_ffmpeg_scplx_mask;
#if LIBAVCODEC_BUILD >= 4647
Widget c_ffmpeg_cmp, c_ffmpeg_subcmp;
static Widget c_ffmpeg_dia;
#endif
#if LIBAVCODEC_BUILD >= 4648
static Widget c_ffmpeg_trell;
#endif
#if LIBAVCODEC_BUILD >= 4649
static Widget c_ffmpeg_last_pred;
#endif
#if LIBAVCODEC_BUILD >= 4651
static Widget c_ffmpeg_predia, c_ffmpeg_precmp;
#endif
#if LIBAVCODEC_BUILD >= 4673
static Widget c_ffmpeg_mbd;
#endif
#if LIBAVCODEC_BUILD >= 4681
static Widget c_ffmpeg_cbp;
#endif
#if LIBAVCODEC_BUILD >= 4698
static Widget c_ffmpeg_ilme;
#endif
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
extern xvid_t xvid;
Widget c_xvid_chromame, c_xvid_hqacpred, c_xvid_trell;

#if XVID_API >= XVID_MAKE_API(4,1)
static Widget c_xvid_bvhq;
#endif

static Widget xvid_shell, c_xvid_modulo;
static Widget c_xvid_vhq, c_xvid_cartoon;
static Widget c_xvid_chroma_opt, c_xvid_turbo;
static Widget c_xvid_gray, c_xvid_interlacing;
static Widget c_xvid_qpel, c_xvid_gmc, c_xvid_packed;
static Widget c_xvid_vmax_b_frames, c_xvid_closed_gop;
static Widget l_xvid_keyint, s_xvid_keyint;
static Widget l_xvidiminquantizer, s_xvidiminquantizer;
static Widget l_xvidimaxquantizer, s_xvidimaxquantizer;
static Widget l_xvidpminquantizer, s_xvidpminquantizer;
static Widget l_xvidpmaxquantizer, s_xvidpmaxquantizer;
static Widget l_xvidbminquantizer, s_xvidbminquantizer;
static Widget l_xvidbmaxquantizer, s_xvidbmaxquantizer;
static Widget l_xvid_qratio, s_xvid_qratio;
static Widget l_xvid_qoffset, s_xvid_qoffset;
static Widget l_xvid_bfthreshold, s_xvid_bfthreshold;
static Widget l_xvid_fdratio, s_xvid_fdratio;

#endif
#endif

enum {
#ifdef HAVE_FFMPEG
  FFMPEG_V4MV,
  FFMPEG_NAQ,
  FFMPEG_GRAY,
  FFMPEG_GMC,
  FFMPEG_QPEL,
  FFMPEG_ILDCT,
  FFMPEG_VMAX_B_FRAMES,
#if LIBAVCODEC_BUILD >= 4647
  FFMPEG_DIA,
  FFMPEG_CMP,
  FFMPEG_SUBCMP,
#endif
#if LIBAVCODEC_BUILD >= 4648
  FFMPEG_TRELL,
#endif
#if LIBAVCODEC_BUILD >= 4649
  FFMPEG_LAST_PRED,
#endif
#if LIBAVCODEC_BUILD >= 4651
  FFMPEG_PREDIA,
  FFMPEG_PRECMP,
#endif
#if LIBAVCODEC_BUILD >= 4673
  FFMPEG_MBD,
#endif
#if LIBAVCODEC_BUILD >= 4681
  FFMPEG_CBP,
#endif
#if LIBAVCODEC_BUILD >= 4698
  FFMPEG_ILME,
#endif
#endif
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
  XVID_VHQ,
#if XVID_API >= XVID_MAKE_API(4,1)
  XVID_BVHQ,
#endif
  XVID_CHROMAME,
  XVID_CHROMA_OPT,
  XVID_HQACPRED,
  XVID_V4MV,
  XVID_TRELL,
  XVID_CARTOON,
  XVID_GRAY,
  XVID_INTERLACING,
  XVID_TURBO,
  XVID_QPEL,
  XVID_GMC,
  XVID_VMAX_B_FRAMES,
  XVID_PACKED,
  XVID_CLOSED_GOP,
#endif
#endif
  DIVXA_MODULO
};

static struct STRTAB on_off_list[] = {
  {1, "On"},
  {0, "Off"},
  {-1, NULL}
};

static struct STRTAB modulo_list[] = {
  {1,"1"},
  {5,"5"},
  {10,"10"},
  {25,"25"},
  {-1, NULL}
};

#ifdef HAVE_FFMPEG
static struct STRTAB ffmpeg_mbd_list[] = {
  {0,"0 - MB comparison"},
  {1,"1 - Fewest bits (vhq)"},
  {2,"2 - Rate distortion"},
  {-1, NULL}
};

static struct STRTAB ffmpeg_otherquality_list[] = {
  {0,"0"},
  {1,"1"},
  {2,"2"},
  {3,"3"},
  {-1, NULL}
};

#if LIBAVCODEC_BUILD >= 4649
static struct STRTAB ffmpeg_last_pred_list[] = {
  {0,"0"},
  {1,"1"},
  {2,"2"},
  {3,"3"},
  {-1, NULL}
};
#endif

#if LIBAVCODEC_BUILD >= 4647
static struct STRTAB ffmpeg_cmp_list[] = {
  {0,"0"},
  {2,"2"},
  {3,"3"},
//  {6,"6"}, too hard for several codecs!! segfault...
  {-1, NULL}
};

static struct STRTAB ffmpeg_dia_list[] = {
  {0,"-1"},  
  {1,"0"},
  {2,"1"},
  {3,"2"},
  {4,"3"},
  {5,"4"},
  {-1, NULL}
};
#endif
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
static struct STRTAB xvid_vhq_list[] = {
  {0,"0 - Disabled"},
  {1,"1 - Mode Selection"},
  {2,"2 - Limited Search"},
  {3,"3 - Medium Search"},
  {4,"4 - Wide Search"},
  {-1, NULL}
};

static struct STRTAB xvid_otherquality_list[] = {
  {0,"0"},
  {1,"1"},
  {2,"2"},
  {3,"3"},
  {4,"4"},
  {-1, NULL}
};

#endif
#endif
/*--------------------------------------------------------------------------*/

struct CALL_ACTION call_videoparams = { 0, "DivX", {NULL} };

/*--------------------------------------------------------------------------*/

#ifdef HAVE_FFMPEG
#define FFMPEG_KEYINT_RANGE             1000.0
#define FFMPEG_VLELIM_RANGE              500.0
#define FFMPEG_VCELIM_RANGE              500.0
#define FFMPEG_LUMI_MASK_RANGE           100.0
#define FFMPEG_DARK_MASK_RANGE           100.0
#define FFMPEG_TCPLX_MASK_RANGE          100.0
#define FFMPEG_SCPLX_MASK_RANGE          100.0
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
#define XVID_KEYINT_RANGE               500.0
#define XVID_MINQUANTIZER_RANGE          31.0
#define XVID_MAXQUANTIZER_RANGE          31.0
#define XVID_QRATIO_RANGE              1000.0
#define XVID_QOFFSET_RANGE             1000.0
#define XVID_BFTHRESHOLD_RANGE          255.0
#define XVID_FDRATIO_RANGE              100.0
#endif
#endif

/*--------------------------------------------------------------------------*/

void update_divxa_commandwidget(Widget widget,int value, char *text)
{
  char label[64];
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, text), value);
  if (widget)
    XtVaSetValues (widget, XtNlabel, label, NULL);
}

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
static void
close_xvid_win (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  XtCallActionProc (xvid_shell, "XviD", NULL, NULL, 0);
}

void
XviDAction (Widget widget, XEvent * event,
              String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;
  
  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (xvid_shell), XtWindow (xvid_shell), NULL,
                       0);
          return;
        }
    }

  if (xvid_shell == NULL)
    {
      fprintf(stderr, "DivxXviDAction: ooops: xvid_shell is NULL\n");
      return;
    }

  if (mapped)
    {
      XtPopdown (xvid_shell);
      mapped = 0;
    }
  else
    {
      XtPopup (xvid_shell, XtGrabNone);
      mapped = 1;
      if (first)
        {
          XSetWMProtocols (XtDisplay (xvid_shell), XtWindow (xvid_shell),
                           wm_protocols, 2);
          first = 0;
        }
    }
}

#endif
#endif

#ifdef HAVE_FFMPEG
static void
close_ffmpeg_win (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  XtCallActionProc (ffmpeg_shell, "FFMpeg", NULL, NULL, 0);
}

void
FFMpegAction (Widget widget, XEvent * event,
              String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;
  
  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (ffmpeg_shell), XtWindow (ffmpeg_shell), NULL,
                       0);
          return;
        }
    }

  if (ffmpeg_shell == NULL)
    {
      fprintf(stderr, "FFMpegAction: ooops: ffmpeg_shell is NULL\n");
      return;
    }

  if (mapped)
    {
      XtPopdown (ffmpeg_shell);
      mapped = 0;
    }
  else
    {
      XtPopup (ffmpeg_shell, XtGrabNone);
      mapped = 1;
      if (first)
        {
          XSetWMProtocols (XtDisplay (ffmpeg_shell), XtWindow (ffmpeg_shell),
                           wm_protocols, 2);
          first = 0;
        }
    }
}
#endif

static void
divx_advanced_menu_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int cd = (int) clientdata;
  int j;
  char label[64];

  switch (cd)
    {
    case DIVXA_MODULO:
      if (-1 != (j = popup_menu (widget, translate("Modulo for ALL"), modulo_list)))
        {
          modulo = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Modulo for ALL")), (int)modulo);
#ifdef HAVE_FFMPEG
          XtVaSetValues (c_ffmpeg_modulo, XtNlabel, label, NULL);
#endif
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
          XtVaSetValues (c_xvid_modulo, XtNlabel, label, NULL);
#endif
#endif
        }
      break;
#ifdef HAVE_FFMPEG
    case FFMPEG_V4MV:
      if (-1 != (j = popup_menu (widget, translate("V4mv : 4 Motion Vector"), on_off_list)))
        {
          ffmpeg.v4mv = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("V4mv : 4 Motion Vector")),
                   int_to_str (ffmpeg.v4mv, on_off_list));
          XtVaSetValues (c_ffmpeg_v4mv, XtNlabel, label, NULL);
        }
      break;
      
    case FFMPEG_NAQ:
      if (-1 != (j = popup_menu (widget, translate("Normalize Quantization"), on_off_list)))
        {
          ffmpeg.naq = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Normalize Quantization")),
                   int_to_str (ffmpeg.naq, on_off_list));
          XtVaSetValues (c_ffmpeg_naq, XtNlabel, label, NULL);
        }
      break;
      
    case FFMPEG_GRAY:
      if (-1 != (j = popup_menu (widget, translate("Grayscale Encoding"), on_off_list)))
        {
          ffmpeg.gray = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Grayscale Encoding")),
                   int_to_str (ffmpeg.gray, on_off_list));
          XtVaSetValues (c_ffmpeg_gray, XtNlabel, label, NULL);
        }
      break;
      
    case FFMPEG_GMC:
      if (-1 != (j = popup_menu (widget, translate("Global Motion Compensation"), on_off_list)))
        {
          ffmpeg.gmc = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Global Motion Compensation")),
                   int_to_str (ffmpeg.gmc, on_off_list));
          XtVaSetValues (c_ffmpeg_gmc, XtNlabel, label, NULL);
        }
      break;
      
    case FFMPEG_QPEL:
      if (-1 != (j = popup_menu (widget, translate("Quarter PixEL"), on_off_list)))
        {
          ffmpeg.qpel = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Quarter PixEL")),
                   int_to_str (ffmpeg.qpel, on_off_list));
          XtVaSetValues (c_ffmpeg_qpel, XtNlabel, label, NULL);
        }
      break;
      
    case FFMPEG_ILDCT:
      if (-1 != (j = popup_menu (widget, translate("InterLaced DCT Encoding"), on_off_list)))
        {
          ffmpeg.ildct = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("InterLaced DCT Encoding")),
                   int_to_str (ffmpeg.ildct, on_off_list));
          XtVaSetValues (c_ffmpeg_ildct, XtNlabel, label, NULL);
        }
      break;
#if LIBAVCODEC_BUILD >= 4698
    case FFMPEG_ILME:
      if (-1 != (j = popup_menu (widget, translate("InterLaced Motion Estimation"), on_off_list)))
        {
          ffmpeg.ilme = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("InterLaced Motion Estimation")),
                   int_to_str (ffmpeg.ilme, on_off_list));
          XtVaSetValues (c_ffmpeg_ilme, XtNlabel, label, NULL);
        }
      break;
#endif
#if LIBAVCODEC_BUILD >= 4648
    case FFMPEG_TRELL:
      if (-1 != (j = popup_menu (widget, translate("Trellis Quantization"), on_off_list)))
        {
#if LIBAVCODEC_BUILD >= 4681
          if (ffmpeg.cbp) {
            sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                     translate("Already managed!"));
            ffmpeg.trell = 0;
            XtVaSetValues (c_ffmpeg_trell, XtNlabel, label, NULL);}
          else
#endif
           {ffmpeg.trell = j;
            sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                     int_to_str (ffmpeg.trell, on_off_list));
            XtVaSetValues (c_ffmpeg_trell, XtNlabel, label, NULL);}
        }
      break;
#endif
#if LIBAVCODEC_BUILD >= 4681
    case FFMPEG_CBP:
      if (-1 != (j = popup_menu (widget, translate("Coded Block Pattern"), on_off_list)))
        {
          ffmpeg.cbp = j;

          if (ffmpeg.cbp) {
            sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                     translate("Already managed!"));
            ffmpeg.trell = 0;
            XtVaSetValues (c_ffmpeg_trell, XtNlabel, label, NULL);}
          else {
            sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                     int_to_str (ffmpeg.trell, on_off_list));
            XtVaSetValues (c_ffmpeg_trell, XtNlabel, label, NULL);}

          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Coded Block Pattern")),
                   int_to_str (ffmpeg.cbp, on_off_list));
          XtVaSetValues (c_ffmpeg_cbp, XtNlabel, label, NULL);
        }
      break;
#endif
    case FFMPEG_VMAX_B_FRAMES:
      if (-1 != (j = popup_menu (widget, translate("MAX B-Frames"), ffmpeg_otherquality_list)))
        {
          ffmpeg.vmax_b_frames = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("MAX B-Frames")), ffmpeg.vmax_b_frames);
          XtVaSetValues (c_ffmpeg_vmax_b_frames, XtNlabel, label, NULL);
        }
      break;

#if LIBAVCODEC_BUILD >= 4647
    case FFMPEG_DIA:
      if (-1 != (j = popup_menu (widget, translate("DIAmond"), ffmpeg_dia_list)))
        {
          ffmpeg.dia = j-1;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("DIAmond")), ffmpeg.dia);
          XtVaSetValues (c_ffmpeg_dia, XtNlabel, label, NULL);
        }
      break;

    case FFMPEG_CMP:
      if (-1 != (j = popup_menu (widget, translate("Full pel ME CoMParison"), ffmpeg_cmp_list)))
        {
          ffmpeg.cmp = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Full pel ME CoMParison")), ffmpeg.cmp);
          XtVaSetValues (c_ffmpeg_cmp, XtNlabel, label, NULL);
        }
      break;
      
    case FFMPEG_SUBCMP:
      if (-1 != (j = popup_menu (widget, translate("SUB pel ME CoMParison"), ffmpeg_cmp_list)))
        {
          ffmpeg.subcmp = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("SUB pel ME CoMParison")), ffmpeg.subcmp);
          XtVaSetValues (c_ffmpeg_subcmp, XtNlabel, label, NULL);
        }
      break;
#endif

#if LIBAVCODEC_BUILD >= 4649
    case FFMPEG_LAST_PRED:
      if (-1 != (j = popup_menu (widget, translate("LAST frame motion PREdictors"), ffmpeg_last_pred_list)))
        {
          ffmpeg.last_pred = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("LAST frame motion PREdictors")), ffmpeg.last_pred);
          XtVaSetValues (c_ffmpeg_last_pred, XtNlabel, label, NULL);
        }
      break;
#endif

#if LIBAVCODEC_BUILD >= 4651
    case FFMPEG_PREDIA:
      if (-1 != (j = popup_menu (widget, translate("PRE-pass DIAmond"), ffmpeg_dia_list)))
        {
          ffmpeg.predia = j-1;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("PRE-pass DIAmond")), ffmpeg.predia);
          XtVaSetValues (c_ffmpeg_predia, XtNlabel, label, NULL);
        }
      break;
      
    case FFMPEG_PRECMP:
      if (-1 != (j = popup_menu (widget, translate("ME PRE-pass CoMParison"), ffmpeg_cmp_list)))
        {
          ffmpeg.precmp = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("ME PRE-pass CoMParison")), ffmpeg.precmp);
          XtVaSetValues (c_ffmpeg_precmp, XtNlabel, label, NULL);
        }
      break;
#endif

#if LIBAVCODEC_BUILD >= 4673
    case FFMPEG_MBD:
      if (-1 != (j = popup_menu (widget, translate("MacroBlock Decision"), ffmpeg_mbd_list)))
        {
          ffmpeg.mbd = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("MacroBlock Decision")), ffmpeg.mbd);
          XtVaSetValues (c_ffmpeg_mbd, XtNlabel, label, NULL);
        }
      break;
#endif
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
    case XVID_VHQ:
      if (-1 != (j = popup_menu (widget, translate("VHQ Mode"), xvid_vhq_list)))
        {
          xvid.vhq = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("VHQ Mode")), xvid.vhq);
          XtVaSetValues (c_xvid_vhq, XtNlabel, label, NULL);
        }
      break;
#if XVID_API >= XVID_MAKE_API(4,1)
    case XVID_BVHQ:
      if (-1 != (j = popup_menu (widget, translate("BVHQ Mode"), on_off_list)))
        {
          xvid.bvhq = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("BVHQ Mode")),
                   int_to_str (xvid.bvhq, on_off_list));
          XtVaSetValues (c_xvid_bvhq, XtNlabel, label, NULL);
        }
      break;
#endif
    case XVID_CHROMAME:
      if (-1 != (j = popup_menu (widget, translate("Chroma Motion Estimation"), on_off_list)))
        {
          if (divx.quality >= 4)
            sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Chroma Motion Estimation")),
                     translate("Already managed!"));
          else {
          xvid.chromame = j;
            sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Chroma Motion Estimation")),
                     int_to_str (xvid.chromame, on_off_list));}
          XtVaSetValues (c_xvid_chromame, XtNlabel, label, NULL);
        }
      break;

    case XVID_CHROMA_OPT:
      if (-1 != (j = popup_menu (widget, translate("Chroma Optimizer"), on_off_list)))
        {
          xvid.chroma_opt = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Chroma Optimizer")),
                   int_to_str (xvid.chroma_opt, on_off_list));
          XtVaSetValues (c_xvid_chroma_opt, XtNlabel, label, NULL);
        }
      break;

    case XVID_HQACPRED:
      if (-1 != (j = popup_menu (widget, translate("HQ-AC Prediction"), on_off_list)))
        {
          if (divx.quality >= 6)
            sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("HQ-AC Prediction")),
                     translate("Already managed!"));
          else {
          xvid.hqacpred = j;
            sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("HQ-AC Prediction")),
                     int_to_str (xvid.hqacpred, on_off_list));}
          XtVaSetValues (c_xvid_hqacpred, XtNlabel, label, NULL);
        }
      break;
      
    case XVID_TRELL:
      if (-1 != (j = popup_menu (widget, translate("Trellis Quantization"), on_off_list)))
        {
          if (divx.quality >= 5)
            sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                     translate("Already managed!"));
          else {
          xvid.trell = j;
            sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                     int_to_str (xvid.trell, on_off_list));}
          XtVaSetValues (c_xvid_trell, XtNlabel, label, NULL);
        }
      break;

    case XVID_CARTOON:
      if (-1 != (j = popup_menu (widget, translate("Cartoon Mode"), on_off_list)))
        {
          xvid.cartoon = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Cartoon Mode")),
                   int_to_str (xvid.cartoon, on_off_list));
          XtVaSetValues (c_xvid_cartoon, XtNlabel, label, NULL);
        }
      break;

    case XVID_GRAY:
      if (-1 != (j = popup_menu (widget, translate("Grayscale Encoding"), on_off_list)))
        {
          xvid.gray = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Grayscale Encoding")),
                   int_to_str (xvid.gray, on_off_list));
          XtVaSetValues (c_xvid_gray, XtNlabel, label, NULL);
        }
      break;

    case XVID_INTERLACING:
      if (-1 != (j = popup_menu (widget, translate("Interlaced Encoding"), on_off_list)))
        {
          xvid.interlacing = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Interlaced Encoding")),
                   int_to_str (xvid.interlacing, on_off_list));
          XtVaSetValues (c_xvid_interlacing, XtNlabel, label, NULL);
        }
      break;      

    case XVID_TURBO:
      if (-1 != (j = popup_menu (widget, translate("Turbo Mode"), on_off_list)))
        {
          xvid.turbo = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Turbo Mode")),
                   int_to_str (xvid.turbo, on_off_list));
          XtVaSetValues (c_xvid_turbo, XtNlabel, label, NULL);
        }
      break;      

    case XVID_QPEL:
      if (-1 != (j = popup_menu (widget, translate("Quarter PixEL"), on_off_list)))
        {
          xvid.qpel = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Quarter PixEL")),
                   int_to_str (xvid.qpel, on_off_list));
          XtVaSetValues (c_xvid_qpel, XtNlabel, label, NULL);
        }
      break;      

    case XVID_GMC:
      if (-1 != (j = popup_menu (widget, translate("Global Motion Compensation"), on_off_list)))
        {
          xvid.gmc = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Global Motion Compensation")),
                   int_to_str (xvid.gmc, on_off_list));
          XtVaSetValues (c_xvid_gmc, XtNlabel, label, NULL);
        }
      break;      

    case XVID_PACKED:
      if (-1 != (j = popup_menu (widget, translate("Packed bitstream"), on_off_list)))
        {
          xvid.packed = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Packed bitstream")),
                   int_to_str (xvid.packed, on_off_list));
          XtVaSetValues (c_xvid_packed, XtNlabel, label, NULL);
        }
      break;      

    case XVID_CLOSED_GOP:
      if (-1 != (j = popup_menu (widget, translate("Closed GOP"), on_off_list)))
        {
          xvid.closed_gop = j;
          sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Closed GOP")),
                   int_to_str (xvid.closed_gop, on_off_list));
          XtVaSetValues (c_xvid_closed_gop, XtNlabel, label, NULL);
        }
      break;

    case XVID_VMAX_B_FRAMES:
      if (-1 != (j = popup_menu (widget, translate("MAX B-Frames"), xvid_otherquality_list)))
        {
          xvid.vmax_b_frames = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("MAX B-Frames")), xvid.vmax_b_frames);
          XtVaSetValues (c_xvid_vmax_b_frames, XtNlabel, label, NULL);
          if (xvid.vmax_b_frames) {
            XtVaSetValues(l_xvid_qratio, XtNsensitive, True, NULL);
            XtVaSetValues(s_xvid_qratio, XtNsensitive, True, NULL);
            XtVaSetValues(l_xvid_qoffset, XtNsensitive, True, NULL);
            XtVaSetValues(s_xvid_qoffset, XtNsensitive, True, NULL);
            XtVaSetValues(l_xvid_bfthreshold, XtNsensitive, True, NULL);
            XtVaSetValues(s_xvid_bfthreshold, XtNsensitive, True, NULL);
            XtVaSetValues(c_xvid_packed, XtNsensitive, True, NULL);
            XtVaSetValues(c_xvid_closed_gop, XtNsensitive, True, NULL);
	    }
          else {
            XtVaSetValues(l_xvid_qratio, XtNsensitive, False, NULL);
            XtVaSetValues(s_xvid_qratio, XtNsensitive, False, NULL);
            XtVaSetValues(l_xvid_qoffset, XtNsensitive, False, NULL);
            XtVaSetValues(s_xvid_qoffset, XtNsensitive, False, NULL);
            XtVaSetValues(l_xvid_bfthreshold, XtNsensitive, False, NULL);
            XtVaSetValues(s_xvid_bfthreshold, XtNsensitive, False, NULL);
            XtVaSetValues(c_xvid_packed, XtNsensitive, False, NULL);
            XtVaSetValues(c_xvid_closed_gop, XtNsensitive, False, NULL);} 
        }
      break;

#endif
#endif
    default:
      break;
    }
}

static void
divx_advanced_jump_scb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  char *name;
  int value = 0;
  float data;
  char label[64];

  data = *(float *) call_data;
  name = XtName (XtParent (widget));
#ifdef HAVE_FFMPEG
  if (0 == strcmp(name, "ffmpegkeyint"))
    {
      value = (int) rint(data * FFMPEG_KEYINT_RANGE);
      if (value == 0)
        value = modulo;

      value /= modulo;
      value *= modulo;
      ffmpeg.keyint = value;

      if (s_ffmpeg_keyint)
        set_float (s_ffmpeg_keyint, XtNtopOfThumb, (float)value/FFMPEG_KEYINT_RANGE);

      sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("KEYframe INTernal")), value);
      if (l_ffmpeg_keyint)
        XtVaSetValues(l_ffmpeg_keyint, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "ffmpegvlelim"))
    {
      value = (int) rint(data * 2 * FFMPEG_VLELIM_RANGE  - FFMPEG_VLELIM_RANGE);
      
      value /= modulo;
      value *= modulo;
      ffmpeg.vlelim = value;

      if (s_ffmpeg_vlelim)
        set_float (s_ffmpeg_vlelim, XtNtopOfThumb, (1+(float)value/FFMPEG_VLELIM_RANGE)/2);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("ELIMination Luminance")), value);
      if (l_ffmpeg_vlelim)
        XtVaSetValues(l_ffmpeg_vlelim, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "ffmpegvcelim"))
    {
      value = (int) rint(data * 2 * FFMPEG_VCELIM_RANGE  - FFMPEG_VCELIM_RANGE);
      
      value /= modulo;
      value *= modulo;
      ffmpeg.vcelim = value;

      if (s_ffmpeg_vcelim)
        set_float (s_ffmpeg_vcelim, XtNtopOfThumb, (1+(float)value/FFMPEG_VCELIM_RANGE)/2);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("ELIMination Chrominance")), value);
      if (l_ffmpeg_vcelim)
        XtVaSetValues(l_ffmpeg_vcelim, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "ffmpeglumimask"))
    {
      value = (int) rint(data * FFMPEG_LUMI_MASK_RANGE);

      value /= modulo;
      value *= modulo;
      ffmpeg.lumi_mask = value/100.0;

      if (s_ffmpeg_lumi_mask)
        set_float (s_ffmpeg_lumi_mask, XtNtopOfThumb, (float)value/FFMPEG_LUMI_MASK_RANGE);

      sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("LUMInance MASKing")), value);
      if (l_ffmpeg_lumi_mask)
        XtVaSetValues(l_ffmpeg_lumi_mask, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "ffmpegdarkmask"))
    {
      value = (int) rint(data * FFMPEG_DARK_MASK_RANGE);

      value /= modulo;
      value *= modulo;
      ffmpeg.dark_mask = value/100.0;

      if (s_ffmpeg_dark_mask)
        set_float (s_ffmpeg_dark_mask, XtNtopOfThumb, (float)value/FFMPEG_DARK_MASK_RANGE);

      sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("DARKness MASKing")), value);
      if (l_ffmpeg_dark_mask)
        XtVaSetValues(l_ffmpeg_dark_mask, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "ffmpegtcplxmask"))
    {
      value = (int) rint(data * FFMPEG_TCPLX_MASK_RANGE);

      value /= modulo;
      value *= modulo;
      ffmpeg.tcplx_mask = value/100.0;

      if (s_ffmpeg_tcplx_mask)
        set_float (s_ffmpeg_tcplx_mask, XtNtopOfThumb, (float)value/FFMPEG_TCPLX_MASK_RANGE);

      sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Temporal ComPLeXity MASKing")), value);
      if (l_ffmpeg_tcplx_mask)
        XtVaSetValues(l_ffmpeg_tcplx_mask, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "ffmpegscplxmask"))
    {
      value = (int) rint(data * FFMPEG_SCPLX_MASK_RANGE);

      value /= modulo;
      value *= modulo;
      ffmpeg.scplx_mask = value/100.0;

      if (s_ffmpeg_scplx_mask)
        set_float (s_ffmpeg_scplx_mask, XtNtopOfThumb, (float)value/FFMPEG_SCPLX_MASK_RANGE);

      sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Spatial ComPLeXity MASKing")), value);
      if (l_ffmpeg_scplx_mask)
        XtVaSetValues(l_ffmpeg_scplx_mask, XtNlabel, label, NULL);
    }
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
  if (0 == strcmp(name, "xvidkeyint"))
    {
      value = (int) rint(data * XVID_KEYINT_RANGE);
      if (value == 0)
        value = modulo;

      value /= modulo;
      value *= modulo;
      xvid.keyint = value;

      if (s_xvid_keyint)
        set_float (s_xvid_keyint, XtNtopOfThumb, (float)value/XVID_KEYINT_RANGE);

      sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("KEYframe INTernal")), value);
      if (l_xvid_keyint)
        XtVaSetValues(l_xvid_keyint, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidiminquantizer"))
    {
      value = (int) rint(data * XVID_MINQUANTIZER_RANGE);
      if (value == 0)
        value = 1;

      xvid.min_quant[0] = value;

      if (s_xvidiminquantizer)
        set_float (s_xvidiminquantizer, XtNtopOfThumb, (float)value/XVID_MINQUANTIZER_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("I-Frame Minimum Quantizer")), value);
      if (l_xvidiminquantizer)
        XtVaSetValues(l_xvidiminquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidimaxquantizer"))
    {
      value = (int) rint(data * XVID_MAXQUANTIZER_RANGE);
      if (value == 0)
        value = 1;

      xvid.max_quant[0] = value;

      if (s_xvidimaxquantizer)
        set_float (s_xvidimaxquantizer, XtNtopOfThumb, (float)value/XVID_MINQUANTIZER_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("I-Frame Maximum Quantizer")), value);
      if (l_xvidimaxquantizer)
        XtVaSetValues(l_xvidimaxquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidpminquantizer"))
    {
      value = (int) rint(data * XVID_MINQUANTIZER_RANGE);
      if (value == 0)
        value = 1;

      xvid.min_quant[1] = value;

      if (s_xvidpminquantizer)
        set_float (s_xvidpminquantizer, XtNtopOfThumb, (float)value/XVID_MINQUANTIZER_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("P-Frame Minimum Quantizer")), value);
      if (l_xvidpminquantizer)
        XtVaSetValues(l_xvidpminquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidpmaxquantizer"))
    {
      value = (int) rint(data * XVID_MAXQUANTIZER_RANGE);
      if (value == 0)
        value = 1;

      xvid.max_quant[1] = value;

      if (s_xvidpmaxquantizer)
        set_float (s_xvidpmaxquantizer, XtNtopOfThumb, (float)value/XVID_MINQUANTIZER_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("P-Frame Maximum Quantizer")), value);
      if (l_xvidpmaxquantizer)
        XtVaSetValues(l_xvidpmaxquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidbminquantizer"))
    {
      value = (int) rint(data * XVID_MINQUANTIZER_RANGE);
      if (value == 0)
        value = 1;

      xvid.min_quant[2] = value;

      if (s_xvidbminquantizer)
        set_float (s_xvidbminquantizer, XtNtopOfThumb, (float)value/XVID_MINQUANTIZER_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("B-Frame Minimum Quantizer")), value);
      if (l_xvidbminquantizer)
        XtVaSetValues(l_xvidbminquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidbmaxquantizer"))
    {
      value = (int) rint(data * XVID_MAXQUANTIZER_RANGE);
      if (value == 0)
        value = 1;

      xvid.max_quant[2] = value;

      if (s_xvidbmaxquantizer)
        set_float (s_xvidbmaxquantizer, XtNtopOfThumb, (float)value/XVID_MINQUANTIZER_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("B-Frame Maximum Quantizer")), value);
      if (l_xvidbmaxquantizer)
        XtVaSetValues(l_xvidbmaxquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidqratio"))
    {
      value = (int) rint(data * 2 * XVID_QRATIO_RANGE  - XVID_QRATIO_RANGE);
      
      value /= modulo;
      value *= modulo;
      xvid.bquant_ratio = value;

      if (s_xvid_qratio)
        set_float (s_xvid_qratio, XtNtopOfThumb, (1+(float)value/XVID_QRATIO_RANGE)/2);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Quantizer Ratio")), value);
      if (l_xvid_qratio)
        XtVaSetValues(l_xvid_qratio, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidqoffset"))
    {
      value = (int) rint(data * 2 * XVID_QOFFSET_RANGE  - XVID_QOFFSET_RANGE);
      
      value /= modulo;
      value *= modulo;
      xvid.bquant_offset = value;

      if (s_xvid_qoffset)
        set_float (s_xvid_qoffset, XtNtopOfThumb, (1+(float)value/XVID_QOFFSET_RANGE)/2);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Quantizer Offset")), value);
      if (l_xvid_qoffset)
        XtVaSetValues(l_xvid_qoffset, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidbfthreshold"))
    {
      value = (int) rint(data * 2 * XVID_BFTHRESHOLD_RANGE  - XVID_BFTHRESHOLD_RANGE);
      
      value /= modulo;
      value *= modulo;
      xvid.bframe_threshold = value;

      if (s_xvid_bfthreshold)
        set_float (s_xvid_bfthreshold, XtNtopOfThumb, (1+(float)value/XVID_BFTHRESHOLD_RANGE)/2);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("B-Frame Threshold")), value);
      if (l_xvid_bfthreshold)
        XtVaSetValues(l_xvid_bfthreshold, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "xvidfdratio"))
    {
      value = (int) rint(data * XVID_FDRATIO_RANGE);

      value /= modulo;
      value *= modulo;
      xvid.frame_drop_ratio = value;

      if (s_xvid_fdratio)
        set_float (s_xvid_fdratio, XtNtopOfThumb, (float)value/XVID_FDRATIO_RANGE);

      sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Frame Drop Tolerance")), value);
      if (l_xvid_fdratio)
        XtVaSetValues(l_xvid_fdratio, XtNlabel, label, NULL);
    }
#endif
#endif

}

static void
divx_advanced_scroll_scb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int move = (int) call_data;
  Dimension length;
  float shown, top1, top2;
  char *name;
  float step = 5.0;
  float range = 65535.0;;
  name = XtName (XtParent (widget));

  XtVaGetValues (widget,
                 XtNlength, &length,
                 XtNshown, &shown, XtNtopOfThumb, &top1, NULL);

#ifdef HAVE_FFMPEG
  if (strcmp(name, "ffmpegkeyint") == 0)
    {
      range = FFMPEG_KEYINT_RANGE;
      step = modulo;
    }
  else if (strcmp(name, "ffmpegvlelim") == 0)
    {
      range = FFMPEG_VLELIM_RANGE;
      step = modulo/2;
    }
  else if (strcmp(name, "ffmpegvcelim") == 0)
    {
      range = FFMPEG_VCELIM_RANGE;
      step = modulo/2;
    }
  else if (strcmp(name, "ffmpeglumimask") == 0)
    {
      range = FFMPEG_LUMI_MASK_RANGE;
      step = modulo;
    }
  else if (strcmp(name, "ffmpegdarkmask") == 0)
    {
      range = FFMPEG_DARK_MASK_RANGE;
      step = modulo;
    }
  else if (strcmp(name, "ffmpegtcplxmask") == 0)
    {
      range = FFMPEG_TCPLX_MASK_RANGE;
      step = modulo;
    }
  else if (strcmp(name, "ffmpegscplxmask") == 0)
    {
      range = FFMPEG_SCPLX_MASK_RANGE;
      step = modulo;
    }
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
  if (strcmp(name, "xvidkeyint") == 0)
    {
      range = XVID_KEYINT_RANGE;
      step = modulo;
    }
  else if (strcmp(name, "xvidiminquantizer") == 0)
    {
      range = XVID_MINQUANTIZER_RANGE;
      step = 1.0;
    }    
  else if (strcmp(name, "xvidimaxquantizer") == 0)   
    {
      range = XVID_MAXQUANTIZER_RANGE;
      step = 1.0 ;
    }
  else if (strcmp(name, "xvidpminquantizer") == 0)
    {
      range = XVID_MINQUANTIZER_RANGE;
      step = 1.0;
    }    
  else if (strcmp(name, "xvidpmaxquantizer") == 0)   
    {
      range = XVID_MAXQUANTIZER_RANGE;
      step = 1.0 ;
    }
  else if (strcmp(name, "xvidbminquantizer") == 0)
    {
      range = XVID_MINQUANTIZER_RANGE;
      step = 1.0;
    }    
  else if (strcmp(name, "xvidbmaxquantizer") == 0)   
    {
      range = XVID_MAXQUANTIZER_RANGE;
      step = 1.0 ;
    }
  else if (strcmp(name, "xvidqratio") == 0)
    {
      range = XVID_QRATIO_RANGE;
      step = modulo/2;
    }
  else if (strcmp(name, "xvidqoffset") == 0)
    {
      range = XVID_QOFFSET_RANGE;
      step = modulo/2;
    }
  else if (strcmp(name, "xvidbfthreshold") == 0)
    {
      range = XVID_BFTHRESHOLD_RANGE;
      step = modulo/2;
    }
  else if (strcmp(name, "xvidfdratio") == 0)
    {
      range = XVID_FDRATIO_RANGE;
      step = modulo;
    }
#endif
#endif

  if (move > 0)
    top2 = top1 + step/range;
  else
    top2 = top1 - step/range;
  if (top2 < 0.0)
    top2 = 0.0;
  if (top2 > 1.0)
    top2 = 1.0;

  divx_advanced_jump_scb (widget, clientdata, &top2);
}

/*--------------------------------------------------------------------------*/

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
void
create_xvid_win()
{
  Widget c, p;
  char label[64];

#ifdef HAVE_XPM
  Pixmap icon_pixmap;
  Pixmap icon_shapemask;
#endif
  
  translate_struct(on_off_list);
  translate_struct(xvid_vhq_list);
  
  xvid_shell = XtVaAppCreateShell ("XviD", "XdTV",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell, XtNvisual,vinfo->visual,
				   XtNcolormap,colormap, XtNdepth,vinfo->depth,
				   NULL);

  xvid_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, xvid_shell, NULL);

  /* XviD Modulo */
  c_xvid_modulo = c = XtVaCreateManagedWidget ("xvidmodulo", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) DIVXA_MODULO);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Modulo for ALL")), (int)modulo);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* XviD General Settings Label */
  c_xvid_me = XtVaCreateManagedWidget ("lxvidme", labelWidgetClass, xvid_paned,
                           PANED_FIX, NULL);

  /* XviD VHQ Mode */
  c_xvid_vhq = c = XtVaCreateManagedWidget ("xvidvhq", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_VHQ);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("VHQ Mode")), xvid.vhq);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* XviD BVHQ Mode */
#if XVID_API >= XVID_MAKE_API(4,1)
  c_xvid_bvhq = c = XtVaCreateManagedWidget ("xvidbvhq", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_BVHQ);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("BVHQ Mode")),
           int_to_str (xvid.bvhq, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif

  /* XviD Chroma Motion Estimation */
  c_xvid_chromame = c = XtVaCreateManagedWidget ("xvidchromame", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_CHROMAME);
  if (divx.quality >= 4 && divx.codec == XVID) {
    sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Chroma Motion Estimation")),
             translate("Already managed!"));
    xvid.chromame = 0;}
  else
    sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Chroma Motion Estimation")),
             int_to_str (xvid.chromame, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* XviD Turbo Mode */
  c_xvid_turbo = c = XtVaCreateManagedWidget ("xvidturbo", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_TURBO);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Turbo Mode")),
           int_to_str (xvid.turbo, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL); 
  
  /* XviD HQ-AC Prediction */
  c_xvid_hqacpred = c = XtVaCreateManagedWidget ("xvidhqacpred", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_HQACPRED);    
  if (divx.quality >= 6 && divx.codec == XVID) {
    sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("HQ-AC Prediction")),
             translate("Already managed!"));
    xvid.hqacpred = 0;}
  else
    sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("HQ-AC Prediction")),
             int_to_str (xvid.hqacpred, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* XviD Frame Drop Tolerance */
  p =
    XtVaCreateManagedWidget ("xvidfdratio", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvid_fdratio = XtVaCreateManagedWidget ("lxvidfdratio", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvid_fdratio = XtVaCreateManagedWidget ("sxvidfdratio", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvid_fdratio, XtNtopOfThumb, (float)xvid.frame_drop_ratio/XVID_FDRATIO_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Frame Drop Tolerance")), xvid.frame_drop_ratio);
  XtVaSetValues(l_xvid_fdratio, XtNlabel, label, NULL);
  XtAddCallback (s_xvid_fdratio, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvid_fdratio, XtNscrollProc, divx_advanced_scroll_scb, NULL); 
    
  /* XviD Keyint */
  p =
    XtVaCreateManagedWidget ("xvidkeyint", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvid_keyint = XtVaCreateManagedWidget ("lxvidkeyint", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvid_keyint = XtVaCreateManagedWidget ("sxvidkeyint", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvid_keyint, XtNtopOfThumb, (float)xvid.keyint/XVID_KEYINT_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("KEYframe INTernal")), xvid.keyint);
  XtVaSetValues(l_xvid_keyint, XtNlabel, label, NULL);
  XtAddCallback (s_xvid_keyint, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvid_keyint, XtNscrollProc, divx_advanced_scroll_scb, NULL);

  /* XviD Cartoon Mode */
  c_xvid_cartoon = c = XtVaCreateManagedWidget ("xvidcartoon", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_CARTOON);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Cartoon Mode")),
           int_to_str (xvid.cartoon, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL); 
  
  /* XviD Advanced Simple Profile Specific Options */
  c_xvid_profset = XtVaCreateManagedWidget ("lxvidprofset", labelWidgetClass, xvid_paned,
                           PANED_FIX, NULL);
  
  /* XviD MAX B-Frames */
  c_xvid_vmax_b_frames = c = XtVaCreateManagedWidget ("xvidvmaxbframes", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_VMAX_B_FRAMES);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("MAX B-Frames")), xvid.vmax_b_frames);
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* XviD Quantizer Ratio */
  p =
    XtVaCreateManagedWidget ("xvidqratio", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvid_qratio = XtVaCreateManagedWidget ("lxvidqratio", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvid_qratio = XtVaCreateManagedWidget ("sxvidqratio", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvid_qratio, XtNtopOfThumb, (1+(float)xvid.bquant_ratio/XVID_QRATIO_RANGE)/2);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Quantizer Ratio")), xvid.bquant_ratio);
  XtVaSetValues(l_xvid_qratio, XtNlabel, label, NULL);
  XtAddCallback (s_xvid_qratio, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvid_qratio, XtNscrollProc, divx_advanced_scroll_scb, NULL);
    if (xvid.vmax_b_frames) {
    XtVaSetValues(l_xvid_qratio, XtNsensitive, True, NULL);
    XtVaSetValues(s_xvid_qratio, XtNsensitive, True, NULL);}
  else {
    XtVaSetValues(l_xvid_qratio, XtNsensitive, False, NULL);
    XtVaSetValues(s_xvid_qratio, XtNsensitive, False, NULL);}  

  /* XviD Quantizer Offset */
  p =
    XtVaCreateManagedWidget ("xvidqoffset", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvid_qoffset = XtVaCreateManagedWidget ("lxvidqoffset", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvid_qoffset = XtVaCreateManagedWidget ("sxvidqoffset", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvid_qoffset, XtNtopOfThumb, (1+(float)xvid.bquant_offset/XVID_QOFFSET_RANGE)/2);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Quantizer Offset")), xvid.bquant_offset);
  XtVaSetValues(l_xvid_qoffset, XtNlabel, label, NULL);
  XtAddCallback (s_xvid_qoffset, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvid_qoffset, XtNscrollProc, divx_advanced_scroll_scb, NULL);
    if (xvid.vmax_b_frames) {
    XtVaSetValues(l_xvid_qoffset, XtNsensitive, True, NULL);
    XtVaSetValues(s_xvid_qoffset, XtNsensitive, True, NULL);}
  else {
    XtVaSetValues(l_xvid_qoffset, XtNsensitive, False, NULL);
    XtVaSetValues(s_xvid_qoffset, XtNsensitive, False, NULL);}  

  /* XviD B-Frame Threshold */
  p =
    XtVaCreateManagedWidget ("xvidbfthreshold", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvid_bfthreshold = XtVaCreateManagedWidget ("lxvidbfthreshold", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvid_bfthreshold = XtVaCreateManagedWidget ("sxvidbfthreshold", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvid_bfthreshold, XtNtopOfThumb, (1+(float)xvid.bframe_threshold/XVID_BFTHRESHOLD_RANGE)/2);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("B-Frame Threshold")), xvid.bframe_threshold);
  XtVaSetValues(l_xvid_bfthreshold, XtNlabel, label, NULL);
  XtAddCallback (s_xvid_bfthreshold, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvid_bfthreshold, XtNscrollProc, divx_advanced_scroll_scb, NULL);
  if (xvid.vmax_b_frames) {
    XtVaSetValues(l_xvid_bfthreshold, XtNsensitive, True, NULL);
    XtVaSetValues(s_xvid_bfthreshold, XtNsensitive, True, NULL);}
  else {
    XtVaSetValues(l_xvid_bfthreshold, XtNsensitive, False, NULL);
    XtVaSetValues(s_xvid_bfthreshold, XtNsensitive, False, NULL);}  
  
  /* XviD Packed bitstream */
  c_xvid_packed = c = XtVaCreateManagedWidget ("xvidpacked", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_PACKED);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Packed bitstream")),
           int_to_str (xvid.packed, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  if (xvid.vmax_b_frames) XtVaSetValues(c_xvid_packed, XtNsensitive, True, NULL);
  else XtVaSetValues(c_xvid_packed, XtNsensitive, False, NULL); 
  
  /* XviD Closed GOP */
  c_xvid_closed_gop = c = XtVaCreateManagedWidget ("xvidclosedgop", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_CLOSED_GOP);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Closed GOP")),
           int_to_str (xvid.closed_gop, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  if (xvid.vmax_b_frames) XtVaSetValues(c_xvid_closed_gop, XtNsensitive, True, NULL);
  else XtVaSetValues(c_xvid_closed_gop, XtNsensitive, False, NULL); 
  
  /* XviD Quarter PixEL */
  c_xvid_qpel = c = XtVaCreateManagedWidget ("xvidqpel", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_QPEL);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Quarter PixEL")),
           int_to_str (xvid.qpel, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* XviD Global Motion Compensation */
  c_xvid_gmc = c = XtVaCreateManagedWidget ("xvidgmc", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_GMC);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Global Motion Compensation")),
           int_to_str (xvid.gmc, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* XviD Interlaced Encoding */
  c_xvid_interlacing = c = XtVaCreateManagedWidget ("xvidinterlacing", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_INTERLACING);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Interlaced Encoding")),
           int_to_str (xvid.interlacing, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* XviD Quantizers Label */
  c_xvid_quantizers = XtVaCreateManagedWidget ("lxvidquantizers", labelWidgetClass, xvid_paned,
                           PANED_FIX, NULL);

  /* XviD I min Quantizer Slider */
  p =
    XtVaCreateManagedWidget ("xvidiminquantizer", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvidiminquantizer = XtVaCreateManagedWidget ("lxvidiminquantizer", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvidiminquantizer = XtVaCreateManagedWidget ("sxvidiminquantizer", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvidiminquantizer, XtNtopOfThumb, (float)xvid.min_quant[0]/XVID_MINQUANTIZER_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("I-Frame Minimum Quantizer")), xvid.min_quant[0]);
  XtVaSetValues(l_xvidiminquantizer, XtNlabel, label, NULL);
  XtAddCallback (s_xvidiminquantizer, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvidiminquantizer, XtNscrollProc, divx_advanced_scroll_scb, NULL);
  
  /* XviD I max Quantizer Slider */
  p =
    XtVaCreateManagedWidget ("xvidimaxquantizer", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvidimaxquantizer = XtVaCreateManagedWidget ("lxvidimaxquantizer", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvidimaxquantizer = XtVaCreateManagedWidget ("sxvidimaxquantizer", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvidimaxquantizer, XtNtopOfThumb, (float)xvid.max_quant[0]/XVID_MAXQUANTIZER_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("I-Frame Maximum Quantizer")), xvid.max_quant[0]);
  XtVaSetValues(l_xvidimaxquantizer, XtNlabel, label, NULL);
  XtAddCallback (s_xvidimaxquantizer, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvidimaxquantizer, XtNscrollProc, divx_advanced_scroll_scb, NULL);

  /* XviD P min Quantizer Slider */
  p =
    XtVaCreateManagedWidget ("xvidpminquantizer", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvidpminquantizer = XtVaCreateManagedWidget ("lxvidpminquantizer", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvidpminquantizer = XtVaCreateManagedWidget ("sxvidpminquantizer", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvidpminquantizer, XtNtopOfThumb, (float)xvid.min_quant[1]/XVID_MINQUANTIZER_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("P-Frame Minimum Quantizer")), xvid.min_quant[1]);
  XtVaSetValues(l_xvidpminquantizer, XtNlabel, label, NULL);
  XtAddCallback (s_xvidpminquantizer, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvidpminquantizer, XtNscrollProc, divx_advanced_scroll_scb, NULL);
  
  /* XviD P max Quantizer Slider */
  p =
    XtVaCreateManagedWidget ("xvidpmaxquantizer", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvidpmaxquantizer = XtVaCreateManagedWidget ("lxvidpmaxquantizer", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvidpmaxquantizer = XtVaCreateManagedWidget ("sxvidpmaxquantizer", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvidpmaxquantizer, XtNtopOfThumb, (float)xvid.max_quant[1]/XVID_MAXQUANTIZER_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("P-Frame Maximum Quantizer")), xvid.max_quant[1]);
  XtVaSetValues(l_xvidpmaxquantizer, XtNlabel, label, NULL);
  XtAddCallback (s_xvidpmaxquantizer, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvidpmaxquantizer, XtNscrollProc, divx_advanced_scroll_scb, NULL);

  /* XviD B min Quantizer Slider */
  p =
    XtVaCreateManagedWidget ("xvidbminquantizer", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvidbminquantizer = XtVaCreateManagedWidget ("lxvidbminquantizer", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvidbminquantizer = XtVaCreateManagedWidget ("sxvidbminquantizer", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvidbminquantizer, XtNtopOfThumb, (float)xvid.min_quant[2]/XVID_MINQUANTIZER_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("B-Frame Minimum Quantizer")), xvid.min_quant[2]);
  XtVaSetValues(l_xvidbminquantizer, XtNlabel, label, NULL);
  XtAddCallback (s_xvidbminquantizer, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvidbminquantizer, XtNscrollProc, divx_advanced_scroll_scb, NULL);
  
  /* XviD B max Quantizer Slider */
  p =
    XtVaCreateManagedWidget ("xvidbmaxquantizer", panedWidgetClass, xvid_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_xvidbmaxquantizer = XtVaCreateManagedWidget ("lxvidbmaxquantizer", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_xvidbmaxquantizer = XtVaCreateManagedWidget ("sxvidbmaxquantizer", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_xvidbmaxquantizer, XtNtopOfThumb, (float)xvid.max_quant[2]/XVID_MAXQUANTIZER_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("B-Frame Maximum Quantizer")), xvid.max_quant[2]);
  XtVaSetValues(l_xvidbmaxquantizer, XtNlabel, label, NULL);
  XtAddCallback (s_xvidbmaxquantizer, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_xvidbmaxquantizer, XtNscrollProc, divx_advanced_scroll_scb, NULL); 

  /* XviD Trellis Quantization */
  c_xvid_trell = c = XtVaCreateManagedWidget ("xvidtrell", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_TRELL);
  if (divx.quality >= 5 && divx.codec == XVID) {
    sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
             translate("Already managed!"));
    xvid.trell = 0;}
  else
    sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
             int_to_str (xvid.trell, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* XviD Other Options */
  c_xvid_otherset = XtVaCreateManagedWidget ("lxvidotherset", labelWidgetClass, xvid_paned,
                           PANED_FIX, NULL); 
    
  /* XviD Grayscale Encoding */
  c_xvid_gray = c = XtVaCreateManagedWidget ("xvidgray", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_GRAY);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Grayscale Encoding")),
           int_to_str (xvid.gray, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* XviD Chroma Optimizer */
  c_xvid_chroma_opt = c = XtVaCreateManagedWidget ("xvidchromaopt", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) XVID_CHROMA_OPT);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Chroma Optimizer")),
           int_to_str (xvid.chroma_opt, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

/* Open the Video Menu */
  c_xvid_videoopen = XtVaCreateManagedWidget ("lxvidvideoopen", commandWidgetClass, xvid_paned,
                          PANED_FIX, NULL);
    XtAddCallback (c_xvid_videoopen, XtNcallback, action_cb, (XtPointer) & call_videoparams);

   /* Button to close XviD Page Menu */

  c_xvid_quit = c = XtVaCreateManagedWidget ("lxvidclose", commandWidgetClass, xvid_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, save_cb, (XtPointer) NULL);
  XtAddCallback (c, XtNcallback, close_xvid_win, (XtPointer) NULL);
  
#ifdef HAVE_XPM
  XpmCreatePixmapFromData(XtDisplay(xvid_shell),
                          RootWindowOfScreen(XtScreen(xvid_shell)),
                          icon_xpm,
                          &icon_pixmap,
                          &icon_shapemask, NULL);
  XtVaSetValues(xvid_shell, XtNiconPixmap, icon_pixmap, NULL);
#endif
  create_xvid_pixmaps(XtDisplay(xvid_paned),&RootWindowOfScreen(XtScreen(xvid_paned)));
}
#endif
#endif

#ifdef HAVE_FFMPEG
void
create_ffmpeg_win()
{
  Widget c, p;
  char label[64];

#ifdef HAVE_XPM
  Pixmap icon_pixmap;
  Pixmap icon_shapemask;
#endif
  
  translate_struct(on_off_list);
  translate_struct(ffmpeg_mbd_list);
  
  ffmpeg_shell = XtVaAppCreateShell ("FFMpeg", "XdTV",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell, XtNvisual,vinfo->visual,
				   XtNcolormap,colormap, XtNdepth,vinfo->depth,
				   NULL);

  ffmpeg_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, ffmpeg_shell, NULL);

  /* FFMpeg Modulo */
  c_ffmpeg_modulo = c = XtVaCreateManagedWidget ("ffmpegmodulo", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) DIVXA_MODULO);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Modulo for ALL")), (int)modulo);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* FFMpeg Motion Estimation Other Options */
  c_ffmpeg_me = XtVaCreateManagedWidget ("lffmpegme", labelWidgetClass, ffmpeg_paned,
                           PANED_FIX, NULL);
    
  /* FFMpeg V4mv : 4 Motion Vector */
  c_ffmpeg_v4mv = c = XtVaCreateManagedWidget ("ffmpegv4mv", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_V4MV);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("V4mv : 4 Motion Vector")),
           int_to_str (ffmpeg.v4mv, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* FFMpeg Quarter PixEL */
  c_ffmpeg_qpel = c = XtVaCreateManagedWidget ("ffmpegqpel", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_QPEL);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Quarter PixEL")),
           int_to_str (ffmpeg.qpel, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* FFMpeg Global Motion Compensation */
  c_ffmpeg_gmc = c = XtVaCreateManagedWidget ("ffmpeggmc", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_GMC);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Global Motion Compensation")),
           int_to_str (ffmpeg.gmc, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

#if LIBAVCODEC_BUILD >= 4698  
  /* FFMpeg InterLaced Motion Estimation */
  c_ffmpeg_ilme = c = XtVaCreateManagedWidget ("ffmpegilme", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_ILME);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("InterLaced Motion Estimation")),
           int_to_str (ffmpeg.ilme, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif
  
  /* FFMpeg Keyint */
  p =
    XtVaCreateManagedWidget ("ffmpegkeyint", panedWidgetClass, ffmpeg_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_ffmpeg_keyint = XtVaCreateManagedWidget ("lffmpegkeyint", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_ffmpeg_keyint = XtVaCreateManagedWidget ("sffmpegkeyint", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_ffmpeg_keyint, XtNtopOfThumb, (float)ffmpeg.keyint/FFMPEG_KEYINT_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("KEYframe INTernal")), ffmpeg.keyint);
  XtVaSetValues(l_ffmpeg_keyint, XtNlabel, label, NULL);
  XtAddCallback (s_ffmpeg_keyint, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_ffmpeg_keyint, XtNscrollProc, divx_advanced_scroll_scb, NULL);

  /* FFMpeg MAX B-Frames */
  c_ffmpeg_vmax_b_frames = c = XtVaCreateManagedWidget ("ffmpegvmaxbframes", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_VMAX_B_FRAMES);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("MAX B-Frames")), ffmpeg.vmax_b_frames);
  XtVaSetValues (c, XtNlabel, label, NULL);

#if LIBAVCODEC_BUILD >= 4647 
  /* FFMpeg DIAmond */
  c_ffmpeg_dia = c = XtVaCreateManagedWidget ("ffmpegdia", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_DIA);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("DIAmond")), ffmpeg.dia);
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif

#if LIBAVCODEC_BUILD >= 4651 
  /* FFMpeg PRE-pass DIAmond */
  c_ffmpeg_predia = c = XtVaCreateManagedWidget ("ffmpegpredia", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_PREDIA);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("PRE-pass DIAmond")), ffmpeg.predia);
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif

#if LIBAVCODEC_BUILD >= 4647 
  /* FFMpeg Full pel ME CoMParison */
  c_ffmpeg_cmp = c = XtVaCreateManagedWidget ("ffmpegcmp", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_CMP);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Full pel ME CoMParison")), ffmpeg.cmp);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* FFMpeg SUB pel ME CoMParison */
  c_ffmpeg_subcmp = c = XtVaCreateManagedWidget ("ffmpegsubcmp", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_SUBCMP);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("SUB pel ME CoMParison")), ffmpeg.subcmp);
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif

#if LIBAVCODEC_BUILD >= 4651 
  /* FFMpeg ME PRE-pass CoMParison */
  c_ffmpeg_precmp = c = XtVaCreateManagedWidget ("ffmpegprecmp", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_PRECMP);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("ME PRE-pass CoMParison")), ffmpeg.precmp);
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif

#if LIBAVCODEC_BUILD >= 4649 
  /* FFMpeg LAST frame motion PREdictors */
  c_ffmpeg_last_pred = c = XtVaCreateManagedWidget ("ffmpeglastpred", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_LAST_PRED);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("LAST frame motion PREdictors")), ffmpeg.last_pred);
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif
 
  /* FFMpeg Masking */
  c_ffmpeg_masking = XtVaCreateManagedWidget ("lffmpegmasking", labelWidgetClass, ffmpeg_paned,
                           PANED_FIX, NULL);
  
/* FFMpeg LUMInance MASKing */
  p =
    XtVaCreateManagedWidget ("ffmpeglumimask", panedWidgetClass, ffmpeg_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_ffmpeg_lumi_mask = XtVaCreateManagedWidget ("lffmpeglumimask", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_ffmpeg_lumi_mask = XtVaCreateManagedWidget ("sffmpeglumimask", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_ffmpeg_lumi_mask, XtNtopOfThumb, ffmpeg.lumi_mask*100.0/FFMPEG_LUMI_MASK_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("LUMInance MASKing")), (int)(ffmpeg.lumi_mask*100.0+0.5));
  XtVaSetValues(l_ffmpeg_lumi_mask, XtNlabel, label, NULL);
  XtAddCallback (s_ffmpeg_lumi_mask, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_ffmpeg_lumi_mask, XtNscrollProc, divx_advanced_scroll_scb, NULL);
  
/* FFMpeg DARKness MASKing */
  p =
    XtVaCreateManagedWidget ("ffmpegdarkmask", panedWidgetClass, ffmpeg_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_ffmpeg_dark_mask = XtVaCreateManagedWidget ("lffmpegdarkmask", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_ffmpeg_dark_mask = XtVaCreateManagedWidget ("sffmpegdarkmask", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_ffmpeg_dark_mask, XtNtopOfThumb, ffmpeg.dark_mask*100.0/FFMPEG_DARK_MASK_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("DARKness MASKing")), (int)(ffmpeg.dark_mask*100.0+0.5));
  XtVaSetValues(l_ffmpeg_dark_mask, XtNlabel, label, NULL);
  XtAddCallback (s_ffmpeg_dark_mask, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_ffmpeg_dark_mask, XtNscrollProc, divx_advanced_scroll_scb, NULL);
  
/* FFMpeg Scplx_mask */
  p =
    XtVaCreateManagedWidget ("ffmpegscplxmask", panedWidgetClass, ffmpeg_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_ffmpeg_scplx_mask = XtVaCreateManagedWidget ("lffmpegscplxmask", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_ffmpeg_scplx_mask = XtVaCreateManagedWidget ("sffmpegscplxmask", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_ffmpeg_scplx_mask, XtNtopOfThumb, ffmpeg.scplx_mask*100.0/FFMPEG_SCPLX_MASK_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Spatial ComPLeXity MASKing")), (int)(ffmpeg.scplx_mask*100.0+0.5));
  XtVaSetValues(l_ffmpeg_scplx_mask, XtNlabel, label, NULL);
  XtAddCallback (s_ffmpeg_scplx_mask, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_ffmpeg_scplx_mask, XtNscrollProc, divx_advanced_scroll_scb, NULL);
    
/* FFMpeg Tcplx_mask */
  p =
    XtVaCreateManagedWidget ("ffmpegtcplxmask", panedWidgetClass, ffmpeg_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_ffmpeg_tcplx_mask = XtVaCreateManagedWidget ("lffmpegtcplxmask", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_ffmpeg_tcplx_mask = XtVaCreateManagedWidget ("sffmpegtcplxmask", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_ffmpeg_tcplx_mask, XtNtopOfThumb, ffmpeg.tcplx_mask*100.0/FFMPEG_TCPLX_MASK_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("Temporal ComPLeXity MASKing")), (int)(ffmpeg.tcplx_mask*100.0+0.5));
  XtVaSetValues(l_ffmpeg_tcplx_mask, XtNlabel, label, NULL);
  XtAddCallback (s_ffmpeg_tcplx_mask, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_ffmpeg_tcplx_mask, XtNscrollProc, divx_advanced_scroll_scb, NULL);

  /* FFMpeg Normalize Quantization */
  c_ffmpeg_naq = c = XtVaCreateManagedWidget ("ffmpegnaq", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_NAQ);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Normalize Quantization")),
           int_to_str (ffmpeg.naq, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  
  /* FFMpeg Vlelim */
  p =
    XtVaCreateManagedWidget ("ffmpegvlelim", panedWidgetClass, ffmpeg_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_ffmpeg_vlelim = XtVaCreateManagedWidget ("lffmpegvlelim", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_ffmpeg_vlelim = XtVaCreateManagedWidget ("sffmpegvlelim", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_ffmpeg_vlelim, XtNtopOfThumb, (1+(float)ffmpeg.vlelim/FFMPEG_VLELIM_RANGE)/2);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("ELIMination Luminance")), ffmpeg.vlelim);
  XtVaSetValues(l_ffmpeg_vlelim, XtNlabel, label, NULL);
  XtAddCallback (s_ffmpeg_vlelim, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_ffmpeg_vlelim, XtNscrollProc, divx_advanced_scroll_scb, NULL);

  /* FFMpeg Vcelim */
  p =
    XtVaCreateManagedWidget ("ffmpegvcelim", panedWidgetClass, ffmpeg_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_ffmpeg_vcelim = XtVaCreateManagedWidget ("lffmpegvcelim", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_ffmpeg_vcelim = XtVaCreateManagedWidget ("sffmpegvcelim", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_ffmpeg_vcelim, XtNtopOfThumb, (1+(float)ffmpeg.vcelim/FFMPEG_VCELIM_RANGE)/2);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("ELIMination Chrominance")), ffmpeg.vcelim);
  XtVaSetValues(l_ffmpeg_vcelim, XtNlabel, label, NULL);
  XtAddCallback (s_ffmpeg_vcelim, XtNjumpProc, divx_advanced_jump_scb, NULL);
  XtAddCallback (s_ffmpeg_vcelim, XtNscrollProc, divx_advanced_scroll_scb, NULL);
    
  /* FFMpeg Quantization */
  c_ffmpeg_quantizers = XtVaCreateManagedWidget ("lffmpegquantizers", labelWidgetClass, ffmpeg_paned,
                           PANED_FIX, NULL);
#if LIBAVCODEC_BUILD >= 4673 
  /* FFMpeg MacroBlock Decision */
  c_ffmpeg_mbd = c = XtVaCreateManagedWidget ("ffmpegmbd", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_MBD);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH5, translate("MacroBlock Decision")), ffmpeg.mbd);
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif
#if LIBAVCODEC_BUILD >= 4648  
  /* FFMpeg Trellis Quantization */
  c_ffmpeg_trell = c = XtVaCreateManagedWidget ("ffmpegtrell", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_TRELL);
#if LIBAVCODEC_BUILD >= 4681
  if (ffmpeg.cbp) {
    sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
             translate("Already managed!"));
    ffmpeg.trell = 0;}
  else
#endif
    sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
             int_to_str (ffmpeg.trell, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif

#if LIBAVCODEC_BUILD >= 4681
  /* FFMpeg Coded Block Pattern */
  c_ffmpeg_cbp = c = XtVaCreateManagedWidget ("ffmpegcbp", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_CBP);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Coded Block Pattern")),
           int_to_str (ffmpeg.cbp, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
#endif

  /* FFMpeg Other Settings Label */
  c_ffmpeg_otherset = XtVaCreateManagedWidget ("lffmpegotherset", labelWidgetClass, ffmpeg_paned,
                           PANED_FIX, NULL);
 
  /* FFMpeg Grayscale Encoding */
  c_ffmpeg_gray = c = XtVaCreateManagedWidget ("ffmpeggray", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_GRAY);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Grayscale Encoding")),
           int_to_str (ffmpeg.gray, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* FFMpeg InterLaced DCT Encoding */
  c_ffmpeg_ildct = c = XtVaCreateManagedWidget ("ffmpegildtc", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_advanced_menu_cb, (XtPointer) FFMPEG_ILDCT);
  sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("InterLaced DCT Encoding")),
           int_to_str (ffmpeg.ildct, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

/* Open the Video Menu */
  c_ffmpeg_videoopen = XtVaCreateManagedWidget ("lffmpegvideoopen", commandWidgetClass, ffmpeg_paned,
                          PANED_FIX, NULL);
    XtAddCallback (c_ffmpeg_videoopen, XtNcallback, action_cb, (XtPointer) & call_videoparams);

/* Button to close FFMpeg Page Menu */

  c_ffmpeg_quit = c = XtVaCreateManagedWidget ("lffmpegclose", commandWidgetClass, ffmpeg_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, save_cb, (XtPointer) NULL);
  XtAddCallback (c, XtNcallback, close_ffmpeg_win, (XtPointer) NULL);
  
#ifdef HAVE_XPM
  XpmCreatePixmapFromData(XtDisplay(ffmpeg_shell),
                          RootWindowOfScreen(XtScreen(ffmpeg_shell)),
                          icon_xpm,
                          &icon_pixmap,
                          &icon_shapemask, NULL);
  XtVaSetValues(ffmpeg_shell, XtNiconPixmap, icon_pixmap, NULL);
#endif
  create_ffmpeg_pixmaps(XtDisplay(ffmpeg_paned),&RootWindowOfScreen(XtScreen(ffmpeg_paned)));
}
#endif
