/*****************************************************************************
 * divx_ui.c: DivX Parameters editor
 *****************************************************************************
 * $Id: divx_ui.c,v 1.112 2007/02/10 16:05:09 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
#include <malloc.h>


#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <sys/time.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <pthread.h>
#include <sys/wait.h>

#include "config.h"

#include "divx_ui.h"
#include "divx.h"
#include "divx_advanced_ui.h"
#include "strtab.h"
#include "toolbox.h"
#include "xdtv.h"
#include "grab.h"
#include "x11.h"
#include "popup_ui.h"
#include "file_sel.h"
#include "audio.h"
#include "container.h"

#ifdef HAVE_XPM
#include <X11/xpm.h>
#include "xdtv.xpm"
#endif

// internationalization
#include "translation.h"
#include "pixmaps_loader.h"

#ifdef BUGTEXTXAW3D
#define STRDUP(x) strdup(x)
#else
#define STRDUP(x) (x)
#endif

struct CONTAINER* cont_handler=NULL;
/*-------------------------------------------------------------------------*/
	
extern void save_cb (Widget widget, XtPointer clientdata, XtPointer call_data);
extern void SetBgAction (Widget, XEvent *, String *, Cardinal *);
extern void RecordMovieAction (Widget , XEvent * , String * , Cardinal *);
extern void action_cb (Widget widget, XtPointer clientdata, XtPointer call_data);
extern void OptionsAction (Widget, XEvent *, String *, Cardinal *);
extern void init_divxwin_labels(void);

extern Widget app_shell, divx_shell, divxaudio_shell, streamparams_shell, c_record;
extern Display *dpy;
extern Atom wm_protocols[2];
extern int debug;
extern int bttvfd,vbifd;
extern int xawpopup;
extern char lastfilename[256];
extern struct STRTAB audio_fmt_names[];

extern divx_t divx;
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
extern xvid_t xvid;
#endif
#endif

static int w_modul = WRECPAD;
static int h_modul = HRECPAD;

#ifdef HAVE_XPM
  Pixmap icon_pixmap;
  Pixmap icon_shapemask;
#endif

/* Widget which are not static are used in pixmaps libraries */
Widget divx_paned;
Widget divxaudio_paned;
Widget streamparams_paned;

// video
static Widget c_file_format_menu;
static Widget c_divxcodec, c_wmodul, c_hmodul;
static Widget l_divxwidth,        s_divxwidth;
static Widget l_divxheight,       s_divxheight;
static Widget l_divxbitrate,      s_divxbitrate;
Widget l_divxmaxsize,      s_divxmaxsize;
static Widget c_video_fps;
static Widget l_divxmaxgap, s_divxmaxgap;
static Widget l_divxdelay, s_divxdelay;
static Widget c_divxquality;
Widget l_divxminquantizer, s_divxminquantizer;
Widget l_divxmaxquantizer, s_divxmaxquantizer;
static Widget t_divxfilepath;
static Widget c_displayframes;
static Widget c_sub;
static Widget c_chg;

// Audio: don't need Lame
static Widget c_stereo;
static Widget c_audio_freq;
static Widget c_audio_fmt;
static Widget c_divxaudiobufsize, s_divxaudiobufsize;
static Widget c_divxaudiofrgmts, s_divxaudiofrgmts;
static Widget c_divxaudiosizefrgmt, s_divxaudiosizefrgmt;

static Widget c_compressaudio;
static Widget c_mp3_vbr;
static Widget p_divxmp3vbrquality,   l_divxmp3vbrquality,   s_divxmp3vbrquality;
static Widget p_divxmp3bitrate;
Widget l_divxmp3bitrate,   s_divxmp3bitrate;
static Widget p_divxmp3quality,   l_divxmp3quality,   s_divxmp3quality;

//divx video
Widget c_filefmt;
Widget c_divxvideocodec;
Widget c_divxsize ;
Widget c_divxdisplay;
Widget c_divxpath ;
Widget c_divxaudioopen ;
Widget c_divxstreamopen ;
Widget c_divxopenmain ;
Widget c_divxsave ;
Widget c_divxpath_selector ;
Widget c_divxschedule;
Widget c_divxplayer;
Widget c_codecaopt;

//audio
Widget c_divxaudiomp3;
Widget c_divxaudiovideo;
Widget c_divxaudio;
Widget c_audiodivxopen;
Widget c_audiostreamopen;
Widget c_audiosave;
Widget c_audioopenmain;

//streaming 
Widget c_streamdivxopen;
Widget c_streamaudioopen;
Widget c_streamopenmain;
Widget c_stropts;
Widget c_http_port;
Widget c_streamparamssave;
Widget c_streamparamsclose;
Widget c_streaming;

static  Widget c_stream_onoff;
static Widget t_http_port_str;

char * http_port_str;
char http_port[256];

static char* displayed_filename=NULL;
static int filename_malloc=0;

enum {
  FILE_FORMAT,
  DIVX_CODEC = 10,
  DIVX_HMODUL,
  DIVX_WMODUL,
  DIVX_QUALITY,
  DIVX_DISPLAY_FRAMES,
  DIVX_SUB,
  DIVX_CHG,
  DIVX_STEREO,
  DIVX_AUDIO_FREQ,
  DIVX_FPS,
  DIVX_AUDIO_FMT,
  DIVX_AUDIO_CODEC,
  DIVX_MP3_VBR,
  DIVX_STREAM_ONOFF
};

static struct STRTAB on_off_list[] = {
  {1, "On"},
  {0, "Off"},
  {-1, NULL}
};

static struct STRTAB mp3_vbr_list[] = {
  {1, "VBR (Variable)"},
  {0, "CBR (Constant)"},
  {-1, NULL}
};

static struct STRTAB stereo_list[] = {
  {1, "Stereo"},
  {0, "Mono"},
  {-1, NULL}
};

static struct STRTAB audio_freq_list[] = {
  {48000, "48000 Hz"},
  {44100, "44100 Hz"},
  {32000, "32000 Hz"},
  {24000, "24000 Hz"},
  {22050, "22050 Hz"},
  {16000, "16000 Hz"},
  {12000, "12000 Hz"},
  {11025, "11025 Hz"},
  {8000,  " 8000 Hz"},
  {-1, NULL}
};

static struct STRTAB video_fps_list[] = {
  {30000,    "30"},
  {29970,    "29.97"},
  {25000,    "25"},
  {24000,    "24"},
  {23980,    "23.98"},
  {15000,    "15"},
  {12500,    "12.5"},
  {10000,    "10"},
  {8333,     "8.33"},
  {7500,     "7.5"},
  {6250,     "6.25"},
  {5000,     "5"},
  {3750,     "3.75"},
  {3125,     "3.12"},
  {1000,     "1"},
  {-1, NULL}
};

static struct STRTAB modulo_list[] = {
  {1,  " 1"},
  {2,  " 2"},
  {4,  " 4"},
  {8,  " 8"},
  {16, "16"},
  {32, "32"},
  {-1, NULL}
};

#ifdef HAVE_DIVX4LINUX
static struct STRTAB divx4linux_quality_list[] = {
  {1,  "Fastest"},
  {2,  "Fast"},
  {3,  "Medium"},
  {4,  "Slow"},
  {5,  "Slowest"},
  {-1, NULL}
};
#endif

#ifdef HAVE_FFMPEG
static struct STRTAB ffmpeg_quality_list[] = {
  {0,  "none (very low quality)"},
  {2,  "log (low quality)"},
  {3,  "phods (low quality)"},
  {4,  "EPZS (medium quality)"},
  {1,  "full (slow)"},
  {5,  "X1 (experimental)"},
  {-1, NULL}
};
#endif

#ifdef HAVE_XVID
static struct STRTAB xvid_quality_list[] = {
  {0,  "H623, No ME"},
  {1,  "MPEG, Low"},
  {2,  "H623, Low"},
  {3,  "H623, Medium"},
  {4,  "H623, High"},
  {5,  "H623, Very High"},
  {6,  "H623, Ultra High"},
  {-1, NULL}
};
#endif

#ifdef HAVE_OGG
static struct STRTAB theora_quality_list[] = {
  {0,  "0"},
  {1,  "1"},
  {2,  "2"},
  {3,  "3"},
  {4,  "4"},
  {5,  "5"},
  {6,  "6"},
  {7,  "7"},
  {8,  "8"},
  {9,  "9"},
  {-1, NULL}
};
#endif

static struct STRTAB uncompressed_quality_list[] = {
  {0, "default"},
  {-1, NULL}
};

static struct STRTAB *quality_list = NULL;

static struct STRTAB *container_list_t=NULL;
static struct STRTAB *codec_list_t=NULL, *audio_codec_list_t;

/*-------------------------------------------------------------------------*/
#define FIX_RIGHT_TOP       \
    XtNleft,XawChainRight,  \
    XtNright,XawChainRight, \
    XtNtop,XawChainTop,     \
    XtNbottom,XawChainTop

#define DIVX_WIDTH_RANGE                768.0
#define DIVX_HEIGHT_RANGE               576.0
#define DIVX_MAXSIZE_RANGE             4096.0
#define DIVX_BITRATE_RANGE            10000.0
#define DIVX_MAXGAP_RANGE               500.0
#define DIVX_AUDIOFRGMTS_RANGE          100.0
#define DIVX_AUDIOSIZEFRGMT_RANGE     65535.0
#define DIVX_DELAY_RANGE                500.0
#define DIVX_QUALITY_RANGE                5.0
#define DIVX_MINQUANTIZER_RANGE          64.0
#define DIVX_MAXQUANTIZER_RANGE          64.0
#define AUDIO_BUFFERSIZE_RANGE        65535.0

#define MP3_BITRATE_RANGE             512.0
#define MP3_QUALITY_RANGE               9.0
#define MP3_VBR_QUALITY_RANGE           9.0

/*-------------------------------------------------------------------------*/

struct CALL_ACTION call_options_5 = { 0, "Options", {NULL} };

#ifdef HAVE_FFMPEG
struct CALL_ACTION call_ffmpeg = { 0, "FFMpeg", {NULL} };
static int cb_ffmpeg = 0; // a callback can only be init once!!
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
struct CALL_ACTION call_xvid = { 0, "XviD", {NULL} };
static int cb_xvid = 0; // a callback can only be init once!!
#endif
#endif
/*--------------------------------------------------------------------------*/

void update_divx_scrollbarwidget(Widget widgeta,Widget widgetb,int value, int range, char *text)
{
  char label[64];
  if (widgetb)
    set_float (widgetb, XtNtopOfThumb, (float)value/range);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH3, text), value);
  if (widgeta)
    XtVaSetValues (widgeta, XtNlabel, label, NULL);
}

/* for some libc (SunOS 5.8) printf("%s",NULL) segfaults.... */
static char* nn(char * s) {
  return s ?s : "(null)";
}

struct STRTAB *
get_quality_list(int codec)
{
#ifdef HAVE_FFMPEG
  if(xdtv_to_ffmpeg(codec)!=-1)
    return ffmpeg_quality_list;
#endif
  switch (codec)
    {
#ifdef HAVE_DIVX4LINUX
    case DIVX4LINUX:
      return divx4linux_quality_list;
#endif
#ifdef HAVE_XVID
    case XVID:
      return xvid_quality_list;
      break;
#endif
#ifdef HAVE_OGG
    case THEORA:
      return theora_quality_list;
#endif
    case UNCOMPRESSED:
      return uncompressed_quality_list;
      break;
    default:
      return NULL;
      break;
    }
}

/*-------------------------------------------------------------------------*/

struct CALL_ACTION call_recordmovie = { 0, "RecordMovie", {NULL} };

/*-------------------------------------------------------------------------*/

// Code from mwxcd http://member.nifty.ne.jp/nmaruichi/
// the purpose is to have in Xaw a file selector to choose the path easily

static void srcSelectFinish(const char *dirName, const char *fileName)
{
	char Path[8192];
	if (fileName[0] != '\0')
	  sprintf(Path, "%s/%s", dirName, fileName);
	else
	  sprintf(Path, "%s", dirName);
	if(t_divxfilepath)
	  XtVaSetValues (t_divxfilepath, XtNstring, STRDUP(Path), NULL);
}


static void srcFileSelector(Widget w, XtPointer client_data, XtPointer call_data)
{
	char dirName[4096];
	char fileName[1024];
	extern selectorPanes filePickupPaneInfo[];
	extern selectorPanes fileSortPaneInfo[];
	static selectorPanes *defPickup = &filePickupPaneInfo[1];
	static selectorPanes *defSort = &fileSortPaneInfo[1];
	dirName[0] = '\0';
	fileName[0] = '\0';
	fileCheck(divx.filename, sizeof(dirName), dirName, sizeof(fileName), fileName);
	fileSelectorCall(w, dirName, defPickup, defSort, srcSelectFinish);
}

/*-------------------------------------------------------------------------*/

void wfc(int i)
{
        waitpid(-1, NULL, WNOHANG);
        signal(SIGCHLD, wfc);
}

void execute(Widget w, XtPointer client_data, XtPointer call_data)
{
  char *cmd = (char *)client_data;
  pthread_t thread;
  pthread_create (&thread, NULL, (void *(*)(void *)) system, cmd);
}

static void
close_divx_win (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  XtCallActionProc (divx_shell, "DivX", NULL, NULL, 0);
}

static void
close_divxaudio_win (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  XtCallActionProc (divxaudio_shell, "DivXAudio", NULL, NULL, 0);
}

static void
close_streamparamswin (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  XtCallActionProc (streamparams_shell, "StreamParams", NULL, NULL, 0);
}

void
DivXAction (Widget widget, XEvent * event,
              String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (divx_shell), XtWindow (divx_shell), NULL,
                       0);
          return;
        }
    }

  if (divx_shell == NULL)
    {
      fprintf(stderr, "DivXAction: ooops: divx_shell is NULL\n");
      popup_mess(translate("DivXAction:\nooops: divx_shell is NULL"));
      return;
    }

  if (is_divx_in_progress())
    {
      fprintf(stderr, "DivXAction: During a record, you can't reach the Video Recorder Options window\n");
      popup_mess(translate("The Video Recorder Options Window\ncan't be reached during a record\n"));
      return;
    }

  if (mapped)
    {
      XtPopdown (divx_shell);
      mapped = 0;
      if (c_record)
        XtVaSetValues (c_record, XtNlabel, translate("Record Movie     :     (Off)          R"), NULL);             
    }
  else
    {
      XtPopup (divx_shell, XtGrabNone);
      mapped = 1;
      if (c_record)
        XtVaSetValues (c_record, XtNlabel, translate("Record Movie     : (Options)      Alt+R"), NULL);
      if (first)
        {
          XSetWMProtocols (XtDisplay (divx_shell), XtWindow (divx_shell),
                           wm_protocols, 2);
          first = 0;
        }
    }
}

void
DivXAudioAction (Widget widget, XEvent * event,
              String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (divxaudio_shell), XtWindow (divxaudio_shell), NULL,
                       0);
          return;
        }
    }

  if (divxaudio_shell == NULL)
    {
      fprintf(stderr, "DivXAudioAction: ooops: divxaudio_shell is NULL\n");
      popup_mess(translate("DivXAudioAction:\nooops: divxaudio_shell is NULL"));
      return;
    }

  if (is_divx_in_progress())
    {
      fprintf(stderr, "DivXAudioAction: During a record, you can't reach the Audio Recorder Options window\n");
      popup_mess(translate("The Audio Recorder Options Window\ncan't be reached during a record\n"));
      return;
    }

  if (mapped)
    {
      XtPopdown (divxaudio_shell);
      mapped = 0;
      if (c_record)
        XtVaSetValues (c_record, XtNlabel, translate("Record Movie     :     (Off)          R"), NULL);      
    }
  else
    {
      XtPopup (divxaudio_shell, XtGrabNone);
      mapped = 1;
      if (c_record)
        XtVaSetValues (c_record, XtNlabel, translate("Record Movie     : (Options)     Ctrl+R"), NULL);
      if (first)
        {
          XSetWMProtocols (XtDisplay (divxaudio_shell), XtWindow (divxaudio_shell),
                           wm_protocols, 2);
          first = 0;
        }
    }
}

void
StreamParamsAction (Widget widget, XEvent * event, String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (streamparams_shell), XtWindow (streamparams_shell), NULL, 0);
          return;
        }
    }

  if (streamparams_shell == NULL)
    {
      fprintf(stderr, "StreamParamsAction: ooops: streamparams_shell is NULL\n");
      popup_mess(translate("StreamParamsAction:\nooops: streamparams_shell is NULL\n"));
      return;
    }

  if (mapped)
    {
      XtPopdown (streamparams_shell);
      mapped = 0;
      if (is_divx_in_progress() && c_record)
        XtVaSetValues (c_record, XtNlabel, translate("Record Movie     :      (On)          R"), NULL); 
      else if (c_record)
        XtVaSetValues (c_record, XtNlabel, translate("Record Movie     :     (Off)          R"), NULL);      
    }
  else
    {
      XtPopup (streamparams_shell, XtGrabNone);
      mapped = 1;
      if (c_record)
        XtVaSetValues (c_record, XtNlabel, translate("Record Movie     : (Options)    Shift+R"), NULL);
      if (first)
        {
          XSetWMProtocols (XtDisplay (streamparams_shell), XtWindow (streamparams_shell), wm_protocols, 2);
          first = 0;
        }
    }
}

static void
divx_menu_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int cd = (int) clientdata;
  int j;
  char label[64];
  int value;
  char *tok;
  size_t argc;

  switch (cd)
    { 
    case FILE_FORMAT:
      if (-1 != (j = popup_menu (widget, translate("Container"), container_list_t)))
        {
	  struct stat buf;
	  char* prec_pointer=NULL;

	  divx.container = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Container")),
                   int_to_str (divx.container, container_list_t));
          XtVaSetValues (c_file_format_menu, XtNlabel, label, NULL);
	  force_container(j);

	  cont_handler=containers[container];

	  update_av_codecs();

	  prec_pointer=displayed_filename;
	  displayed_filename=get_filename();
		  
	  if (stat (divx.filename, &buf) == 0 &&  S_ISDIR(buf.st_mode)) {
		  if (debug > 2)
			fprintf(stderr," divx.filename is a directory, we don't make anything !! \n");
	  }else {

		filename_malloc=1;
  
		if(t_divxfilepath){
   	  		XtVaSetValues (t_divxfilepath, XtNstring,STRDUP(displayed_filename) , NULL);
		}

	  }
	  
	  if (filename_malloc){
		free(prec_pointer);
	  }
	  filename_malloc=1;
	
	}
      break;

   
    // Video DivX Parameters window entries
    case DIVX_CODEC:
      if (-1 != (j = popup_menu (widget, translate("Codec"), translate_struct_copy(cont_handler->v_codecs))))
        {
          update_vcodec(j);
        }
      break;

    case DIVX_WMODUL:
      if (-1 != (j = popup_menu (widget, translate("W-Modul"), modulo_list)))
        {
          w_modul = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("W-Modul")), w_modul);
          XtVaSetValues (c_wmodul, XtNlabel, label, NULL);

          // Recompute new width value
          value = divx.width / w_modul;
          value = value * w_modul;
          divx.width = value;
          if (s_divxwidth)
            set_float (s_divxwidth, XtNtopOfThumb, (float)value/DIVX_WIDTH_RANGE);

          sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Width")), divx.width);
          if (l_divxwidth)
            XtVaSetValues(l_divxwidth, XtNlabel, label, NULL);
        }
      break;

    case DIVX_HMODUL:
      if (-1 != (j = popup_menu (widget, translate("H-Modul"), modulo_list)))
        {
          h_modul = j;
          sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("H-Modul")), h_modul);
          XtVaSetValues (c_hmodul, XtNlabel, label, NULL);

          // Recompute new height value
          value = divx.height / h_modul;
          value = divx.height * h_modul;
          divx.height = value;
          if (s_divxheight)
            set_float (s_divxheight, XtNtopOfThumb, (float)value/DIVX_HEIGHT_RANGE);

          sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Height")), divx.height);
          if (l_divxheight)
            XtVaSetValues(l_divxheight, XtNlabel, label, NULL);
        }
      break;

    case DIVX_QUALITY:
      if (-1 != (j = popup_menu (widget, translate("Quality"), quality_list)))
        {
          divx.quality = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Quality")),
                   int_to_str (divx.quality, quality_list));
          XtVaSetValues (c_divxquality, XtNlabel, label, NULL);
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
          if (divx.codec == XVID) {

            if (divx.quality < 4)
              {
                sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Chroma Motion Estimation")),
                         int_to_str (xvid.chromame, on_off_list));
                XtVaSetValues (c_xvid_chromame, XtNlabel, label, NULL);
                sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                         int_to_str (xvid.trell, on_off_list));
                XtVaSetValues (c_xvid_trell, XtNlabel, label, NULL);
                sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("HQ-AC Prediction")),
                         int_to_str (xvid.hqacpred, on_off_list));
                XtVaSetValues (c_xvid_hqacpred, XtNlabel, label, NULL);
              }

            if (divx.quality >= 4)
              {
                xvid.chromame = 0;
                sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Chroma Motion Estimation")),
                         translate("Already managed!"));
                XtVaSetValues(c_xvid_chromame, XtNlabel, label, NULL);}
            else {  
                sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Chroma Motion Estimation")),
                         int_to_str (xvid.chromame, on_off_list));
                XtVaSetValues(c_xvid_chromame, XtNlabel, label, NULL);}

            if (divx.quality >= 5)
              {
                xvid.trell = 0;
                sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                         translate("Already managed!"));
                XtVaSetValues(c_xvid_trell, XtNlabel, label, NULL);}
            else {  
                sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("Trellis Quantization")),
                         int_to_str (xvid.trell, on_off_list));
                XtVaSetValues(c_xvid_trell, XtNlabel, label, NULL);}

            if (divx.quality >= 6)
              {
                xvid.hqacpred = 0;
                sprintf (label, "%-*s: %18s", ALI(LAB_WIDTH5, translate("HQ-AC Prediction")),
                         translate("Already managed!"));
                XtVaSetValues(c_xvid_hqacpred, XtNlabel, label, NULL);}
            else {  
                sprintf (label, "%-*s: %5s", ALI(LAB_WIDTH5, translate("HQ-AC Prediction")),
                         int_to_str (xvid.hqacpred, on_off_list));
                XtVaSetValues(c_xvid_hqacpred, XtNlabel, label, NULL);}
          }   
#endif
#endif
        }
      break;

    case DIVX_DISPLAY_FRAMES:
      if (-1 != (j = popup_menu (widget, translate("Display Frames"), on_off_list)))
        {
          divx.display_frames = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Display Frames")),
                   int_to_str (divx.display_frames, on_off_list));
          XtVaSetValues (c_displayframes, XtNlabel, label, NULL);
        }
      break;

    case DIVX_SUB:
      if (-1 != (j = popup_menu (widget, translate("Record Subtitles"), on_off_list)))
        {
          divx.sub = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Record Subtitles")),
                   int_to_str (divx.sub, on_off_list));
          XtVaSetValues (c_sub, XtNlabel, label, NULL);
        }
      break;

    case DIVX_CHG:
      if (-1 != (j = popup_menu (widget, translate("Change Channel"), on_off_list)))
        {
          divx.chg = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Change Channel")),
                   int_to_str (divx.chg, on_off_list));
          XtVaSetValues (c_chg, XtNlabel, label, NULL);
        }
      break;
      
    // Audio DivX Parameters window entries  
    case DIVX_STEREO:
      if (-1 != (j = popup_menu (widget, translate("Stereo Mode"), stereo_list)))
        {
          divx.stereo = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Stereo Mode")),
                   int_to_str (divx.stereo, stereo_list));
          XtVaSetValues (c_stereo, XtNlabel, label, NULL);
        }
      break;   

    case DIVX_AUDIO_FREQ:
      if (-1 != (j = popup_menu (widget, translate("Audio Frequency"), audio_freq_list)))
        {
          divx.audio_freq = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Audio Frequency")),
                   int_to_str (divx.audio_freq, audio_freq_list));
          XtVaSetValues (c_audio_freq, XtNlabel, label, NULL);
        }
      break;

    case DIVX_FPS:
      if (-1 != (j = popup_menu (widget, translate("Fps"), video_fps_list)))
        {
          divx.fps = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Fps")),
                   int_to_str (divx.fps, video_fps_list));
          XtVaSetValues (c_video_fps, XtNlabel, label, NULL);
        }
      break;

    case DIVX_AUDIO_FMT:
      if (-1 != (j = popup_menu (widget, translate("Audio Format"), audio_fmt_names)))
        {
          divx.audio_fmt = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Audio Format")),
                   int_to_str (divx.audio_fmt, audio_fmt_names));
          XtVaSetValues (c_audio_fmt, XtNlabel, label, NULL);
        }
	if (int_to_str (divx.audio_fmt, audio_fmt_names) == "u8")
          {
	    update_acodec(AUDIO_UNCOMPRESSED);
          }
        else 
	{
          XtVaSetValues(c_compressaudio, XtNsensitive, True, NULL);
	}
      break;  

    case DIVX_AUDIO_CODEC:
      if (-1 != (j = popup_menu (widget, translate("Audio Codec"),  translate_struct_copy(cont_handler->a_codecs))))
        {
	  update_acodec(j);
        }
      break;

   case DIVX_MP3_VBR:
      if (-1 != (j = popup_menu (widget, translate("Bitrate Mode"), mp3_vbr_list)))
        {
          divx.mp3_vbr = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Bitrate Mode")),
                   int_to_str (divx.mp3_vbr, mp3_vbr_list));
          XtVaSetValues (c_mp3_vbr, XtNlabel, label, NULL);
	  if (divx.audio_codec!=AUDIO_UNCOMPRESSED)
	  {
          	if (j == 1)
            		{
              		XtVaSetValues(l_divxmp3bitrate, XtNsensitive, False, NULL);
              		XtVaSetValues(s_divxmp3bitrate, XtNsensitive, False, NULL);
              		XtVaSetValues(l_divxmp3vbrquality, XtNsensitive, True, NULL);
              		XtVaSetValues(s_divxmp3vbrquality, XtNsensitive, True, NULL);
            		}
          	else
            		{
              		XtVaSetValues(l_divxmp3bitrate, XtNsensitive, True, NULL);
              		XtVaSetValues(s_divxmp3bitrate, XtNsensitive, True, NULL);
              		XtVaSetValues(l_divxmp3vbrquality, XtNsensitive, False, NULL);
              		XtVaSetValues(s_divxmp3vbrquality, XtNsensitive, False, NULL);
            		}
	  }
	  else
	  {
              XtVaSetValues(l_divxmp3bitrate, XtNsensitive, False, NULL);
              XtVaSetValues(s_divxmp3bitrate, XtNsensitive, False, NULL);
              XtVaSetValues(l_divxmp3vbrquality, XtNsensitive, False, NULL);
              XtVaSetValues(s_divxmp3vbrquality, XtNsensitive, False, NULL);
	  }

        }
      break;

     // Streaming DivX Parameters window entries
     case DIVX_STREAM_ONOFF:
      if (-1 != (j = popup_menu (widget, translate("Streaming Server Activation"), on_off_list)))
        {
          if (!is_divx_in_progress())
	  {
	    divx.stream = j;
            sprintf (label, "%-*s:     %s  ", ALI(LAB_WIDTH3, translate("Streaming Server Activation")),
                   int_to_str (divx.stream, on_off_list));
            XtVaSetValues (c_stream_onoff, XtNlabel, label, NULL);
          }

	  if (debug)
	    fprintf(stderr,"Streaming Server Activation: %d\n",divx.stream);

          if (!divx.stream)
          {
	    XtVaSetValues(t_http_port_str, XtNsensitive, False, NULL);
	    XtVaSetValues(c_streaming, XtNsensitive, False, NULL);
	    XtVaSetValues(c_streamaudioopen, XtNsensitive, False, NULL);
	    XtVaSetValues(c_streamdivxopen, XtNsensitive, False, NULL);
            tok = "gray95";
	    argc = 1;
	    SetBgAction (t_http_port_str, NULL, &tok, &argc);
#ifdef HAVE_XPM
	    create_divxstream_off_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
#endif
          }
          else
          {
            XtVaSetValues(t_http_port_str, XtNsensitive, True, NULL);
	    XtVaSetValues(c_streaming, XtNsensitive, True, NULL);
	    XtVaSetValues(c_streamaudioopen, XtNsensitive, True, NULL);
	    XtVaSetValues(c_streamdivxopen, XtNsensitive, True, NULL);
            tok = "white";
	    argc = 1;
	    SetBgAction (t_http_port_str, NULL, &tok, &argc);
#ifdef HAVE_XPM
	    create_divxstream_on_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
#endif
          }
        }
      break;

    default:
      break;
    }
}

static void
divx_jump_scb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  char *name;
  int value = 0;
  float data;
  char label[64];

  data = *(float *) call_data;
  name = XtName (XtParent (widget));
#if 0
  fprintf (stderr, "jump to %f (%s/%d)\n", *(float *) call_data, name, value);
#endif
  // Video Options
  if (0 == strcmp(name, "divxwidth"))
    {
      value = (int) rint(data * DIVX_WIDTH_RANGE);
      if (value == 0)
        value = w_modul; // minimun acceptable value

      // make sure width is divisible by w_modul
      value /= w_modul;
      value *= w_modul;
      divx.width = value;

      if (s_divxwidth)
        set_float (s_divxwidth, XtNtopOfThumb, (float)value/DIVX_WIDTH_RANGE);

      sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Width")), divx.width);
      //sprintf(label, "Width: %3d", divx.width);
      if (l_divxwidth)
        XtVaSetValues(l_divxwidth, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxheight"))
    {
      value = (int) rint(data * DIVX_HEIGHT_RANGE);
      if (value == 0)
        value = h_modul; // minimun acceptable value

      // make sure height is divisible by h_modul
      value /= h_modul;
      value *= h_modul;
      divx.height = value;

      if (s_divxheight)
        set_float (s_divxheight, XtNtopOfThumb, (float)value/DIVX_HEIGHT_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Height")), divx.height);
      if (l_divxheight)
        XtVaSetValues(l_divxheight, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxbitrate"))
    {
      value = (int) rint(data * DIVX_BITRATE_RANGE);
      if (value == 0)
        value = 1;
      divx.bitrate = value*1000;

      if (s_divxbitrate)
        set_float (s_divxbitrate, XtNtopOfThumb, (float)value/DIVX_BITRATE_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Bitrate (kbit/s)")), value);
      if (l_divxbitrate)
        XtVaSetValues(l_divxbitrate, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxminquantizer"))
    {
      value = (int) (data * DIVX_MINQUANTIZER_RANGE);
      if (value == 0)
        value = 1;
      divx.min_quantizer = value;

      if (s_divxminquantizer)
        set_float (s_divxminquantizer, XtNtopOfThumb, (float)value/DIVX_MINQUANTIZER_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Min Quantizer")), divx.min_quantizer);
      if (l_divxminquantizer)
        XtVaSetValues(l_divxminquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxmaxquantizer"))
    {
      value = (int) (data * DIVX_MAXQUANTIZER_RANGE);
      if (value == 0)
        value = 1;
      divx.max_quantizer = value;

      if (s_divxmaxquantizer)
        set_float (s_divxmaxquantizer, XtNtopOfThumb, (float)value/DIVX_MINQUANTIZER_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Max Quantizer")), divx.max_quantizer);
      if (l_divxmaxquantizer)
        XtVaSetValues(l_divxmaxquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxmaxsize"))
    {
      value = (int) rint(data * DIVX_MAXSIZE_RANGE);
      divx.maxsize = value;

      if (s_divxmaxsize)
        set_float (s_divxmaxsize, XtNtopOfThumb, (float)value/DIVX_MAXSIZE_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Max. Size (Mb)")), value);
      if (l_divxmaxsize)
        XtVaSetValues(l_divxmaxsize, XtNlabel, label, NULL);
    }
  // Audio Options
  else if (0 == strcmp(name, "divxaudiobufsize"))
    {
      value = (int) (data * AUDIO_BUFFERSIZE_RANGE);
      if (value == 0)
        value = 441;
      divx.audio_buffer_size = value;

      if (s_divxaudiobufsize)
        set_float (s_divxaudiobufsize, XtNtopOfThumb, (float)value/AUDIO_BUFFERSIZE_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Audio Buffer Size")), divx.audio_buffer_size);
      if (c_divxaudiobufsize)
        XtVaSetValues(c_divxaudiobufsize, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxaudiofrgmts"))
    {
      value = (int) rint(data * DIVX_AUDIOFRGMTS_RANGE);
      if (value == 0)
        value = 1;
      divx.audio_fragments = (float) value;

      if (s_divxaudiofrgmts)
        set_float (s_divxaudiofrgmts, XtNtopOfThumb, (float)value/DIVX_AUDIOFRGMTS_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Audio Fragments")), value);
      if (c_divxaudiofrgmts)
        XtVaSetValues(c_divxaudiofrgmts, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxaudiosizefrgmt"))
    {
      value = (int) rint(data * DIVX_AUDIOSIZEFRGMT_RANGE);
      if (value == 0)
        value = 1024;
      divx.audio_sizefragment = (float) value;

      if (s_divxaudiosizefrgmt)
        set_float (s_divxaudiosizefrgmt, XtNtopOfThumb, (float)value/DIVX_AUDIOSIZEFRGMT_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Audio Size Fragment")), value);
      if (c_divxaudiosizefrgmt)
        XtVaSetValues(c_divxaudiosizefrgmt, XtNlabel, label, NULL);
    }
  // Audio Codec Options
  else if (0 == strcmp(name, "divxmp3bitrate"))
    {
      value = (int) (data * MP3_BITRATE_RANGE);
      if (value == 0)
        value = 16;
      divx.mp3_bitrate = value;

      if (s_divxmp3bitrate)
        set_float (s_divxmp3bitrate, XtNtopOfThumb, (float)value/MP3_BITRATE_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Bitrate")), divx.mp3_bitrate);
      if (l_divxmp3bitrate)
        XtVaSetValues(l_divxmp3bitrate, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxmp3vbrquality"))
    {
      value = (int) (data * MP3_VBR_QUALITY_RANGE);
      if (value == 0)
        value = 1 ;
      divx.mp3_vbr_quality = value;

      if (s_divxmp3vbrquality)
        set_float (s_divxmp3vbrquality, XtNtopOfThumb, (float)value/MP3_VBR_QUALITY_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("VBR Quality")), divx.mp3_vbr_quality);
      if (l_divxmp3vbrquality)
        XtVaSetValues(l_divxmp3vbrquality, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxmp3quality"))
    {
      value = (int) (data * MP3_QUALITY_RANGE);
      if (value == 0)
        value = 1 ;
      divx.mp3_quality = value;

      if (s_divxmp3quality)
        set_float (s_divxmp3quality, XtNtopOfThumb, (float)value/MP3_QUALITY_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Quality")), divx.mp3_quality);
      if (l_divxmp3quality)
        XtVaSetValues(l_divxmp3quality, XtNlabel, label, NULL);
    }
  // Audio/Video Options
  else if (0 == strcmp(name, "divxmaxgap"))
    {
      value = (int) rint(data * DIVX_MAXGAP_RANGE);
      if (value == 0)
        value = 1;
      divx.maxgap = (float) value/1000;

      if (s_divxmaxgap)
        set_float (s_divxmaxgap, XtNtopOfThumb, (float)value/DIVX_MAXGAP_RANGE);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Max. Gap A/V (ms)")), value);
      if (l_divxmaxgap)
        XtVaSetValues(l_divxmaxgap, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxdelay"))
    {
      value = (int) rint(data * 2 * DIVX_DELAY_RANGE - DIVX_DELAY_RANGE);
      divx.delay = value/1000.0;

      if (s_divxdelay)
        set_float (s_divxdelay, XtNtopOfThumb, (1+(float)value/DIVX_DELAY_RANGE)/2);

      sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Delay A/V (ms)")), value);

      if (debug) {
      fprintf(stderr,"\n");
      fprintf(stderr,"Value: %d\n", (int)value);
      fprintf(stderr,"Delay: %f\n", (float)divx.delay);
      }
      if (l_divxdelay)
        XtVaSetValues(l_divxdelay, XtNlabel, label, NULL);
    }
#if 0
  printf("%s: value = %d\n", name, value);
#endif
}

static void
divx_scroll_scb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int move = (int) call_data;
  Dimension length;
  float shown, top1, top2;
  char *name;
  float step;
  float range;
  name = XtName (XtParent (widget));

  XtVaGetValues (widget,
                 XtNlength, &length,
                 XtNshown, &shown, XtNtopOfThumb, &top1, NULL);

  // Video Options
  if (strcmp(name, "divxwidth") == 0)
    {
      range = DIVX_WIDTH_RANGE;
      step = w_modul;
    }
  else if (strcmp(name, "divxheight") == 0)
    {
      range = DIVX_HEIGHT_RANGE;
      step = h_modul;
    }
  else if (strcmp(name, "divxbitrate") == 0)
    {
      range = DIVX_BITRATE_RANGE;
      step = 1.0;
    }
  else if (strcmp(name, "divxminquantizer") == 0)
    {
      range = DIVX_MINQUANTIZER_RANGE;
      step = 1.0;
    }    
  else if (strcmp(name, "divxmaxquantizer") == 0)   
    {
      range = DIVX_MAXQUANTIZER_RANGE;
      step = 1.0;
    }    
      else if (strcmp(name, "divxmaxsize") == 0)
    {
      range = DIVX_MAXSIZE_RANGE;
      step = 16.0;
    }
    
    // Audio Options
  else if (strcmp(name, "divxaudiobufsize") == 0)
    {
      range = AUDIO_BUFFERSIZE_RANGE;
      step = 441.0;
    }    
   else if (strcmp(name, "divxaudiofrgmts") == 0)
    {
      range = DIVX_AUDIOFRGMTS_RANGE;
      step = 1.0;
    }
   else if (strcmp(name, "divxaudiosizefrgmt") == 0)
    {
      range = DIVX_AUDIOSIZEFRGMT_RANGE;
      step = 1024.0;
    }

  // Audio Codec Options
  else if (strcmp(name, "divxmp3bitrate") == 0)
    {
      range = MP3_BITRATE_RANGE;
      step = 16.0;
    }
  else if (strcmp(name, "divxmp3vbrquality") == 0)
    {
      range = MP3_VBR_QUALITY_RANGE;
      step = 1.0;
    }
  else if (strcmp(name, "divxmp3quality") == 0)
    {
      range = MP3_QUALITY_RANGE;
      step = 1.0;
    }
   // Audio/Video Options     
   else if (strcmp(name, "divxmaxgap") == 0)
    {
      range = DIVX_MAXGAP_RANGE;
      step = 1.0;
    }
   else if (strcmp(name, "divxdelay") == 0)
    {
      range = DIVX_DELAY_RANGE;
      step = 1.0;
    }
  else
    {
      range = 65535.0;
      step = 5.0;
    }

  //top2 = top1 + (float) move / length / 5;
  if (move > 0)
    top2 = top1 + step/range;
  else
    top2 = top1 - step/range;
  if (top2 < 0.0)
    top2 = 0.0;
  if (top2 > 1.0)
    top2 = 1.0;

#if 0
  fprintf (stderr, "name: %s, range = %f, step = %f\n", name, range, step);
  fprintf (stderr, "top1 = %f, move = %d, step/range = %f, top2 = %f\n",top1,move,step/range,top2);
  fprintf (stderr, "scroll by %d\tlength %d\tshown %f\ttop %f => %f\n",
           move, length, shown, top1, top2);
#endif

  divx_jump_scb (widget, clientdata, &top2);
}

void
create_divxwin(void)
{
  Widget c,p;
  char label[64];

  translate_struct(on_off_list);
#ifdef HAVE_DIVX4LINUX
  translate_struct(divx4linux_quality_list);
#endif
#ifdef HAVE_FFMPEG
  translate_struct(ffmpeg_quality_list);
#endif
#ifdef HAVE_XVID
  translate_struct(xvid_quality_list);
#endif
/* Nothing to translate: there is only some numbers
"________" is to avoid the do-po.sh script to discover that there is a translation to do....
#ifdef HAVE_OGG
  trans________late_struct(theora_quality_list);
#endif*/

  translate_struct(uncompressed_quality_list);
  container_list_t=translate_struct_copy(container_list);
  codec_list_t=translate_struct_copy(codec_list);
  
  divx_shell = XtVaAppCreateShell ("DivX", "XdTV",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell, XtNvisual,vinfo->visual,
				   XtNcolormap,colormap, XtNdepth,vinfo->depth,
				   NULL);

  divx_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, divx_shell, NULL);

  /* File container Label */
  c_filefmt = XtVaCreateManagedWidget ("lfileformat", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);

  /* File container Menu */
  c_file_format_menu = c = XtVaCreateManagedWidget ("fileformat", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) FILE_FORMAT);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Container")),
           int_to_str (divx.container, container_list_t));
  XtVaSetValues (c, XtNlabel, label, NULL);
  force_container(divx.container);
  cont_handler=containers[container];

  /* DivX Codec Label */
  c_divxvideocodec = XtVaCreateManagedWidget ("ldivxcodec", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);

  /* Divx Codec Menu */
  c_divxcodec = c = XtVaCreateManagedWidget ("divxcodec", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_CODEC);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Codec")),
           int_to_str (divx.codec, codec_list_t));
  quality_list = get_quality_list(divx.codec);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* More Codec Advanced Options */
  c_codecaopt = c = XtVaCreateManagedWidget ("ldivxcodecaopt", commandWidgetClass, divx_paned, PANED_FIX, NULL);
#ifdef HAVE_FFMPEG
    if (
#ifdef HAVE_XVID
        divx.codec != XVID &&
#endif
#ifdef HAVE_DIVX4LINUX
        divx.codec != DIVX4LINUX &&
#endif
#ifdef HAVE_OGG
        divx.codec != THEORA &&
#endif
        divx.codec != UNCOMPRESSED) {
      XtAddCallback (c_codecaopt, XtNcallback, action_cb, (XtPointer) & call_ffmpeg);
      cb_ffmpeg = 1;
    }
#endif
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
if (divx.codec == XVID) {
      XtAddCallback (c_codecaopt, XtNcallback, action_cb, (XtPointer) & call_xvid);
      cb_xvid = 1;
}
#endif
#endif

  /* DivX Size Label */
  c_divxsize = XtVaCreateManagedWidget ("ldivxsize", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);

  /* DivX W-Modul Slider */
  c_wmodul = c = XtVaCreateManagedWidget ("wmodul", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_WMODUL);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("W-Modul")), w_modul);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* DivX Width Slider */
  p =
    XtVaCreateManagedWidget ("divxwidth", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxwidth = XtVaCreateManagedWidget ("ldivxwidth", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxwidth = XtVaCreateManagedWidget ("sdivxwidth", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxwidth, XtNtopOfThumb, (float)divx.width/DIVX_WIDTH_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Width")), divx.width);
  XtVaSetValues(l_divxwidth, XtNlabel, label, NULL);

  XtAddCallback (s_divxwidth, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxwidth, XtNscrollProc, divx_scroll_scb, NULL);

  /* DivX H-Modul Slider */
  c_hmodul = c = XtVaCreateManagedWidget ("hmodul", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_HMODUL);
  sprintf (label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("H-Modul")), h_modul);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* DivX Height Slider */
  p =
    XtVaCreateManagedWidget ("divxheight", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxheight = XtVaCreateManagedWidget ("ldivxheight", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxheight = XtVaCreateManagedWidget ("sdivxheight", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxheight, XtNtopOfThumb, (float)divx.height/DIVX_HEIGHT_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Height")), divx.height);
  XtVaSetValues(l_divxheight, XtNlabel, label, NULL);

  XtAddCallback (s_divxheight, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxheight, XtNscrollProc, divx_scroll_scb, NULL);

   /* DivX Bitrate Slider */
  p =
    XtVaCreateManagedWidget ("divxbitrate", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxbitrate = XtVaCreateManagedWidget ("ldivxbitrate", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxbitrate = XtVaCreateManagedWidget ("sdivxbitrate", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxbitrate, XtNtopOfThumb, (float)divx.bitrate/1000/DIVX_BITRATE_RANGE);
  sprintf(label, "%-*s: %5ld", ALI(LAB_WIDTH3, translate("Bitrate (kbit/s)")), divx.bitrate/1000);
  XtVaSetValues(l_divxbitrate, XtNlabel, label, NULL);

  XtAddCallback (s_divxbitrate, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxbitrate, XtNscrollProc, divx_scroll_scb, NULL);

  /* DivX FPS Slider */
  c_video_fps = c =
    XtVaCreateManagedWidget ("divxfps", commandWidgetClass,
                             divx_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_FPS);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Fps")),
           int_to_str (divx.fps, video_fps_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* DivX Quality Menu */
  c_divxquality = c = XtVaCreateManagedWidget ("divxquality", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_QUALITY);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Quality")),
           translate(nn(int_to_str (divx.quality, quality_list))));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* DivX min Quantizer Slider */
  p =
    XtVaCreateManagedWidget ("divxminquantizer", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxminquantizer = XtVaCreateManagedWidget ("ldivxminquantizer", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxminquantizer = XtVaCreateManagedWidget ("sdivxminquantizer", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxminquantizer, XtNtopOfThumb, (float)divx.min_quantizer/DIVX_MINQUANTIZER_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Min Quantizer")), divx.min_quantizer);
  XtVaSetValues(l_divxminquantizer, XtNlabel, label, NULL);

  XtAddCallback (s_divxminquantizer, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxminquantizer, XtNscrollProc, divx_scroll_scb, NULL);

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
if (divx.codec == XVID) {
  XtVaSetValues(l_divxminquantizer, XtNsensitive, False, NULL);
  XtVaSetValues(s_divxminquantizer, XtNsensitive, False, NULL);}
#endif
#endif

  /* DivX max Quantizer Slider */
  p =
    XtVaCreateManagedWidget ("divxmaxquantizer", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmaxquantizer = XtVaCreateManagedWidget ("ldivxmaxquantizer", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmaxquantizer = XtVaCreateManagedWidget ("sdivxmaxquantizer", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmaxquantizer, XtNtopOfThumb, (float)divx.max_quantizer/DIVX_MAXQUANTIZER_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Max Quantizer")), divx.max_quantizer);
  XtVaSetValues(l_divxmaxquantizer, XtNlabel, label, NULL);

  XtAddCallback (s_divxmaxquantizer, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmaxquantizer, XtNscrollProc, divx_scroll_scb, NULL);
  
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
if (divx.codec == XVID) {
  XtVaSetValues(l_divxmaxquantizer, XtNsensitive, False, NULL);
  XtVaSetValues(s_divxmaxquantizer, XtNsensitive, False, NULL);}
#endif
#endif

  /* Other Options Label */
  c_divxdisplay = XtVaCreateManagedWidget ("ldivxdisplay", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);

   /* DivX Max. Size */
  p =
    XtVaCreateManagedWidget ("divxmaxsize", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmaxsize = XtVaCreateManagedWidget ("ldivxmaxsize", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmaxsize = XtVaCreateManagedWidget ("sdivxmaxsize", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmaxsize, XtNtopOfThumb, (float)divx.maxsize/DIVX_MAXSIZE_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Max. Size (Mb)")), divx.maxsize);
  XtVaSetValues(l_divxmaxsize, XtNlabel, label, NULL);

  XtAddCallback (s_divxmaxsize, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmaxsize, XtNscrollProc, divx_scroll_scb, NULL);
  
  /* DivX Display Frames Menu */
  c_displayframes = c = XtVaCreateManagedWidget ("divxdisplay", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_DISPLAY_FRAMES);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Display Frames")),
           int_to_str (divx.display_frames, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* Record Subtitles Menu */
  c_sub = c = XtVaCreateManagedWidget ("divxsub", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_SUB);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Record Subtitles")),
           int_to_str (divx.sub, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* Change channel during record Menu */
  c_chg = c = XtVaCreateManagedWidget ("divxchg", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_CHG);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Change Channel")),
           int_to_str (divx.chg, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  c_divxschedule = c = XtVaCreateManagedWidget ("ldivxschedule", commandWidgetClass, divx_paned, PANED_FIX, NULL);
  signal(SIGCHLD, wfc);
  XtAddCallback (c, XtNcallback, execute, translate("xterm -geometry 80x35+0+0 -title 'Schedule a Record: (Need AT & CRON)' -e sh xdtv_record.sh"));
  
  c_divxplayer = c = XtVaCreateManagedWidget ("ldivxplayer", commandWidgetClass, divx_paned, PANED_FIX, NULL);
  signal(SIGCHLD, wfc);
  XtAddCallback (c, XtNcallback, execute, lastfilename);
  
  /* select the divx.filename through a GUI */

  c_divxpath= c = XtVaCreateManagedWidget ("ldivxfilepath", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);  
  c_divxpath_selector = c = XtVaCreateManagedWidget ("ldivxfilepathselector", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback(c, XtNcallback, srcFileSelector, NULL);  
    
  /* DivX file path */

  t_divxfilepath = c = XtVaCreateManagedWidget ("divxfilepath", asciiTextWidgetClass, divx_paned,
                                                PANED_FIX, XtNtype, XawAsciiString, NULL);
  if (divx.filename == NULL) divx.filename = getcwd(NULL,0);

  XtVaSetValues (t_divxfilepath, XtNstring, STRDUP(divx.filename), NULL);
  

/* Open DivX Audio Menu */

  c_divxaudioopen = XtVaCreateManagedWidget ("ldivxaudioopen", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c_divxaudioopen, XtNcallback, close_divxaudio_win, (XtPointer) NULL);

/* Open Streaming Menu */

  c_divxstreamopen = XtVaCreateManagedWidget ("lstreamopen", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c_divxstreamopen, XtNcallback, close_streamparamswin, (XtPointer) NULL);
  
/* Open Main Options Menu */
  c_divxopenmain = XtVaCreateManagedWidget ("lopenmain", commandWidgetClass, divx_paned, PANED_FIX, NULL);
    XtAddCallback (c_divxopenmain, XtNcallback, action_cb, (XtPointer) & call_options_5);  

/* Button to close DivX Menu */

  c_divxsave = XtVaCreateManagedWidget ("ldivxsave", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c_divxsave, XtNcallback, save_cb, (XtPointer) NULL);
  XtAddCallback (c_divxsave, XtNcallback, close_divx_win, (XtPointer) NULL);

#ifdef HAVE_XPM
  XpmCreatePixmapFromData(XtDisplay(divx_shell),
                          RootWindowOfScreen(XtScreen(divx_shell)),
                          icon_xpm,
                          &icon_pixmap,
                          &icon_shapemask, NULL);
  XtVaSetValues(divx_shell, XtNiconPixmap, icon_pixmap, NULL);

  create_divx_pixmaps(XtDisplay(divx_paned),&(RootWindowOfScreen(XtScreen(divx_paned))));
  
#endif

}

void
create_divxaudiowin(void)
{
  Widget c,p;
  char label[64];
#ifdef HAVE_XPM  
  Pixmap icon_pixmap;
  Pixmap icon_shapemask;
#endif

  translate_struct(on_off_list);
  translate_struct(stereo_list);
  translate_struct(mp3_vbr_list);
  audio_codec_list_t = translate_struct_copy(audio_codec_list);

  divxaudio_shell = XtVaAppCreateShell ("DivXAudio", "XdTV",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell, XtNvisual,vinfo->visual,
				   XtNcolormap,colormap, XtNdepth,vinfo->depth,
				   NULL);

  divxaudio_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, divxaudio_shell, NULL);


  /* AVI Audio Label */
  c_divxaudio = XtVaCreateManagedWidget ("ldivxaudio", labelWidgetClass, divxaudio_paned,
                           PANED_FIX, NULL);

  /* Select stereo mode */
  c_stereo = c =
    XtVaCreateManagedWidget ("divxstereo", commandWidgetClass,
                             divxaudio_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_STEREO);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Stereo Mode")),
           int_to_str (divx.stereo, stereo_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
  
    /* Select the audio frequency */
  c_audio_freq = c =
    XtVaCreateManagedWidget ("divxaudiofreq", commandWidgetClass,
                             divxaudio_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_AUDIO_FREQ);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Audio Frequency")),
           int_to_str (divx.audio_freq, audio_freq_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

     /* Select the audio Format */
  c_audio_fmt = c =
    XtVaCreateManagedWidget ("divxaudiofmt", commandWidgetClass,
                             divxaudio_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_AUDIO_FMT);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Audio Format")),
           int_to_str (divx.audio_fmt, audio_fmt_names));
  XtVaSetValues (c, XtNlabel, label, NULL);
     
  /* Audio Buffer Size Slider */
  p =
    XtVaCreateManagedWidget ("divxaudiobufsize", panedWidgetClass, divxaudio_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  c_divxaudiobufsize = XtVaCreateManagedWidget ("ldivxaudiobufsize", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxaudiobufsize = XtVaCreateManagedWidget ("sdivxaudiobufsize", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxaudiobufsize, XtNtopOfThumb, (float)divx.audio_buffer_size/AUDIO_BUFFERSIZE_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Audio Buffer Size")), divx.audio_buffer_size);
  XtVaSetValues(c_divxaudiobufsize, XtNlabel, label, NULL);

  XtAddCallback (s_divxaudiobufsize, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxaudiobufsize, XtNscrollProc, divx_scroll_scb, NULL);

  /* DivX Audio Fragments Slider */
  p =
    XtVaCreateManagedWidget ("divxaudiofrgmts", panedWidgetClass, divxaudio_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  c_divxaudiofrgmts = XtVaCreateManagedWidget ("ldivxaudiofrgmts", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxaudiofrgmts = XtVaCreateManagedWidget ("sdivxaudiofrgmts", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxaudiofrgmts, XtNtopOfThumb, (float)divx.audio_fragments/DIVX_AUDIOFRGMTS_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Audio Fragments")), divx.audio_fragments);
  XtVaSetValues(c_divxaudiofrgmts, XtNlabel, label, NULL);

  XtAddCallback (s_divxaudiofrgmts, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxaudiofrgmts, XtNscrollProc, divx_scroll_scb, NULL);

  /* DivX Audio Size Fragment Slider */
  p =
    XtVaCreateManagedWidget ("divxaudiosizefrgmt", panedWidgetClass, divxaudio_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  c_divxaudiosizefrgmt = XtVaCreateManagedWidget ("ldivxaudiosizefrgmt", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxaudiosizefrgmt = XtVaCreateManagedWidget ("sdivxaudiosizefrgmt", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxaudiosizefrgmt, XtNtopOfThumb, (float)divx.audio_sizefragment/DIVX_AUDIOSIZEFRGMT_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Audio Size Fragment")), divx.audio_sizefragment);
  XtVaSetValues(c_divxaudiosizefrgmt, XtNlabel, label, NULL);
  XtAddCallback (s_divxaudiosizefrgmt, XtNjumpProc, divx_jump_scb, NULL);
  XtAddCallback (s_divxaudiosizefrgmt, XtNscrollProc, divx_scroll_scb, NULL);
  
  /* AVI Audio Label */
  c_divxaudiomp3 = XtVaCreateManagedWidget ("ldivxaudiomp3", labelWidgetClass, divxaudio_paned,
                           PANED_FIX, NULL);

  /* Select compressed audio */
  c_compressaudio = c =
    XtVaCreateManagedWidget ("divxcompressedaudio", commandWidgetClass,
                             divxaudio_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_AUDIO_CODEC);
  if (int_to_str (divx.audio_fmt, audio_fmt_names) == "u8")
  {
    divx.audio_codec = AUDIO_UNCOMPRESSED;
    XtVaSetValues(c_compressaudio, XtNsensitive, False, NULL);
  }
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Audio Codec")),
           int_to_str (divx.audio_codec, audio_codec_list_t));
  XtVaSetValues (c, XtNlabel, label, NULL);


  /* Select bitrate mode */
  c_mp3_vbr = c =
    XtVaCreateManagedWidget ("divxmp3vbr", commandWidgetClass,
                             divxaudio_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_MP3_VBR);
  sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Bitrate Mode")),
           int_to_str (divx.mp3_vbr, mp3_vbr_list));
  XtVaSetValues (c, XtNlabel, label, NULL);
    if (divx.audio_codec==AUDIO_UNCOMPRESSED)
      XtVaSetValues(c_mp3_vbr, XtNsensitive, False, NULL);


  /* MP3 Constant Bitrate Slider */
  p_divxmp3bitrate = p =
    XtVaCreateManagedWidget ("divxmp3bitrate", panedWidgetClass, divxaudio_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmp3bitrate = XtVaCreateManagedWidget ("ldivxmp3bitrate", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmp3bitrate = XtVaCreateManagedWidget ("sdivxmp3bitrate", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmp3bitrate, XtNtopOfThumb, (float)divx.mp3_bitrate/MP3_BITRATE_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Bitrate")), divx.mp3_bitrate);
  XtVaSetValues(l_divxmp3bitrate, XtNlabel, label, NULL);
  if (divx.mp3_vbr || divx.audio_codec==AUDIO_UNCOMPRESSED)
    {
	  XtVaSetValues(l_divxmp3bitrate, XtNsensitive, False, NULL);
          XtVaSetValues(s_divxmp3bitrate, XtNsensitive, False, NULL);
    }
  XtAddCallback (s_divxmp3bitrate, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmp3bitrate, XtNscrollProc, divx_scroll_scb, NULL); 
  
  /* MP3 VBR Quality Slider */
  p_divxmp3vbrquality = p =
    XtVaCreateManagedWidget ("divxmp3vbrquality", panedWidgetClass, divxaudio_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmp3vbrquality = XtVaCreateManagedWidget ("ldivxmp3vbrquality", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmp3vbrquality = XtVaCreateManagedWidget ("sdivxmp3vbrquality", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmp3vbrquality, XtNtopOfThumb, (float)divx.mp3_vbr_quality/MP3_VBR_QUALITY_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("VBR Quality")), divx.mp3_vbr_quality);
  XtVaSetValues(l_divxmp3vbrquality, XtNlabel, label, NULL);
  if (!divx.mp3_vbr || divx.audio_codec==AUDIO_UNCOMPRESSED)
    {
	  XtVaSetValues(l_divxmp3vbrquality, XtNsensitive, False, NULL);
          XtVaSetValues(s_divxmp3vbrquality, XtNsensitive, False, NULL);
    }
  XtAddCallback (s_divxmp3vbrquality, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmp3vbrquality, XtNscrollProc, divx_scroll_scb, NULL);

  /* MP3 Quality Slider */
  p_divxmp3quality = p =
    XtVaCreateManagedWidget ("divxmp3quality", panedWidgetClass, divxaudio_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmp3quality = XtVaCreateManagedWidget ("ldivxmp3quality", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmp3quality = XtVaCreateManagedWidget ("sdivxmp3quality", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmp3quality, XtNtopOfThumb, (float)divx.mp3_quality/MP3_QUALITY_RANGE);
  sprintf(label, "%-*s: %5d", ALI(LAB_WIDTH3, translate("Quality")), divx.mp3_quality);
  XtVaSetValues(l_divxmp3quality, XtNlabel, label, NULL);
  if (divx.audio_codec==AUDIO_UNCOMPRESSED)
    {
	  XtVaSetValues(l_divxmp3quality, XtNsensitive, False, NULL);
          XtVaSetValues(s_divxmp3quality, XtNsensitive, False, NULL);
    }
  XtAddCallback (s_divxmp3quality, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmp3quality, XtNscrollProc, divx_scroll_scb, NULL);

  /* AVI Audio Label */
  c_divxaudiovideo = XtVaCreateManagedWidget ("ldivxaudiovideo", labelWidgetClass, divxaudio_paned,
                           PANED_FIX, NULL);

  /* DivX MaxGap Slider */
  p =
    XtVaCreateManagedWidget ("divxmaxgap", panedWidgetClass, divxaudio_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmaxgap = XtVaCreateManagedWidget ("ldivxmaxgap", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmaxgap = XtVaCreateManagedWidget ("sdivxmaxgap", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmaxgap, XtNtopOfThumb, (float)divx.maxgap*1000/DIVX_MAXGAP_RANGE);
  sprintf(label, "%-*s: %5ld", ALI(LAB_WIDTH3, translate("Max. Gap A/V (ms)")), (unsigned long)((float)(divx.maxgap*1000)));
  XtVaSetValues(l_divxmaxgap, XtNlabel, label, NULL);

  XtAddCallback (s_divxmaxgap, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmaxgap, XtNscrollProc, divx_scroll_scb, NULL);

/* DivX delay */
  p =
    XtVaCreateManagedWidget ("divxdelay", panedWidgetClass, divxaudio_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxdelay = XtVaCreateManagedWidget ("ldivxdelay", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxdelay = XtVaCreateManagedWidget ("sdivxdelay", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxdelay, XtNtopOfThumb, (1+divx.delay*1000/DIVX_DELAY_RANGE)/2);
  sprintf(label, "%-*s: %5ld", ALI(LAB_WIDTH3, translate("Delay A/V (ms)")), (unsigned long)((float)(divx.delay*1000)));
  if (debug) {
  fprintf(stderr,"Delay: %f\n", (float)divx.delay);
  }
  XtVaSetValues(l_divxdelay, XtNlabel, label, NULL);

  XtAddCallback (s_divxdelay, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxdelay, XtNscrollProc, divx_scroll_scb, NULL);

/* Open DivX Video Menu */

  c_audiodivxopen = XtVaCreateManagedWidget ("ldivxopen", commandWidgetClass, divxaudio_paned, PANED_FIX, NULL);
  XtAddCallback (c_audiodivxopen, XtNcallback, close_divx_win, (XtPointer) NULL);

/* Open Streaming Menu */

  c_audiostreamopen = XtVaCreateManagedWidget ("lstreamopen", commandWidgetClass, divxaudio_paned, PANED_FIX, NULL);
  XtAddCallback (c_audiostreamopen, XtNcallback, close_streamparamswin, (XtPointer) NULL);
  
/* Open Main Options Menu */
  c_audioopenmain = XtVaCreateManagedWidget ("lopenmain", commandWidgetClass, divxaudio_paned, PANED_FIX, NULL);
    XtAddCallback (c_audioopenmain, XtNcallback, action_cb, (XtPointer) & call_options_5);   

/* Button to close DivXAudio Menu */

  c_audiosave = XtVaCreateManagedWidget ("ldivxsave", commandWidgetClass, divxaudio_paned, PANED_FIX, NULL);
  XtAddCallback (c_audiosave, XtNcallback, save_cb, (XtPointer) NULL);
  XtAddCallback (c_audiosave, XtNcallback, close_divxaudio_win, (XtPointer) NULL);

#ifdef HAVE_XPM
  XpmCreatePixmapFromData(XtDisplay(divxaudio_shell),
                          RootWindowOfScreen(XtScreen(divxaudio_shell)),
                          icon_xpm,
                          &icon_pixmap,
                          &icon_shapemask, NULL);
  XtVaSetValues(divxaudio_shell, XtNiconPixmap, icon_pixmap, NULL);

  create_divxaudio_pixmaps(XtDisplay(divxaudio_paned),&(RootWindowOfScreen(XtScreen(divxaudio_paned))));
  
#endif


  
}

void
create_streamparamswin()
{
  Widget c;
  char label[64];
#ifdef HAVE_XPM  
  Pixmap icon_pixmap;
  Pixmap icon_shapemask;
#endif

  streamparams_shell = XtVaAppCreateShell ("StreamParams", "XdTV",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell, XtNvisual,vinfo->visual,
				   XtNcolormap,colormap, XtNdepth,vinfo->depth,
				   NULL);
  streamparams_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, streamparams_shell, NULL);

/* Streaming Options Label */
  c_stropts = XtVaCreateManagedWidget ("lstropts", labelWidgetClass, streamparams_paned,
                           PANED_FIX, NULL);

  /* Streaming activation Menu */
  c_stream_onoff = c = XtVaCreateManagedWidget ("Streaming_Server", commandWidgetClass, streamparams_paned,
                               PANED_FIX, NULL);
    XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_STREAM_ONOFF);
      sprintf (label, "%-*s:     %s  ", ALI(LAB_WIDTH3, translate("Streaming Server Activation")),
           int_to_str (divx.stream, on_off_list));
      XtVaSetValues (c, XtNlabel, label, NULL);


/* Http Port Label */
  c_http_port = c = XtVaCreateManagedWidget ("lhttp_port", labelWidgetClass, streamparams_paned,
                           PANED_FIX, NULL);
  t_http_port_str = c = XtVaCreateManagedWidget ("http_port", asciiTextWidgetClass, streamparams_paned,
                                                PANED_FIX, XtNtype, XawAsciiString, NULL);
  if (!divx.stream)
    XtVaSetValues(t_http_port_str, XtNsensitive, False, NULL);

  sprintf (http_port, "%d", divx.http_port);
  XtVaSetValues (t_http_port_str, XtNstring, STRDUP(http_port), NULL);


/* launch streaming Action */
  c_streaming = c = XtVaCreateManagedWidget ("lstreaming", commandWidgetClass, streamparams_paned, PANED_FIX, NULL);
  //XtAddCallback (c, XtNcallback, execute, "xterm");
  XtAddCallback (c, XtNcallback, action_cb, (XtPointer) & call_recordmovie);
    if (!divx.stream)
      XtVaSetValues(c_streaming, XtNsensitive, False, NULL);

/* Open DivX Video Menu */
  c_streamdivxopen = XtVaCreateManagedWidget ("ldivxopen", commandWidgetClass, streamparams_paned,
                               PANED_FIX, NULL);
    XtAddCallback (c_streamdivxopen, XtNcallback, close_divx_win, (XtPointer) NULL);
    if (!divx.stream)
      XtVaSetValues(c_streamdivxopen, XtNsensitive, False, NULL);


/* Open DivX Audio Menu */
  c_streamaudioopen = XtVaCreateManagedWidget ("ldivxaudioopen", commandWidgetClass, streamparams_paned,
                               PANED_FIX, NULL);
    XtAddCallback (c_streamaudioopen, XtNcallback, close_divxaudio_win, (XtPointer) NULL);
    if (!divx.stream)
      XtVaSetValues(c_streamaudioopen, XtNsensitive, False, NULL);

    
/* Open Main Options Menu */
  c_streamopenmain = XtVaCreateManagedWidget ("lopenmain", commandWidgetClass, streamparams_paned, PANED_FIX, NULL);
    XtAddCallback (c_streamopenmain, XtNcallback, action_cb, (XtPointer) & call_options_5);  
        

/* Button to apply & save Stream Parameters GUI */
  c_streamparamssave = XtVaCreateManagedWidget ("lstreamparamssave", commandWidgetClass, streamparams_paned, PANED_FIX, NULL);
  XtAddCallback (c_streamparamssave, XtNcallback, save_cb, (XtPointer) NULL);


/* Button to close Stream Parameters GUI */
  c_streamparamsclose = XtVaCreateManagedWidget ("lstreamparamsclose", commandWidgetClass, streamparams_paned, PANED_FIX, NULL);
  XtAddCallback (c_streamparamsclose, XtNcallback, close_streamparamswin, (XtPointer) NULL);

#ifdef HAVE_XPM
  XpmCreatePixmapFromData(XtDisplay(streamparams_shell),
                          RootWindowOfScreen(XtScreen(streamparams_shell)),
                          icon_xpm,
                          &icon_pixmap,
                          &icon_shapemask, NULL);
  XtVaSetValues(streamparams_shell, XtNiconPixmap, icon_pixmap, NULL);
#endif

  create_divxstream_common_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));

  if (!divx.stream)
   create_divxstream_off_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
  else
   create_divxstream_on_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));

}

int divxstream_is_enabled(void)
{
	return divx.stream;
}

void update_divxfilename(void)
{
  if(t_divxfilepath)
    XtVaGetValues (t_divxfilepath, XtNstring, &divx.filename, NULL);
}

void update_http_port(void)
{
  if(!t_http_port_str) return;
  XtVaGetValues (t_http_port_str, XtNstring, &http_port_str, NULL);
  sscanf (http_port_str, "%d", &divx.http_port);
  if (debug)
    fprintf(stderr,"divx.http_port : %d\n",divx.http_port);
}

void update_vcodec(int j){
	  char label[64];
          divx.codec = j;
	  if (!label) return;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, "Codec"),
                   int_to_str (divx.codec, codec_list_t));
          XtVaSetValues (c_divxcodec, XtNlabel, label, NULL);
          quality_list = get_quality_list(divx.codec);
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
          if (divx.codec != XVID) {
            XtVaSetValues(l_divxminquantizer, XtNsensitive, True, NULL);
            XtVaSetValues(s_divxminquantizer, XtNsensitive, True, NULL);
            XtVaSetValues(l_divxmaxquantizer, XtNsensitive, True, NULL);
            XtVaSetValues(s_divxmaxquantizer, XtNsensitive, True, NULL);}
          else {
            XtVaSetValues(l_divxminquantizer, XtNsensitive, False, NULL);
            XtVaSetValues(s_divxminquantizer, XtNsensitive, False, NULL);
            XtVaSetValues(l_divxmaxquantizer, XtNsensitive, False, NULL);
            XtVaSetValues(s_divxmaxquantizer, XtNsensitive, False, NULL);} 
#endif
#endif
#ifdef HAVE_XVID
#ifdef HAVE_XVID10
      if(cb_xvid) {
        XtRemoveCallback (c_codecaopt, XtNcallback, action_cb, (XtPointer) & call_xvid);
        cb_xvid = 0;}     
#endif
#endif

#ifdef HAVE_FFMPEG
    if (
#ifdef HAVE_XVID
        divx.codec == XVID ||
#endif
#ifdef HAVE_DIVX4LINUX
        divx.codec == DIVX4LINUX ||
#endif
#ifdef HAVE_OGG
        divx.codec == THEORA ||
#endif
        divx.codec == UNCOMPRESSED) {
      XtRemoveCallback (c_codecaopt, XtNcallback, action_cb, (XtPointer) & call_ffmpeg);
      cb_ffmpeg = 0;
    }

    if (cb_ffmpeg == 0 && 
#ifdef HAVE_XVID
        divx.codec != XVID &&
#endif
#ifdef HAVE_DIVX4LINUX
        divx.codec != DIVX4LINUX &&
#endif
#ifdef HAVE_OGG
        divx.codec != THEORA &&
#endif
        divx.codec != UNCOMPRESSED) {
      XtAddCallback (c_codecaopt, XtNcallback, action_cb, (XtPointer) & call_ffmpeg);
      cb_ffmpeg = 1;
    }
#endif

#ifdef HAVE_XVID
#ifdef HAVE_XVID10
if (divx.codec == XVID) {    
    XtAddCallback (c_codecaopt, XtNcallback, action_cb, (XtPointer) & call_xvid);
    cb_xvid = 1;}
#endif
#endif
}

void update_acodec(int j){
	char label[64];
          divx.audio_codec = j;
          sprintf (label, "%-*s:       %s", ALI(LAB_WIDTH3, translate("Audio Codec")),
                   int_to_str (divx.audio_codec, audio_codec_list_t));
          XtVaSetValues (c_compressaudio, XtNlabel, label, NULL);
          if (j == AUDIO_UNCOMPRESSED)
            {
              XtVaSetValues(l_divxmp3bitrate, XtNsensitive, False, NULL);
              XtVaSetValues(s_divxmp3bitrate, XtNsensitive, False, NULL);
              XtVaSetValues(l_divxmp3quality, XtNsensitive, False, NULL);
              XtVaSetValues(s_divxmp3quality, XtNsensitive, False, NULL);
              XtVaSetValues(l_divxmp3vbrquality, XtNsensitive, False, NULL);
              XtVaSetValues(s_divxmp3vbrquality, XtNsensitive, False, NULL);
              XtVaSetValues(c_mp3_vbr, XtNsensitive, False, NULL);
            }
          else
            {
              XtVaSetValues(l_divxmp3bitrate, XtNsensitive, True, NULL);
              XtVaSetValues(s_divxmp3bitrate, XtNsensitive, True, NULL);
              XtVaSetValues(l_divxmp3quality, XtNsensitive, True, NULL);
              XtVaSetValues(s_divxmp3quality, XtNsensitive, True, NULL);
              XtVaSetValues(c_mp3_vbr, XtNsensitive, True, NULL);
              if (divx.mp3_vbr == 1)
                {
                  XtVaSetValues(l_divxmp3bitrate, XtNsensitive, False, NULL);
                  XtVaSetValues(s_divxmp3bitrate, XtNsensitive, False, NULL);
                  XtVaSetValues(l_divxmp3vbrquality, XtNsensitive, True, NULL);
                  XtVaSetValues(s_divxmp3vbrquality, XtNsensitive, True, NULL);
                }
              else
                {
                  XtVaSetValues(l_divxmp3bitrate, XtNsensitive, True, NULL);
                  XtVaSetValues(s_divxmp3bitrate, XtNsensitive, True, NULL);
                  XtVaSetValues(l_divxmp3vbrquality, XtNsensitive, False, NULL);
                  XtVaSetValues(s_divxmp3vbrquality, XtNsensitive, False, NULL);
                }
              }
}

static int search_and_update_codec_in_list(int old_codec_id,const struct STRTAB* codec_list){
	int i=0;
	if (!codec_list) return -1;
	// first we search if the (old) codec is in the new list or not ...
	while(codec_list[i].nr!=-1){
		if (codec_list[i].nr == old_codec_id)return old_codec_id; 
		i++;
	}
	// codec is not in the new list : we take the first one in the list ...
	// even if there is no codec (?!?) we should have {-1,null} in the list ...
	return codec_list[0].nr;
}
void update_av_codecs(void){

	int prec_vcodec=divx.codec;
	int prec_acodec=divx.audio_codec;
	int new_vcodec=search_and_update_codec_in_list(prec_vcodec,cont_handler->v_codecs);
	int new_acodec=search_and_update_codec_in_list(prec_acodec,cont_handler->a_codecs);

	update_vcodec(new_vcodec);
	update_acodec(new_acodec);
}
