/*****************************************************************************
 * dvb_init_ui.c: DVB init GUI
 *****************************************************************************
 * $Id: dvb_init_ui.c,v 1.13 2007/01/29 12:29:26 estre Exp $
 *****************************************************************************
 * Copyright (C) 2006 Pingus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <dlfcn.h>
#include <pthread.h>

#include "config.h"

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Scrollbar.h>

#ifdef HAVE_DVB
#include "xdtv.h"
#include "grab.h"
#include "x11.h"
#include "toolbox.h"

# include "popup_ui.h"

// internationalization
#include "translation.h"
#include "pixmaps_loader.h"

#ifdef HAVE_XPM
# include <X11/xpm.h>
#endif

#ifdef HAVE_XPM
# include "xdtv.xpm"
#endif

#include "dvbtools.h"
#include "dvb/scan.h"
#endif

/*-------------------------------------------------------------------------*/
// widget which are not static are used into the xdtv pixmaps lib
Widget dvbinit_shell; // call in conf.c
Widget dvbinit_paned;
Widget c_dvbinit_action;
Widget c_dvbinit_label;
Widget c_dvbinit_close;
Widget c_dvbinit_rc;

#ifdef HAVE_DVB
extern Widget app_shell;
extern Display *dpy;
extern Atom wm_protocols[2];
extern int debug;

static Widget c_dvbinit_country;
static Widget c_dvbinit_city;
static Widget c_dvbinit_range;
static Widget c_dvbinit_offset;

int dvbinit_country;
int dvbinit_city;
static int dvbinit_range;
int dvbinit_offset = 0;

/*------------------------------------------------------------------------*/

enum {
  DVBINIT_COUNTRY,
  DVBINIT_CITY,
  DVBINIT_RANGE,
  DVBINIT_OFFSET
};

#define MAX_CITY_NUM_IN_LIST 45
#define DVBINIT_OFFSET_RANGE 500.0

static  transp_file** all_files=NULL;
static int country_city_are_ok=0;
static struct STRTAB* country_list= NULL ;
static struct STRTAB country_list_dummy[] = {
  {-1,NULL}
};
static struct STRTAB* city_list=NULL; 
static struct STRTAB* partial_city_list=NULL; 
static struct STRTAB city_list_dummy[] = {
  {-1,NULL}
};

static struct STRTAB offset_list[] = {
  {-833 ,"-833 kHz"},
  {-666 ,"-666 kHz"},
  {-500 ,"-500 kHz"},
  {-333 ,"-333 kHz"},
  {-166 ,"-166 kHz"},
  {0 ,"0 kHz"},
  {166 ,"+166 kHz"},
  {333 ,"+333 kHz"},
  {500 ,"+500 kHz"},
  {666 ,"+666 kHz"},
  {833 ,"+833 kHz"},
  {-1,NULL}
};

static struct STRTAB range_list[] = {
  {0 ,"A..G"},
  {1 ,"H..N"},
  {2 ,"O..U"},
  {3 ,"V..Z"},
//  {4 ,"All"},
  {-1,NULL}
};


static int get_entries_number(const struct STRTAB* list, int range , int* begin_p, int* end_p );

/*-------------------------------------------------------------------------*/

static void waitforchild(int i)
{
        waitpid(-1, NULL, WNOHANG);
        signal(SIGCHLD, waitforchild);
}

static void execute_it (Widget w, XtPointer client_data, XtPointer call_data)
{
  char *cmd = (char *)client_data;
  pthread_t thread;
  pthread_create (&thread, NULL, (void *(*)(void *)) system, cmd);
}

/*--------------------------------------------------------------------------*/
static void
save_dvbinitwin (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  char* dvb_init_conf=NULL ; 
  char* current_city=NULL;
  char* current_country=NULL;

  if (! country_city_are_ok ){
   popup_mess(translate("You must choose a country and a city !\n"));
   return;
  }
  XtVaSetValues(c_dvbinit_close,XtNsensitive,False,NULL);
  dvb_init_conf=get_dvbinit_file_name();
  current_country=int_to_str (dvbinit_country, country_list);
  current_city=int_to_str (dvbinit_city, city_list);

  download_and_save_initfile(dvb_init_conf, all_files , current_country , current_city);
  popup_mess(translate("The $HOME/.xdtv/dvb_init_conf was saved !!\n"));
  free(dvb_init_conf);
  XtVaSetValues(c_dvbinit_close,XtNsensitive,True,NULL);
}

static void
close_dvbinitwin (Widget widget, XtPointer clientdata, XtPointer call_data)
{

  XtCallActionProc (dvbinit_shell, "DVBinit", NULL, NULL, 0);
}

int getlist (void) {
  if (all_files == NULL) 
  { // Widget is open
    if (NULL == (all_files = get_initfiles_list () )) {  // get all transponders files on linuxtv.org MERCURIAL
      return 0;
    }

    if (NULL == (country_list=get_country_list(all_files))) {
      if (debug) fprintf(stderr,"Cannot get any list of countries\n");
      return 0; 
    }
    return 1;
  }
  else 
  { // Once the lists are full, it's not needed to download them back ???
    return 1;
  }
}
#endif

void
DVBinitAction (Widget widget, XEvent * event,
              String * params, Cardinal * num_params)
{
#ifdef HAVE_DVB
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug) fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (dvbinit_shell), XtWindow (dvbinit_shell), NULL, 0);
          return;
        }
    }

  if (dvbinit_shell == NULL)
    {
      popup_mess(translate("DVBinitAction:\nooops: dvbinit_shell is NULL\n"));
      return;
    }

  if (mapped)
    {
      XtPopdown (dvbinit_shell);
      mapped = 0;
      if (debug) fprintf(stderr,"Close Window\n");
    }
  else
    {
      if (debug) fprintf(stderr,"Open Window\n");
      if(getlist()) {
        // At the begining, city list is off because we must first choose the country
        XtVaSetValues(c_dvbinit_country,XtNsensitive,True, NULL);
        XtVaSetValues(c_dvbinit_city,   XtNsensitive,False,NULL);
        XtVaSetValues(c_dvbinit_range,  XtNsensitive,False,NULL);
        XtVaSetValues(c_dvbinit_action, XtNsensitive,False,NULL);
        create_dvbinit_off_pixmaps(XtDisplay(dvbinit_paned) , &RootWindowOfScreen(XtScreen(dvbinit_paned)));
      }
      else {
        popup_mess(translate(" Cannot get any file from the linuxtv.org website\n\n For more informations about the DVB support\n read the README.dvb file that can be find into the XdTV tarball\n\n"));
        return; 
      }


      XtPopup (dvbinit_shell, XtGrabNone);
      mapped = 1;

      if (first)
        {
          XSetWMProtocols (XtDisplay (dvbinit_shell), XtWindow (dvbinit_shell), wm_protocols, 2);
          first = 0;
        }
    }

  if (debug) fprintf(stderr,"DVB init action finished\n");
#endif
}

/*-------------------------------------------------------------------------*/
#ifdef HAVE_DVB
static void
dvbinit_menu_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int cd = (int) clientdata;
  int j;
  char label[64];

  switch (cd)
    {
    case DVBINIT_COUNTRY:
      if (-1 != (j = popup_menu (widget, translate("Country"), country_list)))
        {
	  int city_list_size=0;	
          dvbinit_country = j;
          sprintf (label, "%-*s:     %s",
		   ALI(LAB_WIDTH, translate("Country")),
                   int_to_str (dvbinit_country, country_list));
          XtVaSetValues (c_dvbinit_country, XtNlabel, label, NULL);
	  if (debug) fprintf(stderr,"Country: %d\n",dvbinit_country);
	  if (city_list){
		  free_strtab(city_list);
	  }
	  city_list=get_city_list(all_files,int_to_str (dvbinit_country, country_list));
	  
	  city_list_size=srtab_size(city_list);
	  if (debug) fprintf(stderr,"city_list_size = %d\n",city_list_size);
	  country_city_are_ok=0;
  	  XtVaSetValues(c_dvbinit_action,XtNsensitive,False,NULL);
          create_dvbinit_off_pixmaps(XtDisplay(dvbinit_paned) , &RootWindowOfScreen(XtScreen(dvbinit_paned)));
	 
	  if (city_list_size <= MAX_CITY_NUM_IN_LIST){
  	  	XtVaSetValues(c_dvbinit_range,XtNsensitive,False,NULL);
  	  	XtVaSetValues(c_dvbinit_city,XtNsensitive,True,NULL);
		partial_city_list=city_list;
		break;
	  }else {
  	  	XtVaSetValues(c_dvbinit_range,XtNsensitive,True,NULL);
  	  	XtVaSetValues(c_dvbinit_city,XtNsensitive,False,NULL);
		break;
	  }
        }
      break;

    case DVBINIT_CITY:
      if (-1 != (j = popup_menu (widget, translate("City"), partial_city_list)))
        {
          dvbinit_city = j;
          sprintf (label, "%-*s:     %s",
		   ALI(LAB_WIDTH, translate("City")),
                   int_to_str (dvbinit_city, partial_city_list));
          XtVaSetValues (c_dvbinit_city, XtNlabel, label, NULL);
	  country_city_are_ok=1;
  	  XtVaSetValues(c_dvbinit_action,XtNsensitive,True,NULL);
          create_dvbinit_on_pixmaps(XtDisplay(dvbinit_paned) , &RootWindowOfScreen(XtScreen(dvbinit_paned)));
	  if (debug) fprintf(stderr,"City: %d\n",dvbinit_city);
        }
      break;
      
    case DVBINIT_RANGE:
      if (-1 != (j = popup_menu (widget, translate("Range"), range_list)))
        {
	  int beg_l=0;
	  int end_l=0;
	  int num_to_copy=0;
	  struct STRTAB* old_city_list=partial_city_list;
	  
          dvbinit_range = j;
          sprintf (label, "%-*s:     %s",
		   ALI(LAB_WIDTH, translate("Range")),
                   int_to_str (dvbinit_range, range_list));
          XtVaSetValues (c_dvbinit_range, XtNlabel, label, NULL);
	  country_city_are_ok=0;
  	  XtVaSetValues(c_dvbinit_action,XtNsensitive,False,NULL);
          create_dvbinit_off_pixmaps(XtDisplay(dvbinit_paned) , &RootWindowOfScreen(XtScreen(dvbinit_paned)));

	  
//  	  XtVaSetValues (c_dvbinit_action,XtNsensitive,True,NULL);
//          create_dvbinit_on_pixmaps(XtDisplay(dvbinit_paned) , &RootWindowOfScreen(XtScreen(dvbinit_paned)));
	  if (debug) fprintf(stderr,"Range: %d\n",dvbinit_city);

	  if (-1 == get_entries_number(city_list , dvbinit_range , &beg_l ,&end_l)){
		  fprintf(stderr,"get_entries_number() problem\n");
		  return;
	  }
	  num_to_copy=end_l-beg_l+1;
	  if (debug) fprintf(stderr,"Will copy list between num %d and %d : %d\n",beg_l,end_l,num_to_copy);
	  if (!num_to_copy) return ;
	  
	  if (NULL == (partial_city_list=strtab_copy_in_range(city_list,beg_l,num_to_copy))){
	  	fprintf(stderr,"problem in strtab_copy_in_range\n");
		return;
	  }
  	  XtVaSetValues(c_dvbinit_city,XtNsensitive,True,NULL);
	  
	  if (old_city_list);
	  	free_strtab(old_city_list);
        }
      break;

     case DVBINIT_OFFSET:
      if (-1 != (j = popup_menu (widget, translate("Global Offset"), offset_list)))
        {
          dvbinit_offset = j;
          sprintf (label, "%-*s:     %s",
		   ALI(LAB_WIDTH, translate("Global Offset")),
                   int_to_str (dvbinit_offset , offset_list));
          XtVaSetValues (c_dvbinit_offset, XtNlabel, label, NULL);
	  if (debug) fprintf(stderr,"Global Offset: %d kHz\n",dvbinit_offset);
        }
      break;
      

    default:
      break;
    }
}

/*-------------------------------------------------------------------------*/


static int get_entries_number(const struct STRTAB* tab, int range , int* begin_p, int* end_p ){
	int list_size=0;
	int found=0;
	int i=0;
	
	if (!begin_p || ! end_p) return -1;
	list_size = srtab_size(tab);
	

	for (i=0;tab[i].str != NULL; i++){
		if (!strlen(tab[i].str)) {
			fprintf(stderr,"dummy string!\n");
			return -1;
		}
		switch (range) {
			case 0 : 	// A .. G
				if (tab[i].str[0]=='a' || tab[i].str[0]=='A' || tab[i].str[0]=='g' || tab[i].str[0]=='G') {
					if (!found){
						*begin_p=i;
						found=1;
					}	
					*end_p=i;
				}
				break;
				
			case 1 :   	// H .. N
				if (tab[i].str[0]=='h' || tab[i].str[0]=='H' || tab[i].str[0]=='n' || tab[i].str[0]=='N') {
					if (!found){
						*begin_p=i;
						found=1;
					}	
					*end_p=i;
				}
				break;

			case 2 :  // O .. U
				if (tab[i].str[0]=='o' || tab[i].str[0]=='O' || tab[i].str[0]=='u' || tab[i].str[0]=='U') {
					if (!found){
						*begin_p=i;
						found=1;
					}	
					*end_p=i;
				}
				break;

			case 3 :   // V .. Z
				if (tab[i].str[0]=='v' || tab[i].str[0]=='V' || tab[i].str[0]=='z' || tab[i].str[0]=='Z') {
					if (!found){
						*begin_p=i;
						found=1;
					}	
					*end_p=i;
				}
				break;

			default : 
				break;
		
		}
	
	
	}

	return 0;

}


void
create_dvbinitwin()
{
  Widget c;
  char label[64];
#ifdef HAVE_XPM
  Pixmap icon_pixmap;
  Pixmap icon_shapemask;
#endif

  dvbinit_shell = XtVaAppCreateShell ("DVBinit", "XdTV",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell, XtNvisual,vinfo->visual,
				   XtNcolormap,colormap, XtNdepth,vinfo->depth,
				   NULL);

  dvbinit_paned = XtVaCreateManagedWidget ("paned", panedWidgetClass, dvbinit_shell, NULL);

  /* DVB init label */
  c_dvbinit_label = XtVaCreateManagedWidget ("ldvbinitlabel", labelWidgetClass, dvbinit_paned, PANED_FIX, NULL);

  /* Choose the DVB init Country */
  c_dvbinit_country = c = XtVaCreateManagedWidget ("ldvbinitcountry", commandWidgetClass, dvbinit_paned, PANED_FIX, NULL);
    XtAddCallback (c, XtNcallback, dvbinit_menu_cb, (XtPointer) DVBINIT_COUNTRY);
    sprintf (label, "%-*s:     %s", ALI(LAB_WIDTH, translate("Country")), int_to_str (dvbinit_country, country_list_dummy));
    XtVaSetValues (c, XtNlabel, label, NULL);

  /* Choose the DVB init Range */
  c_dvbinit_range = c = XtVaCreateManagedWidget ("ldvbinitrange", commandWidgetClass, dvbinit_paned, PANED_FIX, NULL);
    XtAddCallback (c, XtNcallback, dvbinit_menu_cb, (XtPointer) DVBINIT_RANGE);
    sprintf (label, "%-*s:     %s", ALI(LAB_WIDTH, translate("Range")), int_to_str (dvbinit_range, range_list));
    XtVaSetValues (c, XtNlabel, label, NULL);

  /* Choose the DVB init City */
  c_dvbinit_city = c = XtVaCreateManagedWidget ("ldvbinitcity", commandWidgetClass, dvbinit_paned, PANED_FIX, NULL);
    XtAddCallback (c, XtNcallback, dvbinit_menu_cb, (XtPointer) DVBINIT_CITY);
    sprintf (label, "%-*s:     %s", ALI(LAB_WIDTH, translate("City")), int_to_str (dvbinit_city, city_list_dummy));
    XtVaSetValues (c, XtNlabel, label, NULL);

   /* Choose the DVB init Offset */
   c_dvbinit_offset =c= XtVaCreateManagedWidget ("dvbinitoffset", commandWidgetClass, dvbinit_paned, PANED_FIX , NULL);
    XtAddCallback (c, XtNcallback, dvbinit_menu_cb, (XtPointer) DVBINIT_OFFSET);
    sprintf (label, "%-*s:     %s", ALI(LAB_WIDTH, translate("Global Offset")), int_to_str (dvbinit_offset, offset_list));
    XtVaSetValues (c, XtNlabel, label, NULL);

  /* Edit the dvb_init.conf file */
  c_dvbinit_rc = c = XtVaCreateManagedWidget ("ldvbinitrc", commandWidgetClass, dvbinit_paned, PANED_FIX, NULL);
  signal(SIGCHLD, waitforchild);
  XtAddCallback (c, XtNcallback, execute_it, translate("xterm -geometry 80x35+0+0 -title xdtv_init.conf -e vi $HOME/.xdtv/dvb_init.conf"));

  /* Download the DVB init file & Save */
  c_dvbinit_action = c = XtVaCreateManagedWidget ("ldvbinitaction", commandWidgetClass, dvbinit_paned, PANED_FIX, NULL);
    XtAddCallback (c, XtNcallback, save_dvbinitwin, (XtPointer) NULL);

  /* Quit the DVB init GUI */
  c_dvbinit_close = c = XtVaCreateManagedWidget ("ldvbinitclose", commandWidgetClass, dvbinit_paned, PANED_FIX, NULL);
    XtAddCallback (c, XtNcallback, close_dvbinitwin, (XtPointer) NULL);

#ifdef HAVE_XPM
  XpmCreatePixmapFromData(XtDisplay(dvbinit_shell),
                          RootWindowOfScreen(XtScreen(dvbinit_shell)),
                          icon_xpm,
                          &icon_pixmap,
                          &icon_shapemask, NULL);
  XtVaSetValues(dvbinit_shell, XtNiconPixmap, icon_pixmap, NULL);
#endif
  create_dvbinit_common_pixmaps(XtDisplay(dvbinit_paned) , &RootWindowOfScreen(XtScreen(dvbinit_paned)));
  if (! country_city_are_ok )
    create_dvbinit_off_pixmaps(XtDisplay(dvbinit_paned) , &RootWindowOfScreen(XtScreen(dvbinit_paned)));
  else
    create_dvbinit_on_pixmaps(XtDisplay(dvbinit_paned) , &RootWindowOfScreen(XtScreen(dvbinit_paned)));

}

int dvbinit_is_enabled(void)
{
	return country_city_are_ok;
}

#endif
