/*****************************************************************************
 * dvbtools.c: some tools for the DVB-T support
 *****************************************************************************
 * $Id: dvbtools.c,v 1.12 2007/02/19 18:08:29 estre Exp $
 *****************************************************************************
 * Program under GNU General Public License (see ../COPYING)
 *****************************************************************************/

#include "config.h"
#ifdef HAVE_DVB

#include "dvbtools.h"
#include <regex.h> 
 
#include <string.h>

#include "translation.h"

extern char *strndup (__const char *__string, size_t __n); //dont know why function not detected ?!?
#define MAX_NB_CITY 1000 // FIXME ?

const struct STRTAB inversion_list [] = {
  {INVERSION_OFF,"INVERSION_OFF"},
  {INVERSION_ON,"INVERSION_ON"},
  {INVERSION_AUTO,"INVERSION_AUTO"},
  {-1, NULL}
};

const struct STRTAB bw_list [] = {
  {BANDWIDTH_6_MHZ, "BANDWIDTH_6_MHZ"},
  {BANDWIDTH_7_MHZ, "BANDWIDTH_7_MHZ"},
  {BANDWIDTH_8_MHZ, "BANDWIDTH_8_MHZ"},
  {-1, NULL}
};

const struct  STRTAB fec_list [] = {
  {FEC_1_2,"FEC_1_2"},
  {FEC_2_3,"FEC_2_3"},
  {FEC_3_4,"FEC_3_4"},
  {FEC_5_6,"FEC_5_6"},
  {FEC_6_7,"FEC_6_7"},
  {FEC_7_8,"FEC_7_8"},
  {FEC_8_9,"FEC_8_9"},
  {FEC_AUTO,"FEC_AUTO"},
  {FEC_NONE,"FEC_NONE"},
  {-1, NULL}
};

const struct  STRTAB guard_list [] = {
  {GUARD_INTERVAL_1_32,"GUARD_INTERVAL_1_32"},
  {GUARD_INTERVAL_1_16,"GUARD_INTERVAL_1_16"},
  {GUARD_INTERVAL_1_8,"GUARD_INTERVAL_1_8"},
  {GUARD_INTERVAL_1_4,"GUARD_INTERVAL_1_4"},
  {-1, NULL}
};

const struct  STRTAB hierarchy_list [] = {
  {HIERARCHY_1,"HIERARCHY_1"},
  {HIERARCHY_2,"HIERARCHY_2"},
  {HIERARCHY_4,"HIERARCHY_4"},
  {HIERARCHY_NONE,"HIERARCHY_NONE"},
  {-1, NULL}
};

const struct  STRTAB constellation_list [] = {
  {QPSK,"QPSK"},
  {QAM_128,"QAM_128"},
  {QAM_16,"QAM_16"},
  {QAM_256,"QAM_256"},
  {QAM_32,"QAM_32"},
  {QAM_64,"QAM_64"},
  {-1, NULL}
};

const struct  STRTAB transmissionmode_list [] = {
  {TRANSMISSION_MODE_2K,"TRANSMISSION_MODE_2K"},
  {TRANSMISSION_MODE_8K,"TRANSMISSION_MODE_8K"},
  {-1, NULL}
};

int set_pesfilter (int fd, int pid, dmx_pes_type_t type, int dvr)
{
        struct dmx_pes_filter_params pesfilter;

        if (pid <= 0 || pid >= 0x1fff)
                return 0;

        pesfilter.pid = pid;
        pesfilter.input = DMX_IN_FRONTEND;
        pesfilter.output = dvr ? DMX_OUT_TS_TAP : DMX_OUT_DECODER;
        pesfilter.pes_type = type;
        pesfilter.flags = DMX_IMMEDIATE_START;

        if (ioctl(fd, DMX_SET_PES_FILTER, &pesfilter) < 0) {
               /* PERROR ("ioctl(DMX_SET_PES_FILTER) for %s PID failed",
                        type == DMX_PES_AUDIO ? "Audio" :
                        type == DMX_PES_VIDEO ? "Video" : "??");
			*/
                return -1;
        }

        return 0;
}




int setup_frontend (int fe_fd, struct dvb_frontend_parameters *frontend)
{
	struct dvb_frontend_info fe_info;

	if (ioctl(fe_fd, FE_GET_INFO, &fe_info) < 0) {
	//	PERROR("ioctl FE_GET_INFO failed");
		return -1;
	}

	if (fe_info.type != FE_OFDM) {
	//	ERROR ("frontend device is not a OFDM (DVB-T) device");
		return -1;
	}

	printf ("tuning to %i Hz\n", frontend->frequency);

	if (ioctl(fe_fd, FE_SET_FRONTEND, frontend) < 0) {
	//	PERROR("ioctl FE_SET_FRONTEND failed");
		return -1;
	}

	return 0;
}



int check_frontend (int fe_fd)
{
	fe_status_t status;
	__uint16_t snr, signal;
	__uint32_t ber, uncorrected_blocks;

	do {
		ioctl(fe_fd, FE_READ_STATUS, &status);
		ioctl(fe_fd, FE_READ_SIGNAL_STRENGTH, &signal);
		ioctl(fe_fd, FE_READ_SNR, &snr);
		ioctl(fe_fd, FE_READ_BER, &ber);
		ioctl(fe_fd, FE_READ_UNCORRECTED_BLOCKS, &uncorrected_blocks);

		printf ("status %02x | signal %04x | snr %04x | "
			"ber %08x | unc %08x | ",
			status, signal, snr, ber, uncorrected_blocks);

		if (status & FE_HAS_LOCK){
			printf("FE_HAS_LOCK\n");
			//printf("ok, frontend runing ...\n");
			return 0;
		}

		usleep(1000000);
		printf("\n");
	} while (1);

	return 0;
}


int add_filter(unsigned int pid, const unsigned char* dmxdev)
{
	int fd;
	struct dmx_pes_filter_params f;

	fd = open((const char*)dmxdev, O_RDONLY);
	if (fd == -1) {
		perror("cannot open demux device");
		return -1;
	}

	memset(&f, 0, sizeof(f));
	f.pid = (__uint16_t) pid;
	f.input = DMX_IN_FRONTEND;
	f.output = DMX_OUT_TS_TAP;
	f.pes_type = DMX_PES_OTHER;
	f.flags   = DMX_IMMEDIATE_START;

	if (ioctl(fd, DMX_SET_PES_FILTER, &f) == -1) {
		perror("DMX_SET_PES_FILTER");
		return -1;
	}
	fprintf(stderr, "set filter for PID 0x%04x on fd %d\n", pid, fd);

	return fd;
}


string_t* 
string_new() {
	string_t* s=NULL;
	if (NULL != (s=malloc (sizeof *s))) {
		s->str=NULL;
		s->size=0;
		s->len=0;
	}else {
		perror("malloc");
	}
	return s;
}






static char **
str_to_tab(char *text, char separateur)
{
  char **a;
  int nb_cr = 0;
  int i, j;
  
/* Compte du nombre de lignes */
  for (i = 0; text[i]; i++)
    {
      if (text[i]==separateur )
        nb_cr++;
    }
  
/* Allocation mmoire du tableau */
  if (NULL == (a = malloc ((nb_cr + 2) * sizeof *a))){
    perror("malloc");
    return (NULL);
  } 
  
/* Remplissage du tableau */
  a[0] = text;
  j = 1;
  for (i = 0; text[i]; i++)
    {
      if ( text[i] == separateur)
        {
          text[i] = '\0';
          a[j] = text + i + 1;
          j++;
        }
    }
  a[j] = NULL;
  return (a);
}


size_t
str_append (void *ptr, size_t size, size_t nmemb, void *stream)
{
  string_t *str = stream;
  int len = size * nmemb;
  if (str->size < (str->len + len))
    {
/* Si la chane est trop petite, nous l'agrandissons */
      str->size += (1 + (len / STRING_BLOCK_SIZE)) * STRING_BLOCK_SIZE;
      str->str = realloc (str->str, str->size + 1);
      if (NULL == str->str)
        return -1;
    }
/* Ajout des donnes  la fin de la chane et mise  jour de la chane */
  memcpy (str->str + str->len, ptr, len);
  str->len += len;
  str->str[str->len] = '\0';
  return (len);
}

char *
get_URL (const char *url)
{
  CURL *curl;
  CURLcode res;
  string_t *str;
  char *r;
  
/* Initialisations */
  if (NULL == (str = string_new ()))
    return (NULL);
  curl = curl_easy_init ();
  if (curl)
    {
/* Paramtres pour cURL */
      
/* Fichier  demander */
      curl_easy_setopt (curl, CURLOPT_URL, url);
/* Ne pas vrifier le certificat */
      curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, 0);
/* Ne pas vrifier la correspondance entre la machine distante
 * et le certificat */
      curl_easy_setopt (curl, CURLOPT_SSL_VERIFYHOST, 0);
/* str_append() est notre fonction de rappel pour stocker
 * le fichier en mmoire */
      curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, str_append);
/* str sera le quatrime argument fourni  str_append() */
      curl_easy_setopt (curl, CURLOPT_WRITEDATA, str);
      
/* Recherche le fichier */
      res = curl_easy_perform (curl);
      
/* Nettoyage des variables de cURL; retour */
      curl_easy_cleanup (curl);
      if ((CURLE_OK == res) && str->str)
        {
          r = str->str;
          free (str);
          return (r);
        }
      else
        return (NULL);
    }
  else
    {
      free (str);
      return (NULL);
    }
}

char* get_url_id(const char* all_datas){
	char **tab=NULL;
	int i,j;
	char* tmpstr=NULL;
	regex_t* preg=NULL;

	if (NULL == (preg=malloc(sizeof *preg))){
		perror("malloc");
		goto err;
	}
//	<a href="?cmd=manifest;manifest=efc50388cd8890655ecdff2d4c19979ab56ccb6f;path=/;style=gitweb">tree</a><br/
	if (0 != (regcomp(preg,"^.*<a href=\".cmd=manifest;manifest=.*;path=/;style=gitweb\">tree</a>",REG_NOSUB | REG_EXTENDED))){
		perror("regcomp");
		goto err;
	}

	if (NULL == (tmpstr=strdup(all_datas))){
		perror("strdup");
		goto err;
	
	}
	tab=str_to_tab(tmpstr , '|');  // on oublie tmpstr, le free se fait sur tab !

	for (i=0;tab[i];i++){

		j=regexec (preg , tab[i],0,NULL,0);

		if (0==j) { // le motif est trouve !
			char* str_ori=tab[i];
			char* id_to_return=NULL;
			char* id_begin=NULL;
			char* id_end=NULL;
			int id_len=13;
//			fprintf(stderr,"%s\n",str_ori);

			/* la chaine de caractere commence par : \" <a href="?cmd=manifest;manifest=" , ie 33 car. (Mercurial) */
			/* on doit recuperer l'ID de la derniere page  jour prsentant tous les fichiers !  */
			/* l'ID complet est par exemple efc50388cd8890655ecdff2d4c19979ab56ccb6f 
			 * on ne garde que le debut :   efc50388cd88  , soit les 12 premiers caracteres */
			
			id_begin = str_ori+33;
			id_end   = id_begin+id_len;


			if (NULL == (id_to_return = malloc (id_len*sizeof *id_to_return))){
				perror("malloc");
				goto err;
			}

			strncpy(id_to_return,id_begin,id_len-1);
			id_to_return[id_len-1]='\0';
			
		        	
		//	fprintf (stderr,"%s\n",id_to_return);
			return id_to_return;
		}
	}
	regfree(preg);
	free(preg);
	free(tab[0]);
	free(tab);
	return NULL;

err  :
	fprintf(stderr,"an error has occured, freeing structs\n");
	if (NULL != preg ) {
		regfree(preg);
		free(preg);
	}
	if (NULL != tab ){
		free(tab[0]);
		free(tab);
	}
	return NULL;
}

char* get_corrected_url( char* ori_url, char* old_motif, char* new_motif){
	int ori_url_len=0;
	int old_motif_len=0;
	int new_motif_len=0;
	char* motif_begin=NULL;

	if (!ori_url || ! old_motif || !new_motif   ) return NULL;
	
	ori_url_len   = strlen(ori_url);
	old_motif_len = strlen(old_motif);
	new_motif_len = strlen(new_motif);
	
	if (!ori_url_len || ! old_motif_len || !new_motif_len ){
		fprintf(stderr,"error, a length is 0.\n");	
		return NULL;
	}
	if ( old_motif_len !=  new_motif_len ){
		fprintf(stderr,"error, lengths are differents.\n");	
		return NULL;
	}

	if (ori_url_len <=  old_motif_len ){
		fprintf(stderr,"error, motif is too big.\n");	
		return NULL;
	}

	if (NULL == (motif_begin = strstr(ori_url,old_motif))){
		fprintf(stderr,"error : %s not found in %s\n",old_motif,ori_url);
		return NULL;
	}

	strncpy(motif_begin,new_motif,old_motif_len);

//	fprintf(stderr,"url constructed : %s\n",ori_url);

	return ori_url;
}

transp_file** parse_listefiles(const char* all_datas){
	char **tab=NULL;
	int i,j;
	transp_file** all_city=NULL;
	int current_city=0;
	char* tmpstr=NULL;
	regex_t* preg=NULL;

	if (NULL == (preg=malloc(sizeof *preg))){
		perror("malloc");
		goto err;
	}
//	if (0 != (regcomp(preg,"^<td><a name=\".*\" href=\"/cgi-bin/viewcvs.cgi/dvb-apps/util/scan/dvb-t/.*" ,REG_NOSUB | REG_EXTENDED))){
	if (0 != (regcomp(preg,"^.*<a href=\".fl=.*;file=util/scan/dvb-t/..-.*;style=gitweb\">revisions</a>" ,REG_NOSUB | REG_EXTENDED))){
//	if (0 != (regcomp(preg,".*<a href=\".fl=" ,REG_NOSUB | REG_EXTENDED))){
		perror("regcomp");
		goto err;
	}

	if (NULL == (tmpstr=strdup(all_datas))){
		perror("strdup");
		goto err;
	
	}
	tab=str_to_tab(tmpstr , '|');  // on oublie tmpstr, le free se fait sur tab !

	if (NULL == (all_city = malloc (MAX_NB_CITY * sizeof *all_city))){
		perror("malloc");
		goto err;
	}
	for (i=0;tab[i];i++){

		j=regexec (preg , tab[i],0,NULL,0);

		if (0==j) { // le motif est trouve !
			char* str_ori=tab[i];
			char* file_name_beg=NULL;
			char* file_name_end=NULL;
			int filenamelen=0;
			char* id_begin=NULL;
			char* id_end=NULL;
			int id_len=0;
//			fprintf(stderr,"%s\n",str_ori);

			/* la chaine de caractere commence par :  <td><a name=" , ie 13 car. (CVS)*/
			/* la chaine de caractere commence par : \" <a href="?fl=d595b3d8cb52;file=util/scan/dvb-t/\" , ie 48 car. (Mercurial) */
			/* on doit recuperer l'ID du fichier pour pouvoir telecharger !!! c'est galre !  */
			
			id_begin=str_ori+14;
			id_end = strstr(str_ori,";file=util");

			file_name_beg=str_ori+48;
			file_name_end=strstr(str_ori,";style=gitweb");

			id_len = id_end-id_begin;
			filenamelen = file_name_end-file_name_beg;

			if (id_len <=0){ // ?? problem !
				fprintf(stderr,"id_len = %d !?!?\n",id_len);
				goto err;
			}
			if (filenamelen <=0){ // ?? problem !
				fprintf(stderr,"filenamelen = %d !?!?\n",filenamelen);
				goto err;
			}
			if (NULL == (all_city[current_city] = malloc (sizeof *(all_city[current_city])))){
				perror("malloc");
				goto err;
			}

			strncpy(all_city[current_city]->country, file_name_beg,2);
			file_name_beg+=3;
			all_city[current_city]->city=strndup(file_name_beg,filenamelen -3);
			all_city[current_city]->country[3]='\0';
			all_city[current_city]->city[filenamelen-3]='\0';
			all_city[current_city]->id=strndup(id_begin,id_len);
			all_city[current_city]->id[id_len]='\0';

		//	fprintf(stderr,"file : %s-%s  , %s\n",all_city[current_city]->country,all_city[current_city]->city,all_city[current_city]->id);
			current_city++;
		        	
			if (current_city >= MAX_NB_CITY -1){ // on garde une place pour un NULL
				fprintf (stderr,"  *********** PLEASE INCREASE MAX_NB_CITY  ****** \n");
				break;
			}
		//	fprintf (stderr,"%s\n",filename);
		}
	}
	all_city[current_city] = NULL;
//	fprintf (stderr,"number of city: %d \n",current_city );
//	
	regfree(preg);
	free(preg);
	free(tab[0]);
	free(tab);
	return all_city;

err  :
	fprintf(stderr,"an error has occured, freeing structs\n");
	if (NULL != preg ) {
		regfree(preg);
		free(preg);
	}
	if (NULL != tab ){
		free(tab[0]);
		free(tab);
	}
	if (NULL != all_city ){
		free(all_city); // faire la boucle sur toutes les villes ... idiot !
	}

	return NULL;
}
struct STRTAB* get_country_list(const transp_pp all_city){
	struct STRTAB* country_list=NULL;
	int nb_country=0;
	int i;
	char prev[3]="XX";

	for (i=0; all_city[i] ; i++){
		if (0 != (strncmp(all_city[i]->country,prev,2))){
			strncpy(prev,all_city[i]->country,2);
			nb_country++;
		}
	}

//	fprintf(stderr,"nombre de pays : %d\n",nb_country);
	if (NULL == (country_list=malloc((nb_country+1) * sizeof (*country_list)))){
		perror ("malloc");
		return NULL;
	}
	strncpy(prev,"XX",3);
	nb_country=0;
	for (i=0; all_city[i];i++){
		if (0 != (strncmp(all_city[i]->country,prev,2))){
			strncpy(prev,all_city[i]->country,2);
			all_city[i]->country[2]='\0';
			country_list[nb_country].str=strndup(all_city[i]->country,3);
			country_list[nb_country].nr=nb_country;
//			fprintf(stderr,"country : %s \n",country_list[nb_country].str);
			nb_country++;
		}
	}
	country_list[nb_country].str=NULL ;
	country_list[nb_country].nr=-1;

	return country_list;
}
struct STRTAB* get_city_list(const transp_pp all_city, const char* country){
	struct STRTAB* city_list=NULL;
	int nb_city=0;
	int i;

	for (i=0; all_city[i] ; i++){
		if (0 == (strncmp(all_city[i]->country,country,2))){
			/* we want this city !*/
			nb_city++;
		}
	}

	if (NULL == (city_list=malloc((nb_city+1) * sizeof (*city_list)))){
		perror ("malloc");
		return NULL;
	}
	nb_city=0;
	for (i=0; all_city[i];i++){
		if (0 == (strncmp(all_city[i]->country,country,2))){
			city_list[nb_city].str=strdup(all_city[i]->city);
			city_list[nb_city].nr=nb_city;
			nb_city++;
		}
	}
	city_list[nb_city].str=NULL ;
	city_list[nb_city].nr=-1;

	return city_list;
}

transp_file** get_initfiles_list(){
	char* all_datas1=NULL;
	char* all_datas2=NULL;
	char* url_id=NULL;
	char* final_url=NULL;
	transp_file** all_city=NULL;
//	struct STRTAB* country_list=NULL;
//	int city;
	
/* on doit d'abord recuperer LINUXTV_DVBT_MERCURIAL_URL1, le parser pour recuperer l'ID de la derniere url  jour listant les fichiers dvb*/

	if(NULL==(all_datas1=get_URL (LINUXTV_DVBT_MERCURIAL_URL1))){
		fprintf(stderr,"cannot get url from : '%s'\n", LINUXTV_DVBT_MERCURIAL_URL1);
		return NULL;
	}
	if(NULL==(url_id=get_url_id(all_datas1))){
		fprintf(stderr,"cannot parse url : '%s'\n", LINUXTV_DVBT_MERCURIAL_URL1);
		return NULL;
	}
	final_url=strdup(LINUXTV_DVBT_MERCURIAL_URL2);
	
	if(NULL==(final_url=get_corrected_url(final_url, "xxxxxxxxxxxx",url_id ))){
		fprintf(stderr,"error in finale url construction \n");
		return NULL;
	}
	
	if(NULL==(all_datas2=get_URL (final_url))){
		fprintf(stderr,"cannot get list of init files from : '%s'\n", LINUXTV_DVBT_MERCURIAL_URL2);
		return NULL;
	}
//	fprintf(stderr,"dumps datas : '%s'\n",all_datas);
	
	all_city = parse_listefiles(all_datas2);
	
	free (all_datas1);
	free (all_datas2);
	free (final_url);
	free (url_id);
	
	return all_city;

}



void dvb_write_init_file (const char* datas,FILE* file){ 
	/* simple check : each line should start by ' #' (comment) or ' T' */
	char **tab=NULL;
	char* tmpstr=NULL;
	int i;
	regex_t* preg1=NULL;
	regex_t* preg2=NULL;


	if (NULL == (preg1=malloc(sizeof *preg1))){
		perror("malloc");
		goto err;
	}
	if (NULL == (preg2=malloc(sizeof *preg2))){
		perror("malloc");
		goto err;
	}
	if (0 != (regcomp(preg1,"^.*</span> T" ,REG_NOSUB | REG_EXTENDED))){
		perror("regcomp");
		goto err;
	}
	if (0 != (regcomp(preg2,"^.*</span> #" ,REG_NOSUB | REG_EXTENDED))){
		perror("regcomp");
		goto err;
	}
	if (NULL == (tmpstr=strdup(datas))){
		perror("strdup");
		return ; 
	
	}

	tab=str_to_tab(tmpstr ,'\n');  // on oublie tmpstr, le free se fait sur tab !

	for (i=0;tab[i];i++){
		int j,k;
		
		j=regexec (preg1 , tab[i],0,NULL,0);
		k=regexec (preg2 , tab[i],0,NULL,0);

		if (0 == j || 0 == k){
			char* str_ori=tab[i];
			char* transp_file_start_line=NULL;
			char* delim="</span>";
		//	fprintf(stderr,"line:%s\n",tab[i]);

			transp_file_start_line=strstr(str_ori,delim);
			transp_file_start_line+=strlen(delim);
			
			while (*transp_file_start_line == ' ') { // space in begin of line must be removed !
				transp_file_start_line++;
			}
			
			fprintf(file,"%s\n",transp_file_start_line);
		}
	}
	regfree(preg1);
	free(preg1);
	regfree(preg2);
	free(preg2);
	free(tab[0]);
	free(tab);
	return;

err  :
	fprintf(stderr,"an error has occured, freeing structs\n");
	if (NULL != preg1 ) {
		regfree(preg1);
		free(preg1);
	}
	if (NULL != preg2 ) {
		regfree(preg2);
		free(preg2);
	}
	if (NULL != tab ){
		free(tab[0]);
		free(tab);
	}
}

transp_file* get_transp_file(const transp_pp all_city , const char* country, const char* city){
	int i;

	if (NULL == country || NULL == city)
		return NULL;

	for (i=0; all_city[i] ; i++){
		if ( (0==(strncmp(all_city[i]->country,country,2))) && (0==(strncmp(all_city[i]->city,city,strlen(city))))){
			/* we want this transp file !*/
			return all_city[i];
		}
	}
	return NULL;
}

void download_and_save_initfile(const char* dvb_init_conf, const transp_pp all_files , const char* current_country, const char* current_city  ){

	  /* ok so we have the city, the country and the id -> we know the transp_file file ! */

  char* current_id=NULL;
  transp_file* current_transpfile=NULL; 
  char* url_file_to_download=NULL;
  int tmplen=0;
  char* file_downloaded=NULL;
  FILE* dvb_init_file;

  if (NULL == (current_transpfile = get_transp_file (all_files, current_country,current_city) )){
  	fprintf(stderr,"%s-%s not found in transp list ... problem ! \n",current_country,current_city);
	return ;
  }
  current_id=strdup(current_transpfile->id);
  
  if (NULL == (url_file_to_download = malloc (strlen (LINUXTV_DVBT_MERCURIAL_CO_DEB) + strlen (current_id) +
		  strlen (LINUXTV_DVBT_MERCURIAL_CO_MID)+ strlen (current_country) + strlen (current_city) + 
		  strlen (LINUXTV_DVBT_MERCURIAL_CO_END) +2 ))){  // +1 pour '-' , +1 pour '\0'
  	perror("malloc");
	free(current_id);
	return ;
  }

  	/* construction de l'URL */	  
  tmplen=strlen(LINUXTV_DVBT_MERCURIAL_CO_DEB);
  strncpy(url_file_to_download,LINUXTV_DVBT_MERCURIAL_CO_DEB,tmplen);
  strncpy(url_file_to_download+tmplen,current_id,strlen(current_id));
  tmplen+=strlen(current_id);
  strncpy(url_file_to_download+tmplen,LINUXTV_DVBT_MERCURIAL_CO_MID,strlen(LINUXTV_DVBT_MERCURIAL_CO_MID));
  tmplen+=strlen(LINUXTV_DVBT_MERCURIAL_CO_MID);
  strncpy(url_file_to_download+tmplen,current_country,strlen(current_country));
  tmplen+=strlen(current_country);
  strncpy(url_file_to_download+tmplen,"-",1);
  tmplen++;
  strncpy(url_file_to_download+tmplen,current_city,strlen (current_city));
  tmplen+=strlen(current_city);
  strncpy(url_file_to_download+tmplen,LINUXTV_DVBT_MERCURIAL_CO_END,strlen(LINUXTV_DVBT_MERCURIAL_CO_END));
  tmplen+=strlen(LINUXTV_DVBT_MERCURIAL_CO_END);
  url_file_to_download[tmplen]='\0';
  fprintf(stderr, translate("get dvb init file from url : %s\n"),url_file_to_download);

  if(NULL==(file_downloaded =get_URL (url_file_to_download))){
  	fprintf(stderr,"cannot get the url : '%s'\n",url_file_to_download );
	return ;
  }

  if (NULL == (dvb_init_file = fopen (dvb_init_conf,"w"))){
  	perror("fopen");
	return ;
  }

  dvb_write_init_file(file_downloaded,dvb_init_file);
  fclose(dvb_init_file);

  fprintf(stderr,translate("dvb init file : %s has been created.\n"),dvb_init_conf);

  free(url_file_to_download);
  free(file_downloaded);
  free(current_id);
}

char* get_dvbinit_file_name () {
  char *homedir = getenv ("HOME");
  char* dvb_init_conf=NULL;
  if (!homedir)
    perror ("$HOME not set");
  if (NULL == (dvb_init_conf = malloc ( (strlen(homedir) + strlen(DVB_TRANSPONDER_FILE) + 1)* (sizeof(*dvb_init_conf))))){
    perror("malloc");
    return NULL;
  }
  memcpy (dvb_init_conf, homedir, strlen(homedir));
  memcpy (dvb_init_conf+ strlen(homedir),DVB_TRANSPONDER_FILE , strlen(DVB_TRANSPONDER_FILE) + 1);
  return dvb_init_conf;
}

#endif //HAVE_DVB
