/*****************************************************************************
 * grab-avi.c: avi interface, reading frames from an avi file, only for test
 *  THE AVI FILE MUST BE IN XVID FORMAT.
 *****************************************************************************
 * $Id: grab-avi.c,v 1.48 2006/03/09 21:05:01 alainjj Exp $
 *****************************************************************************
 * Copyright (C) 2003 Alainjj
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "config.h"

#ifdef HAVE_XVID
#include <xvid.h>
#ifdef XVID_API
#define HAVE_XVID10 /* XVID 1.0 */
#else
#define HAVE_XVID09 /* XVID 0.9 */
#endif
#endif
#ifdef HAVE_LAME
#include <lame/lame.h>
#endif

#include "grab-file.h"
#include "channel.h"
#include "avilib.h"
#include "divx.h"

///      to be removed ? ///

extern void **buffers;
extern char* bufcompress;
extern double delay; // delay between two images
extern clock_t t_start;
extern struct tms start;

extern char* audiobuf;
extern int audiobuflen;
extern int audio_buf_max_len;

#define AVIAUDIOBUF 800000

//////////////////////////////////////

extern void video_set_capture_size(int w, int h);
extern int nbufs;

#ifdef HAVE_XVID
extern int fmts_xaw_to_xvidcolorspace[]; /* from divx.c */
#endif

/* variable specific for grab-avi */
char *avifilename;
int avibench = 0;
extern avi_t *   videofile ;
#ifdef HAVE_XVID
#ifdef HAVE_XVID09
static XVID_DEC_PARAM xvid_dec_p;
#else
static xvid_dec_create_t xvid_dec_p;
#endif
#endif

extern divx_t divx;
extern double audiostamp;
void ExitCB (Widget, XtPointer, XtPointer);
extern Widget app_shell;
   
/* in http client mode, the server has a priori any norm */
static struct STRTAB norms[] = {
  {0,   "PAL"},
  {1,  "NTSC"},
  {2, "SECAM"},
  {3, "PAL-Nc"},
  {4, "PAL-M"},
  {5, "PAL-N"},
  {6, "NTSC-JP"},
  {7, "PAL-60"},
  {-1, NULL}
};

/* idem */
static struct STRTAB inputs[] = {
  {0,   "Television"},
  {1,   "Composite1"},
  {2,   "Composite2"},
  {3,   "Composite3"},
  {4,   "Composite4"},
  {5,   "S-Video"},
  {6,   "other"},
  {-1, NULL}
};

extern struct GRABBER grab_avi;

static int raw_direct = 0;

static int avireaduncompressedframe(char *buf) {
  static char* audiobufcompressed;
  int r;
  long len;
  struct timeval tv;
  char *audiobuf2;
  if (NULL == (audiobufcompressed=alloca(audio_buf_max_len))){
  	perror("alloca");
	exit(1);
  }
  if(videofile->a_fmt==WAVE_FORMAT_PCM) 
    audiobuf2=audiobuf;
  else
    audiobuf2=audiobufcompressed;
  do {
    r=AVI_read_data(videofile, buf, 778*576*4, audiobuf2, audio_buf_max_len, &len);
    if(debug) 
      fprintf(stderr,"AVI_read_data: av=%d len=%ld\n",r,len);
    if(r==2) {
      gettimeofday (&tv, NULL);
      audiostamp=tv.tv_usec/1000000.0+tv.tv_sec;
      if(audiobuf==audiobuf2)
	audiobuflen=len;
      else {
#ifdef HAVE_LAME
	if(videofile->a_fmt==WAVE_FORMAT_MPEGLAYER3) {
	  if(videofile->a_chans==1)
	    audiobuflen=lame_decode((unsigned char*)audiobufcompressed,len,
				    (short*)audiobuf,(short*)audiobuf);
	  else {
	    static short *pcm_l,*pcm_r;
	    int i;
	    if (NULL == (pcm_l=(short*)malloc(audio_buf_max_len/sizeof(char)*sizeof(*pcm_l)/2))){
	    	perror("alloca");
		exit (-1);
	    }
	    if (NULL == (pcm_r=(short*)malloc(audio_buf_max_len/sizeof(char)*sizeof(*pcm_r)/2))){
	    	perror("alloca");
		free(pcm_l);
		exit (-1);
	    }
	    audiobuflen=lame_decode((unsigned char*)audiobufcompressed,len,pcm_l,pcm_r);
	    for(i=0;i<audiobuflen;i++) {
	      ((short *)audiobuf)[2*i]=pcm_l[i];
	      ((short *)audiobuf)[2*i+1]=pcm_r[i];
	    }
	    audiobuflen*=2;
	    free(pcm_l);
	    free(pcm_r);
	  }
	  if(audiobuflen==0) 
	    if(debug>=2) {fprintf(stderr,"@@ mp3 frame -- need more data\n");}
	  if(audiobuflen<0) {
	    fprintf(stderr,"PB lame_decode\n");
	    exit(1);
	  }
	  audiobuflen*=2;
	} else 
#endif
	  {
	    fprintf(stderr, "Audio format not implemented \n");
	    fprintf(stderr, "convert with \"mencoder -oac pcm -ovc xvid\"\n");
	    exit(1);
	  }
      }
      if(debug>=2) fprintf(stderr,"audiobuflen=%d\n",audiobuflen);
      if(audiobuflen>0) {
	unlok_lock_mutex();      
      }
    }
    if(r==0) {
      fprintf(stderr,"END OF AVI FILE REACHED\n");
      AVI_seek_start(videofile);
      if(avibench || !videofile->seekable) {
	int clockpersec = sysconf(_SC_CLK_TCK);
	clock_t t_end;
	struct tms end;
	file_audio_stop();
	t_end = times(&end);
	printf("total  %.3lf secs\n",(double)(t_end-t_start)/clockpersec);
	printf("user   %.3lf secs\n",
	       (double)(end.tms_utime-start.tms_utime)/clockpersec);
	printf("system %.3lf secs\n",
	       (double)(end.tms_stime-start.tms_stime)/clockpersec);
	ExitCB(app_shell, NULL, NULL);
	exit(0);
      }
    }
  } while(r==2 || r==0);
  if(r!=1) return r; else return len;
}


static int
grab_open(struct device_t *device)
{
  int i;
#ifdef HAVE_XVID
#ifdef HAVE_XVID09
  XVID_INIT_PARAM ini;
#else
  xvid_gbl_init_t ini;
#endif
#endif

  videofile = AVI_open_input_file(avifilename, 1);
  if(videofile == NULL) {
    AVI_print_error("AVI_open_input_filename");
    return -1;
  }
  if(debug) {
    fprintf(stderr,"##AVI: width=%ld height=%ld fourcc=%4s\n",
	    videofile->width,videofile->height,videofile->compressor);
    fprintf(stderr,"##AVI: a_chans=%ld a_rate=%ld a_fmt=%ld a_bits=%ld\n",
	    videofile->a_chans, videofile->a_rate, videofile->a_fmt, videofile->a_bits);
  }
  video_set_capture_size(videofile->width, videofile->height);
  if(str_to_int(videofile->compressor, fourccs_raw) != -1) {
    raw_direct = 1;
  }
#ifdef HAVE_XVID
  else if(!strncasecmp(videofile->compressor,"XVID",4)) {
    memset(&ini,0,sizeof(ini));
#ifdef HAVE_XVID09
    if(xvid_init(NULL, 0, &ini, NULL)) {
      fprintf(stderr,"initialization xvid failed\n");
      return -1;
    }
    if(ini.api_version != API_VERSION) {
      fprintf(stderr,"xvid has changed since compilation, recompile !\n");
      return -1;
    }
#else
    ini.version = XVID_VERSION;
    if(xvid_global(NULL, 0, &ini, NULL)<0) {
      fprintf(stderr,"initialization xvid failed\n");
      return -1;
    }
    xvid_dec_p.version = XVID_VERSION;
#endif
    xvid_dec_p.width = videofile->width;
    xvid_dec_p.height = videofile->height;
    if(xvid_decore(NULL, XVID_DEC_CREATE, &xvid_dec_p, NULL)) {
      fprintf(stderr, "xvid init failed\n");
      return -1;
    }
    bufcompress=(char *) malloc(6*videofile->width*videofile->height);
  }
#endif
  else {
    fprintf(stderr,"video format (fourcc=%4s) not implemented\n",videofile->compressor);
    fprintf(stderr, "convert with \"mencoder -oac pcm -ovc xvid\"\n");
    exit(1);
  }
  nbufs = nbufs_default;
  if(nbufs > 3) nbufs = 3;
  buffers = (void **)(malloc(nbufs*sizeof(void*)));
  for(i=0;i<nbufs;i++)
    buffers[i] = malloc((videofile->width+15)*videofile->height*4);
  delay = (double)videofile->fps_den / videofile->fps_num;
#ifdef HAVE_LAME
  if(videofile->a_fmt==WAVE_FORMAT_MPEGLAYER3) {
    lame_decode_init();
    videofile->a_bits=16;
  }
#endif
 audio_buf_max_len=AVIAUDIOBUF*sizeof(char);
 if (NULL==(audiobuf=(char*) malloc(audio_buf_max_len))){
 	perror("malloc");
 }

 t_start = times(&start);
  
 init_semaphores_or_mutex();

  file_audio_start();
  return 0;
}

static int
grab_close ()
{
  int i;
  file_audio_stop();
  destroy_semaphores_or_mutex();
  
#ifdef HAVE_XVID
  if(!strncasecmp(videofile->compressor,"XVID",4)) 
    xvid_decore(xvid_dec_p.handle,XVID_DEC_DESTROY, NULL, NULL);
#endif
#ifdef HAVE_LAME_DECODE_EXIT
//  lame_decode_exit();
#endif
  AVI_close(videofile);
  videofile=NULL;
  for(i=0;i<nbufs;i++) free(buffers[i]);
  free(buffers);
  free(audiobuf);
  return 0;
}

/*
static int
grab_overlay (int x, int y, int width, int height, int format,
              struct OVERLAY_CLIP *oc, int count)
{
  return 0;
}
*/

void * get_img(video_fmt format, int width, int height) {
  struct timeval tv;
#ifdef HAVE_XVID
  int r;
#ifdef HAVE_XVID09
  XVID_DEC_FRAME dec;
#else
  xvid_dec_frame_t dec;
#endif
#endif
  int imgprec=img;

  img++;
  if(img==nbufs) img=0;
#ifdef HAVE_XVID
  if(!strncasecmp(videofile->compressor,"XVID",4)) {
    if(width<videofile->width || width>= videofile->width+16) {
      fprintf(stderr,
	      "avi_width=%ld requested_width in [%d %d]\n",videofile->width,
	      width-15,width);
      return NULL;
    }
  } else 
#endif
    if(width!=videofile->width) {
      fprintf(stderr,
	      "avi_width=%ld requested_width=%d !\n",videofile->width,width);
      return NULL;
    }
  if(height!=videofile->height) {
    fprintf(stderr,
	    "avi_height=%ld requested_height=%d !\n",videofile->height,height);
    return NULL;
  }
  if(debug) 
    fprintf(stderr,"extracting frame %ld\n",videofile->video_pos);
  
  if(raw_direct) {
    int l=avireaduncompressedframe(buffers[img]);
    if(l<0) {
      fprintf(stderr, "PB avireaduncompressedframe\n");
      return NULL;
    }
    if(l==0) {
      fast_memcpy(buffers[img], buffers[imgprec], size_img(format,width,height));
      if(debug) fprintf(stderr, "duplicated frame ?\n");
    }
  }
#ifdef HAVE_XVID
  else if(!strncasecmp(videofile->compressor,"XVID",4)) {
    int l=avireaduncompressedframe(bufcompress);
    if(l<0) {
      fprintf(stderr, "PB avireaduncompressedframe\n");
      return NULL;
    }
    if(l==0) {
      fast_memcpy(buffers[img], buffers[imgprec], size_img(format,width,height));
      if(debug) fprintf(stderr, "duplicated frame ?\n");
    } else {
      memset(&dec,0,sizeof(dec));
#ifdef HAVE_XVID09
      dec.colorspace = fmts_xaw_to_xvidcolorspace[format];
      dec.image = buffers[img];
      dec.stride = width;
#else
      dec.version = XVID_VERSION;
      dec.output.csp = fmts_xaw_to_xvidcolorspace[format];
      dec.output.plane[0] = buffers[img];
      dec.output.stride[0] = size_img(format,width,1);
#endif
      dec.bitstream = bufcompress;
      dec.length = l;
      r=xvid_decore(xvid_dec_p.handle, XVID_DEC_DECODE,&dec,NULL);
#ifdef HAVE_XVID09
      if(r>0) r=-1000-r;
#endif
      if(r<0) {
	fprintf(stderr, "xvid decoding error, the file is probably not XVID r=%d\n",r);
	fprintf(stderr, "  convert with 'mencoder -ovc xvid -oac copy'\n");
	return NULL;
      }
    }
  }
#endif
  gettimeofday (&tv, NULL);
  videostampmax=tv.tv_usec/1000000.0+tv.tv_sec;
  if(!avibench && videostampmax<videostampmin+delay) {
    usleep(1000000*(delay-videostampmax+videostampmin));
  }
  videostampmin=videostampmax;
  return buffers[img];
}  

static int fmt_available(video_fmt f) {
  if(raw_direct)
    return f == str_to_int(videofile->compressor,fourccs_raw);
#ifdef HAVE_XVID
  else
    return fmts_xaw_to_xvidcolorspace[f]!=0;
#else
  return 0;
#endif
}


/* ---------------------------------------------------------------------- */

static int
grab_tune (const struct CHANNEL* chan)
{
  
  if(strncmp(avifilename,"http://",7)==0) {
    char buf[100];
    sprintf(buf,"freq %d",chan->freq);
    write(videofile->fdes,buf,strlen(buf)+1);
  }
  return 0;
}

static int
grab_tuned ()
{
  return 1;
}

static int
grab_input (int input, int norm)
{
  if(strncmp(avifilename,"http://",7)==0) {
    char buf[100];
    if(input!=-1) {
      sprintf(buf,"input %s",int_to_str(input,inputs));
      write(videofile->fdes,buf,strlen(buf)+1);
    }
    if(norm!=-1) {
      sprintf(buf,"norm %s",int_to_str(norm,norms));
      write(videofile->fdes,buf,strlen(buf)+1);
    }      
  }
  return 0;
}

static int
grab_picture (int color, int bright, int hue, int contrast)
{
  /* TODO */
  return 0;
}

static int
grab_audio (int mute, int volume, int *mode)
{
  return 0;
}

static int is525(int norm) {
  return norm == 1 || norm == 6 || norm==7;
}

static int issecam(int norm) {
  return norm == 2;
}

static void* get_buf(int i){
  return buffers[i];
}

struct GRABBER grab_avi = {
  "avi",
  norms, inputs,
  grab_open,
  grab_close,
  NULL,
  fmt_available,
  get_img,
  grab_tune,
  grab_tuned,
  grab_input,
  grab_picture,
  grab_audio,
  is525,
  issecam,
  get_buf
};
