/*****************************************************************************
 * grab-file.h: common struct/tools for grab-avi and grab-dvb
 *  THE AVI FILE MUST BE IN XVID FORMAT.
 *****************************************************************************
 * $Id: grab-file.c,v 1.3 2006/02/14 15:48:43 estre Exp $
 *****************************************************************************
 * Copyright (C) 2003 estre
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "grab-file.h"
#include "avilib.h"
#include "divx.h"

void **buffers;
char* bufcompress;
double delay; // delay between two images
clock_t t_start;
struct tms start;
pthread_t audio_thread;
pthread_mutex_t file_mutex;
#ifndef NOTHAVE_SEM
sem_t file_sem,file_sem2;
#else
pthread_mutex_t file_mut,file_mut2,*file_mut_audio,*file_mut_video;
#endif

avi_t *   videofile=NULL ;

char* audiobuf;
int audiobuflen;
int use_file_audio=0;
int audio_buf_max_len=0;
int filenoaudioout=0;

#ifndef HAVE_SUNAUDIO
int file_audio_fragments = 2;
#else
int file_audio_fragments = 16;
#endif
int file_audio_sizefragment = 8192;


int file_extract_audiobuf(void *buf) {
  int r;
//  fprintf(stderr,"call to file_extract_audio\n");
#ifdef _POSIX_THREAD_IS_GNU_PTH
  pthread_yield_np();
#endif
#ifndef NOTHAVE_SEM
  sem_wait(&file_sem);
#else
  pthread_mutex_lock(file_mut_audio);
#endif
  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
  /* The thread must exit at this  point to be sure
     otherwise sem could have the value 2 */
  pthread_testcancel ();
  pthread_setcancelstate(PTHREAD_CANCEL_DISABLE,NULL);
  if(audiobuflen<=0) {
    /* happens with gdb ? */
    int x=-1;
    fprintf(stderr,"PB synchro\n");
#ifndef NOTHAVE_SEM
    while(sem_trywait(&file_sem));
#else
    while(pthread_mutex_trylock(file_mut_audio));
#endif
    pthread_exit(&x);
  }
  fast_memcpy(buf,audiobuf,audiobuflen);
//  only for test !!
//   fwrite(audiobuf,1, audiobuflen, outfileAudio);

  r=audiobuflen;
  audiobuflen=0;
#ifndef NOTHAVE_SEM
  sem_post(&file_sem2);
#else
  pthread_mutex_unlock(file_mut_audio);
  file_mut_audio = file_mut_audio=&file_mut ? &file_mut2 : &file_mut;
#endif
  return r;
}

void file_audio_stop(void){

  if(!use_file_audio) return;
  pthread_cancel (audio_thread);
#ifndef NOTHAVE_SEM
  sem_post(&file_sem);
#else
  pthread_mutex_unlock(file_mut_video);
#endif
  pthread_join (audio_thread, NULL);
  if(!filenoaudioout) audio_close();
  use_file_audio=0;
  pthread_mutex_destroy(&file_mutex);
}

void init_semaphores_or_mutex(void){
 t_start = times(&start);
	
#ifndef NOTHAVE_SEM
  sem_init(&file_sem ,0, 0);
  sem_init(&file_sem2,0,0);
#else
  pthread_mutex_init (&file_mut, NULL);
  pthread_mutex_init (&file_mut2, NULL);
  file_mut_audio = file_mut_video = &file_mut;
  pthread_mutex_lock(&file_mut);
#endif

}
void destroy_semaphores_or_mutex(void){

#ifndef NOTHAVE_SEM
  sem_destroy(&file_sem);
  sem_destroy(&file_sem2);
#else
  pthread_mutex_destroy(&file_mut);
  pthread_mutex_destroy(&file_mut2);
#endif
}
void file_wait_write_audio(void) {
  pthread_mutex_lock(&file_mutex);
  pthread_mutex_unlock(&file_mutex);
}
void unlok_lock_mutex(void){
	      
#ifndef NOTHAVE_SEM
	sem_post(&file_sem);
	sem_wait(&file_sem2);
#else
	pthread_mutex_unlock(file_mut_video);
	file_mut_video = file_mut_video=&file_mut ? &file_mut2 : &file_mut;
#ifdef _POSIX_THREAD_IS_GNU_PTH
	pthread_yield_np();
#endif
	pthread_mutex_lock(file_mut_video);
#endif
}


void* audio_main(void *arg) {
  int l;
  static char* audiobuf2;
  if (NULL == (audiobuf2=alloca(audio_buf_max_len))){
  	perror("alloca");
	exit(-1);
  }
  pthread_setcancelstate(PTHREAD_CANCEL_DISABLE,NULL);
  while(1) {
    l=file_extract_audiobuf(audiobuf2);
    if(!filenoaudioout) audio_write(audiobuf2, l);
    pthread_mutex_lock(&file_mutex);
    if (is_divx_in_progress()) {
      write_audio_to_avi((short*)audiobuf2, l/(videofile->a_bits/8));
    }
    pthread_mutex_unlock(&file_mutex);

  }
  pthread_exit(NULL);
}

void file_audio_start(void) {
  if(videofile==NULL) return;
  use_file_audio=1;
  if(!filenoaudioout) {
    if(!audio_open(AUDIO_WO, videofile->a_bits==8 ? AUDIO_U8 : AUDIO_S16LE,
		   videofile->a_chans, videofile->a_rate, file_audio_fragments,
		   file_audio_sizefragment)<0) {
      fprintf(stderr, "audio_open failed\n");
      pthread_exit(NULL);
    }
  }
  mixer_set_mode(MIXER_MODE_PCM);
  pthread_mutex_init (&file_mutex, NULL);
  pthread_create (&audio_thread, NULL, audio_main, NULL);
}


