 /******************************************************************************
 * pixmaps_loader.c: NLS support
 *******************************************************************************
 * $Id: pixmaps_loader.c,v 1.18 2006/06/18 18:50:00 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2005 Nico
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <dirent.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "config.h"
#include "strtab.h"

#ifndef RTLD_NOW /* For OpenBSD */
#define RTLD_NOW 0
#endif

#include "pixmaps_loader.h"
#include "translation.h"
#include "divx_ui.h"
#include "divx.h"
#include "conf.h"

// DVB!!!!!
#include "dvb_init_ui.h"

/*******************************************/
extern Widget opt_paned; 
extern Widget divx_paned;
extern Widget divxaudio_paned;
extern Widget streamparams_paned;
extern Widget pFileShell;
extern Widget alevtparams_paned;
extern Widget grab_paned;
extern Widget xosdparams_paned;
extern Widget plugin_paned;
extern Widget x11params_paned;
extern Widget popup;
extern Widget subpage_paned;
extern Widget conf_shell;
extern Widget pFileShell;
extern Widget pErrorShell;
#ifdef HAVE_FFMPEG
extern Widget ffmpeg_paned;
#endif
#ifdef HAVE_XVID
extern Widget xvid_paned;
#endif
// DVB!!!!
#ifdef HAVE_DVB
extern Widget dvbinit_paned;
#endif

/***************************************/

static int library_loaded;

extern int debug;

extern char* current_theme;
extern char* current_language;
extern struct STRTAB* theme_list;

static char *libdir = PREFIX "/lib/xdtv";
static char *lib_common_name="libxdtv";

void (*create_main_pix) (Display *display, Drawable* d);
void (*create_divx_pix) (Display *display, Drawable* d);
void (*create_divxaudio_pix) (Display *display, Drawable* d);
void (*create_fileselector_pix) ( Display *display, Drawable* d);
void (*create_fileselector_menu_pix) ( Display *display, Drawable* d);
void (*create_alevt_pix) (Display *display, Drawable* d);
void (*create_grab_pix) (Display *display, Drawable* d);
void (*create_xosd_pix) (Display *display, Drawable* d);
void (*create_plugin_pix) (Display *display, Drawable* d);
void (*create_x11_pix) (Display *display, Drawable* d);
void (*create_popup_pix) (Display *display, Drawable* d);
void (*create_help_pix) (Display *display, Drawable* d);
void (*create_subtitle_pix) (Display *display, Drawable* d);
#ifdef HAVE_FFMPEG
void (*create_ffmpeg_pix) (Display *display, Drawable* d);
#endif
#ifdef HAVE_XVID
void (*create_xvid_pix) (Display *display, Drawable* d);
#endif

// DivX Streaming
void (*create_divxstream_common_pix) (Display *display, Drawable* d);
void (*create_divxstream_on_pix) (Display *display, Drawable* d);
void (*create_divxstream_off_pix) (Display *display, Drawable* d);
void (*create_divxstream_started_pix) (Display *display, Drawable* d);
void (*create_divxstream_stopped_pix) (Display *display, Drawable* d);

// Editor
void (*create_editor_common_pix) (Display *display, Drawable* d);
void (*create_editor_off_pix) (Display *display, Drawable* d);
void (*create_editor_on_pix) (Display *display, Drawable* d);
#ifdef HAVE_DVB
void (*create_editor_dvb_pix) (Display *display, Drawable* d);
#endif

// DVB!!!!!!
#ifdef HAVE_DVB
void (*create_dvbinit_common_pix) (Display *display, Drawable* d);
void (*create_dvbinit_off_pix) (Display *display, Drawable* d);
void (*create_dvbinit_on_pix) (Display *display, Drawable* d);
#endif

static void* handle=NULL;

static struct theme_names {
  char* curr_name;
  struct theme_names *prev;
} *disp_themes = NULL;

void 
load_lib(char* name, int exit_on_error)
{
  char *error;
  int pixlib_error = 0;
  
  if(name == NULL) return;
  if(debug) fprintf(stderr, "Loading %s\n", name);
  
  handle = dlopen(name, RTLD_NOW);
  if (!handle) {
      error = dlerror();
      if(!exit_on_error && !strstr(error,"No such file"))
	fprintf(stderr, "dlopen: %s\n", error);
      if(exit_on_error) exit(1);
  }  else {

    create_main_pix   = dlsym(handle, "create_main_action_pixmap");
    create_divx_pix = dlsym(handle, "create_divx_pixmap");
    create_divxaudio_pix = dlsym(handle, "create_divxaudio_pixmap");
    create_fileselector_pix = dlsym(handle, "create_fileselector_pixmap");
    create_fileselector_menu_pix = dlsym(handle, "create_fileselector_menu_pixmap");
    create_alevt_pix = dlsym(handle, "create_alevt_pixmap");
    create_grab_pix = dlsym(handle, "create_grab_pixmap");
    create_xosd_pix = dlsym(handle, "create_xosd_pixmap");
    create_plugin_pix = dlsym(handle, "create_plugin_pixmap");
    create_x11_pix = dlsym(handle, "create_x11_pixmap");
    create_popup_pix = dlsym(handle, "create_popup_pixmap");
    create_help_pix = dlsym(handle, "create_help_pixmap");
    create_subtitle_pix = dlsym(handle, "create_subtitle_pixmap");
#ifdef HAVE_FFMPEG
    create_ffmpeg_pix = dlsym(handle, "create_ffmpeg_pixmap");
#endif
#ifdef HAVE_XVID
    create_xvid_pix = dlsym(handle, "create_xvid_pixmap");
#endif

// DivX Streaming
    create_divxstream_common_pix = dlsym(handle, "create_divxstream_common_pixmap");
    create_divxstream_on_pix = dlsym(handle, "create_divxstream_on_pixmap");
    create_divxstream_off_pix = dlsym(handle, "create_divxstream_off_pixmap");
    create_divxstream_started_pix = dlsym(handle, "create_divxstream_started_pixmap");
    create_divxstream_stopped_pix = dlsym(handle, "create_divxstream_stopped_pixmap");

// Editor
    create_editor_common_pix = dlsym(handle, "create_editor_common_pixmap");
    create_editor_off_pix = dlsym(handle, "create_editor_off_pixmap");
    create_editor_on_pix = dlsym(handle, "create_editor_on_pixmap");
#ifdef HAVE_DVB
    create_editor_dvb_pix = dlsym(handle, "create_editor_dvb_pixmap");
#endif

// DVB!!!
#ifdef HAVE_DVB
    create_dvbinit_common_pix = dlsym(handle, "create_dvbinit_common_pixmap");
    create_dvbinit_off_pix = dlsym(handle, "create_dvbinit_off_pixmap");
    create_dvbinit_on_pix = dlsym(handle, "create_dvbinit_on_pixmap");
#endif

    if ((error = dlerror()) != NULL){
      if (debug||1)
	fprintf(stderr, "dlsym: %s\n", error);
      fprintf(stderr,
	      "WARNING: A pixmap library has been find but is unable to provide the create_pixmap function.\n");
      pixlib_error = 1;
    }  
    if (pixlib_error) {
      dlclose(handle);
      handle = NULL;
      fprintf(stderr, "!!!!!!!!! %s BADLY LOADED !!!!!!!!!!!.\n", name);
      if(exit_on_error)
	      exit(1);
    }else {
    	library_loaded=1;
    } 
  }
}

void 
close_pixlib(){
  if (handle !=NULL)
    dlclose(handle);
}


void reload_pixlib(int loadpix,char* theme)
{
	if (strcmp(theme,current_theme) == 0  || !library_loaded)
		return;
	
	set_theme_name(theme);
	close_pixlib();
	load_pixlib(loadpix,theme);

	create_main_pixmaps (XtDisplay(opt_paned) , &(RootWindowOfScreen(XtScreen(opt_paned)))) ; 
	create_divx_pixmaps(XtDisplay(divx_paned),&(RootWindowOfScreen(XtScreen(divx_paned))));
	create_divxaudio_pixmaps(XtDisplay(divxaudio_paned),&(RootWindowOfScreen(XtScreen(divxaudio_paned))));
	create_fileselector_pixmaps(XtDisplay(pFileShell),&RootWindowOfScreen(XtScreen(pFileShell)));
	create_fileselector_menu_pixmaps(XtDisplay(pFileShell),&RootWindowOfScreen(XtScreen( pFileShell )));
	create_alevt_pixmaps(XtDisplay(alevtparams_paned) , &RootWindowOfScreen(XtScreen(alevtparams_paned)));
	create_grab_pixmaps(XtDisplay(grab_paned),&RootWindowOfScreen(XtScreen(grab_paned)));
	create_xosd_pixmaps(XtDisplay(xosdparams_paned),&RootWindowOfScreen(XtScreen(xosdparams_paned)));
	create_plugin_pixmaps(XtDisplay(plugin_paned),&RootWindowOfScreen(XtScreen(plugin_paned)));
	create_x11_pixmaps(XtDisplay(x11params_paned),&RootWindowOfScreen(XtScreen(x11params_paned)));
	create_popup_pixmaps (XtDisplay(pErrorShell),&RootWindowOfScreen(XtScreen(pErrorShell)));
//	create_help_pixmaps(XtDisplay(popup),&RootWindowOfScreen(XtScreen(popup)));
	create_subtitle_pixmaps(XtDisplay(subpage_paned),&RootWindowOfScreen(XtScreen(subpage_paned)));
#ifdef HAVE_FFMPEG
        create_ffmpeg_pixmaps(XtDisplay(ffmpeg_paned),&RootWindowOfScreen(XtScreen(ffmpeg_paned)));
#endif
#ifdef HAVE_XVID
        create_xvid_pixmaps(XtDisplay(xvid_paned),&RootWindowOfScreen(XtScreen(xvid_paned)));
#endif

// DivX Streaming
	create_divxstream_common_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
	if (divxstream_is_enabled()){
	  create_divxstream_off_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
	  create_divxstream_on_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
	  if(is_divx_in_progress()) {
	    create_divxstream_started_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
          }else {
	    create_divxstream_stopped_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
          }
	}else {
	  create_divxstream_on_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
	  create_divxstream_off_pixmaps(XtDisplay(streamparams_paned),&RootWindowOfScreen(XtScreen(streamparams_paned)));
	}

// Editor
	create_editor_common_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
	if (complete_scan){
	  create_editor_off_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
	  create_editor_on_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
	}else {
	  create_editor_on_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
	  create_editor_off_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
	}
#ifdef HAVE_DVB
	create_editor_dvb_pixmaps (XtDisplay(conf_shell),&(RootWindowOfScreen(XtScreen(conf_shell))));
#endif

// DVB!!!!!!
#ifdef HAVE_DVB
	create_dvbinit_common_pixmaps(XtDisplay(dvbinit_paned),&(RootWindowOfScreen(XtScreen(dvbinit_paned))));
	if (dvbinit_is_enabled()){
	  create_dvbinit_off_pixmaps(XtDisplay(dvbinit_paned),&(RootWindowOfScreen(XtScreen(dvbinit_paned))));
	  create_dvbinit_on_pixmaps(XtDisplay(dvbinit_paned),&(RootWindowOfScreen(XtScreen(dvbinit_paned))));	
	}else {
	  create_dvbinit_on_pixmaps(XtDisplay(dvbinit_paned),&(RootWindowOfScreen(XtScreen(dvbinit_paned))));
	  create_dvbinit_off_pixmaps(XtDisplay(dvbinit_paned),&(RootWindowOfScreen(XtScreen(dvbinit_paned))));
	}
#endif
}

void 
set_theme_name(char* theme)
{
  if (theme != NULL ){
	if (strlen(theme)>21){
	    fprintf (stderr,"Warning, your theme name is too long (>21): %d\n",strlen(theme));
	    fprintf (stderr,"   => I take the default\n");
	    theme = "default";
	}
    	current_theme=theme;
  }
  else if (current_theme == NULL ){
	current_theme="default";
  }
  else if (strlen(current_theme)==0){
	current_theme="default";    
  }
}
void 
parse_lib_name(char* lib_theme_name,char* lib_lang, const char* relative_name )
{
  char* pch;
  char* parsed_name=NULL;
  if (NULL==(parsed_name=alloca((strlen(relative_name)+1)*sizeof(char)))){
  	perror("alloca");
	return;
  }
  strcpy(parsed_name,relative_name );
  pch = strtok (parsed_name,"-");
  while (pch != NULL)
  {
    pch = strtok (NULL, "-");
    if (strcmp(pch, "i18n")==0){
    	strcpy( lib_theme_name, "default");
    	strcpy( lib_lang, strtok (NULL, ".")  );
	return;
    }else if (strcmp(pch, "theme")==0){
	strcpy(lib_theme_name , strtok (NULL, "-"));
    	strcpy(lib_lang, strtok (NULL, ".")  );
    	return;
    }
  }
}

void 
load_pixlib(int loadpix, char* theme)
{

  struct stat buf;
  DIR *dir;
  struct dirent *dir2;
  int i;
  int  nbCompatibleThemes=0;
  
  library_loaded=0;

  if (loadpix ==-1)
	  return;
  
#ifdef HAVE_PIXMAPS  
  if(libdir==NULL) return;
  dir=opendir(libdir);
  if(dir==NULL) {
    if(errno!=ENOENT) perror("opendir");
    return;
  }
  set_theme_name(theme );
     
  while ((dir2 = readdir(dir)) != NULL) {
    char absolute_name[255];
    char needed_lib[60];
    char lib_theme_name[22];
    char lib_lang[3];

/* default library is something like : libxdtv-i18n-$LANG.so . if theme option is enabled, we want something like : libxdtv-theme-$THEME-$LANG.so
*/
    if (strcmp(current_theme, "default")==0) 
	sprintf(needed_lib,"%s-%s-%s.so",lib_common_name,"i18n", translate("en"));
    else 
    	sprintf(needed_lib,"%s-%s-%s-%s.so",lib_common_name,"theme",current_theme, translate("en"));
    
    sprintf(absolute_name,"%s/%s",libdir,dir2->d_name);
    if (stat (absolute_name, &buf) == 0 && S_ISREG(buf.st_mode)){
            parse_lib_name(lib_theme_name,lib_lang,dir2->d_name);
	    if (strcmp(translate("en"),lib_lang)==0){
            	   struct theme_names *p;
      		   p=malloc(sizeof(struct theme_names));
      		   p->prev=disp_themes;
      		   p->curr_name=strdup(lib_theme_name);
      		   disp_themes=p;
		   nbCompatibleThemes++; 
	    }
	    if ((strcmp(dir2->d_name,needed_lib)==0))
      		load_lib(absolute_name, 0);
    }
  }

  if ( NULL == (theme_list=malloc((nbCompatibleThemes+1)*sizeof(*theme_list))) ) {
  	fprintf(stderr, "theme_list allocation failed");
	exit (EXIT_FAILURE);
  }
  i=0;
  while(disp_themes) {
    struct  theme_names *p= disp_themes;
    theme_list[i].nr=i;
    theme_list[i].str=p->curr_name;
    disp_themes=p->prev;
    free(p); // curr_name is used in theme list, so we dont free it.
    i++; 
  }
  closedir(dir);
  dir=opendir(libdir);
  if(dir==NULL) {
    if(errno!=ENOENT) perror("opendir");
    return;
  }
  theme_list[nbCompatibleThemes].nr=nbCompatibleThemes;
  theme_list[nbCompatibleThemes].str=NULL;
#endif
}

void
create_main_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_main_pix)(display, d);
}

void
create_editor_common_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_editor_common_pix)(display, d);
}

// Editor
void
create_editor_off_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_editor_off_pix)(display, d);
}

void
create_editor_on_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_editor_on_pix)(display, d);
}

#ifdef HAVE_DVB
void
create_editor_dvb_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_editor_dvb_pix)(display, d);
}
#endif
// END Editor

void
create_divx_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_divx_pix)(display, d);
}

void
create_divxaudio_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_divxaudio_pix)(display, d);
}

// DivX Streaming
void
create_divxstream_common_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_divxstream_common_pix)(display, d);
}

void
create_divxstream_on_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_divxstream_on_pix)(display, d);
}

void
create_divxstream_off_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_divxstream_off_pix)(display, d);
}

void
create_divxstream_started_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_divxstream_started_pix)(display, d);
}

void
create_divxstream_stopped_pixmaps (Display *display, Drawable* d ){
	if (library_loaded)
		(*create_divxstream_stopped_pix)(display, d);
}
// END DivX Streaming

void 
create_fileselector_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_fileselector_pix)(display,d);
}

void 
create_fileselector_menu_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_fileselector_menu_pix)(display,d);
}

void 
create_alevt_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_alevt_pix)(display,d);
}

void 
create_grab_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_grab_pix)(display,d);
}

void 
create_xosd_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_xosd_pix)(display,d);
}

void 
create_plugin_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_plugin_pix)(display,d);
}

void 
create_x11_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_x11_pix)(display,d);
}

void 
create_popup_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_popup_pix)(display,d);
}

void 
create_help_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_help_pix)(display,d);
}

void 
create_subtitle_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_subtitle_pix)(display,d);
}

#ifdef HAVE_FFMPEG
void 
create_ffmpeg_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_ffmpeg_pix)(display,d);
}
#endif

#ifdef HAVE_XVID
void 
create_xvid_pixmaps(Display *display, Drawable* d){
	if (library_loaded)
		(*create_xvid_pix)(display,d);
}
#endif

// DVB!!!!!!!!!!!!!!!
#ifdef HAVE_DVB
void
create_dvbinit_common_pixmaps(Display *display, Drawable* d ){
	if (library_loaded)
		(*create_dvbinit_common_pix)(display, d);
}

void
create_dvbinit_off_pixmaps(Display *display, Drawable* d ){
	if (library_loaded)
		(*create_dvbinit_off_pix)(display, d);
}

void
create_dvbinit_on_pixmaps(Display *display, Drawable* d ){
	if (library_loaded)
		(*create_dvbinit_on_pix)(display, d);
}
#endif
// END DVB !!!!!!!!!!!!!
