/*****************************************************************************
 * scan_common.c: check available channels
 *****************************************************************************
 * $Id: scan_common.c,v 1.7 2006/06/26 10:42:10 estre Exp $
 *****************************************************************************
 * Copyright (C) 2003 Alain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************
 *
 * Original code:
 *
 *  (c) Gerd Knorr <kraxel@cs.tu-berlin.de>
 *
 *****************************************************************************/


#include "config.h"


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "scan_common.h"

#ifdef HAVE_DVB
#include "dvbtools.h"
#include "dvb/scan.h"
#endif


// internationalization

#include "translation.h"


#ifdef HAVE_ZVBI
#include <libzvbi.h>
#endif

#define NEW_CHAN_NB_MAX 50

extern struct GRABBER *grabbers[];
extern int grabber;
static char *tvname=NULL;

#ifdef HAVE_DVB
extern struct GRABBER grab_dvb;
#endif
chan_handler_t* chan_handler=NULL;
static int chan_handler_ready=0;

chan_handler_t* get_chan_handler(){
	while (!chan_handler_ready)usleep(100000);
	return chan_handler;
}
void new_channel_accepted(chan_handler_t* chan_hdl){
	if (!chan_hdl) return;
	chan_hdl->new_channel_available=0;
}

int new_channel_available(chan_handler_t* chan_hdl){
	if (!chan_hdl) return 0;
	return chan_hdl->new_channel_available;
}

void chan_hdl_is_in_use(chan_handler_t* chan_hdl){
	if (!chan_hdl) return;
    	pthread_mutex_lock(&(chan_hdl->in_use_mutex));
	chan_hdl->is_in_use=1;
    	pthread_mutex_unlock(&(chan_hdl->in_use_mutex));
}

void chan_hdl_is_not_in_use(chan_handler_t* chan_hdl){
	if (!chan_hdl) return;
    	pthread_mutex_lock(&(chan_hdl->in_use_mutex));
	chan_hdl->is_in_use=0;
    	pthread_mutex_unlock(&(chan_hdl->in_use_mutex));
}

static chan_handler_t * new_chan_handler(){
	chan_handler_t * chan_hdl=NULL;

	if(NULL==(chan_hdl=malloc(sizeof *chan_hdl))){
		perror("malloc");
		return NULL;
	}
	if (!chan_hdl){
		fprintf(stderr, "scan_common.c : chan_hdl must be allocated !\n");
		return NULL;
	}
	if (NULL == (chan_hdl->new_channels=malloc(NEW_CHAN_NB_MAX * sizeof(struct CHANNEL)))){
		perror("malloc");
		return NULL;
	}
	chan_hdl->chan_number=-1;
	chan_hdl->scan_running=0;
	chan_hdl->is_in_use=0;
	chan_hdl->current_freq=0;
	chan_hdl->new_channel_available=0;
	pthread_mutex_init (&(chan_hdl->scan_mutex), NULL);
	pthread_mutex_init (&(chan_hdl->in_use_mutex), NULL);
	chan_handler_ready=1;
	
	return chan_hdl;
}

static int delete_chan_handler(chan_handler_t * chan_hdl){
	int i;
	while (1) {
		 int wait;
		 pthread_mutex_lock(&(chan_hdl->in_use_mutex));
		 wait=chan_hdl->is_in_use;
    		 pthread_mutex_unlock(&(chan_hdl->in_use_mutex));
		 if (wait) usleep(100); else break;
		 if (debug > 2)
		 	fprintf(stderr,"scan_common.c : chan_hdl is in use ...\n");
	}
	pthread_mutex_lock(&(chan_hdl->scan_mutex));
	
	for (i=0;i<chan_hdl->chan_number;i++){
		struct CHANNEL* to_delete=&(chan_hdl->new_channels[i]);
		if (debug > 2)
			fprintf(stderr,"scan_common.c : freeing channel %d\n",i);
		if (to_delete->name)
			free(to_delete->name);
		if (to_delete->key)
			free(to_delete->key);
		if (to_delete->cname)
			free(to_delete->cname);
		if (to_delete->subpage)
			free(to_delete->subpage);
	}

        if (debug > 2)
		fprintf(stderr,"scan_common.c : last freeing ...\n");
	free (chan_hdl->new_channels);
	pthread_mutex_unlock(&(chan_hdl->scan_mutex));
	pthread_mutex_destroy(&(chan_hdl->scan_mutex));
	
	free(chan_hdl);
	

	return 0;
}



#ifdef HAVE_ZVBI
extern int vbifd;

static void
event(struct vbi_event *ev, void *user)
{
  switch (ev->type) {
  case VBI_EVENT_NETWORK:
    if (NULL != tvname)
      free(tvname);
    if (strlen((const char*)(ev->ev.network.name)) > 0)
      tvname = strdup((const char*)(ev->ev.network.name));
    break;
  }
}
struct vbi_capture *vbi_cap;
struct vbi_decoder *vbi_dec;
void * vbi_data;
int vbi_lines;
vbi_sliced *vbi_sli;
#endif

void gettvname(char *channelname, int freq) {
  char tvname_unknown[100];
#ifdef HAVE_ZVBI
  double   vbi_timestamp;
  struct timeval vbi_timeout, initial_time, current_time;
#endif
  
  tvname=NULL;
#ifdef HAVE_ZVBI
  gettimeofday (&initial_time, NULL);
  for(;;) {
    int ret;
    long delay;
    gettimeofday (&current_time, NULL);
    delay=5000000-((1000000*current_time.tv_sec+current_time.tv_usec)-
		   (1000000*initial_time.tv_sec+initial_time.tv_usec));
    if(delay<0) break;
    vbi_timeout.tv_sec=delay/1000000;
    vbi_timeout.tv_usec=delay%1000000;
    fprintf(stderr,translate("channel=%s freq=%.3f timeout=%.1f  %c"),channelname,
	    (float)freq/16,(float)delay/1000000,'\r');
    fflush(stderr);
    ret=vbi_capture_read(vbi_cap, vbi_data, vbi_sli,
			 &vbi_lines, &vbi_timestamp, &vbi_timeout);
    if(ret==0) break;
    if(ret<0) {
      if(errno==EIO) {usleep(100000);continue;}
      perror("\nvbi_capture_read");
      break;
    }
    vbi_decode(vbi_dec, vbi_sli, vbi_lines, vbi_timestamp);
    if(tvname!=NULL) break;
  }
#endif
  if(tvname==NULL) 
    {
      if(channelname!=NULL)
	sprintf(tvname_unknown,translate("unknown (%s)"),channelname);
      else 
	sprintf(tvname_unknown,translate("unknown (%.3f MHz)"),(float) freq / 16);
      tvname = strdup(tvname_unknown);
    }
}

void*  scan_v4l(void * arg){
  int i;
  struct GRABBER *grab = grabbers[grabber];
  v4l_param_t	param_analog;
  v4l_param_t*	param_analog_p= &param_analog;
  
  param_analog_p->fullscan=0;
  param_analog_p->tvnorm_index= cur_norm ; 
  param_analog_p->freqtab_index=  chan_tab ;

  chan_handler=new_chan_handler();

  grab->grab_input(str_to_int("Television",grab->inputs),
		   param_analog_p->tvnorm_index);
  grab->grab_audio(0,65535,NULL);
#ifdef HAVE_ZVBI
  close(vbifd);vbifd=-1;
  param_analog_p->services= VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625
    | VBI_SLICED_TELETEXT_B | VBI_SLICED_CAPTION_525
    | VBI_SLICED_CAPTION_625 | VBI_SLICED_VPS
    | VBI_SLICED_WSS_625 | VBI_SLICED_WSS_CPR1204;
  vbi_cap=vbi_capture_v4l_new(device.vbi,16,&(param_analog_p->services),-1,&(param_analog_p->err),0);
  if(vbi_cap==NULL)
    {
      fprintf(stderr,translate("Unable to open vbi device\n"));
      exit(1);
    }
  vbi_dec=vbi_decoder_new();
  vbi_event_handler_add(vbi_dec,~0,event,NULL);
  param_analog_p->vbi_par=vbi_capture_parameters(vbi_cap);
  param_analog_p->fullscan=*((int*) arg);
  vbi_lines=(param_analog_p->vbi_par->count[0] + param_analog_p->vbi_par->count[1]);
  vbi_data=malloc(vbi_lines * (param_analog_p->vbi_par->bytes_per_line));
  vbi_sli = malloc(vbi_lines * sizeof(vbi_sliced));
#endif

  
  chan_handler->scan_running =1;	

  if(!(param_analog_p->fullscan)){
    for(i=0;i<CHAN_ENTRIES;i++) {
      param_analog_p->freq=tvtuner[i].freq[param_analog_p->freqtab_index];
      if(param_analog_p->freq) {
	struct CHANNEL chan; // Temporary !!	
	chan.freq=param_analog_p->freq * 16 / 1000;
	chan_handler->current_freq=chan.freq;
	fprintf(stderr,"%d/%d %s %.3lf   \r",i,CHAN_ENTRIES,tvtuner[i].name,(float)param_analog_p->freq/1000);
	fflush(stderr);
	grab->grab_tune(&chan);

	usleep(200000);
	if(grab->grab_tuned()) {
	  struct CHANNEL* cur_chan=NULL;
	  gettvname(tvtuner[i].name,param_analog_p->freq*16/1000);
	  printf(translate("channel=%s freq=%.3f tvname=%s        \n"),
		 tvtuner[i].name,(float)param_analog_p->freq/1000,tvname);
	  
	  if (debug > 2 )
	  	fprintf(stderr,"scan_common.c : try to lock mutex\n");
	  pthread_mutex_lock(&(chan_handler->scan_mutex));
	  if (debug > 2 )
	  	fprintf(stderr,"scan_common.c : ok mutex locked\n");
  	  chan_handler->new_channel_available=1;

	  /* channel init */
	  
	  chan_handler->chan_number++;
	  cur_chan=&(chan_handler->new_channels[chan_handler->chan_number]);
	  memset(cur_chan,0,sizeof * cur_chan);
	  cur_chan->name=NULL;
	  cur_chan->key=NULL;
	  cur_chan->cname=NULL;
	  cur_chan->subpage=NULL;

	  cur_chan->name=strdup(tvname);
	  cur_chan->key=strdup("x"); 
  	  cur_chan->cname = strdup(tvtuner[i].name);
	  cur_chan->norm = param_analog_p->tvnorm_index;
	  cur_chan->channel_type = ANALOG;

	  if (debug > 2 )
	  	fprintf(stderr,"scan_common.c : unlock mutex\n");
	  pthread_mutex_unlock(&(chan_handler->scan_mutex));
	}
      }
    } 
  }else{
      int f,min_freq=44,max_freq=958;
      char *channel;
      for (f = min_freq*16; f <= max_freq*16; f += 4) 
	{
          struct CHANNEL chan; // Temporary !!	
	  chan.freq=f;
          chan_handler->current_freq=chan.freq;
	  
	  grab->grab_tune(&chan);
	  channel=NULL;
	  for(i=0;i<CHAN_ENTRIES;i++)
	    if(f*1000==tvtuner[i].freq[param_analog_p->freqtab_index]*16) 
	      {channel=tvtuner[i].name;break;}
	  fprintf(stderr,translate("freq=%.3f (min=%d max=%d) channel=%s     %c"),
		  (float) f / 16,min_freq,max_freq,channel,'\r');
	  usleep(200000);
	  if(grab->grab_tuned()) {

            struct CHANNEL* cur_chan=NULL;
		  
	    gettvname(channel,f);
	    printf(translate("freq=%.3f channel=%s tvname=%s        \n"),
		   (float)f/16,channel,tvname);


	    if (debug > 2 )
		fprintf(stderr,"scan_common.c : try to lock mutex\n");
	    pthread_mutex_lock(&(chan_handler->scan_mutex));
	    if (debug > 2 )
		fprintf(stderr,"scan_common.c : ok mutex locked\n");
	   chan_handler->new_channel_available=1;

	   /* channel init */
		  
	   chan_handler->chan_number++;
	   cur_chan=&(chan_handler->new_channels[chan_handler->chan_number]);
	   memset(cur_chan,0,sizeof * cur_chan);
	   cur_chan->name=NULL;
	   cur_chan->key=NULL;
	   cur_chan->cname=NULL;
	   cur_chan->subpage=NULL;

	   cur_chan->name=strdup(tvname);
	   cur_chan->key=strdup("x"); 
	   cur_chan->cname = strdup(tvtuner[i].name);
	   cur_chan->norm = param_analog_p->tvnorm_index;
	   cur_chan->channel_type = ANALOG;

	   if (debug > 2 )
		fprintf(stderr,"scan_common.c : unlock mutex\n");
	   pthread_mutex_unlock(&(chan_handler->scan_mutex));

	  }
	}
    }

  chan_handler->scan_running=2;	
  if(debug > 2)
  	fprintf(stderr,"scan_common : ok scan finished\n");
#ifdef HAVE_ZVBI
  vbi_capture_delete(vbi_cap);
  vbi_decoder_delete(vbi_dec);
  free(vbi_sli);
  free(vbi_data);
  if(tvname!=NULL) free(tvname);
#endif
  delete_chan_handler(chan_handler);
  if(debug > 2)
  	fprintf(stderr,"scan_common : ok return\n");
  return NULL;

}

#ifdef HAVE_DVB

void* scan_dvb(void * arg){
  char* outname="/tmp/xdtv_dvb_chan";
  FILE* conf=NULL;
  char* dvb_init_conf= get_dvbinit_file_name ();
  
  if (!dvb_init_conf) return NULL;

  conf=fopen(outname,"w");
  
  chan_handler=new_chan_handler();


  chan_handler->scan_running =1;	
//  grab_dvb.grab_close();
  scan_main(dvb_init_conf , conf);
  fclose(conf);
//  conf=fopen(outname,"r");

  chan_handler->scan_running=2;	

  delete_chan_handler(chan_handler);
return NULL;
}

#endif

