/*****************************************************************************
 * xdtv_scantv.c: check available channels
 *****************************************************************************
 * $Id: xdtv_scantv.c,v 1.27 2007/02/15 22:13:50 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2003 Alain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************
 *
 * Original code:
 *
 *  (c) Gerd Knorr <kraxel@cs.tu-berlin.de>
 *
 *****************************************************************************/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

/*#ifdef HAVE_DVB
#include "dvbtools.h"
#include "dvb/scan.h"
#endif
*/

// internationalization

#include "translation.h"

int debug = 0;
int webcam = 0;

/* used in colorspace.c */
void *fast_memcpy(void *dest, const void *src, size_t n){
  return memcpy(dest,src,n);
}



#include "grab.h"
#include "channel.h"
#include "channels.h"
#include "strtab.h"

#ifdef HAVE_ZVBI
#include <libzvbi.h>
#endif

#ifdef HAVE_V4L_1
extern struct GRABBER grab_v4l;
#endif
#ifdef HAVE_V4L_2
extern struct GRABBER grab_v4l2;
#endif

typedef struct v4l_param_t {
  unsigned long freq;
  char *tvnorm;
  char *freqtab;
  int tvnorm_index;
  int freqtab_index;
  int fullscan;
  int forcev4l1;
  int forcev4l2;
/*
#ifdef HAVE_DVB 
  int forcedvb;
#endif
*/
#ifdef HAVE_ZVBI
  struct vbi_raw_decoder *vbi_par;
  size_t services;
  char *err;
#endif
} v4l_param_t;



int nbufs_default = 3, owidth, oheight,img;
int x11_bigendian = 0, x11_byteswap = 0, have_xv, nbufs, vbifd;
double videostampmin,videostampmax;
video_fmt x11_pixmap_format = 0, grab_format;
int get_ybar(int h) {return 0;}

struct device_t  device =
#ifdef HAVE_DEVFS
{"/dev/v4l/video0", "/dev/v4l/vbi0"};
#else
{"/dev/video0", "/dev/vbi0"};
#endif
extern struct STRTAB chan_names[];

char *tvname=NULL;
char *confname="xdtvrc.new";

#ifdef HAVE_ZVBI
extern int vbifd;

static void
event(struct vbi_event *ev, void *user)
{
  switch (ev->type) {
  case VBI_EVENT_NETWORK:
    if (NULL != tvname)
      free(tvname);
    if (strlen((const char*)(ev->ev.network.name)) > 0)
      tvname = strdup((const char*)(ev->ev.network.name));
    break;
  }
}
struct vbi_capture *vbi_cap;
struct vbi_decoder *vbi_dec;
void * vbi_data;
int vbi_lines;
vbi_sliced *vbi_sli;
#endif

void gettvname(char *channelname, int freq) {
  char tvname_unknown[100];
#ifdef HAVE_ZVBI
  double   vbi_timestamp;
  struct timeval vbi_timeout, initial_time, current_time;
#endif
  
  tvname=NULL;
#ifdef HAVE_ZVBI
  gettimeofday (&initial_time, NULL);
  for(;;) {
    int ret;
    long delay;
    gettimeofday (&current_time, NULL);
    delay=5000000-((1000000*current_time.tv_sec+current_time.tv_usec)-
		   (1000000*initial_time.tv_sec+initial_time.tv_usec));
    if(delay<0) break;
    vbi_timeout.tv_sec=delay/1000000;
    vbi_timeout.tv_usec=delay%1000000;
    fprintf(stderr,translate("channel=%s freq=%.3f timeout=%.1f  %c"),channelname,
	    (float)freq/16,(float)delay/1000000,'\r');
    fflush(stderr);
    ret=vbi_capture_read(vbi_cap, vbi_data, vbi_sli,
			 &vbi_lines, &vbi_timestamp, &vbi_timeout);
    if(ret==0) break;
    if(ret<0) {
      if(errno==EIO) {usleep(100000);continue;}
      perror("\nvbi_capture_read");
      break;
    }
    vbi_decode(vbi_dec, vbi_sli, vbi_lines, vbi_timestamp);
    if(tvname!=NULL) break;
  }
#endif
  if(tvname==NULL) 
    {
      if(channelname!=NULL)
	sprintf(tvname_unknown,translate("unknown (%s)"),channelname);
      else 
	sprintf(tvname_unknown,translate("unknown (%.3f MHz)"),(float) freq / 16);
      tvname = strdup(tvname_unknown);
    }
}


/* thanks to xawtv ! */
static int
menu(char *intro, struct STRTAB *tab, char *opt)
{
    int i,ret;
    char line[80];

    if (NULL != opt) {
	/* cmd line option -- non-interactive mode */
	for (i = 0; tab[i].str != NULL; i++)
	    if (0 == strcasecmp(tab[i].str,opt))
		return tab[i].nr;
	fprintf(stderr,translate("%s: not found\n"),opt);
	exit(1);
    }

    fprintf(stderr,"\n%s\n",intro);
    for (i = 0; tab[i].str != NULL; i++)
	fprintf(stderr,"  %2d: %s\n",tab[i].nr,tab[i].str);

    for (;;) {
	fprintf(stderr,translate("number ? "));
	fgets(line,79,stdin);
	ret = atoi(line);
	for (i = 0; tab[i].str != NULL; i++)
	    if (ret == tab[i].nr)
		return ret;
	fprintf(stderr,translate("invalid choice\n"));
    }
}

void
display_help_message (FILE *fp)
{
  fprintf(fp,
	translate("\nXdTV scantv options:\n\n"
	    "   -h           print this text.\n"
	    "   -o conffile  set output file.\n"
	    "   -n norm      set tv norm.\n"
	    "   -f table     set frequency table.\n"
	    "   -c videodev  set video device file.\n"
	    "   -C vbidev    set vbi device file.\n"
	    "   -v4l1        force v4l1 mode.\n"
	    "   -v4l2        force v4l2 mode.\n"
/*#ifdef HAVE_DVB	  
	    "   -dvb         force dvb mode.\n"
#endif	 
*/
	    "   -v           debug mode.\n"
	    "   -a           full scan (all frequencies, not just\n"
	    "                the ones from the frequency table)\n\n"));
}

#define INCRARG(i) if(++i>=argc) \
{display_help_message(stderr); exit(0);} arg=argv[i];

void  scan_v4l(v4l_param_t* param_analog, const char* confname , struct GRABBER *grab , int tvnorm , int freqtab, int output_to_file);
/*#ifdef HAVE_DVB	  
void  scan_dvb( const char* confname);
*/
int main(int argc, char **argv)
{
  int i;
  v4l_param_t param_analog;
  
  memset(&param_analog,0,sizeof param_analog);
  init_localisation();
    
  for (i = 1; i < argc; i++) 
    {
      char *arg=argv[i];
      if (strcmp (arg, "-c") == 0)
	{
	  INCRARG(i);
	  device.video=arg;
	}
      else if(strcmp (arg, "-C") == 0)
	{
	  INCRARG(i);
	  device.vbi=arg;	
	}
      else if(strcmp (arg, "-a") == 0)
	  param_analog.fullscan=1;
      else if(strcmp (arg, "-v") == 0)
	{
	  INCRARG(i);
	  debug=atoi(arg);
	}
      else if(strcmp (arg, "-o") == 0)
	{
	  INCRARG(i);
	  confname=arg;	
	}
      else if(strcmp (arg, "-v4l1") == 0)
	{
	  param_analog.forcev4l1=1;
	}
      else if(strcmp (arg, "-v4l2") == 0)
	{
	  param_analog.forcev4l2=1;
	}
/*
#ifdef HAVE_DVB	   
       else if(strcmp (arg, "-dvb") == 0)
	{
	  param_analog.forcedvb=1;
	}
#endif
*/
      else if(strcmp (arg, "-n") == 0)
	{
	  INCRARG(i);
	  param_analog.tvnorm=arg;
	}
      else if(strcmp (arg, "-f") == 0)
	{
	  INCRARG(i);
	  param_analog.freqtab=arg;
	}
      else {
	display_help_message(stderr);
	exit(1);
      }
    }
    
//    if (!param_analog.forcedvb)
    	scan_v4l(&param_analog , confname, NULL, -1 , -1 , 0);
/*#ifdef HAVE_DVB	   
    else {
    	scan_dvb(confname);
    }
#endif
*/


    return 0;
}

void  scan_v4l(v4l_param_t* param_analog, const char* confname ,struct GRABBER *grab, int tv_norm_index,int freqtab_index, int output_to_conffile){
  FILE *conf=NULL;
  int i;
  int close_grab_at_the_end=0;

#ifdef HAVE_V4L_2
  if(grab == NULL && !(param_analog->forcev4l1)) {
    grab = &(grab_v4l2);
    if(grab->grab_open(&device)==-1)
      grab = NULL;
    close_grab_at_the_end=1;
  }
#endif
#ifdef HAVE_V4L_1
  if(grab == NULL && !(param_analog->forcev4l2)) {
    grab = &(grab_v4l);
    if(grab->grab_open(&device)==-1)
      grab = NULL;
    close_grab_at_the_end=1;
  }
#endif

  if(grab==NULL) {
    fprintf(stderr,translate("cannot open video device\n"));
    exit(1);
  }
      
  if (tv_norm_index == -1)
  	param_analog->tvnorm_index=menu(translate("please select your TV norm"),grab->norms,param_analog->tvnorm);
  else 
	param_analog->tvnorm_index=tv_norm_index;

  if (freqtab_index == -1)
  	param_analog->freqtab_index=menu(translate("please select a frequency table"),
       chan_names,param_analog->freqtab);
  else
	param_analog->freqtab_index=freqtab_index;

  if (output_to_conffile){
	  conf=fopen(confname,"w");
	  if(conf==NULL) {perror("fopen:"); exit(1);}
	  fprintf(conf,"freqtab = %s\n",chan_names[param_analog->freqtab_index].str);
	  fprintf(conf,"source = Television\n");
	  fprintf(conf,"norm = %s\n\n",grab->norms[param_analog->tvnorm_index].str);
	  fflush(conf);
  }

  grab->grab_input(str_to_int("Television",grab->inputs),
		   param_analog->tvnorm_index);
  grab->grab_audio(0,65535,NULL);
#ifdef HAVE_ZVBI
  close(vbifd);vbifd=-1;
  param_analog->services= VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625
    | VBI_SLICED_TELETEXT_B | VBI_SLICED_CAPTION_525
    | VBI_SLICED_CAPTION_625 | VBI_SLICED_VPS
    | VBI_SLICED_WSS_625 | VBI_SLICED_WSS_CPR1204;
  vbi_cap=vbi_capture_v4l_new(device.vbi,16,&(param_analog->services),-1,&(param_analog->err),0);
  if(vbi_cap==NULL)
    {
      fprintf(stderr,translate("Unable to open vbi device\n"));
      exit(1);
    }
  vbi_dec=vbi_decoder_new();
  vbi_event_handler_add(vbi_dec,~0,event,NULL);
  param_analog->vbi_par=vbi_capture_parameters(vbi_cap);
  vbi_lines=(param_analog->vbi_par->count[0] + param_analog->vbi_par->count[1]);
  vbi_data=malloc(vbi_lines * (param_analog->vbi_par->bytes_per_line));
  vbi_sli = malloc(vbi_lines * sizeof(vbi_sliced));
#endif
  if(!(param_analog->fullscan))
    for(i=0;i<CHAN_ENTRIES;i++) {
      param_analog->freq=tvtuner[i].freq[param_analog->freqtab_index];
      if(param_analog->freq) {
	struct CHANNEL chan; // Temporary !!	
	chan.freq=param_analog->freq * 16 / 1000;
	fprintf(stderr,"%d/%d %s %.3lf   \r",i,CHAN_ENTRIES,tvtuner[i].name,(float)param_analog->freq/1000);
	fflush(stderr);
	grab->grab_tune(&chan);
	usleep(200000);
	if(grab->grab_tuned()) {
	  gettvname(tvtuner[i].name,param_analog->freq*16/1000);
	  printf(translate("channel=%s freq=%.3f tvname=%s        \n"),
		 tvtuner[i].name,(float)param_analog->freq/1000,tvname);
	  if (output_to_conffile){
	  	fprintf(conf,"[%s]\nchannel = %s\n\n",tvname,tvtuner[i].name);
	  	fflush(conf);
	  }
	}
      }
    } 
  else
    {
      int f,min_freq=44,max_freq=958;
      char *channel;
      for (f = min_freq*16; f <= max_freq*16; f += 4) 
	{
          struct CHANNEL chan; // Temporary !!	
	  chan.freq=f;
	  grab->grab_tune(&chan);
	  channel=NULL;
	  for(i=0;i<CHAN_ENTRIES;i++)
	    if(f*1000==tvtuner[i].freq[param_analog->freqtab_index]*16) 
	      {channel=tvtuner[i].name;break;}
	  fprintf(stderr,translate("freq=%.3f (min=%d max=%d) channel=%s     %c"),
		  (float) f / 16,min_freq,max_freq,channel,'\r');
	  usleep(200000);
	  if(grab->grab_tuned()) {
	    gettvname(channel,f);
	    printf(translate("freq=%.3f channel=%s tvname=%s        \n"),
		   (float)f/16,channel,tvname);
	    if (output_to_conffile){
	    	if(channel==NULL)
	      		fprintf(conf,"[%s]\nfreq = %.3f\n\n",tvname,(float) f / 16);
	    	else
	      		fprintf(conf,"[%s]\nchannel = %s\n\n",tvname,channel);
	    	fflush(conf);
	    }
	  }
	}
    }
  
#ifdef HAVE_ZVBI
  vbi_capture_delete(vbi_cap);
  vbi_decoder_delete(vbi_dec);
  free(vbi_sli);
  free(vbi_data);
  if(tvname!=NULL) free(tvname);
#endif

  if (close_grab_at_the_end)
  	grab->grab_close();
  if (output_to_conffile)
  	fclose(conf);
}

/* used in grab-v4l2.c..*/
void video_overlay
(int (*set_overlay)(int,int,int,int,int,struct OVERLAY_CLIP *,int)) {
}

/*
#ifdef HAVE_DVB
enum {CREATE_NEW_DVB_INIT,USE_STANDARD_DVB_INIT};

static struct STRTAB init_file_list[] = {
  {CREATE_NEW_DVB_INIT,"Get a new one from web (recommended)"},
  {USE_STANDARD_DVB_INIT,"Use dvb_init.conf in ~/.xdtv"},
  {-1, NULL}
};

static char* construct_dvb_file_name () {
	char *homedir = getenv ("HOME");
	char* dvb_init_conf=NULL;
	if (!homedir)
		perror ("$HOME not set");
	if (NULL == (dvb_init_conf = malloc ( (strlen(homedir) + strlen(DVB_TRANSPONDER_FILE) + 1)* (sizeof(*dvb_init_conf))))){
		perror("malloc");
		return NULL;
	}
	memcpy (dvb_init_conf, homedir, strlen(homedir));
	memcpy (dvb_init_conf+ strlen(homedir),DVB_TRANSPONDER_FILE , 
        	strlen(DVB_TRANSPONDER_FILE) + 1);

	return dvb_init_conf;

}
static void  create_dvb_file_from_web (char* dvb_init_conf) {

  transp_file** all_files=NULL;
  struct STRTAB* country_list=NULL;
  struct STRTAB* current_city_list=NULL;
  char current_country[3]="XX";
  char* current_city=NULL;
  int country_number=0;
  int city_number=0;

  if (NULL == dvb_init_conf)
	  return;
  if (NULL == (all_files = get_initfiles_list () )){ 
  	fprintf(stderr,"cannot get files from linuxtv.org\n");
	return ;
  }
  if (NULL == (country_list=get_country_list(all_files))){
  	fprintf(stderr,"cannot get list of country\n");
	return ;
  }

  country_number = menu(translate("please select your country "), country_list , NULL );

//  fprintf(stderr,"country_number = %d\n",country_number);

  strncpy(current_country , int_to_str(country_number , country_list), 3);

  if (NULL == (current_city_list=get_city_list(all_files, current_country ))){
  	fprintf(stderr,"cannot get list of country\n");
	return ;
  }
  city_number = menu(translate("please select the location of your emitter"), current_city_list , NULL );

  current_city=strdup(int_to_str(city_number,current_city_list));
  download_and_save_initfile(dvb_init_conf, all_files , current_country, current_city);

} 

void  scan_dvb( const char* confname){

  char *dvb_init_conf = NULL;
  FILE* conf=NULL;
  int create_new_init_file=0;

  dvb_init_conf = construct_dvb_file_name();
  if (NULL == dvb_init_conf){
  	fprintf(stderr, "dvb_init_conf = NULL\n");
	return;
  }

  translate_struct(init_file_list);
  create_new_init_file = menu(translate("DVB scan need an initial file : "),init_file_list , NULL );

  if (create_new_init_file == CREATE_NEW_DVB_INIT ) {
  	create_dvb_file_from_web(dvb_init_conf) ;
  }

  conf=fopen(confname,"w");
  if(conf==NULL) {perror("fopen:"); exit(1);}

  scan_main(dvb_init_conf , conf);

  // FIXME !! !free (all_city etc. !!)
  fclose(conf);
  return;
  
}
#endif // HAVE_DVB
*/
